///////////////////////////////////////////////////////////////////////////////
// Quake.cpp
///////////////////////////////////////////////////////////////////////////////



#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include "deep.h"
#include "windeep.h"
#pragma hdrstop

#include <dlgs.h>
#include <commctrl.h>
#include <winspool.h>
#include <commdlg.h>

#include "QuakDEFS.H"

void playWAVFile(HWND hwnd,char* FileName, int4 offset);
void CloseWAVE(void);


typedef struct                 // internal directory entry for BSP
{ char  name[16];              // name generated by me
  int4  start;                 // Offset to entry, in bytes, from start of file
  int4  size;                  // Size of entry in file, in bytes
} QuakeBspDirEntry;

typedef  QuakeBspDirEntry *QuakeBspDirPtr;

//==============================================================================

typedef struct             // QDATA - for passing information to Quake Windows
 { char   name[32];        // The name of the Texture passed to the Browser
 //Bool   b1;
 //int2   i1;  etc. etc.
 } QDATA,  *LPQDATA;


// MOST in Newgraph.cpp

//void ReleaseMouseButton(HWND hwnd);  // in mouse.cpp put in windeep.h   3-1-97

// Local Proto's
static BOOL CALLBACK QViewBspDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
static BOOL CALLBACK QViewPakDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
static BOOL CALLBACK QViewWadDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);  // uses qviewbsp dialog
//static BOOL CALLBACK QViewHexDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam); not yet


static BOOL CALLBACK QLumpBspViewDlgProc   (DIALOG_PARAMS); // Display a BSP lump
static BOOL CALLBACK QLumpWadViewDlgProc   (DIALOG_PARAMS); // Display a WAD lump
static BOOL CALLBACK QLumpPakViewDlgProc   (DIALOG_PARAMS); // Display a PAK lump  (raw dump + export option)

//       BOOL CALLBACK HexDumpDisplayDlgProc (DIALOG_PARAMS); // this does all hex dumps now
//static BOOL CALLBACK QLumpBspDisplayDlgProc(DIALOG_PARAMS); // Actual display Modeless
//static BOOL CALLBACK QLumpPakDisplayDlgProc(DIALOG_PARAMS); // Actual display Modeless
//static BOOL CALLBACK QLumpWadDisplayDlgProc(DIALOG_PARAMS); // Actual display Modeless


// Windows95 Quake Stuff
static QuakeBspHeader QuakeBspControl;                       // control header start of BSP (for 1 bsp file only)


static Bool        MakeNewQuakePal(HWND hwnd,int2 newgamma);
static RGBQUAD     qRGBquad[257];        //quake Quad RGB copy for dib

static HPALETTE    hQuakePal= NULL;   // Quake palette
static char        QViewMsg[128]   = {0};// Dialog Update Text from DisplayQMipTex
static int2        QuakeGamma= 16;    // Quake Gamma Correct for pictures
static Bool        Zoom     = FALSE;  // Temporary for zooming Quake Texture
static int2        Qmode;             // For Paint- 1=wad,2=pak,3=bsp
static char        Qname[20]   = {0}; // Quake WAD Graphic file name - Global for Dlg routines
static char        QBSPname[24]= {0}; // Quake Graphic file name - Global for Dlg routines
static char        bspname[24] = {0}; // Quake PAK-BSP name
static char        QPAKname[60]= {0}; // Quake PAK name - Global for Dlg routines
static int4        BSPoffset;

//static char        BrowseName[24] = {0}; // Browser returned name
// No longer required, now passed in lParam info and LPCREATESTRUCT

static HWND        hBrowseWindow = NULL;
static LRESULT     LumpIndex;
static char        LumpName[62] = {0};
static Bool        Sorted = FALSE;

//static HWND      hLumpModeless = NULL; // Export/Print/View

extern int4        ImageXSize;        // X already 32 bit aligned
extern int4        ImageYSize;        // Extern in Newgraph.cpp for SavePcx sizes

static Bool        OpenQuakePakFile(HWND hwnd);
static Bool        OpenQuakeBspFile(HWND hwnd);
static Bool        OpenQuakeWadFile(HWND hwnd);

static void        CloseAllQuakeFiles(void);
static void        CloseQuakeBspFile(void);
static void        CloseQuakeWadFile(void);
static void        CloseQuakePakFile(void);

static QuakeWadDirPtr FindQuakeWadDir(char *name);
static QuakePakDirPtr FindQuakePakDir(char *name);
static QuakeBspDirPtr FindQuakeBspDir(char *name);

static Bool        SetQuakePalette(HWND hwnd, FILE *PalFile,int4 offset, int2 Mode);

static void        QuakeSaveLumpInFile(HWND hwnd, FILE *infile,char*name,char*suffix,int4 namelen, int4 start, int4 size);


////////// BSP only ///////////////////
static int4   BSPlistcount;                   // count of number of files
static char **BSPlistptrs ;                   // ptr to array of file names
static char  *BSPlistnames;                   // array of char names with info

// global for return to BROWSE_BACK
static char  *BrowseListWork;
static int4   BrowseListCount;                // count of number of files
static char **BrowseListPtrs ;                // ptr to array of file names
static char  *BrowseListNames;                // array of char names with info

//////////////////////////////////////////////////////////////////////
/// local global for PAK-BSP graphics

static int4   PAKlistcount;                   // count of number of files
static char **PAKlistptrs ;                   // ptr to array of file names
static char  *PAKlistnames;                   // array of char names with info

static int4   PAKBSPlistcount;                // count of number of files
static char **PAKBSPlistptrs ;                // ptr to array of file names
static char  *PAKBSPlistnames;                // array of char names with info

void  ViewQuakePakBspGraphics(char*bspname);
void  QuakePakTextureViewer(char*bspname);


static Bool FindQuakeMipPicSize(FILE *MipFile, char*picname, int4 offset,
		                                         int4 *xsize,   int4 *ysize);

static void CreateQuakeWadTextureList (void);

//////////////////////////////////////

// gets the names from the bsp in a pak
//static int2 QuakePAKBSPTexture    ( int2 xstart, int2 ystart, int2 xend, int2 yend,
//                             int2 linesinlist, int2 currentline, char *bspname);
//bsp in a pak
//static int2 ViewQuakePAKBSPTexture( int2 xstart, int2 ystart, int2 xend, int2 yend,
//                             int2 linesinlist, int2 currentline, char *picname);
static void CreateQuakePakTextureList (void);
// bsp only
//static int2 ViewQuakeBSPTexture   ( int2 xstart, int2 ystart, int2 xend, int2 yend,
//                             int2 linesinlist, int2 currentline, char *picname);

static void CreateQuakeBspTextureList (void);

static int2 DisplayQuakeMipTexture(HWND hwnd, HDC hdc, //Bool Saving,
                            FILE *TextureFILE,
                            char *TextureFileName,
                            int4 offset,
                            int2 xstart,int2 ystart,
                            int2 xend,  int2 yend,
                            int4 *RawSize,
                            byte**RawData,
                            int4 *ImageSize,
                            byte**ImageBufr,
                            HBITMAP hDibbm,
                            char *picname,
                            int2  magnify,
                            Bool  ShowInfo,
                            int2  fillcolor,           // transparent use
                            Bool  ShowImage);


static char * BrowseQuakeMipTextures (HWND hwnd, char *UserName);

//---------- local global access -----------------------------------------------
static  int2           QuakePicMagnify  =3;

static  int4           MipValue = 0;             // 0,1,2,3 size of texture shift
static  byte           QuakePaletteBufr[768];    // palette from Quake .grp or directory   3-1-97 simpler and ends up same size no msgs rqd

//////////////////
static  FILE          *QuakeBspFile;             // Quake Bsp file--global--
static  int2           QuakeBspStatus    = 'C '; // open or closed
static  int4           QuakeBspDirNum    = 0;    // number of dir entries in BSP (my own really)
static  QuakeBspDirPtr QuakeBspInternalDir=NULL; // area containing my Quake BSP Directory
//////////////////
static  FILE          *QuakeWadFile;             // Quake Wad file--global--
static  int2           QuakeWadStatus    = 'C '; // open or closed
static  int4           QuakeWadDirNum    = 0;    // number of dir entries in Wad
static  QuakeWadDirPtr QuakeWadInternalDir=NULL; // area containing my Quake Directory
//////////////////
static  FILE          *QuakePakFile;             // Quake Wad file--global--
static  int2           QuakePakStatus    = 'C '; // open or closed
static  int4           QuakePakDirNum    = 0;    // number of dir entries in PAK
static  QuakePakDirPtr QuakePakInternalDir=NULL; // area containing my Quake Directory

//////////////////
/*
//--------------------------------------------------------------------------------
// local globals for drawing routines hacked so they can share

static FILE *devout;             // output device printing

//----------------------------------------------------------------------------


static void DumpQuakeWadLump (int2 lumpnum,char *name);    // dump a dir entry
static void PrintQuakeWadLump(int4 Start, int4 Size,int4 offset, int2 lumpnum,char *name);

static void DumpQuakePakLump (int2 lumpnum,char *name);    // dump a dir entry
static void PrintQuakePakLump(int2 lumpnum,char *name);
*/
//------------------------------------------------------------------------------

int2 QuakeColors[30] = {
0,   //Black       0
208, //Blue        1 xxxxxx  (64 old)
181, //Green       2 xxxxxx
47 , //Cyan        3 xxxxxxxxx
251, //Red         4 xxxxxxxx
147, //Magenta     5 xxxxxxxx
202, //Brown       6 xxxxxxx
10,  //LightGray   7   ccc            22 without gamma
8  , //DarkGray    8   ccc            11 ......  .....
44 , //LightBlue   9   xxx  70
176, //LightGreen  10  xxxx
45 , //LightCyan   11  xxxx
233, //LightRed    12  xxx
145, //LightMagenta13 xxx
111, //Yellow      14  xxx
246, //White       15  xxxx
 7 , // gray       16 xxxxx
 4 , // darkergray 17 xxxxx          9  ...... .......
247, // darkred    18 xxxxxxx
184, // darkgreen  19 xxx   brown really 109
218, // darkblue   20 xxx
152, // darkmagenta21 xxx
108, // orange     22  xx
197, // lightbrown 23  xx            58 without gamma
107, // SALMON       1  24   xx
110, // LIGHTYELLOW
244, // LIGHTERBLUE  3  26=  xxxxxx     mainlines    45?
245, // LIGHTERGREEN 4  27=  xxxxxx     pwads      119
234, // LIGHTERRED   5  28= map browser pwad lines
12 , // LIGHTERGRAY  6  29= menu hollow lines
};
///////////////////////////////////////////////////////////////////////////////
// offset of 4
///////////////////////////////////////////////////////////////////////////////
//static int QsortListName16Offset4( const void *a, const void *b)
//{
//   return strnicmp ( * ( (char **) a)+4, *( (char **) b)+4,16  );
//}
static int QsortListName16( const void *a, const void *b)
{
   return strnicmp ( * ( (char **) a), *( (char **) b),16  );
}

static int QsortListName10( const void *a, const void *b)
{
   return strnicmp ( * ( (char **) a), *( (char **) b),10  );
}

static int QsortListName50( const void *a, const void *b)
{
   return strnicmp ( * ( (char **) a), *( (char **) b),50  );
}


//============================================================================//
// Free browse lists
//============================================================================//
static void FreeBrowseList(void)
{
   FreeMemory(BrowseListNames);
   BrowseListNames = NULL;
   FreeMemory(BrowseListPtrs);
   BrowseListPtrs = NULL;
}


//12-30-96 bk Quake Developement
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//  Q U A K E  Developement, View/Export/Print/Hex Dump Wad,Pak,Bsp    //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
#pragma argsused
int2 QuakeDevelopment(HWND hwnd, POINT p, int2 choice)
{
FILE  *PalFile;
DLG2OPTS   d2;				           		// our structure for passing info to dialogs
//QuakeBspHeader  BspHeader;           // control header start of BSP
MipHeader  MipHeader;                  // within BSP #of mips and offsets
char       PalFilename[80];
//QuakeGraphicHdr QuakeImageHeader;           // header start of mip
//int code; //debug

 ZeroMemory(&d2, sizeof(d2));

   switch (choice)
   {
     // Set Sort Options
     case 1:
       if(Confirm(hwnd," Sort the ListBox's ?"))  Sorted = TRUE;
       else                                  Sorted = FALSE;
       break;
     // Display View/Export/Print/Hex  Wad Graphic
     case 2:
       d2.b1 = 0;                     // 0=Wad, 1=Pak, 2=Bsp
       d2.b2 = Sorted;
       Qmode = 1;
       if (!OpenQuakeWadFile(hwnd) ) break;  // get dir entries
       if (QuakeMode == 1)
       {   strcpy(PalFilename,"QUAKE.PAC");
       }
       else
       {   strcpy(PalFilename,"HEXENII.PAC");
       }
       PalFile = FileOpen( PalFilename,"rb",OPENMISS);

       if (!PalFile || !SetQuakePalette(hwnd, PalFile,0,SETCOLORS))
       {   if (!PalFile)
           {   ErrorMsg(hwnd,"%s Not found!\n\n"
                              "This is the QUAKE/HEXENII palette!\n"
                              "Please find this file and\n"
                              "Put it in the DeeP97 directory",PalFilename);
           }

           CloseQuakeWadFile();
           fclose ( PalFile );
           break;
       }
       fclose ( PalFile );

       //Trace(NULL,600,64,"qviewwad");
       DialogBoxParam( gd.hInst, "QViewBsp", hwnd, QViewWadDlgProc, (LPARAM)(LPDLG2OPTS)&d2 );
       //Trace(NULL,600,64,"qviewwad init code= %d",code);
       break;
     // Display View/Export/Print/Hex  Pak Graphic
     case 3:
       d2.b1 = 1;
       d2.b2 = Sorted;
       Qmode = 2;
       if (!OpenQuakePakFile(hwnd) ) break;  // get dir entries

       if (QuakeMode == 1)
       {   strcpy(PalFilename,"QUAKE.PAC");
       }
       else
       {   strcpy(PalFilename,"HEXENII.PAC");
       }
       PalFile = FileOpen( PalFilename,"rb",OPENMISS);

       if (!PalFile || !SetQuakePalette(hwnd, PalFile,0,SETCOLORS))
       {   if (!PalFile)
           {   ErrorMsg(hwnd,"%s Not found!\n\n"
                              "This is the QUAKE/HEXENII palette!\n"
                              "Please find this file and\n"
                              "Put it in the DeeP97 directory",PalFilename);
           }
           CloseQuakePakFile();
           fclose ( PalFile );
           break;
       }
       fclose ( PalFile );

       DialogBoxParam( gd.hInst, "QViewPak", hwnd, QViewPakDlgProc, (LPARAM)(LPDLG2OPTS)&d2 );
       break;
     // Display View/Export/Print/Hex  Bsp Graphic
     case 4:
       d2.b1 = 2;
       d2.b2 = Sorted;
       Qmode = 3;
       if (!OpenQuakeBspFile(hwnd) ) break;                         // get dir entries

       WadSeek( QuakeBspFile, 0);
       WadRead( QuakeBspFile, &QuakeBspControl,sizeof(QuakeBspControl) );  // read header
       WadSeek( QuakeBspFile,  QuakeBspControl.miptex.offset);
       WadRead( QuakeBspFile, &MipHeader,sizeof(MipHeader) );        // read entries header

       if (MipHeader.numtex <0 || MipHeader.numtex > 256)           // bad data
       {   ErrorMsg(hwnd,"Number of entries bad? %d",MipHeader.numtex);
           break;                                                   // bad data
       }

       //===== do pal ================

       if (QuakeMode == 1)
       {   strcpy(PalFilename,"QUAKE.PAC");
       }
       else
       {   strcpy(PalFilename,"HEXENII.PAC");
       }
       PalFile = FileOpen( PalFilename,"rb",OPENMISS);

       if (!PalFile || !SetQuakePalette(hwnd, PalFile,0,SETCOLORS))
       {   if (!PalFile)
           {   ErrorMsg(hwnd,"%s Not found!\n\n"
                              "This is the QUAKE/HEXENII palette!\n"
                              "Please find this file and\n"
                              "Put it in the DeeP97 directory",PalFilename);
           }

           CloseQuakeBspFile();
           fclose ( PalFile );
           break;
       }
       fclose ( PalFile );

       DialogBoxParam( gd.hInst, "QViewBsp", hwnd, QViewBspDlgProc, (LPARAM)(LPDLG2OPTS)&d2 );

       break;
     // raw dump of any lump in wad=============
     case 5:
       if (!OpenQuakeWadFile(hwnd) ) break;      // get dir entries
       DialogBoxParam(gd.hInst, "QViewLump", hwnd, QLumpWadViewDlgProc, 0);
       break;
     // raw dump of any lump in pak=============
     case 6:
       if (!OpenQuakePakFile(hwnd) ) break;      // get dir entries
       DialogBoxParam(gd.hInst, "QViewLump", hwnd, QLumpPakViewDlgProc, 0);
       break;
     // raw dump of lump in bsp =============
     case 7:
       if (!OpenQuakeBspFile(hwnd) ) break;      // get dir entries
       DialogBoxParam(gd.hInst, "QViewLump", hwnd, QLumpBspViewDlgProc, 0);
       break;
     // EXIT MENU
     case 8:
       CloseAllQuakeFiles();
       CloseWAVE();        // stop any wave
       return FALSE;

     //case 5:  // Palette
   }
   
   if (QuakeMode == 1)
   {   HandleQuakeMenu(hwnd);   // 2-20-97 bk     STAY IN MENU
   }
   else
   {   HandleHexen2Menu(hwnd);  // 8-1-97    STAY IN MENU
   }

   return 1;
}



///////////////////////////////////////////////////////////////////////
//                                                                   //
//   Quake File routines, Open / Close Pak, Wad, Bsp                 //
//                                                                   //
///////////////////////////////////////////////////////////////////////
      // copy of old stuff with   mods, but then not used   3-1-97
      // bk : we don't have to worry about making a new palette like we do in doom,
      // we're just worried about displaying the DIB properly, so just initialize
      // it's color table properly and we're set.
      /*
      int2  LocalGammaCorrect;
      uint2 red,green,blue;

	   for (n = 0; n < 256; n++)
	   {  LocalGammaCorrect= QuakeGamma;
	     	// a copy of the quake Palette for Global DIB access
         red  = QuakePaletteBufr[n*3+0];
         green= QuakePaletteBufr[n*3+1];
         blue = QuakePaletteBufr[n*3+2];
         if ( (red+green+blue)== 0 ) // not black!
         {   LocalGammaCorrect = 0;
         }

	    	qRGBquad[n].rgbRed   = min(red  +LocalGammaCorrect,255);
		   qRGBquad[n].rgbGreen = min(green+LocalGammaCorrect,255) ;
		   qRGBquad[n].rgbBlue  = min(blue +LocalGammaCorrect,255) ;
		   qRGBquad[n].rgbReserved = 0;
      } */

      //QuakeGamma = NewGamma;

/////////////////////           DIALOGS          ///////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//  The Quake Display Graphic in a Wad file
/////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QViewWadDlgProc(DIALOG_PARAMS)
{
 static QuakeWadDirPtr TempDir;

 static  char  *listwork;
         int4   n;
 static  int4   listcount;                // count of number of files

 static  char **listptrs ;                // ptr to array of file names
 static  char  *listnames;                // array of char names with info

         int    Index;
         RECT   rc;
         int2   code;
         HDC    hdc;
//       int2   NewGamma;         // the New Quake Gamma Correct for pictures
//       int2   change;           // change in gamma up or down
         int2   i;

         int    nResult;

static VOID **    lpSBits = NULL; // Pointer to bits in DIBSECTION for SavePCX
static HBITMAP    hSDibbm = NULL; // Handle to the Dib's Bitmap returned to us
static BITMAPINFO Lumpbi;			 // Our DIB Bitmap Info Struct.
       HPALETTE   hOldPal;

       char       TempName[24] = {0};
static QDATA      qdat;           // Pass String to our Browse Window
                                  // don't have to use a global variable then
//-------------------------------------------------------------------------

 ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

 ZeroMemory(&qdat, sizeof(qdat));

//Trace(NULL,600,64,"qviewwad init msg = %d",msg);

 switch( msg )
 {
    case WM_INITDIALOG:
	   SetWindowLong(hDlg, DWL_USER, lParam);		// store our Pointer

      //------------------------------------------------
      // store names in list for dialog selection
      //------------------------------------------------

      listptrs  = (char **)GetMemory( QuakeWadDirNum * sizeof( char*));    // n pointers allocated
      listnames = (char  *)GetMemory( QuakeWadDirNum * 18);                // 18 byte allocations

      listwork  = listnames;                                   // where to store
      listcount = 0;                                           // number of entries

      TempDir = QuakeWadInternalDir;                           // dir buffer

      for (n = 0; n < QuakeWadDirNum; n++,TempDir+=1)
      {
         if (!strnicmp(TempDir->name,"palette",7)) continue;    // not the palette

         memset(listwork,0,18);

         strncpy(listwork,TempDir->name,16);      // name
         listptrs[listcount] = listwork;          // addr of saved
         listcount++;                             // +1 saved
         listwork+=18;                            // next avail loc
      }

      if (Sorted)
      {   qsort( listptrs, listcount, sizeof( char *), QsortListName10);
      }

      for(n = 0; n < listcount; n++)
      {   ListBox_AddString(GetDlgItem(hDlg, lst1), listptrs[n]);
      }

      strncpy ( Qname,(char *) listptrs[0],16); // first entry default

      // set the focus to our list box and then select item 0 or our Current Map
      SetFocus(GetDlgItem(hDlg, lst1));
      SendMessage(GetDlgItem(hDlg, lst1), LB_SETCURSEL, 0, 0);

      // Send SelChange Message to initialize MAKEPARAM(Low,High) Low=ID of EditList, High=LBN_SELCHANGE

      SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,
                  lst1), GWL_ID), LBN_SELCHANGE ), 0);

      MipValue      = 0;                       // start with biggest format

	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETRANGE, (WPARAM)0, (LPARAM) MAKELONG((short) 64, (short) 0));
	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETPOS,   (WPARAM)0, (LPARAM) MAKELONG((short) QuakeGamma, 0));

      LastPicXloc = -1;
      return FALSE; // we've set our own focus

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);//   6-1-97
      break;
    //   6-1-97      changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

	case WM_VSCROLL:  // Our Gamma Correct Up/Down Spinner msg
      QuakeGamma = (int2)SendMessage(GetDlgItem(hDlg, IDD_UPDOWNX), UDM_GETPOS, 0, 0);
      MakeNewQuakePal(hDlg,QuakeGamma);

      InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
      UpdateWindow(GetDlgItem(hDlg,rct1));
      break;

    case WM_COMMAND:

      //lpd2 = (LPDLG2OPTS)GetWindowLong(hDlg, DWL_USER);  	// grab our pointer

      switch LOWORD(wParam) 							// handle of the control
      {
         case lst1:		// Out list box was chosen in some fashion

           switch HIWORD(wParam)
			  {
				 case LBN_SELCHANGE:
       		  Index = (LRESULT)SendMessage(GetDlgItem(hDlg, lst1), LB_GETCURSEL, 0, 0);
         	  ListBox_GetText(GetDlgItem(hDlg, lst1), Index, Qname);

              TempDir = FindQuakeWadDir(Qname);
              if (!TempDir || TempDir->size  <=  0 )             // no size
              {    NotifyMsg(hDlg, "Not found or zero length!");
                   break;
              }

              hdc = GetDC(GetDlgItem(hDlg, rct1));              // our picture rect
              hOldPal = SelectPalette(hdc, hQuakePal, FALSE);

  	           RealizePalette(hdc);
	           GetClientRect(GetDlgItem(hDlg, rct1), &rc);

              DisplayQuakeMipTexture(GetDlgItem(hDlg, rct1),
                                     hdc,
                                     //FALSE,
                                     QuakeWadFile,
                                     QuakeWadFileName,
                                     TempDir->start,//offset
                                     0,0,
                                     rc.right, rc.bottom,
                                     NULL, NULL, NULL, NULL,NULL,
                                     Qname,
                                     QuakePicMagnify,
                                     SHOWINFO,
                                     QuakeColors[GRAY],//MenuColor],
                                     SHOWIMAGE);

              //InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
              //UpdateWindow(GetDlgItem(hDlg,rct1));

              SelectPalette(hdc, hOldPal, FALSE);
              ReleaseDC(GetDlgItem(hDlg, rct1), hdc);
				  break;

				 default:
				  return FALSE;
           }
		     break;

         case WM_CHANGE:
           Static_SetText(GetDlgItem(hDlg, stc1), QViewMsg);
           break;

         //case IDOK:                       // Exit Button
         // Exit button
         case IDCANCEL:
           FreeMemory(listnames);
           FreeMemory(listptrs);
           FreeBrowseList();     // free browse list now

           SetQuakePalette(hDlg, NULL, 0, RESTORECOLORS);
           CloseQuakeWadFile();
           EndDialog( hDlg, TRUE );
           return TRUE;
         // Toggle Mip
         case psh1:
           MipValue++;
           if (MipValue > 3) MipValue = 0;
           InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
           UpdateWindow(GetDlgItem(hDlg,rct1));
           return TRUE;
         // Export PCX
         case psh2:
         // Export BMP
         case psh7:
           TempDir = FindQuakeWadDir(Qname);

           if (!TempDir || TempDir->size  <=  0 )             // no size
           {    NotifyMsg(hDlg, "Not found or zero length!");
                 break;
           }

           hdc = GetDC(GetDlgItem(hDlg, rct1));
           GetClientRect(GetDlgItem(hDlg, rct1), &rc);

    	     // Set up a BITMAPINFO structure for use with CreateDIBSection()
           ZeroMemory(&Lumpbi, sizeof(BITMAPINFO));
           Lumpbi.bmiHeader.biSize        = sizeof(BITMAPINFOHEADER);
           Lumpbi.bmiHeader.biPlanes      = 1;
           Lumpbi.bmiHeader.biBitCount    = 8;
           Lumpbi.bmiHeader.biWidth       =  ImageXSize;  // global size
    	     Lumpbi.bmiHeader.biHeight      = -ImageYSize;
	        Lumpbi.bmiHeader.biSizeImage   =  ((ImageXSize+3)& ~3 )* ImageYSize;
    	     Lumpbi.bmiHeader.biCompression = BI_RGB;

           // Create a DC to hold our DIBSECTION
           hSDibbm = (HBITMAP)CreateDIBSection(hdc, &Lumpbi, DIB_PAL_COLORS, (VOID **)&lpSBits, NULL, 0L);

           if (hSDibbm)    //   8-1-97
           {
               code =DisplayQuakeMipTexture(hDlg, NULL, //TRUE,
                                           QuakeWadFile,
                                           QuakeWadFileName,
                                           TempDir->start,   //offset
                                           0,0,
                                           rc.right, rc.bottom,
                                           NULL,
                                           NULL,
                                           NULL,// Imagesize
                                           (byte **)&lpSBits, hSDibbm,
                                           (char *)Qname,
                                           1,
                                           NOSHOWINFO,
                                           BLACK,
                                           NOSHOWIMAGE);
           }
           else
           {   code = -1;
           }

           if (code == 1)
           {
               // psh2 = PCX
               if ( LOWORD(wParam) == psh2)
               {
	                SavePCX(hDlg, Qname, (byte *)&*lpSBits, (byte *)QuakePaletteBufr);    // use quake palette
               }
               else
               // psh7 = BMP save the DIB to disk file
               {
                  nResult = SaveDIB(hDlg,&Lumpbi, lpSBits, Qname, qRGBquad);

	               if (nResult)
                  {   DIBError(nResult);
                  }
               }
           }
           else
           {   NotifyMsg(hDlg, "Possibly Not enough Memory!");
           }

           if(hSDibbm)
           {  DeleteBitmap(hSDibbm);
              hSDibbm = NULL;
              lpSBits = NULL;
           }

           ReleaseDC(hDlg, hdc);
           return TRUE;

         // View/Print/Save Lump
         case psh3:
           DialogBoxParam(gd.hInst, "QViewLump", hDlg, QLumpWadViewDlgProc, 0);
           break;
         // Zoom Graphic Temp for now
         case psh5:
           QuakePicMagnify++;
           if(QuakePicMagnify > (MipValue+1)*4 ) QuakePicMagnify = 1;

           // Send SelChange Message
           SendMessage(hDlg, WM_COMMAND,
                       MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,lst1), GWL_ID), LBN_SELCHANGE ), 0);

           break;
         // Browse
         case psh6:
           CreateQuakeWadTextureList ();  // create list of textures
           strcpy(qdat.name, Qname);
           gd.hParent = hDlg;             // For WM_BROWSE_BACK below

           hBrowseWindow = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW | WS_EX_DLGMODALFRAME,                //   12-8-96
                           "QBrowse",
                           "Browse Quake Textures - F1 for Help",
                           WS_POPUP | WS_CAPTION,
                           0, 0,
                           200, 200,
                           hDlg, NULL, gd.hInst, &qdat);

           ShowWindow(hBrowseWindow, SW_MAXIMIZE);
           UpdateWindow(hBrowseWindow);
           break;

	   	case WM_BROWSE_BACK:	//After Browsing we return here PostMessage(hParent, WM_BROWSE...
           strcpy(Qname, (char *)(LPARAM)lParam);
           Index = 0;
           for (i=0; i< BrowseListCount; i++)
           {   strncpy(TempName,BrowseListPtrs[i],16);
	            if (!strcmp (TempName, Qname) )
	            {  Index = i;
                   break;
	            }
           }

           SendMessage(GetDlgItem(hDlg, lst1), LB_SETCURSEL, Index, 0);
           SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,
                       lst1), GWL_ID), LBN_SELCHANGE ), 0);
           break;
		 }

       break;
  }
   return FALSE;
}



////////////////////////////////////////////////////////////////////////////////
// Grab the PAK names for our First List Box
// and initialize our pointers
//
// Called from SelChange ( Selection Changed ) and InitDialog
//
// Also fills lst1 with current entries
////////////////////////////////////////////////////////////////////////////////
static Bool GetPakNames(HWND hDlg)
{
 static QuakePakDirPtr TempDir;
        char * PAKlistwork;
        char * dotp;                // array of char names with info
        char   name[60];
        int2   n;
//==============================================================================

     //------------------------------------------------
     // store names in list for dialog selection
     //------------------------------------------------

     PAKlistptrs  = (char **)GetMemory( QuakePakDirNum * sizeof( char*));  // n pointers allocated
     PAKlistnames = (char  *)GetMemory( QuakePakDirNum * 60);              // 60 byte allocations

     PAKlistwork  = PAKlistnames;                                // where to store
     PAKlistcount = 0;                                           // number of entries

     TempDir = QuakePakInternalDir;                              // dir buffer

     for (n = 0; n < QuakePakDirNum; n++,TempDir+=1)
     {
        // see if name ends in .bsp
        strncpy(name,TempDir->name,56);                     // name

        dotp = strrchr( name,'.');                          // find . (reverse)
        if (!dotp || strnicmp(dotp,".bsp",4)  ) continue;

        memset(PAKlistwork,0,60);

		  //sprintf(PAKlistwork, "%-50s",name);

        strncpy(PAKlistwork,name,56);               // name
        PAKlistptrs[PAKlistcount] = PAKlistwork;    // addr of saved

        PAKlistcount++;                             // +1 saved
        PAKlistwork+=60;                            // next avail loc
      }

      if (PAKlistcount <=0)
      {
          ErrorMsg(hDlg,"No Bsp Files Found");
          return FALSE;
      }

      if (Sorted)
      {   qsort( PAKlistptrs, PAKlistcount, sizeof( char *), QsortListName10);
      }

      for(n = 0; n < PAKlistcount; n++)
      {   ListBox_AddString(GetDlgItem(hDlg, lst1), PAKlistptrs[n]);
      }

      strncpy ( QPAKname, (char *) PAKlistptrs[0], 24); // first entry default

      return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// Grab the BSP names from the current PAK selection
// and initialize our pointers
//
// Called from SelChange ( Selection Changed ) in our
// first list box. Means a new PAK entry has been chosen
// so we need a new list of entries in lst2
////////////////////////////////////////////////////////////////////////////////
static Bool GetPakBspNames(HWND hwnd)
{
  QuakeWadDirPtr  TempDir;
  QuakePakDirPtr  BspDir;
  QuakeBspHeader  BspHeader;                  // control header start of BSP
  MipHeader       MipHeader;                  // within BSP #of mips and offsets
  QuakeGraphicHdr QuakeImageHeader;           // header start of mip

 static  char  *listwork;
         int4   n;
         int4   Offset;

//==============================================================================


   BspDir = FindQuakePakDir(QPAKname);
   if (!BspDir) return FALSE;

   WadSeek( QuakePakFile, BspDir->start);
   WadRead( QuakePakFile, &BspHeader,sizeof(BspHeader) );       // read header
   if (BspHeader.version > 29)
   {   ErrorMsg(hwnd,"Version > 29 %d",BspHeader.version);
       return FALSE;
   }

   WadSeek( QuakePakFile, BspDir->start+BspHeader.miptex.offset);
   WadRead( QuakePakFile,&MipHeader,sizeof(MipHeader) );        // read entries header

   if (MipHeader.numtex <0 || MipHeader.numtex > 256)           // bad data
   {   ErrorMsg(hwnd,"Number of entries bad? %d",
                       MipHeader.numtex);
       return FALSE; // bad data
   }



   //------------------------------------------------
   // store names in list for dialog selection
   //------------------------------------------------

   PAKBSPlistptrs  = (char **)GetMemory( MipHeader.numtex * sizeof( char*));     // n pointers allocated
   PAKBSPlistnames = (char *) GetMemory( MipHeader.numtex * 22);                 // 22 byte allocations

   listwork        = PAKBSPlistnames;                            // where to store
   PAKBSPlistcount = 0;                                          // number of entries

   for (n = 0; n < MipHeader.numtex; n++,TempDir+=1)
   {
        Offset = BspDir->start+BspHeader.miptex.offset+MipHeader.offset[n];

        WadSeek( QuakePakFile,Offset);
        WadRead( QuakePakFile, &QuakeImageHeader,40);   // x width, y height, xoff,yoff,1stYoffset

        memset(listwork,0,22);
        memcpy(listwork,QuakeImageHeader.name,16);     // name
        memcpy(listwork+18,&Offset,4);

        if (listwork[0] != 0)                           // one had 1st with leading 0?
        {   PAKBSPlistptrs[PAKBSPlistcount] = listwork; // addr of saved
            PAKBSPlistcount++;                          // +1 saved
            listwork+=22;                               // next avail loc
        }
   }

   if(Sorted)
      qsort( PAKBSPlistptrs, PAKBSPlistcount, sizeof( char *),QsortListName16);

   for(n = 0; n < PAKBSPlistcount; n++)
   {   ListBox_AddString(GetDlgItem(hwnd, lst2), PAKBSPlistptrs[n]);
   }

   return TRUE;

}
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//    Read & fill both list box's with Pak and Bsp names                      //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
static Bool UpdateLists(HWND hDlg)
{
      if( GetPakNames(hDlg) ) // Error getting it -Post ourselves a Quit Msg.
      {
          if( GetPakBspNames(hDlg) ) // Retrieve Bsp Names from PAK
          {
             SetFocus(GetDlgItem(hDlg, lst2)); // the Bsp graphics
             SendMessage(GetDlgItem(hDlg, lst2), LB_SETCURSEL, 0, 0);
             ListBox_GetText(GetDlgItem(hDlg, lst2), 0, bspname);

             // Notify List Box # 2 selection has changed
             SendMessage(hDlg,
                         WM_COMMAND,
                         MAKEWPARAM( (WORD)GetWindowLong(GetDlgItem(hDlg,lst2), GWL_ID), LBN_SELCHANGE ), 0);

             return TRUE;
          }
          else
          {
             FreeMemory(PAKlistnames); // Got Pak names, so make to free first
             PAKlistnames = NULL;
             FreeMemory(PAKlistptrs );
             PAKlistptrs = NULL;

             //PostMessage(hDlg,WM_COMMAND,
             //            MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(GetDlgItem(hDlg,lst1), GWL_ID)),
             //                                                   (LPARAM)GetDlgItem(hDlg, lst1));
             return FALSE;
          }
      }
      else
      {
          //PostMessage(hDlg,WM_COMMAND,
          //            MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(GetDlgItem(hDlg,lst1), GWL_ID)),
          //                                                    (LPARAM)GetDlgItem(hDlg, lst1));
          return FALSE;
      }

}



//////////////////////////////////////////////////////////////////////////////
//  The Quake Display Grapic in a Pak/Bsp file
/////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QViewPakDlgProc(DIALOG_PARAMS)
{
       int4   i;  //n
       int    Index;
// int2   NewGamma;    // the New Quake Gamma Correct for pictures
// int2   change;      // change in gamma up or down
       char   *offsetloc;
       char   workname[24];  // name 16 and offset at +20

       HDC    hdc;
       RECT   rc;
       int2   code;
       int    nResult;

static VOID **    lpSBits = NULL; // Pointer to bits in DIBSECTION for SavePCX
static HBITMAP    hSDibbm = NULL; // Handle to the Dib's Bitmap returned to us
static BITMAPINFO Lumpbi;			 // Our DIB Bitmap Info Struct.
       HPALETTE   hOldPal;
       char       TempName[24] = {0};

static QDATA      qdat;           // Pass String to our Browse Window
                                  // don't have to use a global variable then
//-------------------------------------------------------------------------

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

 ZeroMemory(&qdat, sizeof(qdat));

 switch( msg )
  {
    case WM_INITDIALOG:
	   SetWindowLong(hDlg, DWL_USER, lParam);		// store our Pointer
      //lpd2 = (LPDLG2OPTS)lParam;				// grab it for our use
      // EnableWindow(GetDlgItem(hDlg, psh1), FALSE); // MIPS   -------  3-1-97 ok to do

      MipValue   = 0;                          // start with biggest format

	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETRANGE, (WPARAM)0, (LPARAM) MAKELONG((short) 64, (short) 0));
	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETPOS,   (WPARAM)0, (LPARAM) MAKELONG((short) QuakeGamma, 0));

      if (UpdateLists(hDlg))   // Read Pak list, then if successful read BSP list too
      {   LastPicXloc = -1;
          return FALSE; // we've set our own focus
      }
      else
      {   // PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
          //FreeMemory(PAKlistnames);
          //PAKlistnames = NULL;
          //FreeMemory(PAKlistptrs);
          //PAKlistptrs = NULL;
          //FreeMemory(PAKBSPlistnames);
          //PAKBSPlistnames = NULL;
          //FreeMemory(PAKBSPlistptrs);
          //PAKBSPlistptrs = NULL;

          SetQuakePalette(hDlg, NULL, 0, RESTORECOLORS);
          //CloseQuakePakFile();
          EndDialog( hDlg, TRUE );
          return TRUE;
      }

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);//   6-1-97
      break;
    //   6-1-97      changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

	 case WM_VSCROLL:  // Our Gamma Correct Up/Down Spinner msg
      QuakeGamma = (int2)SendMessage(GetDlgItem(hDlg, IDD_UPDOWNX), UDM_GETPOS, 0, 0);
      MakeNewQuakePal(hDlg,QuakeGamma);

      InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
      UpdateWindow(GetDlgItem(hDlg,rct1));
      break;

    case WM_COMMAND:

       //lpd2 = (LPDLG2OPTS)GetWindowLong(hDlg, DWL_USER);  	// grab our pointer

       switch LOWORD(wParam) 							// handle of the control
       {
         case lst1://The PAK list of BSP's etc. was chosen
           switch HIWORD(wParam)
			  {
				 case LBN_SELCHANGE:

       		  Index = (LRESULT)SendMessage(GetDlgItem(hDlg, lst1), LB_GETCURSEL, 0, 0);
         	  ListBox_GetText(GetDlgItem(hDlg, lst1), Index, QPAKname);

              FreeMemory(PAKBSPlistnames);
              PAKBSPlistnames = NULL;
              FreeMemory(PAKBSPlistptrs );
              PAKBSPlistptrs = NULL;

              ListBox_ResetContent(GetDlgItem(hDlg, lst2));

              if (GetPakBspNames(hDlg) ) // Retrieve Bsp Names from PAK
              {
                  SetFocus(GetDlgItem(hDlg, lst2)); // the Bsp graphics
                  SendMessage(GetDlgItem(hDlg, lst2), LB_SETCURSEL, 0, 0);
                  ListBox_GetText(GetDlgItem(hDlg, lst2), 0, bspname);

                  // Notify List Box # 2 selection has changed
                  SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,lst2),
                                                            GWL_ID), LBN_SELCHANGE ), 0);

              }
              else
              {   ErrorMsg(hDlg, "No BSP names available, use Dump selection for raw data");
                  //PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(GetDlgItem(hDlg,
                  //lst1), GWL_ID)),(LPARAM)GetDlgItem(hDlg, lst1));
              }
				  break;

				 default:
				  return FALSE;
           }
		     break;

         //Graphic Texture Change
         case lst2:

            switch HIWORD(wParam)
			   {
				  case LBN_SELCHANGE:
       		   Index = (LRESULT)SendMessage(GetDlgItem(hDlg, lst2), LB_GETCURSEL, 0, 0);
         	   ListBox_GetText(GetDlgItem(hDlg, lst2), Index, bspname);

               //offsetloc = BSPlistptrs[currentline];      // address of name list
               offsetloc = PAKBSPlistptrs[Index];
               memcpy(workname,offsetloc,22);
               memcpy(&BSPoffset,workname+18,4);

               hdc = GetDC(GetDlgItem(hDlg, rct1));         // our picture rect
               hOldPal = SelectPalette(hdc, hQuakePal, FALSE);

  	            RealizePalette(hdc);
	            GetClientRect(GetDlgItem(hDlg, rct1), &rc);

               DisplayQuakeMipTexture(GetDlgItem(hDlg, rct1), hdc,// FALSE,
                                      QuakePakFile,
                                      QuakePakFileName,
                                      BSPoffset,//offset
                                      0,0,
                                      rc.right, rc.bottom,
                                      NULL, NULL, NULL, NULL,NULL,
                                      bspname,
                                      QuakePicMagnify,
                                      SHOWINFO,
                                      QuakeColors[GRAY], //MenuColor
                                      SHOWIMAGE);

               SelectPalette(hdc, hOldPal, FALSE);
               ReleaseDC(GetDlgItem(hDlg, rct1), hdc);

               //InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
               //UpdateWindow(GetDlgItem(hDlg,rct1));
				   break;

				  default:
				   return FALSE;
            }
		      break;
         case WM_CHANGE:
            Static_SetText(GetDlgItem(hDlg, stc1), QViewMsg);
            break;
         //case IDOK:                       // Exit Button
         // Exit button
         case IDCANCEL:
            FreeMemory(PAKlistnames);
            PAKlistnames = NULL;
            FreeMemory(PAKlistptrs);
            PAKlistptrs = NULL;
            FreeMemory(PAKBSPlistnames);
            PAKBSPlistnames = NULL;
            FreeMemory(PAKBSPlistptrs);
            PAKBSPlistptrs = NULL;

            FreeBrowseList();     // free browse list now

            SetQuakePalette(hDlg, NULL, 0, RESTORECOLORS);
            CloseQuakePakFile();
            EndDialog( hDlg, TRUE );
            return TRUE;

///      case IDCANCEL:                   // Premature EXIT - Posted to ourselves
//         EndDialog( hDlg, FALSE );
//         return TRUE;

         case psh1:                       // Toggle Mip
            MipValue++;
            if (MipValue > 3) MipValue = 0;
            InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
            return TRUE;
         case psh2:                       // Export PCX
         case psh7:                       // Export BMP
            hdc = GetDC(GetDlgItem(hDlg, rct1));
            GetClientRect(GetDlgItem(hDlg, rct1), &rc);

    	      // Set up a BITMAPINFO structure for use with CreateDIBSection()
            ZeroMemory(&Lumpbi, sizeof(BITMAPINFO));
            Lumpbi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
            Lumpbi.bmiHeader.biPlanes    = 1;
            Lumpbi.bmiHeader.biBitCount  = 8;
            Lumpbi.bmiHeader.biWidth     = ImageXSize;  // global size
    	      Lumpbi.bmiHeader.biHeight    = -ImageYSize;
	         Lumpbi.bmiHeader.biSizeImage = ((ImageXSize+3)& ~3 ) * ImageYSize;  //   8-1-97
    	      Lumpbi.bmiHeader.biCompression = BI_RGB;

            // Create a DC to hold our DIBSECTION
            hSDibbm = (HBITMAP)CreateDIBSection(hdc, &Lumpbi, DIB_PAL_COLORS, (VOID **)&lpSBits, NULL, 0L);

            if (hSDibbm)   //   8-1-97
            {
                code =DisplayQuakeMipTexture(hDlg, NULL, //TRUE,
                                           QuakePakFile,
                                           QuakePakFileName,
                                           BSPoffset,   //offset
                                           0,0,
                                           rc.right, rc.bottom,
                                           NULL,
                                           NULL,
                                           NULL,// Imagesize
                                           (byte **)&lpSBits, hSDibbm,
                                           (char *)bspname,1,
                                           NOSHOWINFO,
                                           BLACK,
                                           NOSHOWIMAGE);
            }
            else
            {   code = -1;
            }

            if (code == 1)
            {
                // psh2 = PCX
                if ( LOWORD(wParam) == psh2)
                {
	                 SavePCX(hDlg, bspname, (byte *)&*lpSBits, (byte *)QuakePaletteBufr);    // use quake palette
                }
                else
                // psh7 = BMP save the DIB to disk file
                {
                    nResult = SaveDIB(hDlg,&Lumpbi, lpSBits, bspname,qRGBquad);


	                 if (nResult)
                    {   DIBError(nResult);
                    }
                }
            }
            else
            {   NotifyMsg(hDlg, "Possibly Not enough Memory!");
            }

            if(hSDibbm)
            {  DeleteBitmap(hSDibbm);
               hSDibbm = NULL;
               lpSBits = NULL;
            }

            ReleaseDC(hDlg, hdc);
            return TRUE;
         // Print/export/viewhex
         case psh3:
            DialogBoxParam(gd.hInst, "QViewLump", hDlg, QLumpPakViewDlgProc, 0);
            return TRUE;
         //case psh4:
         //   EndDialog( hDlg, TRUE );
         //   return TRUE;

         // ZOOM
         case psh5:
            QuakePicMagnify++;
            if(QuakePicMagnify > (MipValue+1)*4 ) QuakePicMagnify = 1;

            InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
            UpdateWindow(GetDlgItem(hDlg,rct1));
            break;
         // Browse
         case psh6:
            CreateQuakePakTextureList ();
            strcpy(qdat.name, bspname);
            gd.hParent = hDlg;            // For WM_BROWSE_BACK below

            hBrowseWindow = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW | WS_EX_DLGMODALFRAME,                //   12-8-96
                            "QBrowse",
                            "Browse Quake Textures - F1 for Help",
                            WS_POPUP | WS_CAPTION,
                            0, 0,
                            200, 200,
                            hDlg, NULL, gd.hInst, &qdat);

            ShowWindow(hBrowseWindow, SW_MAXIMIZE);
            UpdateWindow(hBrowseWindow);
            break;

	   	case WM_BROWSE_BACK:	//After Browsing we return here PostMessage(hParent, WM_BROWSE...
            strcpy(bspname, (char *)(LPARAM)lParam);
            Index = 0;
            for (i=0; i< BrowseListCount; i++)
            {  strncpy(TempName,BrowseListPtrs[i],16);
	            if (!strcmp (TempName, bspname) )
	            {  Index = i;
                  break;
	            }
            }

            SendMessage(GetDlgItem(hDlg, lst2), LB_SETCURSEL, Index, 0);

            SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,
            lst2), GWL_ID), LBN_SELCHANGE ), 0);
            break;
		 }

       break;
  }
 return FALSE;
}


//////////////////////////////////////////////////////////////////////////////
//   Quake Display Graphic in a Bsp file
/////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QViewBspDlgProc(DIALOG_PARAMS)
{
static  char   *listwork;
        int4    n,i;

        char   *offsetloc;
        char    workname[24];  // name 16 and offset at +20

         int     Index;
 QuakeBspHeader  BspHeader;                  // control header start of BSP
 MipHeader       MipHeader;                  // within BSP #of mips and offsets
 QuakeGraphicHdr QuakeImageHeader;           // header start of mip
//      int2       NewGamma;    // the New Quake Gamma Correct for pictures
//      int2       change;      // change in gamma up or down

        HDC        hdc;
        RECT       rc;
        int2       code;
        int        nResult;

static  VOID **    lpSBits = NULL; // Pointer to bits in DIBSECTION for SavePCX
static  HBITMAP    hSDibbm = NULL; // Handle to the Dib's Bitmap returned to us
static  BITMAPINFO Lumpbi;		     // Our DIB Bitmap Info Struct.
        HPALETTE   hOldPal;
        char       TempName[24] = {0};

static  QDATA       qdat;         // Pass String to our Browse Window
                                  // don't have to use a global variable then
//-------------------------------------------------------------------------

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

 ZeroMemory(&qdat, sizeof(qdat));

 switch( msg )
 {
    case WM_INITDIALOG:
	   SetWindowLong(hDlg, DWL_USER, lParam);		// store our Pointer
      //lpd2 = (LPDLG2OPTS)lParam;				// grab it for our use

      //EnableWindow(GetDlgItem(hDlg, psh3), FALSE); // View/Print/HexDump   8-1-97

      //if(lpd2->b1 == 0)  // 0 == Wad, 1 == Pak, 2 == Bsp  NOT used

      WadSeek( QuakeBspFile, 0);
      WadRead( QuakeBspFile, &BspHeader,sizeof(BspHeader) );       // read header
      if (BspHeader.version > 29)
      {   ErrorMsg(hDlg,"Version > 29 %d",BspHeader.version);
          CloseQuakeBspFile();
          EndDialog( hDlg, TRUE );
          return FALSE;
      }

      WadSeek( QuakeBspFile, BspHeader.miptex.offset);
      WadRead( QuakeBspFile,&MipHeader,sizeof(MipHeader) );        // read entries header

      //------------------------------------------------
      // store names in list for dialog selection
      //------------------------------------------------

      BSPlistptrs  = (char **)GetMemory( MipHeader.numtex * sizeof( char*));     // n pointers allocated
      BSPlistnames = (char *)GetMemory( MipHeader.numtex * 22);                  // 22 byte allocations

      listwork     = BSPlistnames;                            // where to store
      BSPlistcount = 0;                                          // number of entries

      for (n = 0; n < MipHeader.numtex; n++)
      {
         BSPoffset = BspHeader.miptex.offset+MipHeader.offset[n];

         WadSeek( QuakeBspFile,BSPoffset);
         WadRead( QuakeBspFile, &QuakeImageHeader,40);   // x width, y height, xoff,yoff,1stYoffset

         memset(listwork,0,22);
         memcpy(listwork,QuakeImageHeader.name,16);      // name
         memcpy(listwork+18,&BSPoffset,4);               // offset

         if (listwork[0] != 0)                           // one had 1st with leading 0?
         {   BSPlistptrs[BSPlistcount] = listwork;       // addr of saved
             BSPlistcount++;                             // +1 saved
             listwork+=22;                               // next avail loc
         }
      }

      if(Sorted)
      {  qsort( BSPlistptrs, BSPlistcount, sizeof( char *),QsortListName16);
      }

      for(n = 0; n < BSPlistcount; n++)
      {   ListBox_AddString(GetDlgItem(hDlg, lst1), BSPlistptrs[n]);
      }

      strncpy ( QBSPname,(char *) BSPlistptrs[0],24); // first entry default

      // set the focus to our list box and then select item 0 or our Current Map
      SetFocus(GetDlgItem(hDlg, lst1));
      SendMessage(GetDlgItem(hDlg, lst1), LB_SETCURSEL, 0, 0);

      // Send SelChange Message to initialize MAKEPARAM(Low,High) Low=ID of EditList, High=LBN_SELCHANGE
      SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,lst1),
                                                GWL_ID), LBN_SELCHANGE ), 0);

      MipValue   = 0;                          // start with biggest format

	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETRANGE,(WPARAM)0, (LPARAM) MAKELONG((short) 64, (short) 0));
	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETPOS,  (WPARAM)0, (LPARAM) MAKELONG((short) QuakeGamma, 0));

      LastPicXloc = -1;
      return FALSE; // we've set our own focus

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
      break;
    //   6-1-97      changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

	 case WM_VSCROLL:  // Our Gamma Correct Up/Down Spinner msg
      QuakeGamma = (int2)SendMessage(GetDlgItem(hDlg, IDD_UPDOWNX), UDM_GETPOS, 0, 0);
      MakeNewQuakePal(hDlg,QuakeGamma);

      InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
      UpdateWindow(GetDlgItem(hDlg,rct1));
      break;

    case WM_COMMAND:

       //lpd2 = (LPDLG2OPTS)GetWindowLong(hDlg, DWL_USER);  	// grab our pointer

       switch LOWORD(wParam) 							// handle of the control
       {
         case lst1:		// Out list box was chosen in some fashion

           switch HIWORD(wParam)
			  {
				 case LBN_DBLCLK: // Post IDOK to Dialog ( use MAKEWPARAM MACRO )
				  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(GetDlgItem(hDlg,
                          rct1), GWL_ID)),(LPARAM)GetDlgItem(hDlg, rct1));
				  break;

				 case LBN_SELCHANGE:
       		  Index = (LRESULT)SendMessage(GetDlgItem(hDlg, lst1), LB_GETCURSEL, 0, 0);
         	  ListBox_GetText(GetDlgItem(hDlg, lst1), Index, QBSPname);

              // get offset of picture
              //offsetloc = BSPlistptrs[currentline];      // address of name list
              offsetloc = BSPlistptrs[Index];      // address of name list
              memcpy(workname,offsetloc,22);
              memcpy(&BSPoffset,workname+18,4);

              hdc = GetDC(GetDlgItem(hDlg, rct1)); // our picture rect
              hOldPal = SelectPalette(hdc, hQuakePal, FALSE);

  	           RealizePalette(hdc);
	           GetClientRect(GetDlgItem(hDlg, rct1), &rc);

              DisplayQuakeMipTexture(GetDlgItem(hDlg, rct1), hdc, //FALSE,
                                     QuakeBspFile,
                                     QuakeBspFileName,
                                     BSPoffset,//offset
                                     0,0,
                                     rc.right, rc.bottom,
                                     NULL, NULL, NULL, NULL,NULL,
                                     bspname,
                                     QuakePicMagnify,
                                     SHOWINFO,
                                     QuakeColors[GRAY],//MenuColor
                                     SHOWIMAGE);

              SelectPalette(hdc, hOldPal, FALSE);
              ReleaseDC(GetDlgItem(hDlg, rct1), hdc);
				  break;

				 default:
				  return FALSE;
           }
		     break;

         case WM_CHANGE:
           Static_SetText(GetDlgItem(hDlg, stc1), QViewMsg);
           break;

         //case IDOK:                       // Exit Button
         // Exit button
         case IDCANCEL:
           FreeMemory(BSPlistnames);
           BSPlistnames = NULL;
           FreeMemory(BSPlistptrs);
           BSPlistptrs = NULL;
           FreeBrowseList();     // free browse list now

           SetQuakePalette(hDlg, NULL, 0, RESTORECOLORS);
           CloseQuakeBspFile();

           EndDialog( hDlg, TRUE );
           return TRUE;
         // Toggle Mip
         case psh1:
           MipValue++;
           if (MipValue > 3) MipValue = 0;
           InvalidateRect(GetDlgItem(hDlg, rct1), NULL, FALSE);
           UpdateWindow(GetDlgItem(hDlg,rct1));
           return TRUE;
         // Export PCX
         case psh2:
         // Export BMP
         case psh7:
           hdc = GetDC(GetDlgItem(hDlg, rct1));
           GetClientRect(GetDlgItem(hDlg, rct1), &rc);

    	     // Set up a BITMAPINFO structure for use with CreateDIBSection()
           ZeroMemory(&Lumpbi, sizeof(BITMAPINFO));
           Lumpbi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
           Lumpbi.bmiHeader.biPlanes    = 1;
           Lumpbi.bmiHeader.biBitCount  = 8;
           Lumpbi.bmiHeader.biWidth     = ImageXSize;  // global size
    	     Lumpbi.bmiHeader.biHeight    = -ImageYSize;
	        Lumpbi.bmiHeader.biSizeImage = ((ImageXSize+3)& ~3 ) * ImageYSize;  //   8-1-97
    	     Lumpbi.bmiHeader.biCompression = BI_RGB;

           // Create a DC to hold our DIBSECTION
           hSDibbm = (HBITMAP)CreateDIBSection(hdc, &Lumpbi, DIB_PAL_COLORS, (VOID **)&lpSBits, NULL, 0L);

           if (hSDibbm)     //   8-1-97
           {
               code =DisplayQuakeMipTexture(hDlg, NULL, //TRUE,
                                           QuakeBspFile,
                                           QuakeBspFileName,
                                           BSPoffset,   //offset
                                           0,0,
                                           rc.right, rc.bottom,
                                           NULL,
                                           NULL,
                                           NULL,// Imagesize
                                           (byte **)&lpSBits, hSDibbm,
                                           (char *)QBSPname,
                                           1,
                                           NOSHOWINFO,
                                           BLACK,
                                           NOSHOWIMAGE);
           }
           else
           {   code = -1;
           }

           if (code == 1)
           {
                // psh2 = PCX
                if ( LOWORD(wParam) == psh2)
                {
	                 SavePCX(hDlg, QBSPname, (byte *)&*lpSBits, (byte *)QuakePaletteBufr);    // use quake palette
                }
                else
                // psh7 = BMP save the DIB to disk file
                {   nResult = SaveDIB(hDlg,&Lumpbi, lpSBits, QBSPname,qRGBquad);

	                 if (nResult)
                    {   DIBError(nResult);
                    }
                }
           }
           else
           {   NotifyMsg(hDlg, "Possibly Not enough Memory!");
           }

           if(hSDibbm)
           {  DeleteBitmap(hSDibbm);
              hSDibbm = NULL;
              lpSBits = NULL;
           }

           ReleaseDC(hDlg, hdc);
           return TRUE;

         case psh3:                       // view/print/hexdump later to do
           DialogBoxParam(gd.hInst, "QViewLump", hDlg, QLumpBspViewDlgProc, 0);
           break;
           //EndDialog( hDlg, TRUE );
           //return TRUE;
         //case psh4:                       // Hex Dump n/a
         //  EndDialog( hDlg, TRUE );
         //  return TRUE;
         case psh5:                       // Zoom Graphic Temp for now
           QuakePicMagnify ++;
           if(QuakePicMagnify > (MipValue+1)*4 ) QuakePicMagnify = 1;

           // Send SelChange Message
           SendMessage(hDlg, WM_COMMAND,
                       MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,lst1), GWL_ID), LBN_SELCHANGE ), 0);
           break;
         // Browse
         case psh6:
           CreateQuakeBspTextureList ();
           strcpy(qdat.name, QBSPname);
           gd.hParent = hDlg;             // For WM_BROWSE_BACK below

           hBrowseWindow = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW | WS_EX_DLGMODALFRAME,                //   12-8-96
                           "QBrowse",
                           "Browse Quake Textures - F1 for Help",
                           WS_POPUP | WS_CAPTION,
                           0, 0,
                           200, 200,
                           hDlg, NULL, gd.hInst, &qdat);

           ShowWindow(hBrowseWindow, SW_MAXIMIZE);
           UpdateWindow(hBrowseWindow);
           break;

	   	case WM_BROWSE_BACK:	//After Browsing we return here PostMessage(hParent, WM_BROWSE...
           strcpy(QBSPname, (char *)(LPARAM)lParam);
           Index = 0;
           for (i=0; i< BrowseListCount; i++)
           {   strncpy(TempName,BrowseListPtrs[i],16);
	            if (!strcmp (TempName, QBSPname) )
	            {  Index = i;
                  break;
	            }
           }

           SendMessage(GetDlgItem(hDlg, lst1), LB_SETCURSEL, Index, 0);
           // Send SelChange Message
           SendMessage(hDlg, WM_COMMAND,
                       MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,lst1), GWL_ID), LBN_SELCHANGE ), 0);
           break;
		 }

       break;
  }
 return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
//	Quake view a graphic Window Preview Paint   (startup and gamma correction)
//
// Not used much if any anymore. don't call at the beginning
// so prob. only ever receives 1 WM_PAINT and then again
// only if another window would obsure our preview wnd
// or alt-tab'd away and then back again.
//
//////////////////////////////////////////////////////////////////////////////
static void QView_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   RECT        rc;
   HPALETTE    hOldPal;

   QuakeWadDirPtr dir;
   int2 code;

   //--------------------------------------------------------------------------

   if (Qmode == 1)
   {   dir = FindQuakeWadDir(Qname);
       if (!dir || dir->size  <=  0 )             // no size
       {    NotifyMsg(hwnd, "Not found or zero length!");
            return;
       }
    }

   BeginPaint(hwnd, &ps);

   hOldPal = SelectPalette(ps.hdc, hQuakePal, FALSE);

  	RealizePalette(ps.hdc);
	GetClientRect(hwnd, &rc);

   HBRUSH hBrush = CreateSolidBrush(PALETTEINDEX(QuakeColors[GRAY])); // Gray COLORREF
   FillRect(ps.hdc, &rc, hBrush);

   ///NotifyMsg(NULL,"ONPaint  %d,%d brush=%d",rc.right-rc.left, rc.bottom-rc.top,hBrush);

   if(Qmode == 1)        // 1 = Wad
   code = DisplayQuakeMipTexture(hwnd, ps.hdc, //FALSE,
                                 QuakeWadFile,
                                 QuakeWadFileName,
                                 dir->start,//offset
                                 0,0,
                                 rc.right, rc.bottom,
                                 NULL, NULL, NULL, NULL,NULL,
                                 Qname,
                                 QuakePicMagnify,
                                 SHOWINFO,
                                 QuakeColors[GRAY],//MenuColor
                                 SHOWIMAGE);
   else
   if(Qmode == 2)        // 2 = Pak
   code = DisplayQuakeMipTexture(hwnd, ps.hdc, //FALSE,
                                 QuakePakFile,
                                 QuakePakFileName,
                                 BSPoffset,//offset
                                 0,0, rc.right, rc.bottom,
                                 NULL, NULL, NULL, NULL,NULL,
                                 bspname,
                                 QuakePicMagnify,
                                 SHOWINFO,
                                 QuakeColors[GRAY],//MenuColor
                                 SHOWIMAGE);
   else
   if(Qmode == 3)        // 3 = Bsp
   code = DisplayQuakeMipTexture(hwnd, ps.hdc, //FALSE,
                                 QuakeBspFile,
                                 QuakeBspFileName,
                                 BSPoffset,//offset
                                 0,0, rc.right, rc.bottom,
                                 NULL, NULL, NULL, NULL,NULL,
                                 QBSPname,
                                 QuakePicMagnify,
                                 SHOWINFO,
                                 QuakeColors[GRAY],//MenuColor
                                 SHOWIMAGE);
   if (code < 0)
   {   ErrorMsg(hwnd, "Not enough memory!");
   }

   if (hOldPal)  SelectPalette(ps.hdc, hOldPal, FALSE);
   DeleteBrush(hBrush);
   EndPaint(hwnd, &ps);
}

//////////////////////////////////////////////////////////////////////////////
//  The Deep QuakeView Preview Window Procedure
/////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK QViewWndProc(WINDOWS_PARAMS)
{
  switch(msg)
  {
    case(WM_LBUTTONDOWN):
      PostMessage(GetParent(hwnd), WM_COMMAND, psh6, 0); // click on Txtr = Browse
      return 0;

    HANDLE_MSG(hwnd, WM_PAINT, QView_OnPaint);

	 default:
		return QView_DefProc(hwnd, msg, wParam, lParam);   //DefWindowProc
  }
}

////////////////////////////////////////////////////////////////////////////////
//
// display a Quake Mip texture (either from WAD or BSP, both have same format)
//
// display graphic -- buffers are allocated as required
//                    buffers are passed back to user(optl)
////////////////////////////////////////////////////////////////////////////////

#pragma argsused
static int2 DisplayQuakeMipTexture (HWND hwnd, HDC hdc,
                             //Bool  Saving,
                             FILE  *TextureFILE,// file with data
                             char  *TextureFileName,
                             int4   PAKoffset,           // offset in file
                             int2   xstart, int2 ystart,
                             int2   xend,   int2 yend,
		                       int4 * RawSize,
                             byte** RawData,
                             int4 * ImageSize,
                             byte** ImageBufr,
                             HBITMAP hDibbm,
                             char * picname,
                             int2   magnify,
                             Bool   ShowInfo,
                             int2   BackColor,
		                       Bool   ShowImage)      // show image gets rid of transp
{

   int4  Offset;
   int4  m,k,l;
   int4  n;
   int4  Yheight;
   int4  Xwidth;

   int2  imagemagnify = magnify;

   int4  ScreenWidth = xend - xstart+1;   // Screen width
   int4  ScreenHeight= yend - ystart+1;   // Screen height

   int2  Xadjust;         // precalc actual image size (round even row)  width DWORD aligned

   int4  ThisImageSize;
   int4  NewSize;
   int4  RawWrkSize=-1;
   byte *RawWrkData=NULL;

   int4  WrkImageSize=-1;
   byte *WrkPixelPtr;

   int2  shrink     = 0;
   int2  shrinkSkipX= 0;
   int2  shrinkSkipY= 0;

   int2  Xsize;
   int2  Ysize;
   int4  expandTargetX,expandTargetY;
   int4  shrinkTargetX,shrinkTargetY;

   QuakeGraphicHdr QuakeImageHeader;

   VOID **  	lpBits = NULL; // Global pointer to the bits in our DIBSECTION
   BITMAPINFO 	bi;				// Our DIB Bitmap Info Struct.

	static HDC 			hdcMem     = NULL;
	static HBITMAP		hOldBitmap = NULL;
	static HBITMAP		hDib       = NULL;
   static byte *     WrkPtr     = NULL; // used to be tPtr Pointer to Dib bits

   RECT rc;

   //--------------------------------------------------------------------

   if (!strnicmp(picname,"palette",7)) return 0;     // not the palette

   WadSeek( TextureFILE, PAKoffset);              // seek start of entry
   WadRead( TextureFILE, &QuakeImageHeader,40);   // x width, y height, xoff,yoff,1stYoffset

   // get this TEXTURE offset
   Offset = PAKoffset;

   if (MipValue == 0)
   {   Offset += QuakeImageHeader.offset1;         //offset to start of 1st graphics image
   }
   if (MipValue == 1)
   {   Offset += QuakeImageHeader.offset2;         //offset to start of 1st graphics image
   }
   if (MipValue == 2)
   {   Offset += QuakeImageHeader.offset4;         //offset to start of 1st graphics image
   }
   if (MipValue == 3)
   {   Offset += QuakeImageHeader.offset8;         //offset to start of 1st graphics image
   }

   Xsize =  QuakeImageHeader.xsize>>MipValue;
   Ysize =  QuakeImageHeader.ysize>>MipValue;

   ImageXSize = Xsize;                             // global for SavePCX
   ImageYSize = Ysize;

   // get old buffer if any

   if (RawData)
   {   RawWrkData= *RawData;                      //5-3-96 address of prior
       if (!RawWrkData)
       {   if(RawSize)
	        {   *RawSize = 0;                           // none allocated
	        }
       }
       else
       if (RawSize)
       {   RawWrkSize=  *RawSize;                 //5-3-96 size of prior
       }
   }


   // get old image buffer if any

   if (ImageBufr)
    {
    	WrkPtr = *ImageBufr;	   // Grab pointer to the bits of our DIBSection
      hDib = hDibbm;          // Handle to the bitmap
       if (! WrkPtr)
       {   if(ImageSize)
	        {   *ImageSize = 0;           // none allocated
	        }
       }
       else
       if (ImageSize)
       {   WrkImageSize=  *ImageSize;    //size of prior
       }
   }

   // now adjust so it fits in the window

   if (imagemagnify<=0)
   {   Xwidth  = Xsize/ (abs(imagemagnify)+2);
       Yheight = Ysize/ (abs(imagemagnify)+2);
   }
   else
   {   Xwidth  = Xsize* imagemagnify;
       Yheight = Ysize* imagemagnify;
   }

   if (ShowImage) //   5-1-97 only if showing
   {
       while (Xwidth > ScreenWidth || Yheight > ScreenHeight)
       {  imagemagnify--;
          magnify--;
          if (imagemagnify<=0)
          {
    	        Xwidth  = (Xsize)/ (abs(imagemagnify)+2);
    	        Yheight = (Ysize)/ (abs(imagemagnify)+2);
	       }
	       else
	       {   Xwidth  = (Xsize)* imagemagnify;
	           Yheight = (Ysize)* imagemagnify;
	       }
       }
   }

   if (imagemagnify <= 0)                         // shrinking
   {   imagemagnify = 1;

       Xwidth  = Xsize;                           // set to size
       Yheight = Ysize;
   }

   NewSize = Xsize*Ysize;                         // file data buffer

   if (!RawWrkData
   ||  RawWrkSize != NewSize)
   {
       RawWrkSize = NewSize;                      // size of whole image
       if (RawSize)
       {   *RawSize = RawWrkSize;        //++
       }

       FreeMemory(RawWrkData);                    // free prior data buffer
       RawWrkData = (byte *)malloc( RawWrkSize);          // allocate memory needed
       if (RawData)
       {   *RawData = RawWrkData;        //++
       }

       if (!RawWrkData)
       {
	         if (RawSize)
	         {   *RawSize = 0;             //++
	         }
	         return -1;                             // allocation failed
       }

   }

   //---------------------------------------------------------------------
   //              show a quake texture                                 //
   //---------------------------------------------------------------------

   if (magnify < 1 )
   {
      shrink    = abs(magnify)+1;                 // 0 =1, 1=2 etc
      Xwidth   /= shrink;
      Yheight  /= shrink;

      shrinkSkipX = shrink;                      // skip frequency
      shrinkSkipY = shrink;
   }

   // - make sure no wider than screen image wanted

   if (Xwidth  > ScreenWidth ) Xwidth  = ScreenWidth;
   if (Yheight > ScreenHeight) Yheight = ScreenHeight;

   //ThisImageSize  = imagesize(1,1,Xwidth,Yheight);
   Xadjust  = (( Xwidth + 3 ) & ~3); // DWORD Aligned
   ThisImageSize = Xwidth * Yheight;  //   5-1-97 no align here (just so happens they are at 16)

   if( (ImageSize) && (ThisImageSize != WrkImageSize) ) // if buffer can't hold it why continue ?
    {  NotifyMsg(hwnd, "Supplied Image Size is not identical to calculated size" );
       FreeMemory( RawWrkData );                   // free data buffer
       return FALSE;
    }

   hdcMem = CreateCompatibleDC(hdc);

   if (!ImageBufr)//   ||  ThisImageSize != WrkImageSize)
   {
       //WrkImageSize = ThisImageSize;    // size of whole image
       //if (ImageSize)
       //{   *ImageSize = WrkImageSize;    //++
       //}
    	 // Set up a BITMAPINFO structure for use with CreateDIBSection()
    	 ZeroMemory(&bi, sizeof(BITMAPINFO));
    	 bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    	 bi.bmiHeader.biPlanes     = 1;
    	 bi.bmiHeader.biBitCount   = 8;
    	 bi.bmiHeader.biWidth      =  Xwidth;   //   5-1-97
    	 bi.bmiHeader.biHeight     = -Yheight;
		 bi.bmiHeader.biSizeImage  =  Xadjust * Yheight;
    	 bi.bmiHeader.biCompression=  BI_RGB;

    	 // Create a DC to hold our DIBSECTION
    	 hDib = (HBITMAP)CreateDIBSection(hdc, &bi, DIB_PAL_COLORS, (VOID **)&lpBits, NULL, 0L);

       if (!hDib)                            // not enough memory
       {  ErrorMsg(hwnd, "Unable to create a Valid DIBSECTION in DisplayQuakeMipTexture");  //   5-1-97 id was wrong
          DeleteDC(hdcMem);
          FreeMemory( RawWrkData );                   // free data buffer
          hDib 	   = NULL;
          lpBits   = NULL;
          return FALSE;
       }
    	 WrkPtr = (byte *)(VOID **)&*lpBits;	// Grab pointer to the bits of our DIBSection
   }


   // show image info

   if (ShowInfo)
   { // Post a message to the dialog to update the static text
     wsprintf(QViewMsg,"  Mip%d %4d x %-4d  %-10s  Scale %2hd   %-36.36s",
                          MipValue,
                          QuakeImageHeader.xsize,QuakeImageHeader.ysize,
                          picname,magnify,TextureFileName);

     PostMessage(GetParent(hwnd), WM_COMMAND, WM_CHANGE ,0);
   }

   WadSeek (TextureFILE, Offset);                     // start
   WadRead (TextureFILE, RawWrkData,RawWrkSize);      // size

   hOldBitmap  = SelectBitmap(hdcMem, hDib);	         // Select into our Memory Dc
	SetDIBColorTable(hdcMem, 0, 256, qRGBquad);			// Palette indexes
   FillMemory(WrkPtr, ThisImageSize, QuakeColors[GRAY]); // fill with background
 //  FillMemory(WrkPtr, ThisImageSize, GameColors[BackThingColor]); // fill with background

/*   if (ImageBufr)
   {   *ImageBufr = tPtr;                // tell user where it is
   }

   if (!hQDib)                            // not enough memory
   {
	   if (ImageSize)
	    {   *ImageSize = 0;             //++
	    }

	   FreeMemory( RawWrkData );                   // free data buffer
	   if (RawData)
	    {  *RawData = NULL;
	    }
	   if (RawSize)
	    {   *RawSize = 0;        //++
	    }

        if(hOldBitmap) SelectBitmap(hdcMem, hOldBitmap);// just keep happy now
        hOldBitmap     = NULL;
        if(hQDib)      DeleteBitmap(hQDib);
        hQDib 	        = NULL;
        lpQBits 		  = NULL;
        if(hdcMem)     DeleteDC(hdcMem);
        hdcMem         = NULL;
	   return -1;
   }
*/

   //
   //---get data from image buffer and store in screen buffer
   //

   expandTargetY = 0;
   shrinkTargetY = 0;

   if ( (Xwidth  != LastPicXsize
      || Yheight != LastPicYsize)
      && LastPicXloc != -1)             // not first time
   {
      HBRUSH hBrush = CreateSolidBrush(PALETTEINDEX(QuakeColors[GRAY])); // Gray
      SetRect(&rc, LastPicXloc, LastPicYloc, LastPicXloc+LastPicXsize, LastPicYloc+LastPicYsize);

      FillRect(hdc, &rc, hBrush);
      DeleteBrush(hBrush);
      //NotifyMsg(NULL,"Clear  %d,%d brush=%d",rc.right-rc.left, rc.bottom-rc.top,hBrush);
   }

   for (k = 0;
        k < Ysize;
	     k++, expandTargetY+=imagemagnify)            // Y image source
   {
       // shrink skips
       if (shrink)
       {   shrinkSkipY--;
	        if (shrinkSkipY) continue;

	        shrinkSkipY = shrink;

	        shrinkTargetY++;                               // Y in target
	        if (shrinkTargetY >= Yheight)  break;          // Y clipping
	        WrkPixelPtr = RawWrkData + k*Xsize;            // Yheight Source data
       }
       else
       {
	        if (expandTargetY >= Yheight)  break;          // X clipping
	        WrkPixelPtr = RawWrkData + k*Xsize;            // Yheight Source data
       }

       expandTargetX=0;
       shrinkTargetX=0;

       for (l = 0;
	         l < Xsize;
	         l++, expandTargetX+=imagemagnify,WrkPixelPtr+=1)  // Y
       {
	       if (shrink)
	       {  shrinkSkipX--;
	          if (shrinkSkipX) continue;

	          shrinkSkipX = shrink;
	          shrinkTargetX++;                          // next Y in target
	          if (shrinkTargetX >= Xwidth) break;       // Y clipping
	       }
	       else
	       {  if (expandTargetX >= Xwidth) break;       // Y clipping
	       }

	       if (magnify >=1)
	       {
	          for (m=0; m < imagemagnify; m++)             //y expand
	          {   for (n=0; n< imagemagnify; n++)          //x expand
		           {
                    if(  ( (expandTargetY+m)*Xadjust+(expandTargetX+n) ) >= (ThisImageSize) ) break;
		              WrkPtr[ (expandTargetY+m)*Xadjust+(expandTargetX+n)]=*WrkPixelPtr;//Target
		           }
	          }
	       }
	       else
	       {
             if( (shrinkTargetY+Xadjust + shrinkTargetX) >= ThisImageSize)  break;
		       WrkPtr[ shrinkTargetY*Xadjust + shrinkTargetX]=*WrkPixelPtr;//Target
	       }

       } // end x
   } //end y


   // automatically center image

   xstart = xstart+ ScreenWidth /2 - Xwidth /2;        // center x
   ystart = ystart+ ScreenHeight/2 - Yheight/2;        // center y

   if (xstart < 0) xstart = 0;
   if (ystart < 0) ystart = 0;

   // erase the last area used if it was active

   //if ( (Xwidth != LastPicXsize ||  Yheight != LastPicYsize) &&  ShowImage  &&  LastPicXloc != -1)     // not first time
   {
      // DrawScreenBox( LastPicXloc,LastPicYloc,LastPicXloc+LastPicXsize-1,LastPicYloc+LastPicYsize-1,MenuColor);
   }

   if (ShowImage)
   {
       OrgXstart = xstart;
       OrgYstart = ystart;
       OrgXend   = xstart+Xwidth -1;
       OrgYend   = ystart+Yheight -1;
       //DrawingMagnify = imagemagnify;   // <------for undo to work right

       LastPicXloc  = xstart;
       LastPicYloc  = ystart;
       LastPicXsize = Xwidth;
       LastPicYsize = Yheight;

       int4 x;
       int4 y;

       if(!Zoom)
       {   BitBlt(hdc, xstart, ystart, Xwidth, Yheight,hdcMem, 0, 0, SRCCOPY);
       }
       else
       {
          x = Xwidth;        // don't use xend and yend, try to maintain aspect ratio
          y = Yheight;       // increase both x and y until one of them max's out

          while( x < xend && y < yend)
          {     x++;
                y++;
          }

          xstart = ScreenWidth /2  - x/2;         // center x
          ystart = ScreenHeight/2  - y/2;        // center y
          if (xstart < 0) xstart = 0;
          if (ystart < 0) ystart = 0;

          LastPicXloc  = xstart;
          LastPicYloc  = ystart;
          LastPicXsize = x;
          LastPicYsize = y;

          StretchBlt(hdc, xstart, ystart, x, y, hdcMem, 0, 0, Xwidth, Yheight, SRCCOPY);

       }

  	    GdiFlush();
   }

   // free memory if user doesn't want to see image

   if (!RawData)
   {   FreeMemory(RawWrkData);
       if (RawSize)
       {   *RawSize = 0;        //++
       }
   }

   // free memory if user doesn't want to see image

   if (!ImageBufr)
   {
      //FreeMemory( WrkImageBufr);        // free image buffer (we are done with it)
      if (hDib)
      {
        if(hOldBitmap)  SelectBitmap(hdcMem, hOldBitmap);
        if(hdcMem)      DeleteDC(hdcMem);
        hOldBitmap = NULL;
        hdcMem     = NULL;

        DeleteBitmap(hDib);
        hDib 	    = NULL;
        lpBits 	 = NULL;
      }

      if (ImageSize)
      {   *ImageSize = 0;
      }
   }

   // SAFETY
   if(hOldBitmap)  SelectBitmap(hdcMem, hOldBitmap);
   if(hdcMem)      DeleteDC(hdcMem);
   hOldBitmap = NULL;
   hdcMem     = NULL;
   return TRUE;
}

//------------------------------------------------------------------------------
// set Quake Palette
//------------------------------------------------------------------------------
//   3-1-97 left commented for a bit, but if the "old" code was there,
//            couldn't get bg 1st time to be gray, weird!

static Bool SetQuakePalette(HWND hwnd, FILE *PalFile,int4 offset, int2 Mode)
{
   if (Mode == SETCOLORS
   &&  PalFile)
   {
       WadSeek( PalFile,             offset);    //
       WadRead( PalFile, QuakePaletteBufr,768);  // read data

		 // Create our quake Palette and store Globally

       if(!MakeNewQuakePal(hwnd,QuakeGamma) ) return FALSE;

   }
   else
   {
       DeletePalette(hQuakePal);
       hQuakePal = NULL;
   }

   return TRUE;
}

         /*   old CODE   3-1-97
		   for (n = 0; n < 256; n++)
	      {

           lplp->palPalEntry[n].peRed   = QuakePaletteBufr[n*3+0];
		     lplp->palPalEntry[n].peGreen = QuakePaletteBufr[n*3+1];
	   	  lplp->palPalEntry[n].peBlue  = QuakePaletteBufr[n*3+2];
		     lplp->palPalEntry[n].peFlags = PC_NOCOLLAPSE;

		     lplp->palPalEntry[n].peRed   = (uint2)min (lplp->palPalEntry[n].peRed   + QuakeGamma, 255);
		     lplp->palPalEntry[n].peGreen = (uint2)min (lplp->palPalEntry[n].peGreen + QuakeGamma, 255);
		     lplp->palPalEntry[n].peBlue  = (uint2)min (lplp->palPalEntry[n].peBlue  + QuakeGamma, 255);

	       	// a copy of the quake Palette for Global DIB access
	    	  qRGBquad[n].rgbRed   = lplp->palPalEntry[n].peRed;
		     qRGBquad[n].rgbGreen = lplp->palPalEntry[n].peGreen;
		     qRGBquad[n].rgbBlue  = lplp->palPalEntry[n].peBlue;
		     qRGBquad[n].rgbReserved = 0;
         }*/

//////////////////////////////////////////////////////////////////////////////
//
// Makes a new palette according to the new gamma setting
//
/////////////////////////////////////////////////////////////////////////////
static Bool MakeNewQuakePal(HWND hwnd,int2 newgamma)
{
	LPLOGPALETTE   lplp;						// pointer to our Logical Palette
	HANDLE 			hdpal;		         // handle for memory allocation
   int4           n;
   uint2          red,green,blue;
   uint2          LocalGammaCorrect;

   if (hQuakePal)  DeletePalette(hQuakePal);
   hQuakePal = NULL;

	hdpal = GlobalAlloc(GHND, sizeof(LOGPALETTE)+(256*sizeof(PALETTEENTRY)));

   if(hdpal)  								             // Got our Handle
   {
	   lplp = (LPLOGPALETTE)GlobalLock(hdpal);	 // Get our Memory Pointers
      memset(lplp,0,sizeof(LOGPALETTE)+(256*sizeof(PALETTEENTRY)) );  // clear
	   lplp->palVersion = PALVERSION;
	   lplp->palNumEntries = 256;

      memset(qRGBquad,0,sizeof(qRGBquad));
      
		// Initialize palette entries from saved Palette copy Global RGB Quad (revised again to original palette)

      //new
		for (n = 0; n < 256; n++)
	   {
		   lplp->palPalEntry[n].peFlags = PC_EXPLICIT;  // this has to be

         if (n == 0) continue;     // 0 is always black

         red  = QuakePaletteBufr[n*3+0];
         green= QuakePaletteBufr[n*3+1];
         blue = QuakePaletteBufr[n*3+2];
         //red  = qRGBquad[n].rgbRed;;
         //green= qRGBquad[n].rgbGreen;
         //blue = qRGBquad[n].rgbBlue;
		   lplp->palPalEntry[n].peFlags = PC_NOCOLLAPSE;

	    	// Gamma Correct

         LocalGammaCorrect = newgamma;

         if ( (red+green+blue)== 0 ) // not black!
         {   LocalGammaCorrect = 0;
         }

		   lplp->palPalEntry[n].peRed   = (uint2) min (red  + LocalGammaCorrect, 255);
		   lplp->palPalEntry[n].peGreen = (uint2) min (green+ LocalGammaCorrect, 255);
		   lplp->palPalEntry[n].peBlue  = (uint2) min (blue + LocalGammaCorrect, 255);

         qRGBquad[n].rgbRed      = lplp->palPalEntry[n].peRed  ;       //  gamma adjusted
		   qRGBquad[n].rgbGreen    = lplp->palPalEntry[n].peGreen;
		   qRGBquad[n].rgbBlue     = lplp->palPalEntry[n].peBlue;
		   qRGBquad[n].rgbReserved = 0;
      }

		// Create our Quake Palette and store Globally
      hQuakePal = CreatePalette(lplp);
      //new end
      // Free allocated memory
     	GlobalUnlock(hdpal);
	   GlobalFree(hdpal);
      return TRUE;
   }
   else
   {
      ErrorMsg(hwnd, "Not enough memory for QUAKE palette");
      return FALSE;
   }
}

//-------------------------------------------------------------------------------
// open the Quake Wad file and allocate/initialize directory in memory
//-------------------------------------------------------------------------------
static Bool OpenQuakeWadFile(HWND hwnd)
{
   int4     n;
   char TempName [260];

   QuakeWadDirPtr    TempDir;
   QuakeWadHeader    QuakeHeader;        // file header
   QuakeWadDirEntry  QuakeEntry;         // file dir entry

   //---------------------------------------------------------------------------

   CloseAllQuakeFiles();

   strcpy(TempName,QuakeWadFileName);

   if(!(GetFileName(hwnd,"Quake WAD file", TempName, 127, IDS_FILTERWAD, CurrDir)) )   //   3-1-97 127 +simplify
   {   return FALSE;
   }

   if ((QuakeWadFile = FileOpen( TempName, "rb",OPENMISS)) == NULL)
   {
       return FALSE;
   }

   WadSeek( QuakeWadFile, 0);
   WadRead( QuakeWadFile, &QuakeHeader,12 );          // read header

   if (strnicmp(QuakeHeader.id,"WAD2",4) )
   {   ErrorMsg(hwnd, "Not a Quake WAD2 file!",NULL);
       fclose(QuakeWadFile); //Current
       return FALSE;
   }

   strcpy(QuakeWadFileName, TempName);                // save name for nxt time

   QuakeWadStatus = 'O ';

   QuakeWadDirNum    = QuakeHeader.num;                  // number of entries

   // allocate max dir size

   QuakeWadInternalDir = (QuakeWadDirPtr)GetMemory( QuakeWadDirNum * sizeof(QuakeWadDirEntry ));

   WadSeek( QuakeWadFile, QuakeHeader.offset);         // start of dir

   TempDir         = QuakeWadInternalDir;

   for (n = 0; n < QuakeWadDirNum; n++,TempDir+=1)
   {
       WadRead( QuakeWadFile, &QuakeEntry,32);         // read name+size

       strncpy(TempDir->name,QuakeEntry.name,16);      // name
       TempDir->start = QuakeEntry.start;              // current offset
       TempDir->size  = QuakeEntry.size;               // size
       TempDir->msize = QuakeEntry.msize;              // size in memory
   }

   return TRUE;
}



//-------------------------------------------------------------------------------
// open the Quake Wad file and allocate/initialize directory in memory
//-------------------------------------------------------------------------------

static Bool OpenQuakePakFile(HWND hwnd)
{
   int4     n;
   char     TempName [260];

   QuakePakDirPtr    TempDir;
   QuakePakHeader    QuakeHeader;        // file header
   QuakePakDirEntry  QuakeEntry;         // file dir entry

   //---------------------------------------------------------------------------
   CloseAllQuakeFiles();

   strcpy(TempName,QuakePakFileName);

   if(!GetFileName(hwnd,"Quake PAK file", TempName, 127, IDS_FILTERPAK, CurrDir))  //   3-1-97 +simplify
   {   return FALSE;
   }

   if ((QuakePakFile = FileOpen( TempName, "rb",OPENMISS)) == NULL)
   {
       return FALSE;
   }

   WadSeek( QuakePakFile, 0);
   WadRead( QuakePakFile, &QuakeHeader,12 );          // read header id + offset+size

   if (strnicmp(QuakeHeader.id,"PACK",4) )
   {   ErrorMsg(hwnd, "Not a Quake PAK file!");
       fclose(QuakePakFile); //Current
       return FALSE;
   }

   strcpy(QuakePakFileName,    TempName);             // save name for nxt time

   QuakePakStatus = 'O ';

   QuakeHeader.num =   QuakeHeader.size/64;              // = #entries
   QuakePakDirNum =    QuakeHeader.num;                  // number of entries

   // allocate max dir size

   QuakePakInternalDir = (QuakePakDirPtr)GetMemory( QuakePakDirNum * sizeof(QuakePakDirEntry ));

   WadSeek( QuakePakFile, QuakeHeader.offset);         // start of dir

   TempDir         = QuakePakInternalDir;

   for (n = 0; n < QuakePakDirNum; n++,TempDir+=1)
   {
       WadRead( QuakePakFile, &QuakeEntry,64);         // read name+size

       strncpy(TempDir->name,QuakeEntry.name,56);      // name
       TempDir->start = QuakeEntry.start;              // current offset
       TempDir->size  = QuakeEntry.size;               // size
   }

   return TRUE;
}


//------------------------------------------------------------------------------
// open the Quake Wad file and allocate/initialize directory in memory
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
static Bool OpenQuakeBspFile(HWND hwnd)
{
   char    TempName [260];
   char   *BspNames[] = { "Entities",      //  0
                          "Planes",        //  1
                          "MipTex",        //  2
                          "Vertices",      //  3
                          "VisiList",      //  4
                          "Nodes",         //  5
                          "SurFaces",      //  6
                          "LightMaps",     //  7
                          "BoundNodes",    //  8
                          "Leaves",        //  9
                          "SurfaceList",   //  10
                          "Edges",         //  11
                          "SurfaceEdge",   //  12
                          "Hulls"};        //  13


   QuakeBspEntry *TempBspDir;              // directory entry in BSP (14 of them)

   QuakeBspDirPtr TempDir;

   int4           n;
   //---------------------------------------------------------------------------
   CloseAllQuakeFiles();

   strcpy(TempName,QuakeBspFileName);

   if(!GetFileName(hwnd,"Quake BSP file", TempName, 127, IDS_FILTERBSP, CurrDir)) //   3-1-97 127 +simplify
   {   return FALSE;
   }

   if ((QuakeBspFile = FileOpen( TempName, "rb",OPENMISS)) == NULL)
   {
       return FALSE;
   }

   WadSeek( QuakeBspFile, 0);
   WadRead( QuakeBspFile, &QuakeBspControl,sizeof(QuakeBspControl) );       // read header, offset, etc

   if ( QuakeBspControl.version != 29 )
   {   ErrorMsg(hwnd, "Not a Quake version 29 file!");
       fclose(QuakeBspFile); //Current
       return FALSE;
   }

   strcpy(QuakeBspFileName, TempName);                          // save name for nxt time

   QuakeBspStatus = 'O ';

   QuakeBspDirNum = 14;

   // allocate max dir size

   QuakeBspInternalDir = (QuakeBspDirPtr)GetMemory( 14 * sizeof(QuakeBspDirEntry ));  // my internal dir

   TempDir    = QuakeBspInternalDir;
   TempBspDir = &QuakeBspControl.entities;            // start at entities

   for (n = 0; n < QuakeBspDirNum; n++,TempDir+=1,TempBspDir+=1)
   {
       strcpy(TempDir->name,BspNames[n]);              // name
       TempDir->start = TempBspDir->offset;            // current offset
       TempDir->size  = TempBspDir->size;              // size
   }


   return TRUE;
}

//-------------------------------------------------------------------------------
// close all files
//-------------------------------------------------------------------------------
static void CloseAllQuakeFiles(void)
{
       CloseQuakeWadFile();
       CloseQuakePakFile();
       CloseQuakeBspFile();
}

//-----------------------------------------------------------------------------
// close QuakePAK file
//-----------------------------------------------------------------------------
static void  CloseQuakePakFile(void)
{
  if (QuakePakStatus == '0 ')
  {   fclose(QuakePakFile);
      QuakePakStatus = 'C ';
  }

  FreeMemory(QuakePakInternalDir);
  QuakePakInternalDir = NULL;

}

//-----------------------------------------------------------------------------
// close QuakeBSP file
//-----------------------------------------------------------------------------
static void  CloseQuakeBspFile(void)
{
  if (QuakeBspStatus == '0 ')
  {   fclose(QuakeBspFile);
      QuakeBspStatus = 'C ';
  }

  FreeMemory(QuakeBspInternalDir);
  QuakeBspInternalDir = NULL;
}
//-----------------------------------------------------------------------------
// close QuakeWAD file
//-----------------------------------------------------------------------------
static void  CloseQuakeWadFile(void)
{
  if (QuakeWadStatus == '0 ')
  {   fclose(QuakeWadFile);
      QuakeWadStatus = 'C ';
  }

  FreeMemory(QuakeWadInternalDir);
  QuakeWadInternalDir = NULL;

}

//----------------------------------------------------------------------------
//  find entry in QuakeWadDirectory
//----------------------------------------------------------------------------
static QuakeWadDirPtr FindQuakeWadDir(char *name)
{
   int4 n;
   QuakeWadDirPtr TempDir;
   char        wrkname1[18]={0};
   char        wrkname2[18]={0};
   //-------------------------------------------------------------------------

   strncpy(wrkname1,name,16);

   TempDir = QuakeWadInternalDir;                   // dir buffer

   for (n = 0; n < QuakeWadDirNum; n++,TempDir+=1)
   {
       strncpy(wrkname2,TempDir->name,16);       // if length is 16, no terminator
       if (!stricmp(wrkname1,wrkname2) )         //
       {   return TempDir;
       }
   }
   return NULL;
}

//----------------------------------------------------------------------------
//  find entry in QuakePakDirectory
//----------------------------------------------------------------------------
static QuakePakDirPtr FindQuakePakDir(char *name)
{
   int4 n;
   QuakePakDirPtr TempDir;
   char        wrkname1[60]={0};
   char        wrkname2[60]={0};
   //-------------------------------------------------------------------------

   strncpy(wrkname1,name,56);

   TempDir = QuakePakInternalDir;                   // dir buffer

   for (n = 0; n < QuakePakDirNum; n++,TempDir+=1)
   {
       lstrcpyn(wrkname2,TempDir->name,56);       // if length is 56, no terminator
       if (!stricmp(wrkname1,wrkname2) )          //
       {   return TempDir;
       }
   }
   return NULL;
}

//////////////////////////////////////////////////////////////////////////////
// finds x,y size of texture in   quake WAD, global file and counts
//////////////////////////////////////////////////////////////////////////////
static Bool FindQuakeMipPicSize(FILE *MipFile,char*picname, int4 offset, int4 *xsize,int4 *ysize)
{
   QuakeGraphicHdr QuakeImageHeader;

   if (!strnicmp(picname,"palette",7)) return FALSE;  // not palette

   WadSeek( MipFile, offset );                        // seek start of entry
   WadRead( MipFile, &QuakeImageHeader,40);           // x width, y height, xoff,yoff,1stYoffset
   // get this TEXTURE offset
   //if (offset)
   //{   *offset = QuakeImageHeader.offset1 +dir->start;  //offset to start of 1st graphics image
   //}
   *xsize =  QuakeImageHeader.xsize;
   *ysize =  QuakeImageHeader.ysize;

   return TRUE;
}


//----------------------------------------------------------------------------
//  erase background for browsing
//----------------------------------------------------------------------------

static void SetViewBackGround(HWND hwnd, HDC hdc, HBRUSH hBrush)
{
  RECT rc;
  GetClientRect(hwnd, &rc);
  //SetRect(&rc, 0, 0, ScrMaxX, ScrMaxY);
  FillRect(hdc, &rc, hBrush);
}


//----------------------------------------------------------------------------
//  Browse Quake Wad Textures - browse -ONE- Quake WAD
//----------------------------------------------------------------------------

FILE  *BrowseMipFile;
char  *BrowseMipFileName;

//----------------------------------------------------------------------------

static void CreateQuakeWadTextureList (void)  //BrowseQuakeWadTextures
{
   int4 n;
   QuakeWadDirPtr TempDir;
//----------------------------------

   BrowseMipFile     = QuakeWadFile;
   BrowseMipFileName = QuakeWadFileName;

   FreeBrowseList();      // always make sure it's empty
   
   //------------------------------------------------
   // store names in list for dialog selection
   //------------------------------------------------

   BrowseListPtrs  = (char **)GetMemory( QuakeWadDirNum * sizeof( char*));    // n pointers allocated
   BrowseListNames = (char *) GetMemory( QuakeWadDirNum * 22);                //  byte allocations

   BrowseListWork  = BrowseListNames;                       // where to store
   BrowseListCount = 0;                                     // number of entries

   TempDir = QuakeWadInternalDir;                           // dir buffer

   for (n = 0; n < QuakeWadDirNum; n++,TempDir+=1)
   {
       if (!strnicmp(TempDir->name,"palette",7)) continue;  // not the palette
       memset(BrowseListWork,0,18);

       strncpy(BrowseListWork,TempDir->name,16);            // name
       memcpy(BrowseListWork+18,&TempDir->start,4);         // offset

       BrowseListPtrs[BrowseListCount] = BrowseListWork;    // addr of saved
       BrowseListCount++;                                   // +1 saved
       BrowseListWork+=22;                                  // next avail loc
   }

}


//----------------------------------------------------------------------------
//  Browse Quake PAK BSP Textures - browse -ONE- Quake BSP file in a PAK
//----------------------------------------------------------------------------
static void CreateQuakePakTextureList (void) //BrowseQuakePakTextures old
{
   int4 n;
   char *  TempDir;
//----------------------------------

   BrowseMipFile     = QuakePakFile;
   BrowseMipFileName = QuakePakFileName;

   FreeBrowseList();      // always make sure it's empty

   //------------------------------------------------
   // store names in list for dialog selection
   //------------------------------------------------

   BrowseListPtrs  = (char **)GetMemory( PAKBSPlistcount * sizeof( char*));    // n pointers allocated
   BrowseListNames = (char *) GetMemory( PAKBSPlistcount * 22);                //  byte allocations

   BrowseListWork  = BrowseListNames;                       // where to store
   BrowseListCount = 0;                                     // number of entries

   TempDir = PAKBSPlistnames;                                  // dir buffer

   for (n = 0; n < PAKBSPlistcount; n++,TempDir+=22)
   {
       if (!strnicmp(TempDir,"palette",7)) continue;        // not the palette
       memset(BrowseListWork,0,18);

       strncpy(BrowseListWork,TempDir,16);                  // name
       memcpy(BrowseListWork+18,TempDir+18,4);              // offset

       BrowseListPtrs[BrowseListCount] = BrowseListWork;    // addr of saved
       BrowseListCount++;                                   // +1 saved
       BrowseListWork+=22;                                  // next avail loc
   }
}

//----------------------------------------------------------------------------
//  Browse Quake BSP Textures - browse -ONE- Quake BSP file
//----------------------------------------------------------------------------
static void CreateQuakeBspTextureList (void)  // was BrowseQuakeBspTextures
{
   int4 n;
   char *  TempDir;
//----------------------------------

   BrowseMipFile     = QuakeBspFile;
   BrowseMipFileName = QuakeBspFileName;

   FreeBrowseList();      // always make sure it's empty

   //------------------------------------------------
   // store names in list for dialog selection
   //------------------------------------------------

   BrowseListPtrs  = (char **)GetMemory( BSPlistcount * sizeof( char*));    // n pointers allocated
   BrowseListNames = (char *) GetMemory( BSPlistcount * 22);                //  byte allocations

   BrowseListWork  = BrowseListNames;                       // where to store
   BrowseListCount = 0;                                     // number of entries

   TempDir = BSPlistnames;                                  // dir buffer

   for (n = 0; n < BSPlistcount; n++,TempDir+=22)
   {
       if (!strnicmp(TempDir,"palette",7)) continue;        // not the palette
       memset(BrowseListWork,0,18);

       strncpy(BrowseListWork,TempDir,16);                  // name
       memcpy(BrowseListWork+18,TempDir+18,4);              // offset

       BrowseListPtrs[BrowseListCount] = BrowseListWork;    // addr of saved
       BrowseListCount++;                                   // +1 saved
       BrowseListWork+=22;                                  // next avail loc
   }
}

//=============================================================================
// Help for below
//=============================================================================
static void BrowseHelp(HWND hwnd)
{
    // NOTE :  Can't use HelpMsg since it screws up the Palette !!!!
    /* HelpMsg(hwnd, NOCENTER," HELP  For  Browsing Quake Textures",
           "         Mouse Buttons"
           "",
		     " Left  button = Select texture",
		     " Right button = Exit browser",
           "",
		     "  Click Up/Down Buttons to browse",
           "",
		     "         Keyboard Keys",
           "",
		     "  +, -       = Change display size",
		     "  Esc or C   = Cancel/Exit",
		     "  PageUp/Down= Page display",
		     "  Arrow keys = Position select",
		     "  Enter      = Select texture",NULL);  */
           HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,
                               (DWORD)"Texture Browsing Commands");// index name in help needs changing
     /*
     NotifyMsg(hwnd, " HELP  For  Browsing Quake Textures\n\n"
           "         Mouse Buttons\n\n"
		     " Left  button = Select texture\n"
		     " Right button = Exit browser\n\n"
		     "  Click Up/Down Buttons to browse\n\n"
		     "         Keyboard Keys\n\n"
		     "  +, -       = Change display size\n"
		     "  Esc or C   = Cancel/Exit\n"
		     "  PageUp/Down= Page display\n"
		     "  Arrow keys = Position select\n"
		     "  Enter      = Select texture\n");*/
}

//----------------------------------------------------------------------------
//  BrowseQuakeMIPTextures - browse -ONE- Quake FILE
//
// 2-20-97 bk MODS
//----------------------------------------------------------------------------
#pragma argsused
static char * BrowseQuakeMipTextures (HWND hwnd, char *UserName)
{

static int2 QuakeBrowseTextureMag=1;        //possible put in DeeP.chk later
static int2 BrowseBackGround=BLACK;
static int2 BrowseRectangle =WHITE;

   int4 xsize;         //  width in pixels
   int4 ysize;         //  height in pixels

   int2 MaxMagnify     =  3;
   int2 CurrentMagnify = QuakeBrowseTextureMag;
   int2 oldmagnify     = -1;
   int2 LocalMagnify;

   // save last clear area

   int2 oldLastPicXloc = LastPicXloc;
   int2 oldLastPicYloc = LastPicYloc;
   int2 oldLastPicXsize= LastPicXsize;
   int2 oldLastPicYsize= LastPicYsize;

   int2 CurrYorg;
   int2 CurrXorg;
   int2 CurrXloc;
   int2 CurrYloc;
   int2 Width       = 64;
   int2 Height      = 64;

   int2 FirstTexture=  0;         //   3-1-97 these 5 were static
   int2 ActivePic   =  0;         // current active picture number
   int2 LastActive  = -1;         // prior active
   int2 FirstOrigin =  0;         // first on screen
   int2 LastOrigin ;              // last  on screen

   int2 NumberPerScreen;
   int2 Xacross;
   int2 Ydown;
   static int2 i = -1;
   int2 n;
   int2 key;

   int2 Xloc,Yloc;
   int2 HitLoc;

   int2 HotXloc,HotYloc;

   static Bool notdone = TRUE;

   int2 FontSize = 18;

   int2 Button = 0;
   int4 MouseX;
   int4 MouseY;

   Bool LastPicture;
   Bool KeepGoing   = TRUE;      //   3-1-97 was static

       char picname[18] ={0};
static char newname[18] ={0};    // for return to caller

   int4     offset;
   int2     code;

   int4     newoffset;
   HPALETTE hOldPal;
   HFONT    hOldFont;
   HBRUSH   hOldBrush;
   HBRUSH   hBackGroundBrush, hFrameBrush;

   HDC      hdc;
   MSG      msg;    // Our Message Structure
   RECT     rc;

   int4     MaxX, MaxY;

//-----------------------------------------------------------------------

   hdc       = GetDC(hwnd);
   hOldPal   = SelectPalette(hdc, hQuakePal, FALSE);

  	RealizePalette(hdc);
   hOldBrush = SelectBrush(hdc, GetStockBrush(BLACK_BRUSH) );

   hOldFont = SelectFont(hdc, gd.hStockFont);//GetStockFont(ANSI_VAR_FONT)); // 2-1-97 bk

   GetClientRect(hwnd, &rc);

   MaxX = rc.right;
   MaxY = rc.bottom;

   if (BrowseBackGround == BLACK)                          // 3-1-97  
   {   hBackGroundBrush = GetStockBrush(BLACK_BRUSH);
       hFrameBrush      = GetStockBrush(WHITE_BRUSH);
   }
   else
   {   hBackGroundBrush = GetStockBrush(WHITE_BRUSH);
       hFrameBrush      = GetStockBrush(BLACK_BRUSH);
   }

   SetViewBackGround(hwnd, hdc, hBackGroundBrush);

   if (CurrentMagnify < 0) CurrentMagnify = 1;    //<=0

   // list is already created when called now

   if (Sorted)
   {   qsort( BrowseListPtrs, BrowseListCount, sizeof( char *), QsortListName16);
   }

   if (UserName)
   {   strncpy(newname,UserName,16);              // save entry name

       for (i=0; i< BrowseListCount; i++)
       {	  strncpy(picname,BrowseListPtrs[i],16);
	        if (!strcmp (picname,UserName) )
	        {   FirstTexture = i;                     // start display with this one
	            ActivePic = i;
	            break;
	       }
       }
   }

   //NotifyMsg(NULL,"browsestart");

   EatMouseData(hwnd);

   while(KeepGoing)
   {
     for (i=FirstTexture; i< BrowseListCount; i++)
     {
         if (oldmagnify != CurrentMagnify)
      	{   FirstTexture=0;           // to go backwards

	          Width   = 160;
	          Height  = 128;

	          FirstOrigin=i;
	          HotXloc    =0;

            if (CurrentMagnify > 0)
            {   Width *= CurrentMagnify; //
	             Height*= CurrentMagnify; //
            }
            else
            {   Width /=2; //
	             Height/=2; //
            }

	         oldmagnify = CurrentMagnify;

	         Xacross = (MaxX -4) / (Width + 8);
	         Ydown   = (MaxY   ) / (Height+FontSize);

	         NumberPerScreen = Xacross * Ydown;
	         CurrXorg =  ( MaxX - Xacross*( Width+8  )-4 )/2;
	         CurrYorg =  ( MaxY - Ydown  *( Height+FontSize))/2;
	         if (CurrYorg < 0) CurrYorg = 0;
	         if (CurrXorg < 0) CurrXorg = 0;

	         CurrYloc = CurrYorg;
	         CurrXloc = CurrXorg;

	         if (oldmagnify != -1 && NumberPerScreen != 1)
	         {   SetViewBackGround(hwnd, hdc, hBackGroundBrush);
            }
	         else
	         {   SetRect(&rc, CurrXorg-1,CurrYorg,CurrXorg+Width+6,CurrYorg+Height);
	             FrameRect(hdc, &rc, hBackGroundBrush);
	         }
	      }

         strncpy (picname,(char *) BrowseListPtrs[i],16);   // name
         memcpy  (&offset, BrowseListPtrs[i]+18,4);         // offset

         if (FindQuakeMipPicSize(BrowseMipFile,picname,offset, &xsize, &ysize) )
         {  LocalMagnify = CurrentMagnify;
            if (CurrentMagnify > 0)
            {   LocalMagnify +=1;
	         }

            if (NumberPerScreen ==1)
	         { DrawScreenBox(hdc, CurrXloc,CurrYloc,
		        CurrXloc+Width+5,CurrYloc+Height-1,hBackGroundBrush);
            }

            LastPicXloc= -1;                    // don't erase last picture

            code =DisplayQuakeMipTexture(hwnd, hdc, //FALSE,
                                         BrowseMipFile,
                                         BrowseMipFileName,
                                         offset,
                                         CurrXloc,CurrYloc,
                                         CurrXloc+Width+5,CurrYloc+Height-1,
                                         NULL, NULL, NULL, NULL, NULL,
                                         picname,
                                         LocalMagnify,
                                         NOSHOWINFO, 0, SHOWIMAGE);

            if (code < 0)  ErrorMsg(hwnd, "Not enough memory!");
	      }

	      // erase prior text only if at maximum level
	      if (NumberPerScreen == 1)
	      {
	         SetRect(&rc, CurrXloc,CurrYloc+Height,CurrXloc+wt*24,CurrYloc+Height+ht2);
            FillRect(hdc, &rc, hBackGroundBrush);
	      }

         // Set Text color depending on background
         if (BrowseBackGround == BLACK)
         {   SetTextColor(hdc, PALETTEINDEX(QuakeColors[WHITE]));
         }
         else
         {   SetTextColor(hdc, PALETTEINDEX(QuakeColors[BLACK]));
         }

         if (CurrentMagnify < 1)
         {   DrawScreenText (hdc,CurrXloc,CurrYloc+Height,"%s",picname);    //+64
         }
         else
         {   FindQuakeMipPicSize(BrowseMipFile,picname,offset,&xsize, &ysize);
	          DrawScreenText(hdc,CurrXloc,CurrYloc+Height,"%s %3dx%-3d",picname, xsize, ysize);
	      }

	      LastOrigin = i;

	      // draw a rectangle around active one
       	if  (i == ActivePic)
	      {
	          SetRect(&rc, CurrXloc-1,CurrYloc,CurrXloc+Width+6,CurrYloc+Height);
	          FrameRect(hdc, &rc, hFrameBrush);
	          HotXloc = CurrXloc;
	          HotYloc = CurrYloc;
	      }

         if (i == BrowseListCount -1)
         {   LastPicture = TRUE;
         }
	      else
	      {   LastPicture = FALSE;
         }

         CurrXloc+=(Width+8); //72;

	      if (CurrXloc+Width+8 >=MaxX || LastPicture )
	      {
	          CurrXloc  = CurrXorg;
	          CurrYloc +=(Height+FontSize);

	          if (CurrYloc+Height+FontSize > MaxY
              || LastPicture)
	          {
	             notdone = TRUE;
	             oldmagnify = -1;               // force change
                                               // 2-20-97 bk
                //ValidateRect(hwnd, NULL);      // Make sure NOT REDRAW
                //  3-1-97 WM_PAINT code below caused redraw problem

	             // let white box browse around
	             while(notdone)
	             {
                   WaitMessage();
                   PeekMessage(&msg, hwnd, WM_PAINT, WM_PAINT, PM_REMOVE);
                   // just moving mouse causes this message to appear?
                   //if(msg.message == WM_PAINT)    //   3-1-97 I couldn't see where this does anything??
                   //{ SelectPalette(hdc, hQuakePal, FALSE);// if Window needs update
  	                  // RealizePalette(hdc);
		               // i -=NumberPerScreen;                 //Backup to Top of Screen
		               // if (i < -1) i=-1;                    //   3-1-97
		               // KeepGoing = TRUE;                    //for redraw i++
		                /////////  notdone   = FALSE;   3-1-97 for sure not, cause double display
                     // ActivePic = i;
                     //static times;
                     //Trace(hwnd,10,4,"browse%d",times++);
                   //}

	                // set current name
	                if (ActivePic >=0 && ActivePic < BrowseListCount
	                &&  ActivePic != LastActive )
	                {   LastActive = ActivePic;                                      // MenuColor

		                 strncpy(newname,BrowseListPtrs[ActivePic],16);
		                 memcpy (&newoffset,BrowseListPtrs[ActivePic]+18,4);

		                 //if (FindQuakeMipPicSize(BrowseMipFile,newname,&xsize, &ysize,NULL))
		                 if (FindQuakeMipPicSize(BrowseMipFile,newname,newoffset,&xsize, &ysize))
		                 {
                          DisplayCaptionText(hwnd,300,5," %16s %3d x %-3d ",newname, xsize, ysize);
		                 }
	                }

                   key = GetMouseCoords(hwnd,&msg,NULL,&MouseX,&MouseY,&Button);

                   if(Button & 2)        // right hand mouse click
	                {  KeepGoing=FALSE;
		                ActivePic  = -1;
                   }

		             if(Button & 1)
		             {  //key = GetMouseCoords(hwnd,&msg,NULL, &MouseX, &MouseY, NULL);//
                      MouseX -=CurrXorg;  // -origin X
		                MouseY -=CurrYorg;  // -origin Y
                      if (MouseY >=0 && MouseX >=0)
		                { Xloc =  (MouseX ) / (Width +8);
			               Yloc = ( MouseY ) / (Height+FontSize);
                        HitLoc = Xloc+  (Yloc*Xacross)+FirstOrigin;
			               if (HitLoc >= 0 && HitLoc < BrowseListCount )
			               {
                            ActivePic = HitLoc;
			                   KeepGoing = TRUE;   //

                            SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                         FrameRect(hdc, &rc, hBackGroundBrush);

                            HotXloc =  (Xloc * (Width+8)   ) + CurrXorg;
                            HotYloc =  (Yloc * (Height+FontSize)   ) + CurrYorg;

	                         SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                         FrameRect(hdc, &rc, hFrameBrush);
			               }
		                }
                   }

                   if(Button & 4)        // Dblclk
		             {  //key = GetMouseCoords(hwnd,&msg,NULL, &MouseX, &MouseY, NULL);//
                      MouseX -=CurrXorg;  // -origin X
		                MouseY -=CurrYorg;  // -origin Y
                      if (MouseY >=0 && MouseX >=0)
		                {  Xloc =  (MouseX ) / (Width +8);
			                Yloc = ( MouseY ) / (Height+FontSize);
                         HitLoc = Xloc+  (Yloc*Xacross)+FirstOrigin;
			                if (HitLoc >= 0 && HitLoc < BrowseListCount )
			                {
                            ActivePic = HitLoc;
			                   KeepGoing = FALSE;   //
                         }
                      }
                   }

	               switch(key)
	               {
                   case(VK_BACK):      //backspace
                    if (BrowseBackGround == BLACK)
                    {   BrowseBackGround = WHITE;
                        BrowseRectangle  = BLACK;
                        hBackGroundBrush = GetStockBrush(WHITE_BRUSH);
                        hFrameBrush      = GetStockBrush(BLACK_BRUSH);

                    }
                    else
                    {   BrowseBackGround = BLACK;
                        BrowseRectangle  = WHITE;
                        hBackGroundBrush = GetStockBrush(BLACK_BRUSH);
                        hFrameBrush      = GetStockBrush(WHITE_BRUSH);
                    }

                    SetViewBackGround(hwnd, hdc, hBackGroundBrush);

		              KeepGoing = TRUE;
		              notdone   = FALSE;
                    break;
		             case(VK_RETURN):
		              KeepGoing=FALSE;
		              break;
	             	 case(VK_ESCAPE):
		              KeepGoing  = FALSE;
		              ActivePic  = -1;
		              break;
		             case(VK_RBUTTON):
		              KeepGoing  = FALSE;
		              ActivePic  = -1;
		              break;
		             case('=')://(0x00bb):     // +
	             	 case('+')://  2-1-97 bk
		              if (CurrentMagnify < MaxMagnify) //3
		              {
		                 CurrentMagnify++;

		                 i -=NumberPerScreen;   // 12 per page

		                 if (i < -1) i=-1;
		                 KeepGoing = TRUE;
		                 notdone   = FALSE;

		                 if (CurrentMagnify == MaxMagnify)
		                 {
                          SetViewBackGround(hwnd, hdc, hBackGroundBrush);
		                 }
		              }
		              break;
		             case('-')://(0x00bd):     // -
		             case('_')://    2-1-97 bk
		              if (CurrentMagnify > 0)//1) //0
		              {
		                 CurrentMagnify--;
		                 i -=NumberPerScreen;;  // 12 per page

		                 if (i < -1) i=-1;
		                 KeepGoing = TRUE;
		                 notdone   = FALSE;
		              }
		              break;
		             case(VK_PRIOR):  // Page Up    (prior page)
		              i -=NumberPerScreen*2; //# per page

		              if (i < -1) i=-1;
		              KeepGoing = TRUE;
		              notdone   = FALSE;
		              break;
	               case(VK_NEXT):  // Page Down  (next page)
		              notdone   = FALSE;
		              break;
		            case(VK_HOME):  // Home      (move first entry)
		              i = -1;
		              KeepGoing = TRUE;
		              notdone   = FALSE;
		              break;
		            case(VK_END):  // End       (move last entry)
		              i = BrowseListCount - NumberPerScreen -1;

		              if (i < -1) i = -1;
		              KeepGoing = TRUE;
		              notdone   = FALSE;
		              break;
	               // uparrow--------------------------------------------------
		            case(VK_UP):
		             // erase active one if any on this screen
		             if (HotXloc)
		             {  SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                   FrameRect(hdc, &rc, hBackGroundBrush);
		             }
		             else // nothing active yet, make it first one
		             {
			             ActivePic = FirstOrigin;      // make First active
		                HotXloc   = CurrXorg;//32;
			             HotYloc   = CurrYorg;//16;
		             }

		             // has to be on this screen
		             if ( ActivePic -Xacross >= FirstOrigin )
		             {   ActivePic -= Xacross;
		                 HotYloc   -= (Height+FontSize);
		             }
	                SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                FrameRect(hdc, &rc, hFrameBrush);
		             break;
		            // down arrow-----------------------------------------------
		            case(VK_DOWN):
		             // erase active one if any on this screen
		             if  (HotXloc)
 		             {
	                   SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                   FrameRect(hdc, &rc, hBackGroundBrush);
		             }
		             else // nothing active yet, make it first one
		             {
			            ActivePic = FirstOrigin;      // make First active
			            HotXloc   = CurrXorg;//32;
			            HotYloc   = CurrYorg;//16;
		             }

		             // has to be on this screen
		             if ( ActivePic +Xacross <= LastOrigin )
		             {   ActivePic  += Xacross;
			              HotYloc    += (Height+FontSize);
		             }

	                SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                FrameRect(hdc, &rc, hFrameBrush);
		             break;
		            // left arrow ----------------------------------------------
		            case(VK_LEFT):
		             // erase active one if any on this screen
		             if (HotXloc)
		             {   SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                    FrameRect(hdc, &rc, hBackGroundBrush);
		             }
		             else // nothing active yet, make it first one
		             {
			              ActivePic = FirstOrigin;      // make First active
			              HotXloc   = CurrXorg;//32;
			              HotYloc   = CurrYorg;//16;
		             }

		             // backup up picture

		             if (ActivePic -1  >= FirstOrigin )
		             {   ActivePic --;
		                 HotXloc -= (Width+8);
		                 if (HotXloc < CurrXorg)//32  )
		                 {   HotXloc  = (Xacross-1)*(Width+8)+CurrXorg;//32;
			                  HotYloc -= (Height+FontSize);
		                 }
		             }

	                SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                FrameRect(hdc, &rc, hFrameBrush);
		             break;
		            // right arrow---------------------------------------------------
		            case(VK_RIGHT):
		             // erase active one if any on this screen
		             if (HotXloc)
		             {   SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                    FrameRect(hdc, &rc, hBackGroundBrush);
		             }
		             else // nothing active yet, make it first one
		             {
			              ActivePic = FirstOrigin;      // make First active
			              HotXloc   = CurrXorg;//32;
			              HotYloc   = CurrYorg;//16;
		             }

		             // see if past end of screen-------------
		             if (ActivePic+1 <= LastOrigin)
		             {   ActivePic ++;
		                 HotXloc += (Width+8);
		                 if (HotXloc+Width+8 >= MaxX )
		                 {   HotXloc  = CurrXorg;//32;
			                  HotYloc +=(Height+FontSize);
		                 }
		             }
	                SetRect(&rc, HotXloc-1,HotYloc,HotXloc+Width+6,HotYloc+Height);
	                FrameRect(hdc, &rc, hFrameBrush);
		             break;
		           case(VK_F1):                //F1
		             BrowseHelp(hwnd);             //
		             break;
		           // look for a match--------------------------
		           default: // wm_char
		            key &=0x00ff;
		            if (key && isprint (key))       // not extended, check it
		            {   i = -1;
		                KeepGoing = TRUE;
		                for (n=0; n< BrowseListCount; n++)
		                {
			                if ( !strnicmp(&BrowseListPtrs[n][0],(char *)&key,1 ) )
			                {   i = n-1;
				                 break;
			                }
			                else         //find closest so far
			                if ( strnicmp(&BrowseListPtrs[n][0],(char *)&key,1 ) < 0 )
			                {   i= n-1;
			                }
		                }
		                notdone = FALSE;         // get next
		            }
		            break;
	            }  // end switch key

	            if (!KeepGoing) notdone = FALSE;         // always stops
	         }

	         if (!KeepGoing) break;
	         CurrYloc = CurrYorg;
	      }
	     }
     }

   }

   QuakeBrowseTextureMag = CurrentMagnify;
   //NotifyMsg(NULL,"browseexit");

   //if (ActivePic >=0 && ActivePic < BrowseListCount)
   //{   strncpy(newname,BrowseListPtrs[ActivePic],16);
   //}

   // free is done here           3-1-97 can't be done here! Used by caller
   /*
   FreeMemory(BrowseListNames);
   BrowseListNames = NULL;
   FreeMemory(BrowseListPtrs);
   BrowseListPtrs = NULL;
    */
   // restore so can clear ok

   LastPicXloc =  oldLastPicXloc;
   LastPicYloc =  oldLastPicYloc;
   LastPicXsize=  oldLastPicXsize;
   LastPicYsize=  oldLastPicYsize;

   SelectBrush(hdc, hOldBrush);
   SelectFont(hdc, hOldFont);
   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hwnd, hdc);

   if (ActivePic >=0 && ActivePic < BrowseListCount)
   {   strncpy(newname,BrowseListPtrs[ActivePic],16);
       return (newname);
   }
   else
   {   return NULL;
   }
}


///////////////////////////////////////////////////////////////////////////////
// Quake Browser WndProc
///////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK QBrowseWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{

LPQDATA qdat;                   // This all works fine now, just not implemented

static char  oldname[24]={0};
       char* newname;

  switch(msg)
  {
	case WM_BROWSE:
     newname = BrowseQuakeMipTextures (hwnd, oldname);

     DeePDestroyWindow(hwnd);

     if(newname)
     {  PostMessage(gd.hParent, WM_COMMAND, WM_BROWSE_BACK, (LPARAM)(char *)newname);
     }
     else
     {  PostMessage(gd.hParent, WM_COMMAND, WM_BROWSE_BACK, (LPARAM)(char *)oldname);
     }
     break;

   case WM_CREATE:
     SetWindowLong(hwnd, GWL_USERDATA,(LPARAM)((LPCREATESTRUCT)lParam)->lpCreateParams);
     qdat = (LPQDATA)GetWindowLong(hwnd, GWL_USERDATA);
     strcpy(oldname, qdat->name);        // copy string passed in Create Struct.
     PostMessage(hwnd, WM_BROWSE, 0, 0); // give window time to create
     return 0;

   case WM_PAINT:             // at end of browse
 	  PAINTSTRUCT ps;
     BeginPaint(hwnd, &ps);
     //NotifyMsg(NULL,"Qbrowsewndproc");
	  EndPaint(hwnd, &ps);
     return 0;

   HANDLE_MSG(hwnd, WM_GETMINMAXINFO, Browse_OnGetMinMaxInfo);  //   3-1-97

	default:
	  break;
  }

  return QView_DefProc(hwnd, msg, wParam,lParam);
}



//----------------------------------------------------------------------------
//  find entry in QuakePakDirectory
//----------------------------------------------------------------------------
static QuakeBspDirPtr FindQuakeBspDir(char *name)
{
   int4 n;
   QuakeBspDirPtr TempDir;

   //-------------------------------------------------------------------------

   TempDir = QuakeBspInternalDir;                      // dir buffer

   for (n = 0; n < 14; n++,TempDir+=1)
   {
       if (!stricmp(name,TempDir->name) )          //
       {   return TempDir;
       }
   }
   return NULL;
}

/*
//////////////////////////////////////////////////////////////////////////////
//  The Deep Quake Display Lump Modeless Dialog
//////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QLumpBspDisplayDlgProc(DIALOG_PARAMS)
{
   static QuakeBspDirPtr TempDir;
   static char  dirname[38]={0};

   //int4  DirOffset= 0;
   int4  byteoffset=0;  // byte offset

   static FILE *DumpFile;
   int2  i,j;

   //buf is 80 but read text for 260
   char  buf[260];   // 1234 | 12 34 12 34 12 34 12 34 .. .. | 12345678 12345678 |
   char  wrkbuf [16];
   byte  databuf[16]; //data


//===================================================================
  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

   switch (msg)
   {
    case WM_INITDIALOG:
      AdjustDlgPos(hDlg, 236,-20);       // max left (1024) and down a bit, for less res it's on top
      TempDir = FindQuakeBspDir(LumpName);
      if (!TempDir) return -1;

      SendMessage(hDlg, WM_CHANGE, (WPARAM)NULL, (LPARAM)0);
      return TRUE;

   case WM_DESTROY:
      return TRUE;

   case WM_CHANGE:
      TempDir = FindQuakeBspDir(LumpName); // Can't find, don't do anything
      if (!TempDir) break;

      strncpy( dirname, TempDir->name, 38);
      wsprintf( buf, "Contents of entry# %hd %s (%d bytes)",
			             LumpIndex, dirname, TempDir->size);

      Static_SetText(GetDlgItem(hDlg, stc1), buf ); // Update Text

      ListBox_ResetContent(GetDlgItem(hDlg, lst2)); // Get rid of Old contents

      if(TempDir->size > 64000)
      {  if (! Confirm(hDlg,"Files over 64k may take a while to load\n\n"
               "And/Or cause your machine to become unstable\n"
               "If you do not have enough RAM. Are Your Sure ?"))
         return TRUE;
      }

      DumpFile = FileOpen(QuakeBspFileName, "rb",OPENMISS);    // new open binary

      if (!DumpFile) break;     // failed

      // starting here

      WadSeek ( DumpFile, TempDir->start);

      while(byteoffset < TempDir->size)
      {
         WadSeek ( DumpFile, TempDir->start + byteoffset);

	      if (byteoffset < TempDir->size)
	      {   wsprintf( buf, "%04X |",byteoffset);               // offset 
	      }
	      else
	      {   strcpy ( buf, "      ");
	      }

	      for (i = 0; i < 16 && byteoffset < TempDir->size; i++) // 16 bytes of hex
	      {
	          WadRead( DumpFile, &(databuf[i]), 1);
	          wsprintf( wrkbuf, " %02X", databuf[i]);
	          strcat(buf,wrkbuf);
	          byteoffset++;                          //+1 byte listed (offset)
	      }

	      for (j=i; j< 16; j++)                      // filler at the end
	      {   strcat(buf,"   ");
	      }

	      strcat (buf, " |");
	      for (j = 0; j < i; j++)                    // sixteen bytes char
	      {
	        if (databuf[j] >= 32 && databuf[j] < 127 )
	        {  wsprintf( wrkbuf, "%c",databuf[ j]);  // printable
		        strcat(buf,wrkbuf);
	        }
	        else
	        {  strcat( buf, ".");                    // nonprintable
	        }
	      }

	      for (j=i; j< 16; j++)                      // filler at the end
	      {    strcat(buf," ");
	      }

	      strcat( buf, "|");   // ""

         ListBox_AddString(GetDlgItem(hDlg, lst2), buf);
      }

      if (DumpFile)  fclose(DumpFile);
     	break;

   default:
      break;
  }
  return FALSE;
}
*/

//*******************************************************************
// SetupScroll - setup scroll ranges
//
// Setup the vertical and horizontal scroll ranges and positions
// of the applicatons main window based on:
//
//     numlines - The maximum number of lines to display in a hexdump
//                of the selected file.
//     maxwidth - The width of each line to display as formated for
//                the hexdump.
//
// The resulting variables, nVscrollPos and nPageMaxLines, are used
// by the function HdumpPaint to determine what part of the selected
// file to display in the window.
//
// parameters:   hWnd          - The callers window handle
//
//*******************************************************************

static int4  nVscrollMax, nHscrollMax;           // scroll ranges
static int4  nVscrollPos, nHscrollPos;           // current scroll positions
static int4  xClient, yClient;                   // client window size
static int4  numlines;                           // number of lines in file
static int4  maxwidth;                           // width of display format
static int4  nVscrollInc;//, nHscrollInc;           // scroll increments
static int4  nLinesPerPage;                      // max lines on screen
static int4  xChar, yChar, yCharHeight;          // character size

static char  HeadingBuf[100];   // 1234 | 12 34 12 34 12 34 12 34 .. .. | 12345678 12345678 |

static Bool  DumpFmode;        // 0 = hex, 1 = text  (later)
static char  DumpFileName[260];
static FILE *DumpFile;
static int2  DumpFindex;
static char  DumpLumpName[64];
static int4  DumpFstart;                         // start of offset into file
static int4  DumpFsize;                          // size of entry
       int4  DumpFoffset;                        // current offset just shown accessible for now 3-1-98

static void  SetupScroll(HWND hWnd)
{
    RECT rc;

    GetClientRect(hWnd,&rc);
    yClient = rc.bottom;
    xClient = rc.right;

    // numlines established during open
    nVscrollMax = max(0, numlines - yClient / yChar);
    //if (nVscrollMax > 65535)
    //{   nVscrollMax = 65535;
    //}
    nVscrollPos = min(nVscrollPos, nVscrollMax);

    nHscrollMax = max(0, maxwidth - xClient / xChar);
    nHscrollPos = min(nHscrollPos, nHscrollMax);
/*
    SetScrollRange (hWnd, SB_VERT, 0, nVscrollMax, FALSE);
    SetScrollPos   (hWnd, SB_VERT,    nVscrollPos, TRUE);

    SetScrollRange (hWnd, SB_HORZ, 0, nHscrollMax, FALSE);
    SetScrollPos   (hWnd, SB_HORZ,    nHscrollPos, TRUE);
*/
    nLinesPerPage = min(numlines, yClient / yChar);     //lines per page
    /*
    NotifyMsg(NULL,"numlines = %d\n"
                   "Vscroll  = %d\n"
                   "Hscroll  = %d\n"
                   "lines/pg = %d\n"
                   "x %d, y %d, yh %d\n"
                   "client x %d, y %d",
                   numlines,nVscrollMax,nHscrollMax,nLinesPerPage,
                   xChar, yChar, yCharHeight,
                   xClient, yClient);
    */
}

//*******************************************************************
// HdumpPaint - paint the main window
//
// If a file has been selected, as indicated by numlines being
// greater than 0, this module will read and display a portion
// of the file as determined by the current size and scroll
// position of the window.
//
// paramaters:
//             hWnd          - The callers window handle
//
//*******************************************************************
static char *DumpLine(char *szBuf, LPSTR mem, int len, int dwid, char *offset);

static HWND hHexDumpAllDlg;            // to post keys to and other stuff

static void HdumpPaint(HWND hWnd)
{
    PAINTSTRUCT  ps;
    HDC          hDC;
    uint4        offset;
    int4         bytesRead;
    int4         lpos;
    char         buf[128];
    int4         i;
    int4         len;
    //HFILE        hfile;
    LPSTR        lpbuff;
    LPSTR        lpCurrLoc;
    HFONT        hOldFont;
    RECT         rc;
    int4         DumpFmaxoffset;                         // relative end of file

    BeginPaint(hWnd, (LPPAINTSTRUCT) &ps);
    hDC = ps.hdc;

    // Establish fixed font in display context.
    
    GetClientRect(hWnd, &rc);
    FillRect(hDC, &rc, gd.hBrushes[WHITE]);//MenuColor]);//GetStockBrush(BLACK_BRUSH));

    hOldFont = SelectFont(hDC, gd.hHexDumpFont);

    if (numlines
    &&  DumpFile)
    {
        // Get offset into file to start display.

        offset = nVscrollPos * 16;                     //current position (relative to 0)
        DumpFoffset = offset;                          //remember for printing

        if (hHexDumpAllDlg)
        {   sprintf(buf,"%d",offset);
            Static_SetText(GetDlgItem(hHexDumpAllDlg, stc2),buf ); // Update offset
        }
        offset +=DumpFstart;                           // +offset of data

        len    = nLinesPerPage * 16;                   // amount of data to fill a page
        lpbuff =(LPSTR) malloc(len);

        DumpFmaxoffset = DumpFstart+DumpFsize-1;

        if ( (offset+len) > DumpFmaxoffset )   //
        {    len = DumpFmaxoffset - offset +1;
        }

        if (lpbuff)
        {
             if (SeekBytes(DumpFile, offset) )
             {
                 bytesRead = len;

                 if (ReadBytes( DumpFile, lpbuff, len) )
                 {
                     lpos      = 0;                      // starting at line 0
                     lpCurrLoc = lpbuff;

                     for (i = nVscrollPos; i < (nVscrollPos + nLinesPerPage); i++)
                     {
                          if (bytesRead > 16)  len = 16;
                          else            len = bytesRead;

                          bytesRead -= len;

                          DumpLine(buf, lpCurrLoc, len, 16, (char *) (i * 16));

                          TextOut(hDC, xChar * (-nHscrollPos + 0),
                                       yChar * lpos++,
                                       buf,
                                       strlen(buf));

                          lpCurrLoc += 16;     // next buffer address
                     }
                 }
                 free(lpbuff);
             }
        }
        else // no memory
        {
            MessageBox(NULL, "Not enough memory for file",
                                "HDUMP", MB_OK | MB_TASKMODAL);
        }
    }

    SelectFont(hDC, hOldFont);

    EndPaint(hWnd, (LPPAINTSTRUCT) &ps);
}

//*******************************************************************
// DumpLine - dump a line of memory
//
// parameters:
//             szBuf         - buffer to hold result
//             mem           - pointer to memory to format
//             len           - length to format
//             dwid          - max display width (8 or 16 recomended)
//             olbl          - label to put at start of line
//
// returns:
//             A pointer to szBbuf.
//
//*******************************************************************
static char *DumpLine(char *szBuf, LPSTR mem, int len, int dwid, char *offset)
{
    int4  i;
    int4  j;
    byte  c;
    byte  buff[100];
    byte  tbuf[80];

    if (len > dwid) len = dwid;  // set to how many left to do

    *szBuf = 0;

    // Show offset for this line.
    // sprintf((char *)tbuf, " %8d ", offset);            // dec offset
    // strcpy(szBuf, (char *)tbuf);
    sprintf((char *)tbuf, "%06X  ", offset);           // hex offset
    strcat(szBuf, (char *)tbuf);

    // Format hex portion of line and save chars for ascii portion

    memset(buff,' ',16);

    for (i = 0; i < len; i++)
    {
        c = *mem++;

        sprintf((char *)tbuf, "%02X ", c);
        strcat(szBuf, (char *)tbuf);

        if (c >= 32 && c < 127) buff[i] = c;
        else                    buff[i] = '.';        //46;
    }

    j = dwid - i;

    //buff[i] = 0;
    buff[16] = 0;

    // Fill out hex portion of short lines.

    for (i = j; i > 0; i--)
    {    strcat(szBuf, "   ");
    }

    // Add ascii portion to line.
    sprintf((char *)tbuf, " |%s|", (char *)buff);
    strcat(szBuf, (char *)tbuf);

    // Fill out end of short lines.
    for (i = j; i > 0; i--)
    {    strcat(szBuf, " ");
    }

    return(szBuf);
}


//////////////////////////////////////////////////////////////////////////////
//  HexDump Display Generic Lump Modeless Dialog
//  expects : lpd2->sz3);     // filename
//            lpd2->sz4);     // lumpname
//            lpd2->Int4A;    // offset start in file
//            lpd2->Int4B;    // total size
//            lpd2->i1;       // index value
//////////////////////////////////////////////////////////////////////////////

static HWND hHexDumpDisplayDlg;            // to post keys to and other stuff

#pragma argsused
BOOL CALLBACK HexDumpDisplayDlgProc(DIALOG_PARAMS)
{
   //static QuakeBspDirPtr TempDir;
   //static char  dirname[38]={0};

   //static  char  HeadingBuf[100];   // 1234 | 12 34 12 34 12 34 12 34 .. .. | 12345678 12345678 |
   static  HWND  ListArea;
   static  LPDLG2OPTS lpd2;	      // Pointer to control info

   TEXTMETRIC tm;

//===================================================================

   ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97
   hHexDumpDisplayDlg = hDlg;

   switch (msg)
   {
     case WM_INITDIALOG:
       AdjustDlgPos(hDlg, 236,-20);       // max left (1024) and down a bit, for less res it's on top
       //lpd2 = (LPDLG2OPTS)GetWindowLong(hDlg, DWL_USER);  	// grab our pointer
       lpd2 = (LPDLG2OPTS)lParam;		        		// grab it for our use
       ListArea =  GetDlgItem(hDlg, lst2);
       //SelectFont( GetDC(ListArea), gd.hHexDumpFont);
       //SendMessage(hDlg, WM_CHANGE, (WPARAM)NULL, (LPARAM)0);
       return TRUE;

     case WM_DESTROY:
       if (DumpFile) fclose(DumpFile);
       DumpFile = NULL;
       hHexDumpDisplayDlg = NULL;
       return TRUE;

     case WM_CHANGE:
       strcpy(DumpFileName, lpd2->sz3);     // filename
       strcpy(DumpLumpName, lpd2->sz4);     // lumpname
       DumpFstart= lpd2->Int4A;             // TempDir->start;
       DumpFsize = lpd2->Int4B;             // TempDir->size;
       DumpFindex= lpd2->i1;                // index
       DumpFmode = lpd2->b1;                // 0 = hex, 1 = text

       // get text metrics for paint
       ListArea    =  GetDlgItem(hDlg, lst2);
       GetTextMetrics( GetDC(ListArea), &tm);
       xChar       = tm.tmAveCharWidth;
       yChar       = tm.tmHeight+ tm.tmExternalLeading;
       yCharHeight = tm.tmHeight;

       numlines = (int) ((DumpFsize + 15) / 16);           // number of lines required to display this file (rounded)
       DumpLine(HeadingBuf, DumpFileName, 16, 16,NULL);    // get display width by simulating output

       if (DumpFile) fclose(DumpFile);
       DumpFile = FileOpen(DumpFileName, "rb",OPENMISS);    // new open binary

       maxwidth = strlen(HeadingBuf);
       nVscrollPos = 0;
       nHscrollPos = 0;

       SetupScroll( ListArea );//hDlg);

       //strncpy( dirname, TempDir->name, 38);
       //wsprintf(HeadingBuf, "Contents of entry# %hd %s (%d bytes)",
		 //	                   LumpIndex, dirname, TempDir->size);
       wsprintf(HeadingBuf, "Entry# %hd %s (%d)",
		                       DumpFindex, DumpLumpName, DumpFsize);

       SetWindowText(hDlg,DumpFileName);
       Static_SetText(GetDlgItem(hDlg, stc1), HeadingBuf ); // Update Text

       // Show first part of file.
       InvalidateRect(ListArea, NULL, TRUE);
       HdumpPaint(ListArea);
       ValidateRect(hDlg,NULL);
       //UpdateWindow(hDlg);
       return TRUE;

     case WM_PAINT:
       // Go paint the client area of the window with the appropriate
       // part of the selected file.
       SetWindowText(hDlg,DumpFileName);
       Static_SetText(GetDlgItem(hDlg, stc1), HeadingBuf ); // Update Text
       InvalidateRect(ListArea, NULL, TRUE);
       HdumpPaint(ListArea);
       ValidateRect(hDlg,NULL);
       return TRUE;

     default:
       return FALSE;
  }
}


//////////////////////////////////////////////////////////////////////////////
//  Hex Dump / Browse Display Lump Modeless Dialog (dump all) generic
//  expects : lpd2->sz3);     // filename
//            lpd2->sz4);     // lumpname
//            lpd2->Int4A;    // offset start in file
//            lpd2->Int4B;    // total size
//            lpd2->i1;       // index value
//////////////////////////////////////////////////////////////////////////////

typedef struct scrollkeys
{
    WORD wVirtkey;
    int  iMessage;
    WORD wRequest;
} SCROLLKEYS;

SCROLLKEYS key2scroll [] =
{
    { VK_HOME,  WM_VSCROLL, SB_TOP    },
    { VK_END,   WM_VSCROLL, SB_BOTTOM },
    { VK_PRIOR, WM_VSCROLL, SB_PAGEUP },
    { VK_NEXT,  WM_VSCROLL, SB_PAGEDOWN },

    { VK_INSERT,WM_VSCROLL, SB_LINEUP  },
    { VK_DELETE,WM_VSCROLL, SB_LINEDOWN},

    { VK_UP,    WM_VSCROLL, SB_LINEUP },
    { VK_DOWN,  WM_VSCROLL, SB_LINEDOWN },
    { VK_LEFT,  WM_HSCROLL, SB_PAGEUP },
    { VK_RIGHT, WM_HSCROLL, SB_PAGEDOWN }
};


#define NUMKEYS (sizeof key2scroll / sizeof key2scroll[0])

#pragma argsused
BOOL CALLBACK HexDumpAllDlgProc(DIALOG_PARAMS)
{
   // static QuakeBspDirPtr TempDir;
   // static char  dirname[38]={0};

   static  HWND  ListArea;
   static  LPDLG2OPTS lpd2;	      // Pointer to control info

// int4       i;
   TEXTMETRIC tm;

//===================================================================

   ScreenSaverTimer = 0;                    // reset screensaver timer   8-1-97
   hHexDumpAllDlg = hDlg;                      // for processing keys for this class
   // Trace(NULL,10,4,"msg %d",msg);

   switch (msg)
   {
     case WM_INITDIALOG:
       CenterDlgPos(hDlg,0);                // just center
       lpd2 = (LPDLG2OPTS)lParam;		     // grab it for our use

       ListArea =  GetDlgItem(hDlg, lst2);
       //SelectFont( GetDC(ListArea), gd.hHexDumpFont);

       SendMessage(hDlg, WM_CHANGE, (WPARAM)NULL, (LPARAM)0);  // trigger next
       return TRUE;
     case WM_RBUTTONUP:
       PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
       return TRUE;
     //case lst2: //testing
     //  Trace(NULL,10,4,"lst2");
     //  return TRUE;
     case WM_CHANGE:
       strcpy(DumpFileName, lpd2->sz3);     // filename
       strcpy(DumpLumpName, lpd2->sz4);     // lumpname
       DumpFstart= lpd2->Int4A;             // TempDir->start;
       DumpFsize = lpd2->Int4B;             // TempDir->size;
       DumpFindex= lpd2->i1;                // index
       DumpFmode = lpd2->b1;                // 0 = hex, 1 = text

       // get text metrics for paint
       ListArea    =  GetDlgItem(hDlg, lst2);
       GetTextMetrics( GetDC(ListArea), &tm);
       xChar       = tm.tmAveCharWidth;
       yChar       = tm.tmHeight+ tm.tmExternalLeading;
       yCharHeight = tm.tmHeight;

       numlines = (int) ((DumpFsize + 15) / 16);         // number of lines required to display this file (rounded)
       DumpLine(HeadingBuf, DumpFileName, 16, 16,NULL);  // get display width by simulating output

       if (DumpFile) fclose(DumpFile);
       DumpFile = FileOpen(DumpFileName, "rb",OPENMISS);    // new open binary

       maxwidth = strlen(HeadingBuf);
       nVscrollPos = 0;
       nHscrollPos = 0;

       SetupScroll( ListArea );//hDlg);
       SetScrollRange (hDlg, SB_VERT, 0, nVscrollMax, FALSE);
       SetScrollPos   (hDlg, SB_VERT,    nVscrollPos, TRUE);

       //strncpy( dirname, TempDir->name, 38);
       //wsprintf(HeadingBuf, "Contents of entry# %hd %s (%d bytes)",
		 //	                   LumpIndex, dirname, TempDir->size);
       SetWindowText(hDlg,DumpFileName);
       wsprintf(HeadingBuf, "Entry# %hd %s (%d)",
		                       DumpFindex, DumpLumpName, DumpFsize);
       Static_SetText(GetDlgItem(hDlg, stc1), HeadingBuf ); // Update Text

       // Show first part of file.
       InvalidateRect(ListArea, NULL, TRUE);
       HdumpPaint(ListArea);
       ValidateRect(hDlg,NULL);
       //UpdateWindow(ListArea);
       //SetFocus(hDlg);           //ListArea);
       SetFocus(ListArea);           //ListArea);
       //SetCapture(ListArea);		 // testing
       return FALSE;//TRUE;

     case WM_VSCROLL:
       // React to the various vertical scroll related actions.
       switch (GET_WM_VSCROLL_CODE(wParam, lParam))
       {
         case SB_TOP:
          nVscrollInc = -nVscrollPos;
          break;

         case SB_BOTTOM:
          nVscrollInc = nVscrollMax - nVscrollPos;
          break;

         case SB_LINEUP:
          nVscrollInc = -1;
          break;

         case SB_LINEDOWN:
          nVscrollInc = 1;
          break;

         case SB_PAGEUP:
          nVscrollInc = -max(1, yClient / yChar);
          break;

         case SB_PAGEDOWN:
          nVscrollInc = max(1, yClient / yChar);
          break;
         /*
         case SB_THUMBPOSITION:
          nVscrollInc = GET_WM_VSCROLL_POS(wParam, lParam) - nVscrollPos;        //  HIWORD
          break;
         case SB_THUMBTRACK:
          nVscrollInc = GET_WM_VSCROLL_POS(wParam, lParam) - nVscrollPos;
          break; */
         // Windows has a limit of 65535 for these,so big values don't work right
         // would have to replicate/redo the whole bar logic, think of something later?
         case SB_THUMBPOSITION:
         case SB_THUMBTRACK:
           nVscrollInc = GET_WM_VSCROLL_POS(wParam, lParam) - nVscrollPos;
           //PostMessage(hDlg, IDOK, (WPARAM)NULL, (LPARAM)0);  // trigger next didn't work
           break;
         default:
          nVscrollInc = 0;
       }

       nVscrollInc = max(-nVscrollPos,
                     min( nVscrollInc, nVscrollMax - nVscrollPos ) );
/*
typedef struct tagSCROLLINFO {  // for new stuff

    UINT cbSize;
    UINT fMask;
    int  nMin;
    int  nMax;
    UINT nPage;
    int  nPos;
    int  nTrackPos;
}   SCROLLINFO;
typedef SCROLLINFO FAR *LPSCROLLINFO;
*/
        if (nVscrollInc)
        { //SCROLLINFO ScrollControl ={0};
          nVscrollPos += nVscrollInc;
          SetScrollPos   (hDlg, SB_VERT,  nVscrollPos, TRUE);
          //ScrollControl.cbSize = sizeof(ScrollControl);
          //ScrollControl.fMask = SIF_POS;
          //ScrollControl.nPos  = nVscrollPos;
          //SetScrollInfo( hDlg,SB_VERT,&ScrollControl,TRUE);
          InvalidateRect(ListArea, NULL, TRUE);
          HdumpPaint(ListArea);
          ValidateRect(hDlg,NULL);
       }
       return TRUE;
    /*   // not needed yet, leave for docs
    case WM_HSCROLL:
      // React to the various horizontal scroll related actions.
      switch (GET_WM_HSCROLL_CODE(wParam, lParam))
      {
        case SB_LINEUP:
         nHscrollInc = -1;
         break;

        case SB_LINEDOWN:
         nHscrollInc = 1;
         break;

        case SB_PAGEUP:
         nHscrollInc = -8;
         break;

        case SB_PAGEDOWN:
         nHscrollInc = 8;
         break;

        case SB_THUMBPOSITION:
         nHscrollInc = GET_WM_HSCROLL_POS(wParam, lParam) - nHscrollPos;
         break;

        case SB_THUMBTRACK:
         nHscrollInc = GET_WM_HSCROLL_POS(wParam, lParam) - nHscrollPos;
         break;

        default:
         nHscrollInc = 0;
      }

      nHscrollInc = max(-nHscrollPos,
                    min(nHscrollInc, nHscrollMax - nHscrollPos));
      if (nHscrollInc)
      {
         nHscrollPos += nHscrollInc;
         ScrollWindow(ListArea, -xChar * nHscrollInc, 0, NULL, NULL); // hDlg
         SetScrollPos(ListArea, SB_HORZ, nHscrollPos, TRUE);
         UpdateWindow(ListArea);
      }
      break;*/
    /*
    case WM_KEYDOWN:
      //Trace(NULL,10,4,"keydown");
      // Translate various keydown messages to appropriate horizontal
      // and vertical scroll actions.
      for (i = 0; i < NUMKEYS; i++)
      {
          if (wParam == key2scroll[i].wVirtkey)
          {
              SendMessage(hDlg, key2scroll[i].iMessage,
              key2scroll[i].wRequest, 0L);
              break;
          }
      }
      return TRUE; */

    case WM_PAINT:
      // Go paint the client area of the window with the appropriate
      // part of the selected file.
      SetWindowText(hDlg,DumpFileName);
      Static_SetText(GetDlgItem(hDlg, stc1), HeadingBuf ); // Update Text
      InvalidateRect(ListArea, NULL, TRUE);
      HdumpPaint(ListArea); //hDlg);
      ValidateRect(hDlg,NULL);
      return TRUE;         //break;

    //case WM_DESTROY:
	 case WM_COMMAND:

      switch LOWORD(wParam)
      {
         case IDCANCEL:
           if (hHexDumpDisplayDlg)                         // sidedialog is around
           {   SendMessage(hHexDumpDisplayDlg,WM_PAINT, (WPARAM)NULL, (LPARAM)0);
           }
           else
           {   if (DumpFile) fclose(DumpFile);
               DumpFile = NULL;
           }
           hHexDumpAllDlg = NULL;
 	 	     EndDialog(hDlg, IDOK);
	        return TRUE;
         //case IDOK:    //test for scroll didn't work
         //  nVscrollPos = GetScrollPos(hDlg,SB_VERT);  // thumbtrack for too big
         //  InvalidateRect(ListArea, NULL, TRUE);
         //  HdumpPaint(ListArea);
         //  ValidateRect(hDlg,NULL);
         //  return TRUE;
         default:
           //Trace(NULL,10,4,"key =%hu",LOWORD(wParam));
           break;
      }
      break;
    default:
      //Trace(NULL,10,4,"msg %d",msg);
      return FALSE;//break;
  }
  return FALSE;
}
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
/*
#pragma argsused
static void HexDump_OnKey(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags)
{
   int4 i;

   //NotifyMsg(NULL,"key = %d",vk);
   // Translate various keydown messages to appropriate horizontal
   // and vertical scroll actions.
   for (i = 0; i < NUMKEYS; i++)
   {
        if (vk == key2scroll[i].wVirtkey)
        {
            SendMessage(hHexDumpAllDlg, key2scroll[i].iMessage,
                        key2scroll[i].wRequest, 0L);
            return;//break;
        }
   }
   //NotifyMsg(NULL,"key miss = %d",vk);
}
*/
//------------------------------------------------------------------------------
// this processes all the VirtKeys, not WM_KEYDOWN
//------------------------------------------------------------------------------
#pragma argsused
static UINT HexDump_OnDlgCode (HWND hwnd, LPMSG lParam )
{  //static int count;
   //Trace(NULL,10,22,"msg=%d %d",count++,lParam);// lParam->message);// lParam);
   int4 i;

   if (lParam        // have a MSG
   &&  hHexDumpAllDlg)  // and dialog safety
   {
        // Translate various keydown messages to appropriate horizontal
        // and vertical scroll actions.     MSG
        //Trace(NULL,10,22,"msg=%d  %d  %d",lParam->message, lParam->wParam, lParam->lParam);// lParam);
        for (i = 0; i < NUMKEYS; i++)
        {
             if (lParam->wParam == key2scroll[i].wVirtkey)
             {
                  SendMessage(hHexDumpAllDlg, key2scroll[i].iMessage,
                              key2scroll[i].wRequest, 0L);
                  return TRUE;//break;
             }
        }
   }
   return FALSE;
}

//==============================================================================
#pragma argsused
static void HexDump_OnRBDown(HWND hwnd, BOOL fDoubleClick, int MouseX, int MouseY, UINT kFlag)
{
   if (hHexDumpAllDlg)  // and dialog safety
   {
       PostMessage(hHexDumpAllDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hHexDumpAllDlg, GWL_ID)), 0);
   }
   return;
}

//////////////////////////////////////////////////////////////////////////////
// keys
//////////////////////////////////////////////////////////////////////////////
/*
#pragma argsused
static void HexDump_OnChar(HWND hwnd, UINT ch, int cRepeat)
{
   int4 i;

   //NotifyMsg(NULL,"char = %d",ch);
   // Translate various keydown messages to appropriate horizontal
   // and vertical scroll actions.
   for (i = 0; i < NUMKEYS; i++)
   {
        if (ch == key2scroll[i].wVirtkey)
        {
            SendMessage(hHexDumpAllDlg, key2scroll[i].iMessage,
                        key2scroll[i].wRequest, 0L);
            return;//break;
        }
   }
}  */
/////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK HexDumpWndProc(WINDOWS_PARAMS)
{
  //Trace(NULL,10,10,"msg=%d w=%hd",msg, LOWORD(wParam));// lParam);
  switch(msg)
  {
	 //  HANDLE_MSG(hwnd, WM_CREATE,       Help_OnCreate);
	 //  HANDLE_MSG(hwnd, WM_DESTROY, 		Help_OnDestroy);
	 //  HANDLE_MSG(hwnd, WM_COMMAND,     	Help_OnCommand);
    //  HANDLE_MSG(hwnd, WM_CLOSE, 	      Help_OnClose);
    //  HANDLE_MSG(hwnd, WM_PAINT,          Help_OnPaint);
    //  HANDLE_MSG(hwnd, WM_LBUTTONDOWN, 	Help_OnLButtonDown);
    //  HANDLE_MSG(hwnd, WM_RBUTTONDOWN, 	Help_OnRButtonDown);
    //  HANDLE_MSG(hwnd, WM_RBUTTONUP,    HexDump_OnRButtonUp);
        HANDLE_MSG(hwnd, WM_RBUTTONDOWN,	HexDump_OnRBDown);
	 //  HANDLE_MSG(hwnd, WM_KEYDOWN, 	   HexDump_OnKey);
        HANDLE_MSG(hwnd, WM_GETDLGCODE,   HexDump_OnDlgCode);   // the secret to getting keys here
    //  HANDLE_MSG(hwnd, WM_CHAR, 	    	HexDump_OnChar);
	 //  HANDLE_MSG(hwnd, WM_KEYUP, 		   Help_OnKeyUp);
	 //  HANDLE_MSG(hwnd, WM_MOUSEMOVE,    Help_OnMouseMove);

	 default:
      //Trace(NULL,10,22,"msg=%d w=%hd",msg, LOWORD(wParam));// lParam);
		return Deep_DefProc(hwnd, msg, wParam, lParam);
  }
}



////////////////////////////////////////////////////////////////////////////////
//  Display a BSP Lump allow Export as lmp and Print with modeless View
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QLumpBspViewDlgProc(DIALOG_PARAMS)
{
   QuakeBspDirPtr TempDir;
static DLG2OPTS    d2;		        		// our structure for passing info to dialogs

   int4   i,n;

   char   wrkstart[]  = "12,345,678";   //length  = 10
   char   wrksize []  = "12,345,678";   //length  = 10

   char   szbuf[52]={0};

   static int4   listcount;                // count of number of files
   static char **listptrs ;                // ptr to array of file names
   static char  *listnames;                // array of char names with info
   char  *listwork;

   //char  outfile[260];

   static HWND hLumpModeless = NULL;
   static HWND hList;

//===============================================================

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

  switch(msg)
  {

	 case WM_INITDIALOG:
      AdjustDlgPos(hDlg, -512,-20);       // max left (1024) and down a bit, for less res it's on top

      hList = GetDlgItem(hDlg, lst1);

      //------------------------------------------------
      // store names in list for dialog selection
      //------------------------------------------------
      listptrs  = (char **)GetMemory( QuakeBspDirNum * sizeof( char*));     // n pointers allocated
      listnames = (char *) GetMemory( QuakeBspDirNum * 52);                 // 46 byte allocations

      listwork  = listnames;                                  // where to store
      listcount = 0;                                          // number of entries

      TempDir = QuakeBspInternalDir;                          // dir buffer

      for (n = 0; n < QuakeBspDirNum; n++,TempDir+=1)
      {
          memset(listwork,0,52);

          Format(',4', (char*)&TempDir->start , wrkstart, 10);  // start
          Format(',4', (char*)&TempDir->size  , wrksize , 10);  // size

			 //                    16,18,28,38,
          sprintf( listwork, "%-16s  %10s%10s",TempDir->name,wrkstart,wrksize);

          listptrs[listcount] = listwork;         // addr of saved
          listcount++;                            // +1 saved
          listwork+=52;                           // next avail loc
      }

      if (Sorted)
      {   qsort( listptrs, listcount, sizeof( char *), QsortListName16);//Offset4);
      }

      // Fill our list box with Lump Names

	   for(i = 0; i < listcount; i++)
      {   strncpy(szbuf, listptrs[i], 52);
          ListBox_AddString(hList, szbuf);
      }

      LumpIndex = 0;

		ListBox_SetCurSel(hList, LumpIndex);

      //if(hLumpModeless == NULL)
	   //{ 	hLumpModeless = CreateDialogParam(gd.hInst, (LPSTR)"QDisplayLump",
      // 				                         hDlg, QLumpBspDisplayDlgProc, (LPARAM)0);
      //}
      d2.Int4A = 0;                     // store lump parameters
      d2.Int4B = 0;                     // to pass to both dialogs
      d2.i1    = 0;
      d2.b1    = 0;                     // hex
      strcpy(d2.sz3,QuakeBspFileName);  // filename
      strcpy(d2.sz4,"none");            // lump

      if(hLumpModeless == NULL)
	   { 	hLumpModeless = CreateDialogParam(gd.hInst, (LPSTR)"HexDisplayLump1",
       				                         hDlg, HexDumpDisplayDlgProc,  (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);

      }
                                             //1234567890123456 234567890 234567890
      Static_SetText(GetDlgItem(hDlg, stc1), "  N A M E            Offset      Size");

      SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,
                  lst1), GWL_ID), LBN_SELCHANGE ), 0);

	   return TRUE; // or FALSE if we set our own focus

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
      break;
    //   6-1-97      changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

	 case WM_COMMAND:

		switch LOWORD(wParam)
      {
        case lst1:		// The Main Texture List box
        	 switch HIWORD(wParam)
			 {  case LBN_SELCHANGE:
               LumpIndex = (LRESULT)ListBox_GetCurSel(hList);
         	   ListBox_GetText(hList, LumpIndex, LumpName);   // update display

               // have to truncate ( remove spaces ) so FindQuakeWadDir(LumpName)
               // will match "name" to "name    " on stricmp !!
               // Remember this one BRAD

               for(i = 0; i < 16; i++)
               {  if( (LumpName[i] == ' ' ) )
                    break;
               }
               LumpName[i] = '\0';

               TempDir = FindQuakeBspDir(LumpName);
               if (TempDir)
               {
                   d2.Int4A = TempDir->start;  // store lump parameters
                   d2.Int4B = TempDir->size;
                   d2.i1    = LumpIndex;
                   strcpy(d2.sz4,LumpName);

                   SendMessage(hLumpModeless, WM_CHANGE, (WPARAM)NULL, (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);
               }
				   break;
          }
          break;


        //case IDOK:
        // Exit button
        case IDCANCEL:
		    DeePDestroyWindow(hLumpModeless);
 		    hLumpModeless = 0;

          FreeMemory(listnames);
          FreeMemory(listptrs);
		  	 EndDialog(hDlg, IDOK);
		    return TRUE;
        // Print
        case psh1:
          d2.Int4C = DumpFoffset;          // current offset
          HexDumpPrint(gd.hInst,hDlg,&d2);
          //if(!QPrintLump (gd.hInst, hDlg, GetDlgItem(hLumpModeless, lst2), LumpName) )
          //{   ErrorMsg(hDlg, "Unable to Print for some reason");
          //}
		    break;
        // Export
        case psh2:
          TempDir = FindQuakeBspDir(LumpName);

          if (!TempDir)
          {   ErrorMsg(hDlg, "Lump -  %s  - not found", LumpName);
          }
          else
          {   QuakeSaveLumpInFile(hDlg, QuakeBspFile, LumpName,".LMP",16,TempDir->start,TempDir->size); // 3-1-98 was WadFile
          }
		    break;
        // Hex dump all of the file
        case psh3:
	       DialogBoxParam(gd.hInst, (LPSTR)"HexDisplayLump2",
       				       hDlg, HexDumpAllDlgProc,  (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);
          break;
      }
	   break;
  }
  return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
//  Display a Wad Lump allow Export as lmp and Print with modeless View
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QLumpWadViewDlgProc(DIALOG_PARAMS)
{

   QuakeWadDirPtr TempDir;
   int4   i,n;

   int4   xsize,ysize;
   char   wrkstart[]  = "12,345,678";   //length  = 10
   char   wrksize []  = "12,345,678";   //length  = 10
   char   wrksizem[]  = "12,345,678";   //length  = 10

   char   name[16]={0};
   char   szbuf[52]={0};

   static DLG2OPTS    d2;		          // our structure for passing info to dialogs
   static int4   listcount;             // count of number of files
   static char **listptrs ;             // ptr to array of file names
   static char  *listnames;             // array of char names with info
   char  *listwork;

   //char  outfile[260];

   static HWND hLumpModeless = NULL;
   static HWND hList;

//===============================================================

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

  switch(msg)
  {

	 case WM_INITDIALOG:
      AdjustDlgPos(hDlg, -512,-20);       // max left (1024) and down a bit, for less res it's on top

      hList = GetDlgItem(hDlg, lst1);

      //------------------------------------------------
      // store names in list for dialog selection
      //------------------------------------------------

      listptrs  = (char **)GetMemory( QuakeWadDirNum * sizeof( char*));     // n pointers allocated
      listnames = (char *) GetMemory( QuakeWadDirNum * 52);                 // 46 byte allocations

      listwork  = listnames;                                  // where to store
      listcount = 0;                                          // number of entries

      TempDir = QuakeWadInternalDir;                          // dir buffer

      for (n = 0; n < QuakeWadDirNum; n++,TempDir+=1)
      {
          memset(listwork,0,52);

          strncpy(name,TempDir->name,16);                     // name
          Format(',4', (char*)&TempDir->start, wrkstart, 10); // start
          Format(',4', (char*)&TempDir->size , wrksize , 10); // size
          Format(',4', (char*)&TempDir->msize, wrksizem, 10); // memory size

          xsize = 0;                     // for miss (eg palette)
          ysize = 0;
          FindQuakeMipPicSize(QuakeWadFile,name,TempDir->start,&xsize,&ysize);
          //FindQuakeMipPicSize(QuakeWadFile,name,&xsize,&ysize,NULL);

          Format(',4', (char*)&TempDir->size , wrksize , 10); // size
			 //   16,17,20,21,24,25,35,45
          sprintf( listwork, "%-16s %3dx%-3d %10s%10s",name,xsize,ysize, wrksize,wrksizem);

          listptrs[listcount] = listwork;         // addr of saved
          listcount++;                            // +1 saved
          listwork+=52;                           // next avail loc
      }

      if (Sorted)
      {   qsort( listptrs, listcount, sizeof( char *), QsortListName16);//Offset4);
      }

      // Fill our list box with Lump Names
	   for(i = 0; i < listcount; i++)
      {   strncpy(szbuf, listptrs[i], 52);
          ListBox_AddString(hList, szbuf);
      }

      LumpIndex = 0;

		ListBox_SetCurSel(hList, LumpIndex);

      //if(hLumpModeless == NULL)
	   //{ 	hLumpModeless = CreateDialogParam(gd.hInst, (LPSTR)"QDisplayLump",
      // 				                         hDlg, QLumpWadDisplayDlgProc, (LPARAM)0);
      //}
      d2.Int4A = 0;                     // store lump parameters
      d2.Int4B = 0;                     // to pass to both dialogs
      d2.i1    = 0;
      d2.b1    = 0;                     // hex
      strcpy(d2.sz3,QuakeWadFileName);  // filename
      strcpy(d2.sz4,"none");            // lump

      if(hLumpModeless == NULL)
	   { 	hLumpModeless = CreateDialogParam(gd.hInst, (LPSTR)"HexDisplayLump1",
       				                         hDlg, HexDumpDisplayDlgProc,  (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);

      }

      Static_SetText(GetDlgItem(hDlg, stc1), "  N A M E        X  *  Y      Size      Msize");

      SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,
                  lst1), GWL_ID), LBN_SELCHANGE ), 0);
	   return TRUE; // or FALSE if we set our own focus

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
      break;
    //   6-1-97      changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

	 case WM_COMMAND:

		switch LOWORD(wParam)
      {
        case lst1:		// The Main Texture List box
        	 switch HIWORD(wParam)
			 {  case LBN_SELCHANGE:
               LumpIndex = (LRESULT)ListBox_GetCurSel(hList);
         	   ListBox_GetText(hList, LumpIndex, LumpName);   // update display

               // have to truncate ( remove spaces ) so FindQuakeWadDir(LumpName)
               // will match "name" to "name    " on stricmp !!
               // Remember this one BRAD

               for(i = 0; i < 16; i++)
               {  if( (LumpName[i] == ' ' ) )
                    break;
               }
               LumpName[i] = '\0';

               TempDir = FindQuakeWadDir(LumpName); // Can't find, don't do anything

               if (TempDir)
               {
                   d2.Int4A = TempDir->start;  // store lump parameters
                   d2.Int4B = TempDir->size;
                   d2.i1    = LumpIndex;
                   strcpy(d2.sz4,LumpName);
                   SendMessage(hLumpModeless, WM_CHANGE, (WPARAM)NULL, (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);
               }
				   break;
          }
          break;


        //case IDOK:
        // Exit button
        case IDCANCEL:
		    DeePDestroyWindow(hLumpModeless);
 		    hLumpModeless = 0;

          FreeMemory(listnames);
          FreeMemory(listptrs);
		  	 EndDialog(hDlg, IDOK);
		    return TRUE;
        // Print
        case psh1:
          d2.Int4C = DumpFoffset;          // current offset
          HexDumpPrint(gd.hInst,hDlg,&d2);
          //if(!QPrintLump (gd.hInst, hDlg, GetDlgItem(hLumpModeless, lst2), LumpName) )
          //{   ErrorMsg(hDlg, "Unable to Print for some reason");
          //}
		    break;
        // Export
        case psh2:
          TempDir = FindQuakeWadDir(LumpName);

          if (!TempDir)
          {   ErrorMsg(hDlg, "Lump -  %s  - not found", LumpName);
          }
          else
          {   QuakeSaveLumpInFile(hDlg, QuakeWadFile, LumpName,".LMP",16,TempDir->start,TempDir->size);
          }
		    break;
        // Hex dump all of the file
        case psh3:
	       DialogBoxParam(gd.hInst, (LPSTR)"HexDisplayLump2",
       				       hDlg, HexDumpAllDlgProc,  (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);
          break;
      }
	   break;
  }
  return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
//  Quake Display Wad Lump Modeless Dialog
////////////////////////////////////////////////////////////////////////////////
/*
#pragma argsused
static BOOL CALLBACK QLumpWadDisplayDlgProc(DIALOG_PARAMS)
{

   static QuakeWadDirPtr TempDir;
   static char  dirname[18]={0};

   //int4  DirOffset= 0;
   int4  byteoffset=0;  // byte offset

   static FILE *DumpFile;
   int2  i,j;

   //buf is 80 but read text for 260
   char  buf[260];   // 1234 | 12 34 12 34 12 34 12 34 .. .. | 12345678 12345678 |
   char  wrkbuf [16];
   byte  databuf[16]; //data

//===================================================================
  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

   switch (msg)
   {
    case WM_INITDIALOG:
      AdjustDlgPos(hDlg, 236,-20);       // max left (1024) and down a bit, for less res it's on top

      TempDir = FindQuakeWadDir(LumpName);
      if (!TempDir) return -1;

      SendMessage(hDlg, WM_CHANGE, (WPARAM)NULL, (LPARAM)0);
      return TRUE;

   case WM_DESTROY:
      return TRUE;

   case WM_CHANGE:
      TempDir = FindQuakeWadDir(LumpName); // Can't find, don't do anything
      if (!TempDir) break;

      strncpy( dirname, TempDir->name, 16);
      wsprintf( buf, "Contents of entry# %hd %s (%d bytes)",LumpIndex, dirname, TempDir->size);

      Static_SetText(GetDlgItem(hDlg, stc1), buf ); // Update Text

      ListBox_ResetContent(GetDlgItem(hDlg, lst2)); // Get rid of Old contents

      if(TempDir->size > 64000) // 1-25-97 bk
      { if (! Confirm(hDlg,"Files over 64k may take a while to load\n\n"
              "And/Or cause your machine to become unstable\n"
              "If you do not have enough RAM. Are Your Sure ?"))
           return TRUE;
      }

      DumpFile = FileOpen(QuakeWadFileName, "rb",OPENMISS);    // new open binary

      // starting here

      if (DumpFile)
      {   WadSeek ( DumpFile, TempDir->start);
      }
      else
      {   break;   // open failed
      }

      while(byteoffset < TempDir->size)
      {
         WadSeek ( DumpFile, TempDir->start + byteoffset);

	      if (byteoffset < TempDir->size)
	      {   wsprintf( buf, "%04X |",byteoffset);               // offset 
	      }
	      else
	      {   strcpy ( buf, "      ");
	      }

	      for (i = 0; i < 16 && byteoffset < TempDir->size; i++) // 16 bytes of hex
	      {
	         WadRead( DumpFile, &(databuf[i]), 1);
	         wsprintf( wrkbuf, " %02X", databuf[i]);
	         strcat(buf,wrkbuf);
	         byteoffset++;                          //+1 byte listed (offset)
	      }

	      for (j=i; j< 16; j++)                      // filler at the end
	      {   strcat(buf,"   ");
	      }

	      strcat (buf, " |");   //" "
	      for (j = 0; j < i; j++)                    // sixteen bytes char
	      {
	        if (databuf[j] >= 32 && databuf[j] < 127 )
	        {  wsprintf( wrkbuf, "%c",databuf[ j]); // printable
		        strcat(buf,wrkbuf);
	        }
	        else
	        {  strcat( buf, ".");                  // nonprintable
	        }
	      }

	      for (j=i; j< 16; j++)                      // filler at the end
	      {   strcat(buf," ");
	      }

	      strcat( buf, "|");   // ""

         ListBox_AddString(GetDlgItem(hDlg, lst2), buf);
      }
      if (DumpFile)  fclose(DumpFile);
    	break;

   default:
     break;
  }
  return FALSE;
}
*/

////////////////////////////////////////////////////////////////////////////////
//  The Deep95 Display a Pak Lump allow Export as lmp and Print with modeless View
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK QLumpPakViewDlgProc(DIALOG_PARAMS)
{

   QuakePakDirPtr TempDir;
   int4   i,n;

   char   wrkstart[]  = "12,345,678";   //length  = 10
   char   wrksize []  = "12,345,678";   //length  = 10

   char   name[82]={0};
   char  *listwork;

   static DLG2OPTS    d2;		          // our structure for passing info to dialogs
   static int4   listcount;             // count of number of files
   static char **listptrs ;             // ptr to array of file names
   static char  *listnames;             // array of char names with info

   //char  outfile[260];

   static HWND hLumpModeless = NULL;
   static HWND hList;

//===============================================================

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

  switch(msg)
  {

	 case WM_INITDIALOG:
      AdjustDlgPos(hDlg, -512,-20);       // max left (1024) and down a bit, for less res it's on top

      hList = GetDlgItem(hDlg, lst1);

      //------------------------------------------------
      // store names in list for dialog selection
      //------------------------------------------------

      listptrs  = (char **)GetMemory( QuakePakDirNum * sizeof( char*));     // n pointers allocated
      listnames = (char *) GetMemory( QuakePakDirNum * 80);                 // 46 byte allocations

      listwork  = listnames;                                  // where to store
      listcount = 0;                                          // number of entries

      TempDir = QuakePakInternalDir;                          // dir buffer

      for (n = 0; n < QuakePakDirNum; n++,TempDir+=1)
      {
         memset(listwork,0,80);

         strncpy(name,TempDir->name,56);                     // name
         Format(',4', (char*)&TempDir->start, wrkstart, 10); // start
         Format(',4', (char*)&TempDir->size , wrksize , 10); // size
			//  70
         //sprintf( listwork, "%-50s%10s%10s",name, wrkstart,wrksize);
         sprintf( listwork, "%-40s%10s%10s",name, wrkstart,wrksize);

         listptrs[listcount] = listwork;         // addr of saved
         listcount++;                            // +1 saved
         listwork+=80;                           // next avail loc
      }

      if (Sorted)
      {   qsort( listptrs, listcount, sizeof( char *), QsortListName50);
      }

      // Fill our list box with Lump Names
	   for(i = 0; i < listcount; i++)
      { strncpy(name, listptrs[i], 60);
        ListBox_AddString(hList, name);
      }

      LumpIndex = 0;

		ListBox_SetCurSel(hList, LumpIndex);

      //if(hLumpModeless == NULL)
	   //  	hLumpModeless = CreateDialogParam(gd.hInst, (LPSTR)"QDisplayLump",
      // 				                         hDlg, QLumpPakDisplayDlgProc, (LPARAM)0);
      d2.Int4A = 0;                     // store lump parameters
      d2.Int4B = 0;                     // to pass to both dialogs
      d2.i1    = 0;
      d2.b1    = 0;                     // hex
      strcpy(d2.sz3,QuakePakFileName);  // filename
      strcpy(d2.sz4,"none");            // lump

      if(hLumpModeless == NULL)
	   { 	hLumpModeless = CreateDialogParam(gd.hInst, (LPSTR)"HexDisplayLump1",
       				                         hDlg, HexDumpDisplayDlgProc,  (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);

      }

      Static_SetText(GetDlgItem(hDlg, stc1), "     N A M E                                   Start    Size");

      SendMessage(hDlg, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(GetDlgItem(hDlg,lst1),
                        GWL_ID), LBN_SELCHANGE ), 0);
	   return TRUE; // or FALSE if we set our own focus

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);//   6-1-97
      break;
    //   6-1-97      changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

	 case WM_COMMAND:
		switch LOWORD(wParam)
      {
        case lst1:		// The Main Texture List box
        	 switch HIWORD(wParam)
		    {  case LBN_SELCHANGE:
               LumpIndex = (LRESULT)ListBox_GetCurSel(hList);
         	   ListBox_GetText(hList, LumpIndex, LumpName);   // update display

               // have to truncate ( remove spaces ) so FindQuakeWadDir(LumpName)
               // will match "name" to "name    " on stricmp !!
               // Remember this one BRAD

               for(i = 0; i < 40; i++)
               {  if( (LumpName[i] == ' ' ) )
                    break;
               }
               LumpName[i] = '\0';
               TempDir = FindQuakePakDir(LumpName); // Can't find, don't do anything

               if (TempDir)
               {
                   d2.Int4A = TempDir->start;  // store lump parameters
                   d2.Int4B = TempDir->size;
                   d2.i1    = LumpIndex;
                   strcpy(d2.sz4,LumpName);

                   SendMessage(hLumpModeless, WM_CHANGE, (WPARAM)NULL, (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);
               }
				   break;
          }
          break;

        //case IDOK:
        // Exit
        case IDCANCEL:
   	    DeePDestroyWindow(hLumpModeless);
   		 hLumpModeless = 0;

          FreeMemory(listnames);
          FreeMemory(listptrs);
		  	 EndDialog(hDlg, IDOK);
		    return TRUE;
        //PRINT
        case psh1:
          d2.Int4C = DumpFoffset;          // current offset
          HexDumpPrint(gd.hInst,hDlg,&d2);
          //if(!QPrintLump (gd.hInst, hDlg, GetDlgItem(hLumpModeless, lst2),  LumpName) )
          //{   ErrorMsg(hDlg, "Unable to Print for some reason");
          //}
		    break;
        //EXPORT
        case psh2:
          TempDir = FindQuakePakDir(LumpName);

          if (!TempDir)
          {   NotifyMsg(hDlg, "Lump -  %s  - not found", LumpName);
          }
          else
          {   QuakeSaveLumpInFile(hDlg, QuakePakFile, LumpName,NULL,56,TempDir->start,TempDir->size);
          }
		    break;
        // Hex dump all of the file
        case psh3:
	       DialogBoxParam(gd.hInst, (LPSTR)"HexDisplayLump2",
       				       hDlg, HexDumpAllDlgProc,  (LPARAM)(LPDLG2OPTS)&d2);//(LPARAM)0);
          break;
      }
	   break;
  }
  return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
//  The Deep Quake Display Lump Modeless Dialog
//////////////////////////////////////////////////////////////////////////////
/*
#pragma argsused
static BOOL CALLBACK QLumpPakDisplayDlgProc(DIALOG_PARAMS)
{

   static QuakePakDirPtr TempDir;
   static char  dirname[38]={0};

   //int4  DirOffset= 0;
   int4  byteoffset=0;  // byte offset

   static FILE *DumpFile;
   int2  i,j;

   //buf is 80 but read text for 260
   char  buf[260];   // 1234 | 12 34 12 34 12 34 12 34 .. .. | 12345678 12345678 |
   char  wrkbuf [16];
   byte  databuf[16]; //data

   char *dotp;
//===================================================================
  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

   switch (msg)
   {
    case WM_INITDIALOG:
      AdjustDlgPos(hDlg, 236,-20);       // max left (1024) and down a bit, for less res it's on top

      TempDir = FindQuakePakDir(LumpName);
      if (!TempDir) return -1;

      SendMessage(hDlg, WM_CHANGE, (WPARAM)NULL, (LPARAM)0);
      return TRUE;

   case WM_DESTROY:
      return TRUE;

   case WM_CHANGE:
      TempDir = FindQuakePakDir(LumpName); // Can't find, don't do anything
      if (!TempDir) break;

      strncpy( dirname, TempDir->name, 38);
      wsprintf( buf, "Contents of entry# %hd %s (%d bytes)",
			       LumpIndex, dirname, TempDir->size);

      Static_SetText(GetDlgItem(hDlg, stc1), buf ); // Update Text

      ListBox_ResetContent(GetDlgItem(hDlg, lst2)); // Get rid of Old contents

      if(TempDir->size > 64000) // 1-25-97 bk
      {  if (! Confirm(hDlg,"Files over 64k may take a while to load\n\n"
               "And/Or cause your machine to become unstable\n"
               "If you do not have enough RAM. Are Your Sure ?"))
         return TRUE;
      }

      // 1-3-97 bk  Stop from loading a 1.3 mb. bsp into the list box !
      // will handle proper file seeking right away. What about making
      // it for registered ?  Can't use WM_VSCROLL with list box's, would
      // have to have a window class to handle it. As long as the user has
      // enough ram and patience might as well just load the whole thing
      // in. Faster once its loaded that's for sure.


      DumpFile = FileOpen(QuakePakFileName, "rb",OPENMISS);    // new open binary

      if (!DumpFile) break;     // failed

      // if wave file play it now

      dotp = strrchr( LumpName,'.');                          // find . (reverse)
      if (dotp )
      {
          if(!strnicmp(dotp,".wav",4)  )
          {
              playWAVFile(hDlg,QuakePakFileName, TempDir->start);
          }
          //else  can't yet but figure out later
          //if(!strnicmp(dotp,".mid",4)  )
          //{
          //    playMIDFile(hDlg,QuakePakFileName);
          //}
      }

      // starting here

      WadSeek ( DumpFile, TempDir->start);

      while(byteoffset < TempDir->size)
      {
         WadSeek ( DumpFile, TempDir->start + byteoffset);

	      if (byteoffset < TempDir->size)
	      {   wsprintf( buf, "%04X |",byteoffset);               // offset 
	      }
	      else
	      {   strcpy ( buf, "      ");
	      }

	      for (i = 0; i < 16 && byteoffset < TempDir->size; i++) // 16 bytes of hex
	      {
	          WadRead( DumpFile, &(databuf[i]), 1);
	          wsprintf( wrkbuf, " %02X", databuf[i]);
	          strcat(buf,wrkbuf);
	          byteoffset++;                          //+1 byte listed (offset)
	      }

	      for (j=i; j< 16; j++)                      // filler at the end
	      {   strcat(buf,"   ");
	      }

	      strcat (buf, " |");
	      for (j = 0; j < i; j++)                    // sixteen bytes char
	      {
	        if (databuf[j] >= 32 && databuf[j] < 127 )
	        {  wsprintf( wrkbuf, "%c",databuf[ j]);  // printable
		        strcat(buf,wrkbuf);
	        }
	        else
	        {  strcat( buf, ".");                    // nonprintable
	        }
	      }

	      for (j=i; j< 16; j++)                      // filler at the end
	      {    strcat(buf," ");
	      }

	      strcat( buf, "|");   // ""

         ListBox_AddString(GetDlgItem(hDlg, lst2), buf);
      }

      if (DumpFile)  fclose(DumpFile);
     	break;

   default:
      break;
  }
  return FALSE;
}
*/

//---------------------------------------------------------------------------------
//  save a Quake lump in a file  -- works for any file now so share later!!!
//---------------------------------------------------------------------------------

static void QuakeSaveLumpInFile(HWND hwnd,
                                FILE *infile,
                                char *name,
                                char *suffix,
                                int4 namelen, int4 start, int4 size)
{
  FILE *outfile;
//char type[]=".LMP";

  char newname[60] = {0};
  char CurName[60] = {0};

  char drive     [ 4];//= {0};  // 2 'C:'  (+leading 03 for drive picture control)
  char dir      [260];//= {0};  // 31 (28)Includes leading, trailing backslashes (\)
  char filename [260];//= {0};  // 39 12345678 (260 win32)
  char ext      [260];//= {0};  // 43 .xxx     (260 win32)

  char TempName[260] = "*";
  //-----------------------------------------------------------------------------

  strncpy(CurName,name,namelen);     //16  or 56 for now
  strncpy(newname,name,namelen);     //16 if new name required

  // break "filename" into parts------------------

  fnsplit(CurName,drive,dir,filename,ext);

  if (suffix)
  {   strcpy(ext,suffix);
  }

  _makepath(TempName,NULL,NULL,filename,ext);      // merge

  GetSaveName(hwnd,"Quake Lump", TempName,256, IDS_FILTERLMP);//  3-1-97  checks for duplicates too

  if (TempName[0] != '\0')
  {
      // see if name is in use anywhere

      if (!FileIsInUse(TempName,TRUE,FALSE) )   // msg + no set TempName to '\0'
      {
	       if ((outfile = fopen( TempName, "wb")) == NULL)
	       {
		        ErrorMsg(hwnd, "%s - Creation Error?", TempName);
	       }
	       else
	       {
              WadSeek( infile,start);
              CopyBytes(outfile, infile,size);  // data
		        fclose( outfile);
	       }
      }
  }
}

//////////////////////////   end  /////////////////////////////////////
