///////////////////////////////////////////////////////////////////////////////
//  Program Name: Tips.cpp
//  Manages normal tips and tutorial tips
///////////////////////////////////////////////////////////////////////////////

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include "deep.h"
#include "windeep.h"

#include <dlgs.h>							// header for control defines
#include <mmsystem.h>               // for PlaySound

#pragma hdrstop
#pragma warning (disable: 4068)
/*
                 Modeless
TipsDlgProc                   Startup Tip Window
TipsTutorDlgProc              Startup Tutorial Tip
*/

//---------------------------------------------------------------------------


/////////////////////////////////////////////////////////////////////////////
//  The Deep Startup Tips Dialog Procedure
/////////////////////////////////////////////////////////////////////////////

#pragma argsused
BOOL CALLBACK TipsDlgProc(DIALOG_PARAMS)
{
   static oldTip=0;

   ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

   switch (msg)
   {
    case WM_INITDIALOG:
      if (ShowTips)
      {   Button_SetCheck(GetDlgItem(hDlg, chx1), 1 );
      }
      InvalidateRect(GetDlgItem(hDlg, rct1), NULL, TRUE ); // show our tip
      return TRUE;

    case WM_DESTROY:
      return TRUE;

    case WM_COMMAND:
	   if (LOWORD(wParam) == psh1)     // prev tip
      {
        TipNum --;
        if(TipNum < 1) TipNum = MAXTIPS;  // <<< modify above as increased
        oldTip = TipNum;

        InvalidateRect(GetDlgItem(hDlg, rct1), NULL, TRUE );
		  break;
      }

	   if (LOWORD(wParam) == psh2)     // next tip
      {
        if(TipNum >= MAXTIPS) TipNum = 0;  // <<< modify above as increased
        TipNum ++;
        oldTip = TipNum;

        InvalidateRect(GetDlgItem(hDlg, rct1), NULL, TRUE );
		  break;
      }

      if( LOWORD(wParam) == psh3)     // close button
	   {
        PostMessage(GetParent(hDlg), WM_COMMAND, WM_CLOSETIPS, 0 );
        //   12-2-96 for next time
        if (!oldTip)                      // user did not forward yet
        {   if(TipNum >= MAXTIPS) TipNum = 0;  //<< modify as changed   3-1-97 was hardcoded
            TipNum ++;
        }
		  break;
	   }

      if( LOWORD(wParam) == chx1)  // Our Check button for ShowTips
	   {
		  if(Button_GetCheck(GetDlgItem(hDlg, chx1)))
          ShowTips = TRUE;
        else
          ShowTips = FALSE;
		   break;
	   }
	   break;

    default:
       break;
   }
   return FALSE;
}


//////////////////////////////////////////////////////////////////////////////
//    bdk 12-18-96                                                           /
//   Modified to ensure the DOOM palette is selected into the Tip Window     /
//   if it is available, if not, the default palette is used                 /
//     - 3-1-97  slightly larger font --                                   /
//////////////////////////////////////////////////////////////////////////////

#pragma argsused
void Tips_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   RECT     rc;
   HFONT    hOldFont;
   HBRUSH   hBrush;
   HPALETTE hOldPal;
   char     tipNumbers[10]={0};

//---------------------------------------------------------------------------

   BeginPaint(hwnd, &ps);

   if(gd.hDoomPal)
    {  hOldPal = SelectPalette(ps.hdc, gd.hDoomPal, FALSE);
       RealizePalette(ps.hdc);
       hBrush  = CreateSolidBrush(PALETTEINDEX(GameColors[YELLOW]));  //LIGHTYELLOW looks bad in others
    }
   else
    {  hOldPal = SelectPalette(ps.hdc, (HPALETTE)GetStockObject(DEFAULT_PALETTE), FALSE);
       RealizePalette(ps.hdc);
       hBrush = CreateSolidBrush(PALETTERGB(192, 192, 0));    //192,192,0 orange
    }

   GetClientRect(hwnd, &rc);

   FillRect(ps.hdc, &rc, hBrush);

   SetBkMode(ps.hdc, TRANSPARENT);
   SetTextColor(ps.hdc, RGB(0,0,0));

   hOldFont = SelectFont(ps.hdc, gd.hTipsFont);
   rc.left = 10;
   rc.top  = 20;
   rc.right = rc.right - 10;
   // safety   3-1-97
   if(TipNum >  MAXTIPS)  TipNum = 1;
   if(TipNum <= 0      )  TipNum = 1;

   DrawText(ps.hdc,  (LPCTSTR) szLoadString(TipNum + 3000),	// address of string to draw
                      -1, &rc, DT_CENTER | DT_WORDBREAK);
   // 3-1-97
   wsprintf(tipNumbers, "%2d / %2d", TipNum, MAXTIPS);      // Show   Current / Total
   TextOut(ps.hdc, 0,0, tipNumbers, 7);

   SelectFont(ps.hdc, hOldFont);
   DeleteBrush(hBrush);
   SelectPalette(ps.hdc, hOldPal, FALSE);
   EndPaint(hwnd, &ps);
}

//////////////////////////////////////////////////////////////////////////////
//  The Deep Tips Custom Window Proc
//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK TipsWndProc(WINDOWS_PARAMS)
{
  switch(msg)
  {
	 HANDLE_MSG(hwnd, WM_PAINT, Tips_OnPaint);
	 default:
		return Tips_DefProc(hwnd, msg, wParam, lParam);
  }
}


//  2-20-97 bk Added Tutorial Tips, First & Last, Tutorial etc.
//  modifications to and Deep.rc files Dialogs.rc

// 2-20-97 bk Added Left Gun and Right Gun for Arrows, Drawing Tool Window BMP's

HBITMAP     hLGunBmp    = NULL;
HBITMAP     hRGunBmp    = NULL;
HBITMAP     hStopBmp    = NULL;
HBITMAP     hLGDownBmp  = NULL;
HBITMAP     hRGDownBmp  = NULL;
HBITMAP     hStopDownBmp= NULL;

// Our loaded Resource in Deep.cpp

extern LPCSTR ButtonRes;         // In Drawing.cpp for Button clicking

extern HBITMAP hVertexBmp;			// Loaded in Deep.cpp

POINT p;                         // X,Y of the First Button for hit testing

Bool     TutNum = 0;             // Current Page of Tutorial  Start - NOT YET


/////////////////////////////////////////////////////////////////////////////
//  The Deep Startup Tips Dialog Procedure                                  /
/////////////////////////////////////////////////////////////////////////////
static   Bool TutSave1    = FALSE;
static   Bool TutSave2    = FALSE;
static   Bool TutNewLevel = FALSE;
static   Bool TutEditLine = FALSE;
#pragma argsused
BOOL CALLBACK TipsTutorDlgProc(DIALOG_PARAMS)
{
//   static oldTip=0;
   static RECT rcTipPos;
   RECT rc;

//==============================================================================

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

   switch (msg)
   {
    case WM_INITDIALOG:
      //InvalidateRect(GetDlgItem(hDlg, rct1), NULL, TRUE ); // show our tip
      TutNum      = 0;
      TutSave1    = FALSE;
      TutSave2    = FALSE;
      TutNewLevel = FALSE;
      TutEditLine = FALSE;
      return TRUE;


    case WM_DESTROY:          // In case X out while still in Tut.
     if(hLGunBmp)     DeleteBitmap(hLGunBmp);
     if(hRGunBmp)     DeleteBitmap(hRGunBmp);
     if(hLGDownBmp)   DeleteBitmap(hLGDownBmp);
     if(hRGDownBmp)   DeleteBitmap(hRGDownBmp);
     if(hStopBmp)     DeleteBitmap(hStopBmp);
     if(hStopDownBmp) DeleteBitmap(hStopDownBmp);
     hLGunBmp    = NULL;
     hRGunBmp    = NULL;
     hStopBmp    = NULL;
     hLGDownBmp  = NULL;
     hRGDownBmp  = NULL;
     hStopDownBmp= NULL;
     return TRUE;

    case WM_COMMAND:
     switch(LOWORD(wParam) )
      {
	    case psh1:     // Start Tutorial  Runs from MAX + 1 to MAXTUT( MAXTIPS + 10 )

        TutNum = 0;
        if(hLGunBmp)     DeleteBitmap(hLGunBmp);
        if(hRGunBmp)     DeleteBitmap(hRGunBmp);
        if(hLGDownBmp)   DeleteBitmap(hLGDownBmp);
        if(hRGDownBmp)   DeleteBitmap(hRGDownBmp);
        if(hStopBmp)     DeleteBitmap(hStopBmp);
        if(hStopDownBmp) DeleteBitmap(hStopDownBmp);

        hLGunBmp    = LoadBitmap(gd.hInst, "LGUN");
        hRGunBmp    = LoadBitmap(gd.hInst, "RGUN");
        hLGDownBmp  = LoadBitmap(gd.hInst, "LGUNDOWN");
        hRGDownBmp  = LoadBitmap(gd.hInst, "RGUNDOWN");
        hStopBmp    = LoadBitmap(gd.hInst, "STOP");
        hStopDownBmp= LoadBitmap(gd.hInst, "STOPDOWN");


        GetWindowRect(hDlg, &rc);        // Move to top Left or Top Right ?
        CopyRect(&rcTipPos, &rc);        // Save our Position

        MoveWindow(hDlg, gd.rcClient.right - RECTWIDTH(&rc), gd.rcClient.bottom - RECTHEIGHT(&rc), RECTWIDTH(&rc), RECTHEIGHT(&rc), TRUE);
        PostMessage(GetDlgItem(hDlg, rct1), WM_COMMAND, WM_CHANGE, 1);
        InvalidateRect(GetDlgItem(hDlg, rct1), NULL, TRUE );
		  break;

       case psh2:    // close
        PostMessage(GetParent(hDlg), WM_COMMAND, WM_CLOSETIPS, 0 );
        break;
	   }

    break;

    default:
       break;
   }
   return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
// Draw the 3 Back, Stop and Forward Buttons
///////////////////////////////////////////////////////////////////////////////

// Button 1
void DrawButton1( HWND hwnd, HDC hdc, Bool state)    // BACK BUTTON
{
  HDC myhdc;

  if(hdc == NULL)
    myhdc = GetDC(hwnd);
  else
    myhdc = hdc;

  if(state)
   DrawBitmap(myhdc, hLGunBmp, p.x + 0,  1, SRCCOPY);
  else
   DrawBitmap(myhdc, hLGDownBmp, p.x + 0,  1, SRCCOPY);

  if(hdc == NULL)
    ReleaseDC(hwnd, hdc);

}
// Button 2
void DrawButton2( HWND hwnd, HDC hdc, Bool state)    // STOP BUTTON
{
  HDC myhdc;

  if(hdc == NULL)
    myhdc = GetDC(hwnd);
  else
    myhdc = hdc;

  if(state)
    DrawBitmap(myhdc, hStopBmp, p.x + 16,  1, SRCCOPY);
  else
    DrawBitmap(myhdc, hStopDownBmp, p.x + 16,  1, SRCCOPY);

  if(hdc == NULL)
    ReleaseDC(hwnd, hdc);

}

// Button 3
void DrawButton3( HWND hwnd, HDC hdc, Bool state)    // FORWARD BUTTON
{
  HDC myhdc;

  if(hdc == NULL)
    myhdc = GetDC(hwnd);
  else
    myhdc = hdc;

  if(state)
    DrawBitmap(myhdc, hRGunBmp, p.x + 32,  1, SRCCOPY);
  else
    DrawBitmap(myhdc, hRGDownBmp, p.x + 32,  1, SRCCOPY);

  if(hdc == NULL)
    ReleaseDC(hwnd, hdc);

}
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Various Sub's for DoTipTutorial instead of in line code                    //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

void DoTutText(HWND hwnd, HDC hdc, LPRECT rc)
{
  char     tipNumbers[10]={0};

  rc->left  = 6;
  rc->top   = 20;
  rc->right = rc->right - 6;

  wsprintf(tipNumbers, "%2d / %2d", TutNum, MAXTUT);      // Show   Current / Total
  TextOut(hdc, 0,0, tipNumbers, 7);

  DrawText(hdc,  (LPCTSTR) szLoadString(TutNum + 4000), -1, rc, DT_LEFT | DT_WORDBREAK|DT_NOPREFIX);

  DrawButton1(hwnd, hdc, UP);
  DrawButton2(hwnd, hdc, UP);
  DrawButton3(hwnd, hdc, UP);
}

// Draw a slow line for the user to watch ( Tutorial )

void DrawTutLine(HDC hdc, int xs, int ys, int xe, int ye, int2 color)
{

 int x = xs;
 int y = ys;
 static DWORD DrawSpeed=10;
//========================================================

 if(xe > xs)       // Line - Left to Right
 {
    while(x != xe)
    {
       SetPixel(hdc, x, y, PALETTEINDEX(GameColors[color]) );
       Sleep(DrawSpeed);
       x++;
     }
 }
 else if(ye > ys)       // Line - Top to Bottom
 {
    while(y != ye)
     {
       SetPixel(hdc, x, y, PALETTEINDEX(GameColors[color]) );
       Sleep(DrawSpeed);
       y ++;
     }
 }
 else if(xe < xs)       // Line - Right to Left
 {
    while(x != xe)
     {
       SetPixel(hdc, x, y, PALETTEINDEX(GameColors[color]) );
       Sleep(DrawSpeed);
       x --;
     }
 }
 else if(ye < ys)       // Line - Bottom to Top
 {
    while(y != ye)
     {
       SetPixel(hdc, x, y, PALETTEINDEX(GameColors[color]) );
       Sleep(DrawSpeed);
       y --;
     }
 }

}

// Place A Vertex on the Tip window========================================
void Draw1Vertex(HDC hdc, int x, int y)
{
  RECT rcBmp = {0,0,16,16}; // size of our Vertex
  int2 oldVSize;

 //=========================================================

 oldVSize = VertexSize;
 VertexSize = 4;

 HDC BitmapDC=CreateCompatibleDC(hdc);
 HBITMAP OldBitmap=SelectBitmap(BitmapDC, hVertexBmp);
 HPALETTE hOldPal = SelectPalette(BitmapDC, gd.hDoomPal, FALSE);

 FillRect(BitmapDC, &rcBmp, gd.hBrushes[VertexColor]);   // Color the Vertex
 BitBlt(hdc, x, y, VertexSize, VertexSize, BitmapDC, 0, 0, SRCCOPY);

 SelectBitmap(BitmapDC, OldBitmap);	// Restore Old Bitmap Data
 SelectPalette(BitmapDC, hOldPal, FALSE);
 DeleteDC(BitmapDC);						// Delete Temp DC

 VertexSize = oldVSize;              // Restore the proper size of the Vertex
}
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// The Actual Tips Tutorial Code, draws the text and graphics                 //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
void DoTipTutorial(HWND hwnd)
{
   HFONT    hOldFont;
   HBRUSH   hOldBrush;
   HPEN     hOldPen;
   HPALETTE hOldPal;
   int      hOldBkMode;

   HBRUSH   hBrush1, hBrush2;
   HDC      hdc;
   RECT     rc, rc1;
   int      xs, xe, ys, ye;
   int2     oldthing;
   LevelPtr level = &LevelWad.level;

   int2     LevelXCenter;
   int2     LevelYCenter;

//==============================================================================

   hdc = GetDC(hwnd);

   hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
   RealizePalette(hdc);

   hBrush1  = CreateSolidBrush(PALETTEINDEX(GameColors[LIGHTERGRAY]));
   hBrush2  = CreateSolidBrush(PALETTEINDEX(GameColors[BLACK]));

   GetClientRect(hwnd, &rc);
                                  //  15 = size of buttons
   p.x = ((rc.right / 2) - 24 );  // 24 = 15 + 15 + 15 + 3 spaces in betweeen / 2
   p.y = 2;


   if (TutNum > MAXTUT || TutNum < 1)
   {   TutNum = 1;
   }

//   SetFocus(gd.hMainWindow);   // give keyboard focus to main windows !

   if (TutNum == 7)
   {   if (level->NumLineDefs < 4)
       {   ErrorMsg(hwnd,"You didn't draw 4 Lines?\nPlease Press Ctrl+D and Draw.");
           TutNum--;
       }
   }
   else
   if (TutNum == 8)
   {   if (level->NumThings <= 0)
       {   ErrorMsg(hwnd,"You didn't Insert a THING?\nPlease Press Ins in Thing MODE.");
           TutNum--;
       }
   }
   if (TutNum == 18)
   {   if (level->NumThings < 4)
       {   ErrorMsg(hwnd,"You didn't Insert a 4 THINGs?\nPlease Press Ins in Thing MODE.");
           TutNum--;
       }
   }

   if (TutNum == 19)
   {   if (level->NumLineDefs < 4)
       {   ErrorMsg(hwnd,"You didn't draw 4 lines?\nPlease Press  Ctrl+D and Draw.");
           TutNum--;
       }
   }
   if (TutNum == 23)
   {   if (level->NumLineDefs < 8)
       {   ErrorMsg(hwnd,"You didn't draw 4 more lines?\nPlease Press  Ctrl+D and Draw.");
           TutNum--;
       }
   }

   if (TutNum == 27)
   {   if (level->NumLineDefs < 10)
       {   ErrorMsg(hwnd,"You didn't draw 2 more lines?\nPlease Press Ctrl+D and Draw.");
           TutNum--;
       }
   }

   // notify msgs below should never be done, since above are supposed to catch mistakes

   // always output the text first
   hOldFont  = SelectFont(hdc, gd.hTipsFont);//hMedFont);
	hOldBrush = SelectBrush(hdc, hBrush1);
   hOldBkMode= SetBkMode(hdc, TRANSPARENT);

   SetTextColor(hdc, RGB(0,0,0));  // black text
   FillRect(hdc, &rc, hBrush1);    // light gray background
   DoTutText(hwnd, hdc, &rc);

   switch(TutNum)
   {
     case 1:                            // Introduction
     case 3:
     case 4:
     case 5:
     case 7:                           // something was drawn
     case 8:
     case 9:
     case 11:
     case 12:
     case 13:
     case 14:
     case 15:
     case 16:
     case 17:
     case 18:
     case 20:
     case 21:
     case 23:
     case 24:
     case 25:
     case 27:
     case 28:
     case 29:
     case 30:
     case 31:
     case 32:
     case 34:
     case 35:
     case 36:
     case 37:
     case 38:
     case 39:
     case 40:
     case 41:
//      SetTextColor(hdc, RGB(0,0,0));  // black text
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
            //    SetFocus(gd.hMainWindow);   // give keyboard focus to main windows !
      break;
     // File / New Level selected for user
     case 2:
//      SetTextColor(hdc, RGB(0,0,0));
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
      if (TutNewLevel == FALSE)      // make sure not 2x from paint
      {   TutNewLevel = TRUE;
          Sleep(1000);
          PostMessage(gd.hMainWindow, WM_COMMAND, CM_NEWLEVEL, 0); // Start a new level
      }
      break;

     case 6:
//      SetTextColor(hdc, RGB(0,0,0));  // black text
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);

      CopyRect(&rc1, &rc);
      rc.top = 20;
      FillRect(hdc, &rc, hBrush2);       // black background

      xs = 40;     // TOP LINE
      xe = 140;
      ys = 40;
      ye = 40;

      Draw1Vertex(hdc, xs-2, ys-2 );
      DrawTutLine(hdc, xs, ys, xe, ye, YELLOW);
      SetTextColor(hdc, RGB(255,255,255));          //white text
      TextOut(hdc, xs + 30, ys-14, "512", 3);
      //Sleep(10);   //10

      xs = 140;    // Right Side
      xe = 140;
      ys = 40;
      ye = 140;

      Draw1Vertex(hdc, xs-2, ys-2 );
      DrawTutLine(hdc, xs, ys, xe, ye, YELLOW);
      TextOut(hdc, xs + 4, ys+30, "512", 3);

      //Sleep(10); // was 10

      xs = 140;    // Bottom Line
      xe = 40;
      ys = 140;
      ye = 140;

      Draw1Vertex(hdc, xs-2, ys-2 );
      DrawTutLine(hdc, xs, ys, xe, ye, YELLOW);
      Sleep(10);
      Draw1Vertex(hdc, 38, 138 );

      //Sleep(10);
      rc1.left = 4;
      rc1.top  = 110;
      rc1.right = rc1.right - 6;

      SetTextColor(hdc, RGB(255,255,255));
      DrawText(hdc,  (LPCTSTR) szLoadString(TutNum + 4000), -1, &rc1, DT_LEFT | DT_WORDBREAK);
      Sleep(100);

      xs = 40;
      xe = 40;
      ys = 140;
      ye = 40;

      //Sleep(10);
      DrawTutLine(hdc, xs, ys, xe, ye, YELLOW);
      break;
     case 10:
//      SetTextColor(hdc, RGB(0,0,0));
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
      if (level->NumLineDefs >= 4)   // these are just extra safety lines
      {
         if(TutSave1 == FALSE) // in case re-posted from WM_PAINT
         {  TutSave1 = TRUE;
            Sleep(1000);
            PostMessage(gd.hMainWindow, WM_COMMAND, CM_SAVELEVEL, 0); // Save level
         }
      }
      else
      {  ErrorMsg(hwnd,"You didn't draw 4 lines?");
      }
      break;
     case 19:
//      SetTextColor(hdc, RGB(0,0,0));
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
      if (level->NumLineDefs >= 4)
      {
          CalculateMinMax( level);            //   3-1-97 just in case

          LevelXCenter =  level->minX +(level->maxX - level->minX)/2;
          LevelYCenter =  level->minY +(level->maxY - level->minY)/2;

          oldthing    = DefaultThing;         // save default
	       DefaultThing= THING_DEATHMATCH;     // set to new
          InsertObject(level, OBJ_THINGS, -1, LevelXCenter -20, LevelYCenter-20); //  3-1-97
          InsertObject(level, OBJ_THINGS, -1, LevelXCenter -10, LevelYCenter-10);
          InsertObject(level, OBJ_THINGS, -1, LevelXCenter +10, LevelYCenter+10);
          InsertObject(level, OBJ_THINGS, -1, LevelXCenter +20, LevelYCenter+20);
          DefaultThing= oldthing;

          //        hdc , RECT    , Erase Bk, Palette in
          RedrawMap(NULL, &gd.rcClient, TRUE, FALSE);
          if(TutSave2 == FALSE) // in case re-posted from WM_PAINT
          {  TutSave2 = TRUE;
             Sleep(1000);
             PostMessage(gd.hMainWindow, WM_COMMAND, CM_SAVELEVEL, 0); // Start a new level
          }
      }
      else
      {  ErrorMsg(hwnd,"You didn't draw at least 4 lines?");
      }
      break;
     case 22:
//      SetTextColor(hdc, RGB(0,0,0));
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
      rc.top = 60;
      SetTextColor(hdc, RGB(255,255,255));
      FillRect(hdc, &rc, hBrush2);             // black background

      Draw1Vertex(hdc, 10,  70 );    // First Sector
      Draw1Vertex(hdc, 90,  70 );
      Draw1Vertex(hdc, 90,  130 );
      Draw1Vertex(hdc, 10,  130 );

      Draw1Vertex(hdc, 110,  70 );    // Second one
      Draw1Vertex(hdc, 180,  70 );
      Draw1Vertex(hdc, 180, 130 );
      Draw1Vertex(hdc, 110, 130 );

      hOldPen = SelectPen(hdc, gd.hPens[YELLOW] );

      MoveToEx(hdc,14, 72, NULL);
      LineTo(hdc, 90, 72);
      LineTo(hdc, 90, 130);
      LineTo(hdc, 14, 130);
      LineTo(hdc, 14, 72);

      MoveToEx(hdc,114, 72, NULL);
      LineTo(hdc, 180, 72);
      LineTo(hdc, 180, 130);
      LineTo(hdc, 114, 130);
      LineTo(hdc, 114, 72);

      SelectPen(hdc, hOldPen);
      break;

     case 26:
//      SetTextColor(hdc, RGB(0,0,0));
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
      rc.top = 60;

      SetTextColor(hdc, RGB(255,255,255));
      FillRect(hdc, &rc, hBrush2);

      Draw1Vertex(hdc, 10,  70 );    // First Sector
      Draw1Vertex(hdc, 90,  70 );
      Draw1Vertex(hdc, 90,  130 );
      Draw1Vertex(hdc, 10,  130 );

      Draw1Vertex(hdc, 110,  70 );    // Second one
      Draw1Vertex(hdc, 180,  70 );
      Draw1Vertex(hdc, 180, 130 );
      Draw1Vertex(hdc, 110, 130 );

      hOldPen = SelectPen(hdc, gd.hPens[YELLOW] );

      MoveToEx(hdc,14, 72, NULL);
      LineTo(hdc, 90, 72);
      LineTo(hdc, 90, 130);
      LineTo(hdc, 14, 130);
      LineTo(hdc, 14, 72);

      MoveToEx(hdc,114, 72, NULL);
      LineTo(hdc, 180, 72);
      LineTo(hdc, 180, 130);
      LineTo(hdc, 114, 130);
      LineTo(hdc, 114, 72);

      SelectPen(hdc, hOldPen);

      TextOut(hdc, 80, 80, "A", 1);
      xs = 90;    // TOP Line
      xe = 114;
      ys = 90;
      ye = 90;
      DrawTutLine(hdc, xs, ys, xe, ye, YELLOW);
      Sleep(500);
      TextOut(hdc, 120, 80, "B", 1);
      xs = 114;    // RIGHT SIDE Line
      xe = 114;
      ys = 90;
      ye = 110;
      DrawTutLine(hdc, xs, ys, xe, ye, RED);
      Sleep(500);
      TextOut(hdc, 120, 100, "C", 1);
      xs = 114;    // BOTTOM Line
      xe = 90;
      ys = 110;
      ye = 110;
      DrawTutLine(hdc, xs, ys, xe, ye, YELLOW);

      TextOut(hdc, 80, 100, "D", 1);
      TextOut(hdc, 60, 112, "RClick here", 11);
      Sleep(1000);
      xs = 90;    // LEFT SIDE Line
      xe = 90;
      ys = 110;
      ye = 90;
      DrawTutLine(hdc, xs, ys, xe, ye, RED);
      break;
     case 33:
//      SetTextColor(hdc, RGB(0,0,0));
//      FillRect(hdc, &rc, hBrush1);
//      DoTutText(hwnd, hdc, &rc);
      // only if something was done   3-1-97
      if (level->NumLineDefs >= 8)
      {
         if(TutEditLine == FALSE) // in case re-posted from WM_PAINT
         {  TutEditLine = TRUE;
            PostMessage(gd.hMainWindow, WM_COMMAND, CM_LINEDEFSSIDEDEFS, 0);
            GoToObject( OBJ_LINEDEFS, 0, TRUE);
            EditMode  = OBJ_LINEDEFS;
	         CurObject = 1;
            OldObject = 1;
            //        hdc , RECT    , Erase Bk, Palette in
            RedrawMap(NULL, &gd.rcClient, TRUE, FALSE);
            Sleep(350);
            PostMessage(gd.hMainWindow, WM_COMMAND, CM_ENTER, 0);
         }
      }
      else
      {  ErrorMsg(hwnd,"You didn't draw at least 8 lines?");
      }

     // SetFocus(gd.hMainWindow);   // give keyboard focus to main windows !
      break;
   }

   GdiFlush();                    // for safety

   SelectFont(hdc, hOldFont);
   SelectBrush(hdc, hOldBrush);
   SelectPalette(hdc, hOldPal, FALSE);
   SetBkMode(hdc, hOldBkMode);

   DeleteBrush(hBrush1);
   DeleteBrush(hBrush2);

   ReleaseDC(hwnd, hdc);

   SetFocus(gd.hMainWindow);   // give keyboard focus to main windows !

}
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Tip window Clicked on, find where and update appropriately                 //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
void TipsTutor_OnLButtonUp(HWND hwnd, int x, int y, UINT keyFlags)
{
 if( (x > p.x) && (x < (p.x + 15)) && ( y < 18 ) )
 {
//   if(!PlaySound(ButtonRes, gd.hInst, SND_MEMORY | SND_ASYNC | SND_NODEFAULT))
//	   MessageBeep(MB_ICONEXCLAMATION);		// Use this if NOT multimedia

   DrawButton1(hwnd, NULL, UP);
   PostMessage( hwnd, WM_COMMAND, WM_CHANGE, 0);
 }
 else if( (x > p.x + 16) && (x < (p.x + 32)) && ( y < 18 ) )
 {
//   if(!PlaySound(ButtonRes, gd.hInst, SND_MEMORY | SND_ASYNC | SND_NODEFAULT))
//	    MessageBeep(MB_ICONEXCLAMATION);		// Use this if NOT multimedia

   DrawButton2(hwnd, NULL, UP);
   TutNum = 0;
   PostMessage( GetParent(hwnd), WM_COMMAND, psh2, 0);   // close  was psh6
 }
 else if( (x > p.x + 32) && (x < (p.x + 32 + 15)) && ( y < 18 ) )
 {
//   if(!PlaySound(ButtonRes, gd.hInst, SND_MEMORY | SND_ASYNC | SND_NODEFAULT))
//	   MessageBeep(MB_ICONEXCLAMATION);		// Use this if NOT multimedia

   DrawButton3(hwnd, NULL, UP);
//	MessageBeep(MB_ICONEXCLAMATION);		// Use this if multimedia
   PostMessage( hwnd, WM_COMMAND, WM_CHANGE, 1);
 }
}

//
//  Draw our buttons down if the mouse passes over them
//
#pragma argsused
void TipsTutor_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
 static Bool mask = 0;

//============================================================

 if(TipNum < MAXTIPS)  return;

 if( ( x < p.x       || x > p.x + 48  || y > 18 )  ||   // OR
     ( x < p.x + 48  && x > p.x       && y > 18 )  )    // NOT in Buttons
  {
   if(mask)       // Button was down but now away from buttons
    {
      DrawButton1(hwnd, NULL, UP);
      DrawButton2(hwnd, NULL, UP);
      DrawButton3(hwnd, NULL, UP);
      mask = 0;
    }
  }

 else if( (x > p.x) && (x < (p.x + 15)) && ( y < 18 ) )
  {
   DrawButton1(hwnd, NULL, DOWN);
   mask |= 1;
   DrawButton2(hwnd, NULL, UP);
   DrawButton3(hwnd, NULL, UP);
  }
 else if( (x > p.x + 16) && (x < (p.x + 32)) && ( y < 18 ) )
  {
   DrawButton2(hwnd, NULL, DOWN);
   mask |= 2;
   DrawButton1(hwnd, NULL, UP);
   DrawButton3(hwnd, NULL, UP);
  }
 else if( (x > p.x + 32) && (x < (p.x + 32 + 15)) && ( y < 18 ) )
  {
   DrawButton1(hwnd, NULL, UP);
   DrawButton2(hwnd, NULL, UP);
   DrawButton3(hwnd, NULL, DOWN);
   mask |= 4;
  }
}

//////////////////////////////////////////////////////////////////////////////
// 2-20-97 bk, changed color - would not display on some cards properly,
// fooled around with fonts, added Tutorial Tips for new users.
//////////////////////////////////////////////////////////////////////////////
#pragma argsused
void TipsTutor_OnPaint(HWND hwnd)
{
     PAINTSTRUCT ps;
//   RECT     rc;
//   HFONT    hOldFont;
//   HBRUSH   hBrush;
//   HPALETTE hOldPal;
//   char tp[8] = {0};

//==============================================
   BeginPaint(hwnd, &ps);
   /*
   if(gd.hDoomPal)
    {  hOldPal = SelectPalette(ps.hdc, gd.hDoomPal, FALSE);
       RealizePalette(ps.hdc);
       hBrush  = CreateSolidBrush(PALETTEINDEX(GameColors[YELLOW]));  //LIGHTYELLOW looks bad in others
    }
   else
    {  hOldPal = SelectPalette(ps.hdc, (HPALETTE)GetStockObject(DEFAULT_PALETTE), FALSE);
       RealizePalette(ps.hdc);
       hBrush = CreateSolidBrush(PALETTERGB(192, 192, 0));
    }

   GetClientRect(hwnd, &rc);

   FillRect(ps.hdc, &rc, hBrush);

   SetBkMode(ps.hdc, TRANSPARENT);
   SetTextColor(ps.hdc, RGB(0,0,0));

 // Tutorial Active, notify the Tip/Tutorial Window

   hOldFont = SelectFont(ps.hdc, gd.hTipsFont);
   */
   DoTipTutorial(hwnd);
   //SelectFont(ps.hdc, hOldFont);

   //DeleteBrush(hBrush);
   //SelectPalette(ps.hdc, hOldPal, FALSE);

   EndPaint(hwnd, &ps);
}

//////////////////////////////////////////////////////////////////////////////
//  The Deep Tips Custom Window Proc
//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK TipsTutorWndProc(WINDOWS_PARAMS)
{
  switch(msg)
  {
    case WM_COMMAND:
     switch(LOWORD(wParam))
      {
        case WM_CHANGE:

         if(lParam == 1)
         {  TutNum++;             // Increase our Tutorial Page #
             if(TutNum > MAXTUT)
                PostMessage(GetParent(hwnd), WM_COMMAND, psh2, 0);  //exit all done
         }
         else
         {  TutNum --;
            if(TutNum < 0)
            TutNum = 0;
         }

         DoTipTutorial(hwnd);  // All requests for the Tutorial come here
        break;                 // Dlg Button psh6, LBUTTONUP and WM_PAINT
      }
    break;

	 HANDLE_MSG(hwnd, WM_PAINT,     TipsTutor_OnPaint);
	 HANDLE_MSG(hwnd, WM_LBUTTONUP, TipsTutor_OnLButtonUp);
	 HANDLE_MSG(hwnd, WM_MOUSEMOVE, TipsTutor_OnMouseMove);

	 default:
		return Tips_DefProc(hwnd, msg, wParam, lParam);
  }
		return Tips_DefProc(hwnd, msg, wParam, lParam);
}


///////////   END OF MODIFIED CODE     2-20-97 bk




