///////////////////////////////////////////////////////////////////////////////
//  Program Name: Drawing.CPP
//  Collection of assorted DIB (Bmp), PCX, Image routines
///////////////////////////////////////////////////////////////////////////////

// you'll notice the global ImageXsize gives a bit of trouble here, so having a structure
// for each image with raw size info would make for easier and safer code.


// Major changes now to the way the program stores and uses BMP files, since I
// had to modify to display and copy to buffer for IMPORT in Drawing. There was no
// Color Palette Translation done before since it was just part of the viewer
// and utilized it's own palette. Now Translates the DIB upon READ and uses the
// Doom Palette - It will only do this for 256 color DIB's of course .

extern char LViewMsg[128];    // Dialog Update Text from DisplayTex's
extern char MapLevel[10];     // For Preview on Hook Dlg, LumpName browsed, referenced in Menu, paste

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include "deep.h"
#include "windeep.h"
#pragma hdrstop

#include <dlgs.h>
#include <commctrl.h>
#include <mmsystem.h>
#include <math.h>

#define MAXZOOM 16        //   6-1-97


int2 DisplayFileGraphicTexture(HWND hwnd, HDC hdc,
                               int2 xstart,int2 ystart,
                               int2 xend,  int2 yend,
                               //int4  RawSize,  // raw size of data
                               byte *RawData,    // raw data of image
                               byte**ImageBufr,  // Pointer to Dib bits
                               HBITMAP hDibbm,   // Handle to Dib Bitmap
                               char *picname,
                               int2  magnify,
                               Bool  ShowInfo,
                               int2  fillcolor,  // actual 0-255 color, not indexed   5-1-97
                               Bool  ShowImage);

int2 DisplayFileGraphicFlat(HWND hwnd, HDC hdc,
                            int2 xstart,int2 ystart,
                            int2 xend,  int2 yend,
                            byte *RawData,       // addr of raw data buffer
                            byte**ImageBufr,     // Pointer to Dib bits
                            HBITMAP hDibbm,      // Handle to Dib Bitmap
                            char *picname,
                            int2  magnify,
                            Bool  ShowInfo,
                            int2  fillcolor,     //not used, just for consistency
                            Bool  ShowImage);


/*
  void     DrawBitmap
  HBITMAP  LoadDIBSection(HWND hwnd, LPSTR lpFileName,int2 imagemagnify,Bool Translate)
  WORD     SaveDIB(HWND hwnd,HDIB hDib, LPVOID lpvDIBits, LPSTR lpFileName,RGBQUAD* RGBquad)
  void     DIBError(int nErrNo)
  void     RemoveDIBSection(void)
  BOOL     PaintBitmap(HDC hDC, LPRECT lpDCRect, HBITMAP hDDB,  LPRECT lpDDBRect, HPALETTE hPal)
  HPALETTE CreateDIBPalette(HDIB hDIB)
  void     ViewPCXImage (int2 x0, int2 y0,char *userfilename,Bool Translate)
  Bool     SetPCXPalette(HWND hwnd, byte * PCXPaletteBufr);
  HBITMAP  ReadImage (HWND hwnd, char *imagetype,  Bool   Translate, ... openmust)

  void     LoadDrawOnTextures(HWND hwnd);//, HWND hList)   COMBINES WALL and FLAT
  void     InitGFXWindow(HWND hwnd)

  Bool     PasteImage(HWND hwnd,int2  magnify, int2  transparentfill, Bool fromSelect)    // fill background duke
  void     MatchPixels(byte *UserImageBufr,int2 oldcolor,int2 newcolor,int2 type,
  void     DrawCircle (HDC hdc, int4 centerX, int4 centerY, double radius)
  void     FlipImage(HWND hwnd,byte *UserImageBufr,int2 fliptype, int2 transparent, int4 Width, int4 Height)
  void     RotateXY( double *x, double *y, double angleChange)//angledelta)
  void     RotateImage(byte *UserImageBufr,int2 xoffset, int2 yoffset)
  Bool     ImageToRaw(byte *ImageBufr,DOOMImagePtr *DoomBufr)

  Bool     SaveDrawGraphic(HWND hwnd, int2   DoSaveAs,  char*  picname,  char*  outfilename, int4   diroffset,


  void     EyeDropper(void)
  void     ZoomGraphic(BOOL in)
  void     CircleDraw(HWND hwnd,Bool solid)
  void     DrawTheLine(int4 x0, int4 y0,   // from
  void     StorePixel()
  void     UndoPixel()
  void     TrackUndo(int2 xloc, int2 yloc, byte color)
  Bool     CheckForTransParent(HWND hwnd, int4 Xsize, byte*RawWrkData,int2 colortocheck,Bool TellMsg)
  byte *   ImageBufferOffset(int4 Xcoord,int4 Ycoord)
  byte *   RawOffset(int4 Xcoord,int4 Ycoord)

  UINT     GetGFXMouseCoords (HWND hwnd, MSG *msg, Bool RawKey, int4 *xnew, int4 *ynew,int2 *Button)
  void     XY_Update         (HWND hwnd, int x, int y)
  Bool     CheckIfInsideImage(int *DrawX, int *DrawY,  int Width,int Height)

  void     ReInitList(HWND hList)
  void     EditSetup(HWND hwnd)
  void     DrawFree(BOOL freelist)
  void     ToggleTool()
  void     TogglePal(HWND hwnd, int2 init)
  void     FreeConvert()
  void     DrawHelp(HWND hwnd)
  void     RefreshWindowButtons()

  void     CreateMenuWindow1(HWND hwnd)     // 1st menu
  void     CreateMenuWindow2(HWND hwnd)  // 2rd Drawing select
  void     CreateMenuWindow3(HWND hwnd)     // 3nd menu drawing
  void     CreateConvertWindows(HWND hwnd)  // 4th Conversion

  void     ClearButtons(void)
  void     GFXDrawPicture()
  void     GFXDrawBar(HWND hwnd)
  void     GFXDrawTools(HWND hwnd)
  void     GFXDrawColors(HWND hwnd)
  void     GFXDrawStatus(HWND hwnd)
  void     DrawLineList(HWND hwnd, Bool On)
*/


#define ERR_MIN                         4096
#define ERR_NOT_DIB                     4096
#define ERR_UNSUPPORTEDFORMAT           4097
#define ERR_MEMORY                      4098
#define ERR_READ                        4099
#define ERR_SAVE                        5000
#define ERR_LOCK                        5001
#define ERR_OPEN                        5002
#define ERR_CREATEPAL                   5003
#define ERR_GETDC                       5004
#define ERR_FILENOTFOUND                5005
#define ERR_INVALIDHANDLE               5006
#define ERR_DIBFUNCTION                 5007
#define ERR_CREATEDIBSECTION            5008
#define ERR_FILLCOLORTABLE              5009
#define ERR_WIDTH                       5010
#define ERR_HEIGHT                      5011
#define ERR_STRETCHDIBITS               5012
#define ERR_UNDEFINEDERROR              5013
#define ERR_MAX                         5013

// Dib Header Marker - used in writing DIBs to files
#define DIB_BMP_HEADER_MARKER   ((WORD)('M' << 8) | 'B')        // BM
#define WIDTHBYTES(bits)    ((((bits) + 31) & ~31) >> 3)

#define BARWIDTH                        170   // width of left window box
#define COLORBOXWIDTH    (BARWIDTH - 40)/3    // for color boxes with 10 on each side

//#define BUTTONSTART                   300   // buttons on left
#define BUTTONWIDTH                     120   // was 140
#define XBUTTONLOCATION  (BARWIDTH - BUTTONWIDTH) / 2  //
#define BUTTONHEIGHT                     23   // spacing
#define BUTTONHEIGHT2                    20   // actual button size

#define STATHEIGHT                       20   // bottom msg area

int SortListName8( const void *a, const void *b);

extern int4       ImageType;              // In Dialogs.cpp (hook func) and Preview_OnPaint   5-1-97
extern RGBQUAD    dRGBquad[257];          // In utils for Doom Palette  2-10-97 bk

//================================

//extern WadFiles GraphicWad;        // Global in NewGraph.cpp
//extern int4     GraphicDirsize;    //          ''
//extern int4     GraphicDirstart;;  // offset  flat start in file

// In ColorCVT\.cpp
//void ColorPaletteMatch(byte *convertpalette,byte*gamepalette, byte *ConvertedColor);
//extern byte ColorTranslation[256]; //256 converted palette <<---- shared for now

static char GFXWadName[260]={0};          // save names here

// Our globals from ReadImage

       HBITMAP    hViewBitmap   = NULL;   // Handle to the bitmap BMP and PCX
       HDIB       hViewDibInfo  = NULL;   // (HANDLE) for CreateDibPalette and SaveDib
       VOID **    lpViewBits    = NULL;   // Pointer to the actual bits
static BITMAPINFO ViewBi;                 // Our DIB Bitmap Info Struct.

static Bool       PasteActive = FALSE;    // doing actual paste
static int4       PasteWidth;             //   5-1-97 a bit of a hack for now
static int4       PasteHeight;
static Bool       Pasting       = FALSE;
static HPALETTE   hViewPalette  = NULL;   // Logical palette
static byte       PaletteBufr[768];       //   5-1-97 simpler
//RGBQUAD    PCXrgbq[257];         // PCX Viewer RGB copy for dib in dialogs,not needed right now   5-1-97

//static byte *CopyPastePalette =NULL;

//=============   these all need to be global for drawing to work    ===========
static RECT       GfxRC;              // relative screen coordinates of GFX whole drawing screen
static RECT       EditRC;             // relative screen coordinates of Edit drawing screen
static HWND       hGFXWindow = NULL;  //
static HWND       hEditWindow= NULL;

static BITMAPINFO DrawBi;             // Our DIB Bitmap Info Struct. used all over here
static HBITMAP    hDrawBitmap = NULL; // Handle to the Dib's Bitmap returned to us
static VOID **    lpDrawBits  = NULL; // Pointer to bits in DIBSECTION for SavePCX

//--------------------------------------------------------------------------

static byte      *PixelPtr;           //location of this pixel in raw buffer
static byte      *Yorigin;            // area where offsets are
static int4      *Yoffsets;
//static int4     YoffsetOrigin ;     // low offset resident
static byte       YcolumnStart;
static byte       YdataBytes;

struct UnDoList
{
   int2  xloc;                        // location of color
   int2  yloc;
   byte  color;
};

typedef struct UnDoList *UnDoListPtr;

static Bool        PasteTranslate = TRUE;  // to translate past stuff
static int4        DidTransp    = 0;       // number of pixels drawn into transparent
static int4        UnDoMax      = 20000;   // maximum
static int4        UnDoMaxShift = 4000;    // shift this many down
static int4        UnDoActive   = 0;       // number active
static UnDoListPtr UnDoArray    = NULL;    // allocate

static Bool        ImageHasHoles= FALSE; // original image has -holes- =transparent data
static Bool        RecalculateImage   = FALSE; // redo image from Graphics image buffer
static Bool        DidRotate    = FALSE;
static Bool        DidCopy      = FALSE;

static int4        oldActive = -1;
//--------------------------------------------------------------------------

static Bool   InsideImage  = FALSE; // cursor is inside image
static int    RelativeXcoord;       //   5-1-97 for storing into UndoBuffr
static int    RelativeYcoord;       //
static int    oldRelativeXcoord=-1; //
static int    oldRelativeYcoord;    //

static int    oldXcoord, oldYcoord;//
static int    LastcxRange;
static int    LastcyRange;         // Range needed for horz and vert

static int    cxHorzScrollPos = 0; // scrolling x
static int    cyVertScrollPos = 0; // scrolling y

static  int4  OrgXstart;           // Drawing box limits
static  int4  OrgYstart;           //
static  int4  OrgXend  ;           //
static  int4  OrgYend  ;           //

static  int4  DrawX;
static  int4  DrawY;

static  int4  oldDrawX =-1;
static  int4  oldDrawY;
static  int4  prevDrawX=-1;
static  int4  prevDrawY;

static  int4  RotateX;
static  int4  RotateY;

static  int4  MagnifyAdj;
static  int2  DrawingMagnify = 3;   // magnify for drawing   5-1-97 only used here for now

static  byte *RawLoc;              // location in image buffer
static  byte *RawRaw;              // location in raw buffer

//static  int4  RawXcoordStart;
//static  int4  RawYcoordStart;
//static  int4  RawXcoordEnd;
//static  int4  RawYcoordEnd;

static  int4  RawXlast=-1;
static  int4  RawYlast=-1;

static  int4  RawImageSize;           // raw graphic size in file
static  byte *RawImageData =NULL;     // raw graphic data as stored in the file
static  byte *RawImageUndoBufr=NULL;  // backup copy buffer
static  byte *ImageUndoBufr=NULL;     // backup copy buffer
static  int4  MainImageSize = 0;      // graphic at mag of 1 size

// special info box

static  int2  XinfoBoxStart = ((BARWIDTH - 140 ) / 2);  // 140 = Button Widths
static  int2  XinfoBoxEnd   = 120;

typedef struct DoomImage *DOOMImagePtr;
struct DoomImage
{
   byte yorigin;     // 1st Y value
   byte ybytes;      // # of Y data bytes (followed by 00)
   byte *ydata;      // Y data            (followed by 00 ff)
   DOOMImagePtr next;
};

//-------------------    Local Proto's       -----------------------------------
static void     FreeDoomGraphic( DOOMImagePtr *list);
static Bool     ImageToRaw(byte *ImageBufr,DOOMImagePtr *Yoffsets);  // image->DOOM

static void     XY_Update(HWND hwnd, int x, int y);
static HCURSOR  CursorDropper, CursorEraser, CursorFill, CursorMag, CursorPen;

static void     RefreshWindowButtons(void);

static HBITMAP  LoadDibSection(HWND hwnd, LPSTR lpFileName, int2 imagemagnify,Bool Translate);
static HPALETTE CreateDIBPalette(HWND hwnd, HDIB hDIB);
static int4     DIBNumColors(LPSTR lpDIB);
static Bool     SetPCXPalette(HWND hwnd, byte * PCXPaletteBufr);
static void     RemoveDIBSection(void);          // safety check if trans pix in image

static Bool     CheckForTransParent(HWND hwnd, int4 Xsize,
                                               byte *RawImageData,
                                               int2 colortocheck,Bool TellMsg);

static Bool     CheckBitMapForHoles( int4 Xsize,int4 Ysize,
                                            byte* ImageData,
                                            int2  colortocheck);

static void     DrawTheLine(int4 x0, int4 y0, int4 x1, int4 y1);
static void     UndoPixel(void);

static void     RotateImage(byte *UserImageBufr,int2 xoffset, int2 yoffset);
static void     FlipImage  (HWND hwnd,byte *UserImageBufr,int2 fliptype,int2 transparent, int4 Width, int4 Height );
static void     DrawCircle (HDC  hdc,int4 centerX,int4 centerY, double radius);
static void     FillCircle (byte *UserImageBufr,int4 Xloc,int4 Yloc, double radius,
                            int2 fillcolor, int2 newcolor, int2 type,Bool solid);

static void     MatchPixels(byte *UserImageBufr,int2 oldcolor,int2 newcolor,int2 type, int2 width, int2 height);

static Bool     SaveDrawGraphic(HWND hwnd,
                                int4  DoSaveAs,
                                char* picname,
                                char* outfilename,
                                int4  diroffset,
                                int4* ActualRawDataSize, byte**ActualRawDataBufr,
                                int4* ActualImageSize,   byte**ActualImageBufr);

static Bool     PasteImage(HWND hwnd,int2  magnify, int2  transparentfillcolor);    // fill background duke

static Bool     ChangePatchTextureSize(HWND hwnd);
static Bool     MakeBlankTexture(HWND hwnd);
static Bool     ImportBMPtoPWAD(HWND hwnd);

static void     CreateEditWindow(HWND hwnd);
//static void     DestroyEditWindow(void);

static void     GFXEdit_OnPaint(HWND hwnd);  // while editing onpaint

static int4     PasteWadLump(HWND hwnd, char* PasteWadFileName, char* PasteWadLumpName);

static BOOL     CALLBACK PasteLumpDlgProc   (DIALOG_PARAMS);

//static BOOL     CALLBACK PasteWadLumpDlgProc(DIALOG_PARAMS);

//=====================    END  HEADER    ======================================


static void RandomInitial(void)  // only ONCE init number generator ?
{
static Bool FirstTime;
   if (FirstTime)
   {   FirstTime = FALSE;
       randomize();
   }
}


////////////////////////////////////////////////////////////////////////////////
// special mouse coordinates for drawing
// coordinates returned are relative to hEDitWindow
////////////////////////////////////////////////////////////////////////////////
                                                   //*MouseStatus
static UINT GetGFXMouseCoords(HWND hwnd, MSG *msg, Bool RawKey, int4 *xnew, int4 *ynew,int2 *Button)
{
static  POINT  p;
static  UINT   keycode;

   //WaitMessage();                  //MouseStatus    7-1-97 new meaning
   keycode = GetMouseCoords(hwnd,msg,RawKey,NULL,NULL, Button);
   GetCursorPos(&p);
   ScreenToClient(hEditWindow, &p);
//   Trace(hEditWindow,20,4,"raw = %d,%d",p.x,p.y);
   //p.x = (  (p.x+ (DrawingMagnify-1)/2 )/DrawingMagnify) *DrawingMagnify;
   //p.y = (  (p.y+ (DrawingMagnify-1)/2 )/DrawingMagnify) *DrawingMagnify;
//   Trace(hEditWindow,200,4,"new = %d,%d",p.x,p.y);

   if(msg->message == WM_MOUSEMOVE)
   {  XY_Update(hGFXWindow, p.x, p.y);
   }

   if (xnew)
   {   *xnew =p.x;
   }
   if (ynew)
   {   *ynew =p.y;
   }
   return keycode;
}

//-----------------------------------------------------------------------
// Show Current Undo Status   BOX on bottom right
//-----------------------------------------------------------------------
static void ShowCurrUnDo(void)
{
  HDC  hdc;
  HPALETTE hOldPal;
  char msg1[30];
  char msg2[30];
  char msg3[30];
  char msg4[30];
  char msg5[30] = "                            ";
  BITMAP   bm;
  int2    YInfoBoxStart;
//------------------------------------------


     // scrmaxy = 683 for 1024 x 768
     //           515 for 800  x 600

     if (ScrMaxY < 500) return;      // don't do for 640 x 480

     hdc = GetDC(hGFXWindow);
     hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
     RealizePalette(hdc);

     GetObject(hDrawBitmap, sizeof(BITMAP), &bm);

     if (ScrMaxY < 600)              // 800 x 600
     {   YInfoBoxStart = ScrMaxY -38;   //58
     }
     else
     {   YInfoBoxStart = ScrMaxY -68;
     }

     DrawScreenBox3D( hdc, XinfoBoxStart, YInfoBoxStart, XinfoBoxEnd + 34, YInfoBoxStart+70, gd.hBrushes[GRAY]);

     wsprintf(msg1, "UNDO Pixels\t- %5d",    UnDoActive);
     wsprintf(msg2, "Trans. Pixels\t- %5d",  DidTransp);
     wsprintf(msg3, "Magnification\t- %5d",  DrawingMagnify);
     wsprintf(msg4, "Image Size\t= %3dx%3d", bm.bmWidth, bm.bmHeight);

     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+4  , WHITE, msg5);  // clear
     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+18 , WHITE, msg5);  //
     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+34 , WHITE, msg5);  //
     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+50 , WHITE, msg5);  //

     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+4 , WHITE, msg1);
     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+18, WHITE, msg2);
     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+34, WHITE, msg3);
     ScreenColorText( hdc, XinfoBoxStart+4, YInfoBoxStart+50, WHITE, msg4);

     SelectPalette(hdc, hOldPal, FALSE);
     ReleaseDC(hGFXWindow, hdc);
}

//-----------------------------------------------------------------------------
// Helper routines for below
//-----------------------------------------------------------------------------

static int BitmapWidth(HBITMAP hbm)
{
	BITMAP bm;

	if (!hbm) return 0;

	GetObject(hbm, sizeof(bm), &bm);
	return abs(bm.bmWidth);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
static int BitmapHeight(HBITMAP hbm)
{
	BITMAP bm;

	if (!hbm) return 0;

	GetObject(hbm, sizeof(bm), &bm);
	return abs(bm.bmHeight);
}

//-----------------------------------------------------------------------------
// magnify image in original image buffer ( !! Has to use EditWindow !! )
//
// Need to supply a pointer to the dib bits - UserImageBufr
// and a handle to the bitmap so routine can calc size
//
// 3-31-97 bk  Worked fine except as noted Jack
//
//-----------------------------------------------------------------------------
int2 MagnifyGraphicTexture(HWND hwnd,
                           int2  xstart,int2 ystart,
                           int2  xend,  int2 yend,
                           byte *UserImageBufr,
                           HBITMAP Bitmap,
                           int2  magnify,
                           int2  fillcolor)
{
register x, y;
   int4    i,j,k;//,l;
   int4    WrkImageSize;
   byte   *WrkImageBufr;          // buffer containing image
   byte   *baseYcolor;            // start of Y data

   //int2  xwidth = xend - xstart +1;
   //int2  yheight= yend - ystart +1;

   int4    xwidth = xend;       // CAUTION, all calls use GetClientRect   6-1-97
   int4    yheight= yend;       // which has width already in right and bottom, thus prior is off by 1

   byte    color;
   int4    PixLoc;
   int4    xloc,yloc;
   int4    xsize,ysize;

   int4    magXadjust1;         // precalc actual image size (round even row?)
   int4    magXadjust ;
   int4    magYadjust ;
   int4    Xalign;              // row bytes aligned original image

   int4    width, height;

	static     HBITMAP	hDib     = NULL;
   static     VOID ** lpMBits    = NULL;   // Pointer to the actual bits
   BITMAPINFO biM;                 // Our DIB Bitmap Info Struct.
   HBITMAP    hOldBitmap;
   HPALETTE   hOldPal;
   HDC        hdc, hdcMem;
   BITMAP     bm;
   RECT       rc, rcDib, rcClear;
   int        cxRange, cyRange;    // Range needed for horz and vert
	int        xScroll, yScroll;    // Scroll positions

//-----------------------------------------------------------------------------

   //Trace(NULL,10,44,"x=%hd,%hd  y=%hd,%hd",xstart,xend,ystart,yend);

   hdc = GetDC(hwnd);

   hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
   RealizePalette(hdc);

   // dynamic create size of image from xsize
   // get size of the buffer to store image in

   if (!GetObject(Bitmap, sizeof(BITMAP), &bm) ) return 0; //   6-1-97

   xsize   =     bm.bmWidth;
   ysize   = abs(bm.bmHeight);
   Xalign  =(bm.bmWidth+3) & ~3;        // dword row bytes aligned

   magXadjust = xsize*magnify;
   magYadjust = ysize*magnify;
   magXadjust1=(magXadjust+3) & ~3;     // DWORD Aligned

   WrkImageSize = magXadjust1 * magYadjust;

   // Set up a BITMAPINFO structure for use with CreateDIBSection()
   ZeroMemory(&biM, sizeof(BITMAPINFO));
   biM.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
   biM.bmiHeader.biPlanes     =  1;
   biM.bmiHeader.biBitCount   =  8;
   biM.bmiHeader.biWidth      =  magXadjust;   //   5-1-97 magXadjust1;
   biM.bmiHeader.biHeight     = -magYadjust;
	biM.bmiHeader.biSizeImage  =  magXadjust1 * magYadjust;
   biM.bmiHeader.biCompression=  BI_RGB;

  	hdcMem = CreateCompatibleDC(hdc);

   hDib = (HBITMAP)CreateDIBSection(hdc, &biM, DIB_PAL_COLORS, (VOID **)&lpMBits, NULL, 0L);

   if (!hDib || !lpMBits)                            // not enough memory
   {   OutOfMemoryMsg(hwnd);
       hDib    = NULL;
       lpMBits = NULL;
       SelectPalette(hdc, hOldPal, FALSE);
       ReleaseDC(hwnd, hdc);
       return -1;
   }

   WrkImageBufr = (byte *)&*lpMBits;                  // Grab pointer to the bits of our DIBSection

   hOldBitmap  = SelectBitmap(hdcMem, hDib);	         // Select into our Memory Dc
	SetDIBColorTable(hdcMem, 0, 256, dRGBquad);			// Palette indexes

   FillMemory(WrkImageBufr, WrkImageSize, fillcolor); // fill with background

   // automatically center image

   xloc = xstart +xwidth /2 - magXadjust/2;           // center x
   yloc = ystart +yheight/2 - magYadjust/2;           // center y

   GetClientRect(hwnd, &rc);                          // this has to be hEditWindow ALL the time

   // setup scrollbar ranges if graphic is larger than window, gives negative size

   cxRange = magXadjust - rc.right;     // Graphic Xsize - Xsize of window
   cyRange = magYadjust - rc.bottom;    //         Ysize - Ysize of Window

   // reset global variable if a scrollbar is turned off
   if (cxRange <=0)
   {   cxRange = 0;
       cxHorzScrollPos = 0;
   }
   if (cyRange <=0)
   {   cyRange = 0;
       cyVertScrollPos = 0;
   }

   // Set the ranges we've calculated (0 to 0 means invisible scrollbar) (SetCrollinfo new 95 later)

   cxRange = ( (cxRange+magnify-1)/magnify )*magnify;
   cyRange = ( (cyRange+magnify-1)/magnify )*magnify;

   SetScrollRange(hwnd, SB_HORZ, 0, cxRange, TRUE);
   SetScrollRange(hwnd, SB_VERT, 0, cyRange, TRUE);

   // erase the last area used if it was active

   if (LastPicXloc != -1)                                                  // not first time
   {
      if ( magXadjust != LastPicXsize ||  magYadjust != LastPicYsize       // size differ
       ||  cxRange    != LastcxRange  ||  cyRange    != LastcyRange)       // scroll differ
      {
         //  scroll bars mess up clear if going from scroll to nonscroll

         if (cxRange == LastcxRange && cyRange == LastcyRange)             // scroll is the same
         {   if(LastPicXloc < 0)    LastPicXloc = 0;
             if(LastPicYloc < 0)    LastPicYloc = 0;

             //Trace(hwnd,10,10,"%hd,%hd",LastPicXsize,LastPicYsize);

             width  = LastPicXsize  + LastPicXloc;
             height = LastPicYsize  + LastPicYloc;

             if(width  > xend ) width  = xend;
             if(height > yend ) height = yend;

             if (magXadjust < width || magYadjust < height)                  //   6-1-97
             {   SetRect(&rcClear, LastPicXloc, LastPicYloc, width, height); //   5-1-97 -1
                 FillRect(hdc, &rcClear, gd.hBrushes[MenuColor]); //1-1-98
             }
         }
         else
         // scroll size change
         {   FillRect(hdc, &rc, gd.hBrushes[MenuColor]);  //1-1-98
         }
      }
   }
   else
   // special case for zooming since scroll bars come and go!
   if (LastcxRange == -1)
   {   FillRect(hdc, &rc, gd.hBrushes[MenuColor]); //1-1-98
   }

   // copy mag 1 image buffer into this one

   y = 0;
   for (i = 0; i < ysize; i++, y+=magnify)         // Y ------
   {   x = 0;
       baseYcolor = &UserImageBufr[ i*Xalign];     // source
       for (j=0; j< xsize; j++, x+=magnify)        // X-------
       {  color  = *(baseYcolor+j);                // source
	       for (k=0; k < magnify; k++)              // PixLoc+=magXadjust1) //y
	       {	 PixLoc = (y+k)*magXadjust1 + x;       // target
             //for (l=0; l< magnify; l++)            //x
		       //{    WrkImageBufr[PixLoc++] = color;
		       //}
             memset(&WrkImageBufr[PixLoc],color,magnify);  //   5-1-97
	       }
       }   // end x
   } //end y


   if(xloc < 0)  xloc = 0;
   if(yloc < 0)  yloc = 0;

   // Get scroll bar positions
	xScroll  = GetScrollPos(hEditWindow, SB_HORZ);
   yScroll  = GetScrollPos(hEditWindow, SB_VERT);

   // Set up the necessary rectangles -- i.e. the rectangle
   // we're rendering into, and the rectangle in the bitmap

   rcDib.left   = xScroll;
   rcDib.top    = yScroll;
   rcDib.right  = xScroll + RECTWIDTH(&rc) -1;   //   5-1-97
   rcDib.bottom = yScroll + RECTHEIGHT(&rc)-1;


   //  Destination  X, Y ,  Width               Height,      Source Start   X  ,   Y
   //  BitBlt(hdc, xloc, yloc, magXadjust-(magXoffset * 2), magYadjust-(magYoffset * 2),hdcMem, magXoffset, magYoffset, SRCCOPY);
   //  BitBlt(hdc, xloc, yloc, RECTWIDTH(&rc), RECTHEIGHT(&rc),hdcMem, rcDib.left, rcDib.top, SRCCOPY);

   //   5-1-97 don't know if this matters since it appeared to work either way (haven't chked since all the changes)

   if ( magXadjust < RECTWIDTH(&rc)
   ||   magYadjust < RECTHEIGHT(&rc))
   {
        BitBlt(hdc, xloc, yloc, magXadjust, magYadjust, hdcMem, rcDib.left, rcDib.top, SRCCOPY);  //   5-1-97

        OrgXend      = xloc + magXadjust -1;     //   5-1-97   check this out ,see if recwidth if past size?
        OrgYend      = yloc + magYadjust -1;
        LastPicXsize = magXadjust;
        LastPicYsize = magYadjust;
   }
   else
   {
       BitBlt(hdc, xloc, yloc, RECTWIDTH(&rc), RECTHEIGHT(&rc),hdcMem, rcDib.left, rcDib.top, SRCCOPY);

       OrgXend      = xloc + RECTWIDTH (&rc) -1; //   5-1-97  -1
       OrgYend      = yloc + RECTHEIGHT(&rc) -1;
       LastPicXsize = RECTWIDTH(&rc);
       LastPicYsize = RECTHEIGHT(&rc);
   }

   GdiFlush();

   //Trace(hGFXWindow,10,10,"  %d,%d  %d,%d",cxRange,cyRange,LastcxRange,LastcyRange);

   OrgXstart    = xloc ;
   OrgYstart    = yloc ;
   // had to add for new way of stuff
   //Trace(hGFXWindow,10,10," range x=%d,%d y=%d,%d",OrgXstart,OrgXend,OrgYstart,OrgYend);

   LastPicXloc  = xloc;
   LastPicYloc  = yloc;
   LastcxRange  = cxRange;
   LastcyRange  = cyRange;

   //Trace(hEditWindow, 12,12, "xS=%d, yS=%d, xR=%d, yR=%d", xScroll, yScroll, cxRange, cyRange);
   //Trace(hwnd,10,2,"x=%d %d  y= %d %d  scroll x=%d y=%d  width=%d %d",
   //      OrgXstart,OrgXend,OrgYstart,OrgYend,cxHorzScrollPos,cyVertScrollPos,RECTWIDTH(&rc),RECTHEIGHT(&rc) );

   SelectBitmap(hdcMem, hOldBitmap);
   DeleteDC(hdcMem);
   DeleteBitmap(hDib);
   hDib 	    = NULL;
   lpMBits 	 = NULL;

   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hwnd, hdc);

   //ValidateRect(hwnd, NULL);
   ValidateRect(hwnd,&rc);
   return 1;

}

//-----------------------------------------------------------------------------
// scroll     (has to be hEditWindow
//-----------------------------------------------------------------------------

#define DRAWINGTHUMBPOSITION  4444
#define SCROLL_RATIO          4

static int GFXScroll(HWND hwnd, int nHorzOrVert, UINT code, int pos)
{
	 int nPos;          // Scrollbar position.
	 int nMin;          // Minumum scroll bar value.
    int nMax;          // Maximum scroll bar value.
    int nMove;         // How much to move.
    int nLine;         // # of pixels for LINEUP/LINEDOWN
    int nPage;

    RECT rc;           // Client area.

//-----------------------------------------------------------------------------

    GetClientRect(hwnd, &rc);

    if (nHorzOrVert == SB_HORZ)      //   check out both 6-1-97
    {   nPage = (  ( BitmapWidth(hDrawBitmap) * DrawingMagnify) - RECTWIDTH(&rc) );
    }
    else
    {   nPage = (  ( BitmapHeight(hDrawBitmap)* DrawingMagnify) - RECTHEIGHT(&rc) );
    }

    if (nPage < 0) return 0;

    // On a SB_LINEUP/SB_LINEDOWN we will move the picture by
    //  1 / SCROLL_RATIO of the page amount (i.e. if SCROLL_RATIO
    //  is 4, it will scroll the picture a quarter of the way)

    ShowCursor(FALSE);                //   6-1-97

    nLine = nPage / SCROLL_RATIO;     // 1/4th of size

    if (nLine == 0) nLine = 1;

    nPos = GetScrollPos(hwnd, nHorzOrVert);

	 GetScrollRange(hwnd, nHorzOrVert, &nMin, &nMax);

    switch (code)
    {
        case SB_LINEDOWN:             // One line right.
            nMove = nLine;
            break;

        case SB_LINEUP:               // One line left.
            nMove = -nLine;
            break;

		  case SB_PAGEDOWN:             // One page right.
            nMove = nPage;
            //wsprintf(msg, "Move=%d, pos=%d nPos=%d, min=%d max=%d h=%d dh=%d", nMove, pos, nPos, nMin, nMax, height, dheight);
            //MessageBox(hEditWindow, msg, "Hello", MB_OK);
            break;

        case SB_PAGEUP:               // One page left.
            nMove = -nPage;
            break;

        case SB_THUMBPOSITION:        // Absolute position.
            nMove = pos - nPos;
            break;
        case DRAWINGTHUMBPOSITION:    // Absolute position. internal
            nMove = pos;
            break;

		  default:                      // No change.
            nMove = 0;
            break;
    }

    //Trace(hGFXWindow,20,40,"nmove=%d min=%d max=%d",nMove,nMin,nMax);
    if (nMove)
    {
        nPos += nMove;
        nPos   =( (nPos+DrawingMagnify-1)/DrawingMagnify)*DrawingMagnify; //   5-1-97 req'd

        if (nPos < nMin)
        {
            nMove -= nPos - nMin;
				nPos = nMin;
            if (nHorzOrVert == SB_HORZ)
            {   cxHorzScrollPos = nPos;   // set the global variable now   6-1-97 else it might not update
            }
            else
            {  cyVertScrollPos = nPos;    // set the global variable
            }
        }

        if (nPos > nMax)
        {
            nMove -= nPos - nMax;
            nPos = nMax;
            if (nHorzOrVert == SB_HORZ)   //these probably will, just in case
            {   cxHorzScrollPos = nPos;   // set the global variable now   6-1-97 else it might not update
            }
            else
            {   cyVertScrollPos = nPos;    // set the global variable
            }
        }
        //Trace(hGFXWindow,20,70,"nmove=%d min=%d max=%d",nMove,nMin,nMax);

        if (nMove)
        {
            //int SetScrollInfo( HWND  hwnd,	// handle of window with scroll bar
            //                     int  fnBar,	// scroll bar flag
            //                     LPSCROLLINFO  lpsi,	// pointer to structure with scroll parameters
            //                      BOOL  fRedraw	// redraw flag );
            SetScrollPos(hwnd, nHorzOrVert, nPos, TRUE);

            if (nHorzOrVert == SB_HORZ)
            {   cxHorzScrollPos = nPos;    // set the global variable
            }
            else
            {   cyVertScrollPos = nPos;   // set the global variable
            }

            MagnifyGraphicTexture(hEditWindow, EditRC.left,  EditRC.top,
                                               EditRC.right, EditRC.bottom,
                                              (byte *)&*lpDrawBits,
                                               hDrawBitmap,
                                               DrawingMagnify, Transparent);

            //Trace(hGFXWindow,40,40,"xscroll=%d,yscroll=%d",cxHorzScrollPos,cyVertScrollPos);
        }
    }

    //Trace(hGFXWindow,40,80,"gfx xscroll=%d,yscroll=%d",cxHorzScrollPos,cyVertScrollPos);

    ShowCursor(TRUE);   //   6-1-97
    return nMove;
}

//-----------------------------------------------------------------------------
#pragma argsused
static void GFXEdit_OnHScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
     GFXScroll(hwnd,SB_HORZ,code,pos);
}
//-----------------------------------------------------------------------------
#pragma argsused
static void GFXEdit_OnVScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
     GFXScroll(hwnd,SB_VERT,code,pos);
}


//-----------------------------------------------------------------------------
// Premature Exit in LoadDibSection comes here
//-----------------------------------------------------------------------------
static void LoadDibSectionErrExit(HANDLE hDIBNew, HANDLE hFile, Bool UnlockDib, Bool FreeDib)
{
   if (hDIBNew)   // have a dib     5-1-97
   {   if(UnlockDib)  GlobalUnlock(hDIBNew);
       if(FreeDib  )  GlobalFree  (hDIBNew);
   }

   CloseHandle(hFile);
   SetCursor(LoadCursor(NULL, IDC_ARROW));
   ImageType = INVALIDIMAGE;
}

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  Load a windows BMP in and return the handle to the Bitmap                 //
//    5-1-97 added translate option                                         //
////////////////////////////////////////////////////////////////////////////////
static HBITMAP LoadDibSection(HWND hwnd, LPSTR lpFileName,int2 imagemagnify,Bool Translate)
{
    HDC                 hdc;
    HANDLE              hFile;
    BITMAPFILEHEADER    bmfHeader;
    UINT                nNumColors;
    HDIB                hDIBNew;
    LPBITMAPINFOHEADER  lpbih;
    DWORD               dwBytesRead;
    Bool                DoFlip = FALSE;
    char                msg1[80];
    // open the DIB file

    hFile = CreateFile(lpFileName,  GENERIC_READ,
                                    FILE_SHARE_READ,
                                    NULL,
                                    OPEN_EXISTING,
                                    FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);

    if (hFile == INVALID_HANDLE_VALUE)
    {
        DIBError(ERR_FILENOTFOUND);
        return NULL;
    }

    // read the BITMAPFILEHEADER from our file

    if (!ReadFile(hFile,&bmfHeader,sizeof(BITMAPFILEHEADER),&dwBytesRead,NULL)
      || dwBytesRead != sizeof(BITMAPFILEHEADER))
    {
        DIBError(ERR_READ);
        LoadDibSectionErrExit(NULL, hFile, FALSE, FALSE);  // nounlock, nofree
        return NULL;
    }


    if (bmfHeader.bfType != DIB_BMP_HEADER_MARKER)        //0x4d42)  // 'BM'
    {
        DIBError(ERR_NOT_DIB);
        LoadDibSectionErrExit(NULL, hFile, FALSE, FALSE);  // nounlock, nofree,
        return NULL;
    }

    // Allocate memory for header & color table.

    hDIBNew = GlobalAlloc(GHND, sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD));

    if (!hDIBNew)
    {
        DIBError(ERR_MEMORY);
        LoadDibSectionErrExit(NULL, hFile, FALSE, FALSE);  // nounlock, nofree
        return NULL;
    }

    lpbih = (LPBITMAPINFOHEADER)GlobalLock(hDIBNew);

    if (!lpbih)
    {
        DIBError(ERR_LOCK);
        LoadDibSectionErrExit(hDIBNew, hFile, FALSE, TRUE);  // nounlock, free
        return NULL;
    }

    // read the BITMAPINFOHEADER
    if (!ReadFile(hFile,lpbih,sizeof(BITMAPINFOHEADER),&dwBytesRead,NULL)
      || dwBytesRead != sizeof(BITMAPINFOHEADER))
    {
        DIBError(ERR_READ);
        LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE); // unlock, free
        return NULL;
    }

    /* //     ====== debug stuff === leave in
    NotifyMsg(NULL,//"bheadr=%d\n"
                   "bsize =%d\n"
                   //"bres1 =%hd\n"
                   //"bres2 =%d\n"
                   "boffb =%d\n\n"

                   "size  =%d\n"
                   "width =%d\n"
                   "height=%d\n"
                   "planes=%hd\n"
                   "bitcnt=%hd\n"
                   "comprs=%d\n"
                   "size  =%d\n"
                   "xpels =%d\n"
                   "ypels =%d\n"
                   "clrusd=%d\n"
                   "clrImp=%d",

      bmfHeader.bfSize,           //BITMAPFILEHEADER
      //bmfHeader.bfReserved1,
      //bmfHeader.bfReserved2,
      bmfHeader.bfOffBits,

      lpbih->biSize          ,//= sizeof(BITMAPINFOHEADER);
      lpbih->biWidth         ,//= lpbi->bmiHeader.biWidth;
      lpbih->biHeight        ,//= lpbi->bmiHeader.biHeight;
      lpbih->biPlanes        ,//= 1;              // must be 1
	   lpbih->biBitCount      ,//= 8;
      lpbih->biCompression   ,//= BI_RGB;
	   lpbih->biSizeImage     ,//= lpbi->bmiHeader.biSizeImage;
      lpbih->biXPelsPerMeter ,//= 0;
      lpbih->biYPelsPerMeter ,//= 0;
      lpbih->biClrUsed       ,//= 256;
      lpbih->biClrImportant );//= 0;
     // == end debug stuff */

    // Check to see that it's a Windows DIB -- an OS/2 DIB would cause
    // problems. If it's not a Windows DIB (e.g. if biSize is wrong), return NULL.
    if( lpbih->biCompression != BI_RGB)
    {
        HelpMsg(hwnd,CENTER|HELPINFO, "RLE BMP",
                                      "RLE BMPs not supported",
                                      NULL);
        
        LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);// unlock, free
        return NULL;
    }

    if (lpbih->biSize != sizeof(BITMAPINFOHEADER))
    {
        DIBError(ERR_NOT_DIB);
        LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);// unlock, free
        return NULL;
    }

    // #bits/check to see if it's a format we support

    switch (lpbih->biBitCount)
    {
        case 1:
        case 4:
        case 8:
        case 16:
        case 24:
        case 32:
            // the format is supported so continue
            break;

        default:
            // the format isn't supported so fail
            DIBError(ERR_UNSUPPORTEDFORMAT);
            LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);// unlock, free
            return NULL;
    }

    // Now determine the size of the color table and read it.  Since the
    // bitmap bits are offset in the file by bfOffBits, do some special
    // processing to make sure the bits directly follow the color table
    //if (! (nNumColors = (UINT)lpbih->biClrUsed) )

    nNumColors = (UINT)lpbih->biClrUsed;

    if(!nNumColors)
    {
        // color table only for 8 bits per pixel or less
         if (lpbih->biBitCount <= 8)
         {   nNumColors = 1 << lpbih->biBitCount; // standard size table
         }
    }

    // fill in some default values if they are zero
    if (lpbih->biClrUsed == 0)
        lpbih->biClrUsed = nNumColors;

    if (lpbih->biSizeImage == 0)
    {   lpbih->biSizeImage = WIDTHBYTES(lpbih->biWidth * lpbih->biBitCount) * abs(lpbih->biHeight);  //   7-1-97
    }

    // if more than 256 - resize color table
    if (nNumColors > 256)   // very rare, but possible (does user need > 256 colors ... need to test  )
    {
        HDIB hDIBInfoTemp;

        // get a proper-sized buffer for header and color table
        GlobalUnlock(hDIBNew);                     // unlock prior allocation first
        hDIBInfoTemp = GlobalReAlloc(hDIBNew,lpbih->biSize + nNumColors * sizeof(RGBQUAD),0);

        if (!hDIBInfoTemp) // can't resize buffer for loading
        {
            DIBError(ERR_MEMORY);
            LoadDibSectionErrExit(hDIBNew, hFile, FALSE, TRUE); // nounlock, free
            return NULL;
        }
        else
        {   hDIBNew = hDIBInfoTemp;
        }

        lpbih = (LPBITMAPINFOHEADER)GlobalLock(hDIBNew);  // lock
        if (!lpbih)
        {
            DIBError(ERR_LOCK);
            LoadDibSectionErrExit(hDIBNew, hFile, FALSE, TRUE); // nounlock, free
            return NULL;
        }
    }

    // since colors < 256 require more work (+freewrk blows up?), don't do

    if (Pasting
    &&  lpbih->biBitCount != 8)                                //   8-1-97
    {   //if (nNumColors < 256)
        {
           if (lpbih->biBitCount > 8)
           {
                sprintf(msg1,"%d colors paste not supported (256 only)",1 << lpbih->biBitCount);
                HelpMsg(hwnd,CENTER|HELPHELP, "",
                                              msg1,
                                              NULL);
           }
           else
           {
                sprintf(msg1,"%d colors paste not supported (256 only)",nNumColors);
                HelpMsg(hwnd,CENTER|HELPHELP, "",
                                              msg1,
                                              NULL);
           }
           LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);      // unlock, free
           return NULL;
        }
    }

    // read the color table
    if (!ReadFile(hFile, (LPSTR)(lpbih) + lpbih->biSize, nNumColors * sizeof(RGBQUAD), &dwBytesRead, NULL)
      || dwBytesRead != nNumColors * sizeof(RGBQUAD))
    {
        DIBError(ERR_READ);
        LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);      // unlock, free
        return NULL;
    }

    // we've reached the point of no return
    // remove existing DIB section

    RemoveDIBSection();

    // set global variable to new DIB header
    hViewDibInfo = hDIBNew;

    // create new DIB section (match our convention if reqd)

    if (lpbih->biHeight > 0
     && lpbih->biBitCount == 8)
    {   lpbih->biHeight = -lpbih->biHeight;
        DoFlip = TRUE;
    }

    hdc = GetDC(hwnd);
    hViewBitmap = CreateDIBSection(hdc, (LPBITMAPINFO)lpbih, DIB_RGB_COLORS,(VOID **)&lpViewBits, 0, 0L);
    ReleaseDC(hwnd, hdc);

    if (!hViewBitmap || !lpViewBits)
    {
        DIBError(ERR_CREATEDIBSECTION);

        if (hViewBitmap)
        {   DeleteObject(hViewBitmap);
            hViewBitmap = NULL;
        }

        LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);     // unlock, free
        return NULL;
    }

    // fill the buffer allocated by CreateDIBSection with bitmap bits
    SetFilePointer(hFile, bmfHeader.bfOffBits, NULL, FILE_BEGIN);

    if (!ReadFile(hFile, lpViewBits, lpbih->biSizeImage, &dwBytesRead, NULL)
     || dwBytesRead != lpbih->biSizeImage)
    {
        // failed to read the DIB bits into the DIB section so get rid of it

        DIBError(ERR_READ);
        DeleteObject(hViewBitmap);
        hViewBitmap = NULL;
        LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);    // unlock, free
        return NULL;
    }

    if (hViewPalette)
    {   DeletePalette(hViewPalette);            // remove existing palette object   5-1-97 safety
    }

    // create a palette for the DIB section from the DIB color table

    hViewPalette = CreateDIBPalette(hwnd,hViewDibInfo);                        // req'd
    // if hPalette is NULL it won't lead to failure condition, so don't check it

    // Could store global access here but . . .
    //CopyWidth  = lpbih->biWidth;    Save for global Access ??
    //CopyHeight = lpbih->biHeight;   Right now we GetObject Bitmap(hViewBitmap)

    /////////////////////// new   5-1-97 ////////////////////////////////////

    // Set the cursor to a hourglass, in case the loading operation
    // takes more than a sec, the user will know what's going on.

    SetCursor(LoadCursor(NULL, IDC_WAIT));

    //match our conventions   7-1-97
    if (DoFlip)
    {   FlipImage  (hwnd,(byte *) &*lpViewBits, 1, 0 , lpbih->biWidth ,lpbih->biHeight );   // flip Y
    }

    if (imagemagnify > 1 )
    {
      int2    Width,Height,WidthAlign;
      BITMAP  bm;
      byte   *CopyImageBufr;
      int2    CopyWidth;
      int2    CopyHeight;
      int2    CopyWidthAlign;

      byte   *BMPImageBufr;      // Mag buffer
      byte   *baseYcolor;        // start of Y data

      int4    i,j,x,k,y;//l
      byte    color;
      int4    PixLoc;
      int2    TempColor;

      HBITMAP    BhViewBitmap;   // Handle to the bitmap BMP
      VOID **    BlpViewBits ;   // Pointer to the actual bits
      BITMAPINFO BViewBi;        // Our DIB Bitmap Info Struct.

      GetObject(hViewBitmap, sizeof(BITMAP), &bm);

      CopyWidth      =     bm.bmWidth;
      CopyHeight     = abs(bm.bmHeight);
      CopyWidthAlign = (CopyWidth+3) & ~3;

      Width      = CopyWidth  * imagemagnify;
      Height     = CopyHeight * imagemagnify;
      WidthAlign = ((Width + 3) & ~3);

      CopyImageBufr = (byte *) &*lpViewBits;         // actual BMP bits

      //ColorPaletteMatch(PaletteBufr,NULL,ColorTranslation);  // convert, store in ColorTrans   5-1-97

      // Set up a BITMAPINFO structure for use with CreateDIBSection()
      ZeroMemory(&BViewBi, sizeof(BITMAPINFO));
      BViewBi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      BViewBi.bmiHeader.biPlanes     =   1;
      BViewBi.bmiHeader.biBitCount   =   8;
      BViewBi.bmiHeader.biWidth      =   Width;
      BViewBi.bmiHeader.biHeight     =  -Height;
     	BViewBi.bmiHeader.biSizeImage  =   WidthAlign * abs(Height); //   5-1-97 was width
      BViewBi.bmiHeader.biCompression=   BI_RGB;

      // Create a DC to hold our DIBSECTION
      hdc = GetDC(hwnd);                           // use DIB_PAL instead of RGB
      BhViewBitmap = (HBITMAP)CreateDIBSection(hdc, &BViewBi, DIB_PAL_COLORS, (VOID **)&BlpViewBits, NULL, 0L);
      ReleaseDC(hwnd, hdc);

      if (!BhViewBitmap || !BlpViewBits)                            // not enough memory
      {    OutOfMemoryMsg(hwnd);
           DIBError(ERR_CREATEDIBSECTION);

           if (BhViewBitmap)
           {   DeleteObject(BhViewBitmap);
           }
           DeleteObject(hViewBitmap);
           hViewBitmap = NULL;
           lpViewBits  = NULL;

           LoadDibSectionErrExit(hDIBNew, hFile, TRUE, TRUE);    // unlock, free
           return NULL;
      }

      BMPImageBufr = (byte *)&*BlpViewBits;	// Grab pointer to the bits of our DIBSection

      y = 0;

      for (i = 0; i < CopyHeight; i++, y+=imagemagnify)     // Y ------
      {
           x = 0;
           baseYcolor = &CopyImageBufr[ i*CopyWidthAlign];  // source

           for (j=0; j< CopyWidth; j++, x+=imagemagnify)    // X-------
           {
              if (Translate)
              {   TempColor      = *(baseYcolor+j)&0x0ff;
                 *(baseYcolor+j) = ColorTranslation[TempColor];
              }

              color  = *(baseYcolor+j);                     // source

	           for (k=0; k < imagemagnify; k++)              // PixLoc+=magXadjust1) //y
	           {   PixLoc = (y+k)*WidthAlign + x;            // target
                  //for (l=0; l< imagemagnify; l++)           //x
		            //{    BMPImageBufr[PixLoc++] = color;
		            //}
                  memset(&BMPImageBufr[PixLoc],color,imagemagnify);  //   5-1-97
	           }
           }     // end x
      }  //end y

      // now replace bitmap with this one

      DeleteObject(hViewBitmap);                             // delete old one
      hViewBitmap = BhViewBitmap;                            // new BMP
      lpViewBits  = BlpViewBits;

      //if (hPalette)                                          //   8-1-97
      {   GlobalUnlock(hViewDibInfo);
      }
      CloseHandle(hFile);
    }
    /////////////////////////// end new   5-1-97 /////////////////////////////
    else
    {
        BITMAP  bm;
        int2    Width,Height,WidthAlign;

        GetObject(hViewBitmap, sizeof(BITMAP), &bm);
        Width      =     bm.bmWidth;
        Height     = abs(bm.bmHeight);
        WidthAlign = (Width+3) & ~3;

        // Set up a BITMAPINFO structure for use with CreateDIBSection()
        ZeroMemory(&ViewBi, sizeof(BITMAPINFO));
        ViewBi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
        ViewBi.bmiHeader.biPlanes     =   1;
        ViewBi.bmiHeader.biBitCount   =   8;
        ViewBi.bmiHeader.biWidth      =   Width;
        ViewBi.bmiHeader.biHeight     =  -Height;
     	  ViewBi.bmiHeader.biSizeImage  =   WidthAlign * abs(Height); //   5-1-97 was width
        ViewBi.bmiHeader.biCompression=   BI_RGB;

        if (Translate)
        {   byte *CopyImageBufr;
            int4 CopySize,i;

            CopySize   = ((LPBITMAPINFOHEADER)lpbih)->biSizeImage;
            CopyImageBufr = (byte *) &*lpViewBits;         // actual BMP bits

            for(i = 0; i < CopySize; i++)
            {
	             CopyImageBufr[i]= ColorTranslation[CopyImageBufr[i]&0x00ff];
            }
        }

        //if (hPalette)                                          //   8-1-97
        {   GlobalUnlock(hViewDibInfo);
        }
        CloseHandle(hFile);
    }

    SetCursor(LoadCursor(NULL, IDC_ARROW));

    ImageType = BMPIMAGE;

    return hViewBitmap; // Success
}

////////////////////////////////////////////////////////////////////////////////
//
//    hDib       - Handle to BITMAPFINFO and color table (if any) of dib
//    lpvDIBBits - Pointer to bitmap bits
//
//  RETURN VALUE:
//    0 if successful, or one of ERR_INVALIDHANDLE, ERR_OPEN, ERR_LOCK or ERR_SAVE.
//
// 2-10-97 bk  Major Mods
// 3-31-97 bk  Noticed a small discrepancy with SavePCX, one graphic
// was 41 wide in SavePCX and when I looke in ACDC the same BMP was 44
// probably due to padding of the DIB.  Don't know if there is any way around
// that, but I'll look into it to see exactly what is happening.
////////////////////////////////////////////////////////////////////////////////

#pragma argsused   //  3-1-97 hwnd
WORD SaveDIB(HWND hwnd,LPBITMAPINFO lpbi, LPVOID lpvDIBits, char * lpFileName,RGBQUAD* RGBquad)
{
    HDIB                hDIBSave;
    LPBITMAPINFOHEADER  lpbih;
    LPRGBQUAD           lpColorTable;

	 BITMAPFILEHEADER    bmfHdr;     // Header for Bitmap file
	 HANDLE              hFile;      // file handle for opened file
	 DWORD               dwDIBHeaderSize, dwDIBBitsSize, dwBytesWritten;
	 BOOL                bError=FALSE; //   5-1-97 initialize so no msg on cancel

    char TempName[260] = {0};
    char * dotp;
    int4 i;

//==============================================================================

   char  drive     [ 4]= {0};  // 2 'C:'  (+leading 03 for drive picture control)   3-1-97
   char  dir      [260]= {0};  // 31 (28)Includes leading, trailing backslashes (\)
   char  filename [260]= {0};  // 39 12345678 (260 win32)
   char  ext      [260]= {0};  // 43 .xxx     (260 win32)

   int   flags;

static  char PriorDir[260]={0};

//------------------------------------------------------------------------------

   if (!DeePRegistered)
   {  if (ImageXSize > 64
      ||  ImageYSize > 64)
      {
          OnlyRegBigGraphicSave();     // tell only registered this big
          return FALSE;
      }
   }

   strcpy(TempName,PriorDir);       // prior drive:\dir +
   strcat(TempName,lpFileName);     // what user passed

   dotp = strrchr( TempName, '.');  // find . (reverse)

   if (dotp == NULL)
   {  strcat( TempName, ".BMP");    // add .BMP
   }
   else
   {  strcpy( dotp, ".BMP");        // replace .any
   }

   //GetSaveName(hwnd,"BMP name", TempName,256, (char *)szLoadString(IDS_FILTERBMP) );//gd.hMainWindow   3-1-97
   GetSaveName(hwnd,"BMP name", TempName,256, IDS_FILTERBMP );//gd.hMainWindow   3-1-97

   if(TempName[0] != '\0'
   && !FileIsInUse(TempName,TRUE,FALSE) )   // msg + no set TempName to '\0'   5-1-97
   {
      //   3-1-97
      flags = fnsplit(TempName,drive,dir,filename,ext);

      // keep any dir info now set

      if ((flags & DRIVE)
      &&  (flags & DIRECTORY))
      {    _makepath(PriorDir,drive,dir,NULL,NULL);  // create default dir
      }

      hFile = CreateFile(TempName, GENERIC_READ | GENERIC_WRITE,
                          0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

      if (hFile == INVALID_HANDLE_VALUE)    return ERR_OPEN;

	   // Allocate memory for header & color table (if needed)
      hDIBSave = GlobalAlloc(GHND, (DWORD)(sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD)));

      lpbih = (LPBITMAPINFOHEADER)GlobalLock(hDIBSave);

      // initialize bitmap header
      lpbih->biSize          = sizeof(BITMAPINFOHEADER);
      lpbih->biWidth         = lpbi->bmiHeader.biWidth;
      lpbih->biHeight        = abs(lpbi->bmiHeader.biHeight);  //   10-1-97
      lpbih->biPlanes        = 1;                              // must be 1
	   lpbih->biBitCount      = 8;
      lpbih->biCompression   = BI_RGB;
	   lpbih->biSizeImage     = lpbi->bmiHeader.biSizeImage;
      lpbih->biXPelsPerMeter = 0;
      lpbih->biYPelsPerMeter = 0;
      lpbih->biClrUsed       = 256;
      lpbih->biClrImportant  = 256;                            // was 0 (which also means all)

      // find the color table - immediately after the header
      lpColorTable = (LPRGBQUAD)((LPSTR)(lpbih) + lpbih->biSize);

      // fill the color table with our stored colors from the palette
      if (!RGBquad)
      {
          for (i = 0; i < 256; i++, lpColorTable++)
          {
            lpColorTable->rgbRed   = dRGBquad[i].rgbRed;
            lpColorTable->rgbGreen = dRGBquad[i].rgbGreen;
            lpColorTable->rgbBlue  = dRGBquad[i].rgbBlue;
          }
      }
      else
      {
          for (i = 0; i < 256; i++, lpColorTable++)
          {
            lpColorTable->rgbRed   = RGBquad[i].rgbRed;
            lpColorTable->rgbGreen = RGBquad[i].rgbGreen;
            lpColorTable->rgbBlue  = RGBquad[i].rgbBlue;
          }
      }

      // Fill in file type (first 2 bytes must be "BM" for a bitmap)
      bmfHdr.bfType = DIB_BMP_HEADER_MARKER;  // "BM"

      // Calculate size of header and color table (if any)
      dwDIBHeaderSize = lpbih->biSize + (WORD)(256 * sizeof(RGBQUAD)) ;

      // It's not RLE, so size is (DWORD aligned) Width * Height
      //      dwDIBBitsSize = WIDTHBYTES(lpbih->biWidth * lpbih->biBitCount) * abs(lpbih->biHeight); //   7-1-97
      //      lpbih->biSizeImage = dwDIBBitsSize;

      dwDIBBitsSize =lpbih->biSizeImage ;//   7-1-97 size is already set earlier, minimize mistakes

      // Calculate file size by adding DIB size to sizeof(BITMAPFILEHEADER)

      bmfHdr.bfSize = dwDIBHeaderSize + dwDIBBitsSize + sizeof(BITMAPFILEHEADER);
      bmfHdr.bfReserved1 = 0;
      bmfHdr.bfReserved2 = 0;

      // Now, calculate the offset the actual bitmap bits will be in the file
      // It's the Bitmap file header plus the DIB header plus the size of the color table.

      bmfHdr.bfOffBits = (DWORD)sizeof(BITMAPFILEHEADER) + dwDIBHeaderSize;

      // Write the file header

      bError = (!WriteFile(hFile, &bmfHdr, sizeof(BITMAPFILEHEADER), &dwBytesWritten, NULL)
                || dwBytesWritten != sizeof(BITMAPFILEHEADER));

      // Write the DIB header and the bits

	   if (!bError)
          bError = (!WriteFile(hFile, lpbih, dwDIBHeaderSize, &dwBytesWritten, NULL)
                    || dwBytesWritten != dwDIBHeaderSize);

      // looks like by convention bmp are drawn opposite of what we did

      if (lpbih->biHeight > 0)
      {   FlipImage (hwnd, (byte*) lpvDIBits, 1, 0 , lpbih->biWidth ,lpbih->biHeight );   // flip Y to match us   7-1-97
      }

      if (!bError)
          bError = (!WriteFile(hFile, lpvDIBits, dwDIBBitsSize, &dwBytesWritten, NULL)
                    || dwBytesWritten != dwDIBBitsSize);

      CloseHandle(hFile);

      GlobalUnlock(hDIBSave);
      GlobalFree(hDIBSave);

      if(!bError)  ShowStatus(hwnd,"BMP Saved Successfully");
   }

	 return (WORD)(bError ? ERR_SAVE : 0);
}

//----------------------------------------------------------------------//
//                view an image
// Just used to pass a valid filename to the function ReadImage
//----------------------------------------------------------------------//

                                  // 2-10-97 bk Free Header info.
void ViewImage (HWND hwnd)
{
static char  tempname[260] = {0}; // keep last name read here

  GetWadName(hwnd,"Open Image",tempname, 256, IDS_FILTERSTRING, PatchDir );

  RemoveDIBSection();

  return;

  //MessageBox(hwnd, (char *)&tempname, "You loaded file : ", MB_OK); 2-1-97 bk
  // Viewer ?
}

//----------------------------------------------------------------------//
//                view WAVE
//----------------------------------------------------------------------//

void ViewWAVE(HWND hwnd)
{
static char  tempname[260] = {0}; // keep last name read here

 GetWadName(hwnd,"Open Sound",tempname, 256, IDS_FILTERWAV, PatchDir );

 return;
}

//----------------------------------------------------------------------//
//                view MIDI
//----------------------------------------------------------------------//

void ViewMIDI(HWND hwnd)
{
static char  tempname[260] = {0}; // keep last name read here

 GetWadName(hwnd,"Open Midi",tempname, 256, IDS_FILTERMID, PatchDir );

 return;
}

//----------------------------------------------------------------------//
//                view CD
//----------------------------------------------------------------------//
void ViewCD(HWND hwnd)
{
  char CDPlayerExe[260];

  GetWindowsDirectory(CDPlayerExe, sizeof(CDPlayerExe) );      // windows directory
  strcat(CDPlayerExe,"\\CDPLAYER.EXE");                        // should be here

  SpawnProcess(hwnd,CurrDir,CDPlayerExe,SW_SHOWDEFAULT,NOWAIT);//SPAWNGUI- NOWAIT

}

/*
//////////////////////////////////////////////////////////////////////////////
//  experimental play CD audio
//////////////////////////////////////////////////////////////////////////////
static  int  iNumTracks;        // temp hack

#pragma argsused
BOOL CALLBACK CDTrackDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{

  switch(msg)
  { // start of dialog
	 case WM_INITDIALOG:
      char CDText[6];
      if(bCDTrack <=0) bCDTrack = 1;
      itoa(bCDTrack,CDText,10);
	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETRANGE, (WPARAM)0, (LPARAM) MAKELONG((short) iNumTracks, (short) 1));
	   SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETPOS,   (WPARAM)0, (LPARAM) MAKELONG((short) bCDTrack,           0));
      //NotifyMsg(NULL,CDText);
      Edit_SetText(GetDlgItem(hDlg, edt1), CDText);	// set our edit controls
      SetFocus(GetDlgItem(hDlg, edt1));
   	return FALSE;

   // scrolling
	case WM_VSCROLL:
      bCDTrack = (int2)SendMessage(GetDlgItem(hDlg, IDD_UPDOWNX), UDM_GETPOS, 0, 0);
      break;

   case WM_RBUTTONUP:
     PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
     break;
   //   6-1-97
   case WM_LBUTTONUP:
     PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
     break;

   case WM_COMMAND:
	  //HANDLE_WM_COMMAND(hDlg, wParam, lParam, Color_OnCommand);
	  switch LOWORD(wParam)
     {
        case edt1:              // tracksetting
          switch HIWORD(wParam) // the message
          {
             case EN_CHANGE:
               bCDTrack = (int2)GetDlgItemInt(hDlg, edt1, NULL, TRUE); // New cd Track

	            SendDlgItemMessage(hDlg, IDD_UPDOWNX, UDM_SETPOS, (WPARAM)0, (LPARAM) MAKELONG((short) bCDTrack, 0));
               break;
          }
          break;
        case IDOK:
		  	 EndDialog(hDlg, TRUE);
		    return TRUE;

        case IDCANCEL:
		    EndDialog(hDlg, FALSE);
		    return FALSE;
     }
     break;

   //case WM_NOTIFY:
   //  return HANDLE_WM_NOTIFY(hDlg, wParam, lParam, CDTrack_OnNotify);

   default:
     return FALSE;

  }
  return TRUE;
}


//----------------------------------------------------------------------//
//                view CD
//----------------------------------------------------------------------//
void ViewCD(HWND hwnd)
{

  SpawnProcess(hwnd,CurrDir,"C:\\win95\\CDPLAYER.EXE",SW_SHOWDEFAULT,SPAWNGUI);//NOWAIT);

}

DWORD playCDTrack(HWND hWndNotify, BYTE bTrack);

void ViewCDx(HWND hwnd)
{
//DLG2OPTS d2;								// our structure for passing info to dialogs

// * Uses the MCI_STATUS command to get and
// * display the starting times for the tracks
// * on a compact disc. Returns 0L on success;
// * otherwise, it returns an MCI error code.

//DWORD getCDTrackStartTimes(void)
//{
    int     i;
    DWORD   dwReturn;
    DWORD  *pMem;
    char    szTempString[64];
    char    szTimeString[1048] = "\0"; // enough for 40 tracks

    MCI_OPEN_PARMS   mciOpenParms;
    MCI_SET_PARMS    mciSetParms;
    MCI_STATUS_PARMS mciStatusParms;

    int2    minutes;
    int2    seconds;
    //int2    frames;
    int4    currentTime;
    int4    previousTime=0;
    int4    duration;
    int4    totalTime = 0;

    // Open the compact disc device by specifying the device name.

    mciOpenParms.lpstrDeviceType = "cdaudio";
    dwReturn = mciSendCommand(NULL, MCI_OPEN,
                                    MCI_OPEN_TYPE,// |MCI_OPEN_SHAREABLE,
                                    (DWORD)(LPVOID) &mciOpenParms);

    if (dwReturn)
    {
        // * Failed to open device.
        // * Don't have to close it; just return error.
        NotifyMsg(hwnd,"CD is in use -or-\n\n\No CD audio device found? (%d)",dwReturn);
        return;// dwReturn;
    }

    // Device opened successfully. Get the device ID.

    wCDDeviceID = mciOpenParms.wDeviceID;

    // Set the time format to minute/second/frame format.
    mciSetParms.dwTimeFormat = MCI_FORMAT_MSF;
    dwReturn = mciSendCommand(wCDDeviceID, MCI_SET,
                                         MCI_SET_TIME_FORMAT,
                                        (DWORD)(LPVOID) &mciSetParms);
    if (dwReturn)
    {
        CloseCD();
        NotifyMsg(hwnd,"CD audio set format failed? (%d)",dwReturn);
        return;// dwReturn;
    }

    //  Get the number of tracks; limit to number
    //  we can display (20).

    mciStatusParms.dwItem = MCI_STATUS_NUMBER_OF_TRACKS;
    dwReturn = mciSendCommand(wCDDeviceID, MCI_STATUS,
                                           MCI_STATUS_ITEM,
                                          (DWORD)(LPVOID) &mciStatusParms);
    if (dwReturn)
    {
        CloseCD();
        NotifyMsg(hwnd,"CD audio track status failed? (%d)",dwReturn);
        return;// dwReturn;
    }

    iNumTracks = mciStatusParms.dwReturn;
    iNumTracks = min(iNumTracks, 40);

    //  Allocate memory to hold starting positions.
    pMem = (DWORD *)LocalAlloc(LPTR, iNumTracks * sizeof(DWORD));

    if (pMem == NULL)
    {
        CloseCD();
        NotifyMsg(hwnd,"CD audio track allocate failed");
        return;// 0xFFFFFFFF;
    }


    // * For each track, get and save the starting position and
    // * build a string containing starting positions.

    for(i = 1; i <= iNumTracks; i++)
    {
        mciStatusParms.dwItem = MCI_STATUS_POSITION;
        mciStatusParms.dwTrack = i;
        dwReturn = mciSendCommand(wCDDeviceID, MCI_STATUS,
                                               MCI_STATUS_ITEM | MCI_TRACK,
                                              (DWORD)(LPVOID) &mciStatusParms);
        if (dwReturn)
        {   CloseCD();
            return;
        }

        pMem[i-1] = mciStatusParms.dwReturn;
        minutes   = MCI_MSF_MINUTE(pMem[i-1]);
        seconds   = MCI_MSF_SECOND(pMem[i-1]);
        //frame     = MCI_MSF_FRAME (pMem[i-1]);

        currentTime  = minutes*60+seconds;
        duration     = currentTime - previousTime;
        previousTime = currentTime;

        if (i == 1 ) continue;

        totalTime   += duration;

        minutes      = duration/60;
        seconds      = duration -(minutes*60);

        wsprintf(szTempString, "Track%2d - %02d:%02d\n",i-1,minutes,seconds);  // length = 20

        //wsprintf(szTempString, "T%2d - %02d:%02d:%02d\n",  // length = 20
        //                               i,
        //                               MCI_MSF_MINUTE(pMem[i-1]),
        //                               MCI_MSF_SECOND(pMem[i-1]),
        //                               MCI_MSF_FRAME (pMem[i-1]));

        //mciStatusParms.dwItem = MCI_INFO;
        //mciStatusParms.dwTrack = i;
        //dwReturn = mciSendCommand(wCDDeviceID, MCI_STATUS,
        //                                       MCI_STATUS_ITEM | MCI_TRACK,
        //                                      (DWORD)(LPVOID) &mciStatusParms);

        lstrcat(szTimeString, szTempString);
    }


    minutes      = totalTime/60;
    seconds      = totalTime -(minutes*60);
    wsprintf(szTempString, "\nTotal time - %02d:%02d\n",minutes,seconds);  // length = 20
    lstrcat(szTimeString, szTempString);
    // Use MessageBox to display starting times.

    MessageBox(gd.hMainWindow, szTimeString, "Track Starting Position",
                                        MB_ICONINFORMATION);

    // Free memory and close the device. //
    LocalFree((HANDLE) pMem);
    //if (dwReturn = mciSendCommand(wDeviceID,
    //        MCI_CLOSE, 0, NULL)) {
    //        return dwReturn;
    //}

    //return 0;
//}

   for (;;)
   {
       //d2.high1 = 4096;
       //d2.low1  = 1;
       //d2.i1    = bCDTrack;
       //strcpy(d2.sz4, "Select CD track number");

       //if( DialogBoxParam( gd.hInst, MAKEINTRESOURCE( Input1 ), hwnd,
   	 //                 Input1NumberDlgProc, (LPARAM)(LPDLG2OPTS)&d2 ) )

       if (DialogBoxParam( gd.hInst, "CD_Track_Select", hwnd, CDTrackDlgProc, 1))
       {
           //bCDTrack = d2.i1;

           if (playCDTrack(hwnd, bCDTrack))
           {   NotifyMsg(hwnd,"Track not found\n\nAudio CD not mounted?");
           }
       }
       else
       {   break;
       }
   }

   return;
} */

//---------------------------------------------------------------------------//
//  Returns a Handle to the Bitmap read in. Must use the files palette created
//  hViewPalette and PaintBitmap to show on the DC
//  if Translate, translate existing pcx palette to the game palette
//  if NOT, use the PCX palette to show the pic
//
// 1-10-97 bk  Reads and Previews BMP's now
// 2-20-97 Put in Translate
//
// NOTE: Calling Proc MUST free the pcxpalettebufr (if used)!!
//---------------------------------------------------------------------------
                           // PCX,BMP   // get palette from pcxpal and match to gamepal
#pragma argsused
HBITMAP ReadImage (HWND hwnd, int2  TypeOfImage,
                              Bool  Translate,
                              int2  imagemagnify,
                              byte **pcxpalettebufr,
                              byte  *gamepalette,
                              char  *userfilename,
                              Bool   openmust)
{

    FILE *FilePCX;

    int2  imagcolor;
    byte *ImageBufr;
    byte *WrkPixelPtr;
    char  msg[100];

    int2  num_bytes;
    int4  count;
    int2  data;

    int2  Xactual,Yactual;
    int2  Xactualadjust;
    int4  Yheight;             //
    int4  Xwidth ;             //

    int4  m,k,l,x,y,n;         // was 2

    int4  RawImageSize;        // raw image size (expanded)
    byte *RawImageBufr;        // contains 1st pass of image (as is)

    int4  datasize;            // size of file
    byte *DataBufr;            // contains 1st pass of image (as is)
    byte *PaletteBufr= NULL;   //   5-1-97

    PCXHeader image;

    HDC hdc;

//--------------------------------------------------------------------------

    ImageType = INVALIDIMAGE;

    if (TypeOfImage == BMPIMAGE)
    {
       return LoadDibSection(hwnd, userfilename,imagemagnify,Translate);  //gives hBitmap
    }

    if (TypeOfImage != PCXIMAGE) return NULL;//   5-1-97 cosmetic only keep cleaner indent

    // PCX left

    FilePCX = fopen(userfilename, "rb");        // and open
    if (!FilePCX) return NULL;

    datasize =  SizeOfFile(FilePCX,SEEK_SET);     // this is size sets to beginning   5-1-97
    if (datasize <= 130)                          // too small
    {   fclose(FilePCX);                          // close
        HelpMsg(hwnd,CENTER|HELPINFO, "Size",
                                      "Size is too small?",
                                      NULL);
        return NULL;
    }
    // load the header

    fread( &image,128, 1, FilePCX);              // read header
    fclose(FilePCX);                             // close now in case of error

    image.xwidth = image.xend -image.xstart +1;
    image.yheight= image.yend -image.ystart +1;

    // check for legitimate image------------

    if (image.xstart < 0
    || image.xend   < 0
    || image.ystart < 0
    || image.yend   < 0
    || image.manufacturer   != 0x0a
    || image.version        != 0x05
    || image.encoding       != 0x01
    || image.bits_per_pixel != 0x08  // must be 8
    || (datasize <=  sizeof (struct PCXHeaderType) -768 ) )  // not big enough
    {

        if (openmust == OPENMISS)     // ok to be bad
        {   return NULL;
        }

        wsprintf(msg,"size=%d X=%hd,%hd Y=%hd,%hd  control=%hd,%hd,%hd,%hd",
                   datasize,
                   image.xstart,image.xend,
                   image.ystart,image.yend,
                   (int2)image.manufacturer  ,    //!= 0x0a
                   (int2)image.version       ,    //!= 0x05
                   (int2)image.encoding      ,    //!= 0x01
                   (int2)image.bits_per_pixel);   // = 0x08

        HelpMsg(hwnd,CENTER|HELPHELP, "Not Supported",
                                      msg,
                                      "Image size or type not supported (must be 10,5,1,8)!",
                                      NULL);
        return NULL;
    }


    if (image.xwidth < image.bytes_per_line)
    {   Xactual = image.bytes_per_line;
    }
    else
    {   Xactual = image.xwidth ;
    }

    Yactual       = image.yheight;

    RawImageSize  = Xactual * Yactual;                  // data bytes required
    RawImageBufr  = (byte *)malloc(RawImageSize) ;      // raw buffer

    if (!RawImageBufr)                                  // not enough
    {   OutOfMemoryMsg(hwnd);
        return NULL;
    }

    // set to dimensions we need
    image.xstart= 0;
    image.ystart= 0;
    Xactual       = image.xwidth  * imagemagnify;
    Yactual       = image.yheight * imagemagnify;
    Xactualadjust = ( Xactual + 3 ) & ~3;              //DWORD aligned

    DataBufr = (byte *)malloc(datasize);               // buffer for data

    //if(PaletteBufr)                  // Free our previous memory for Palette
    //{   FreeMemory(PaletteBufr);
    //    PaletteBufr  = NULL;
    //}
    //PaletteBufr    = (byte *)malloc(768);    // palette in PCX

    //   3-1-97 redid
    if (pcxpalettebufr)                                 // user wants palette
    {   PaletteBufr = *pcxpalettebufr;                  // get prior if any
    }

    if (!PaletteBufr)                                   // no buffer yet
    {   PaletteBufr    = (byte *)malloc(768);           // palette in PCX
        if (pcxpalettebufr)                             // user wants it
        {   *pcxpalettebufr = PaletteBufr;              // return to calling proc so
        }
    }

    if (!DataBufr || (pcxpalettebufr && !PaletteBufr ) )
    {   FreeMemory( RawImageBufr);
        FreeMemory( PaletteBufr);
        FreeMemory( DataBufr);
        //PaletteBufr = NULL;

        OutOfMemoryMsg(hwnd);
        return NULL;
    }

    // read the data--------------

    FilePCX = FileOpen(userfilename, "rb",OPENMUST);        // open again

    fseek( FilePCX, 128, SEEK_SET);          // +128
    fread( DataBufr,datasize, 1, FilePCX);   // read data

    if (PaletteBufr)
    {
        fseek( FilePCX,   -768, SEEK_END);   // -768
        fread( PaletteBufr,768, 1, FilePCX); // read data

        SetPCXPalette(hwnd, PaletteBufr);   // Initialize hViewPalette and grgbPCX
    }

    fclose(FilePCX);                         // close

    // build the original image in RawImageBufr

   count  = 0;
   WrkPixelPtr = DataBufr;

   while (count <  RawImageSize )
   {  // get first piece of data
      data = *WrkPixelPtr++;

      // is this an RLE?
      if (data >= 192 && data <= 256)
      {  num_bytes = data - 192;                 // how many bytes in run (replicate)
         data = *WrkPixelPtr++;                  // get the data
         // replicate the data in the buffer num_bytes times
         while(num_bytes-- > 0)
	      {  RawImageBufr[count++] = data;
	         if (count >= RawImageSize) break;    // too big
	      }
      }
      else
      {  // copy the actual data into the buffer at next loc
	      RawImageBufr[count++] = data;
      }
   }

   //------------------------------------------------------------------------
   // if user file - find the palette
   //------------------------------------------------------------------------

   if (PaletteBufr)
   {   ColorPaletteMatch(PaletteBufr,gamepalette,ColorTranslation);  // convert, store in ColorTrans
   }
   else
   {   Translate = FALSE;                     // overrride any user setting
   }

   // Set up a BITMAPINFO structure for use with CreateDIBSection()
   ZeroMemory(&ViewBi, sizeof(BITMAPINFO));
   ViewBi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
   ViewBi.bmiHeader.biPlanes     = 1;
   ViewBi.bmiHeader.biBitCount   = 8;
   ViewBi.bmiHeader.biWidth      =  Xactual;
   ViewBi.bmiHeader.biHeight     = -Yactual;
	ViewBi.bmiHeader.biSizeImage  =  Xactualadjust* Yactual; //  5-1-97Xactual
   ViewBi.bmiHeader.biCompression= BI_RGB;

   // Create a DC to hold our DIBSECTION
   hdc = GetDC(hwnd);                           // use DIB_PAL instead of RGB
   hViewBitmap = (HBITMAP)CreateDIBSection(hdc, &ViewBi, DIB_PAL_COLORS, (VOID **)&lpViewBits, NULL, 0L);
   ReleaseDC(hwnd, hdc);

   if (!hViewBitmap || !lpViewBits)                            // not enough memory
   {    OutOfMemoryMsg(hwnd);
        FreeMemory( RawImageBufr);
        FreeMemory( DataBufr);
        FreeMemory( PaletteBufr);               //   3-1-97
        //PaletteBufr = NULL;                     //
        if (pcxpalettebufr)                     //
        {   *pcxpalettebufr=NULL;               //
        }                                       //
        if (hViewBitmap)
        {   DeleteObject(hViewBitmap);
            hViewBitmap = NULL;
        }
        lpViewBits   = NULL;
        return NULL;
   }

   ImageBufr = (byte *)&*lpViewBits;	// Grab pointer to the bits of our DIBSection

   // now expand from RawBuffer into real one

   Yheight = image.yheight;   // make int4
   Xwidth  = image.xwidth;    // make int4

   for (y=0, k=0; k< Yheight; k++, y+=imagemagnify)                     // Y
   {
       WrkPixelPtr = RawImageBufr + k*image.bytes_per_line; //Xwidth  raw data

       for (x=0, l=0; l < Xwidth; l++, x+=imagemagnify,WrkPixelPtr+=1)  // X
       {
          if (Translate)
          {   imagcolor    = *WrkPixelPtr;
              *WrkPixelPtr = ColorTranslation[imagcolor];
          }

	       for (m=0; m < imagemagnify; m++)            //y expand
	       {  for (n=0; n< imagemagnify; n++)          //x expand
		       { ImageBufr[ (y+m)*Xactualadjust + (x+n)]=*WrkPixelPtr;
		       }
	       }
       }    // end x
   }   //end y

   //--------------------------------------------------------------------------

   //   3-1-97 free if only used here
   if (!pcxpalettebufr)
   {   FreeMemory(PaletteBufr);
       //PaletteBufr = NULL;                      //
   }

   FreeMemory( RawImageBufr);
   FreeMemory( DataBufr);

   ImageType = PCXIMAGE;

   //if(PaletteBufr)                  // Free our previous memory for Palette
   //{  FreeMemory(PaletteBufr);
   //    PaletteBufr  = NULL;
   //}
   return hViewBitmap;

   // NOTE: Calling Proc MUST free the pcxpalettebufr !!

}

////////////////////////////////////////////////////////////////////////////////
//
//  FUNCTION: CreateDIBPalette(HDIB)
//
//  PURPOSE: Creates a palette from a DIB by allocating memory for the
//           logical palette, reading and storing the colors from the DIB's
//           color table into the logical palette, creating a palette from
//           this logical palette, and then returning the palette's handle.
//           This allows the DIB to be displayed using the best possible
//           colors (important for DIBs with 256 or more colors).
//
//   ????  > 256 was not in table ???   7-1-97 needs more code
//
////////////////////////////////////////////////////////////////////////////////
static HPALETTE CreateDIBPalette(HWND hwnd,HDIB hDIB)
{
    LPLOGPALETTE  lpPal;          // pointer to a logical palette
    HANDLE        hLogPal;        // handle to a logical palette
    HPALETTE      hPal;           // handle to a palette
	 int4          i;              // loop index
	 int4          wNumColors;     // number of colors in color table
	 LPSTR         lpbih;          // pointer to packed-DIB
	 LPBITMAPINFO  lpbi;           // pointer to BITMAPINFO structure (Win3.0)
    char          msg1[80];
//  static byte * PalBufr = NULL; // Copy of Palette for Translate
//  static  DWORD size;
//  byte *        bufr;           // Address of Dib Bits

//==============================================================================

	 // if handle to DIB is invalid, return NULL
	 if (!hDIB)  return NULL;

    lpbih = (char *)GlobalLock(hDIB);// lock DIB memory block and get a pointer to it
    lpbi  = (LPBITMAPINFO)lpbih;     // get pointer to BITMAPINFO (Win 3.0)

    // get the number of colors in the DIB
    wNumColors = DIBNumColors(lpbih);
    if ( ( (LPBITMAPINFOHEADER)lpbih)->biBitCount!=8 )
    {
    //if (wNumColors <= 0                //   5-1-97  ? bad
    //||  wNumColors > 256)              //   7-1-97 256
    //{
        GlobalUnlock(hDIB);
        if (Pasting)
        {
            sprintf(msg1,"%hd bit paste not supported (8 bit only)",( (LPBITMAPINFOHEADER)lpbih)->biBitCount);
            HelpMsg(hwnd,CENTER|HELPHELP, "",
                                          msg1,
                                          NULL);
        }
        //if (wNumColors <  0)
        //{   ErrorMsg(NULL,"Number of colors < 0 not supported");
        //}
        //else
        //{   ErrorMsg(NULL,"%d colors not supported",wNumColors);  //256
        //}
        return NULL;
    }

    // allocate memory block for logical palette
    hLogPal = GlobalAlloc(GHND, sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) * wNumColors);

    // if not enough memory, clean up and return NULL
    if (!hLogPal)
    {   GlobalUnlock(hDIB);
        return NULL;
    }

    lpPal = (LPLOGPALETTE)GlobalLock(hLogPal);  // lock memory block and get pointer to it
    lpPal->palVersion    = PALVERSION;          // set version
	 lpPal->palNumEntries = wNumColors;          // set number of palette entries

    // this is palette of BMP
    
    memset(PaletteBufr,0,768);                  // if short
    for (i = 0; i < wNumColors; i++)
    {
         lpPal->palPalEntry[i].peRed   = lpbi->bmiColors[i].rgbRed;
         lpPal->palPalEntry[i].peGreen = lpbi->bmiColors[i].rgbGreen;
         lpPal->palPalEntry[i].peBlue  = lpbi->bmiColors[i].rgbBlue;
         lpPal->palPalEntry[i].peFlags = 0;
         if (i< 256)  //   5-1-97
         {  PaletteBufr[i*3+0] = lpbi->bmiColors[i].rgbRed;
            PaletteBufr[i*3+1] = lpbi->bmiColors[i].rgbGreen;
            PaletteBufr[i*3+2] = lpbi->bmiColors[i].rgbBlue;
         }
    }

    ColorPaletteMatch(PaletteBufr,NULL,ColorTranslation);  // convert, store in ColorTrans   5-1-97 for pastetranslate

// this screws up color matching I had in preview   5-1-97--
// since this changes the colors again
// made a new palette but preview not as good as before for BMP, now same
// as PCX, so that's ok. Need to find out how to use windows conversion process      <<<<<<<<<need for drawing
/*
        // 3-31-97 bk  HAD to do this for drawing !
        if(((LPBITMAPINFOHEADER)lpbih)->biBitCount == 8) // only 256 color DIBS
        {
           // Now make our Translation Palette according to the Game palette
           // Rather than change the data bits as in the PCX Import, since we
           // Don't really import like that ( PCX copies the bits from the raw
           // buffer one by one into the dib bits ) we'll just change the palette
           // here and then update the dib bits manually as we load.

           PalBufr    = (byte *)malloc(768);    // palette for Translation
            //   3-1-97
            if (!PalBufr)
            {   GlobalUnlock(hDIB);
                GlobalUnlock(hLogPal);
                return NULL;
            }
            // test preset to doom so if less than 256 have some defaults   5-1-97
          //  for (i = 0; i < 256; i++)
          //  {
          //     PalBufr[i*3+0] = dRGBquad[i].rgbRed;
          //     PalBufr[i*3+1] = dRGBquad[i].rgbGreen;
          //     PalBufr[i*3+2] = dRGBquad[i].rgbBlue;
          //  }

            for(i = 0; i < 256; i ++)
            //for(i = 0; i < wNumColors; i ++)
            {
               PalBufr[i*3+0] = lpPal->palPalEntry[i].peRed;
               PalBufr[i*3+1] = lpPal->palPalEntry[i].peGreen;
               PalBufr[i*3+2] = lpPal->palPalEntry[i].peBlue;
            }

            ColorPaletteMatch(PalBufr, NULL,ColorTranslation);  // convert, store in ColorTrans

            // Calculate size of header and color table (if any)
            //width  = ((LPBITMAPINFOHEADER)lpbih)->biWidth;
            //height = abs(((LPBITMAPINFOHEADER)lpbih)->biHeight);
            //offset = ((LPBITMAPINFOHEADER)lpbih)->biHeight + (WORD)(256 * sizeof(RGBQUAD)) ;

            size   = ((LPBITMAPINFOHEADER)lpbih)->biSizeImage;

            bufr = (byte *) &*lpViewBits;

            for(i = 0; i < size; i++)
            {
		          bufr[i]= ColorTranslation[bufr[i]];
            }

            FreeMemory(PalBufr);
            PalBufr  = NULL;
            // set palette to DOOM instead   5-1-97
            for (i = 0; i < 256; i++)
            //for(i = 0; i < wNumColors; i ++)
            {
               lpPal->palPalEntry[i].peRed   = dRGBquad[i].rgbRed;
               lpPal->palPalEntry[i].peGreen = dRGBquad[i].rgbGreen;
               lpPal->palPalEntry[i].peBlue  = dRGBquad[i].rgbBlue;
               //lpPal->palPalEntry[i].peFlags = 0;
            }
        }
*/
//---------------------------------------------------------

    // create the palette and get handle to it
    hPal = CreatePalette(lpPal);

    // if error getting handle to palette, clean up and return NULL
    if (!hPal)
    {
         GlobalUnlock(hLogPal);
         GlobalFree(hLogPal);
         GlobalUnlock(hDIB);   //   5-1-97
         return NULL;
    }

    // clean up
    GlobalUnlock(hLogPal);
    GlobalFree(hLogPal);
    GlobalUnlock(hDIB);

    // return handle to DIB's palette
    return hPal;
}

////////////////////////////////////////////////////////////////////////////////
//    Used by BMP preview
//    Painting routine for a DDB.  Calls BitBlt() or StretchBlt() to paint
//    the DDB.  The DDB is output to the specified DC, at the coordinates
//    given in lpDCRect.  The area of the DDB to be output is given by
//    lpDDBRect.  The specified palette is used.
//
//  NOTE:  Must have copy of dib bits palette index's in dRGBquad as well now !
//
// Same as DrawBitmap but does Stretch as well
////////////////////////////////////////////////////////////////////////////////

BOOL PaintBitmap(HDC hDC, LPRECT lpDCRect, HBITMAP hDDB,  LPRECT lpDDBRect, HPALETTE hPal)
{
    HDC         hMemDC;            // Handle to memory DC
    HBITMAP     hOldBitmap;        // Handle to previous bitmap
    HPALETTE    hOldPal1 = NULL;   // Handle to previous palette
    HPALETTE    hOldPal2 = NULL;   // Handle to previous palette
	 BOOL        fSuccess;          // Success/fail flag

    float ratio;                   //   3-1-97
    int   Height,Width;
    int   SourceHeight;
    int   SourceWidth;
    int   TargetHeight;
    int   TargetWidth;

//    byte *PaletteBufr=NULL;
//    Bool Translate = FALSE;
//=============================================================================
    // Create a memory DC
    hMemDC = CreateCompatibleDC(hDC);
    if (!hMemDC) return FALSE;

    // If we have a palette, select and realize it

    if (hPal)
    {   hOldPal1 = SelectPalette(hDC,    hPal, TRUE); //FALSE    TRUE matches colors for us
        RealizePalette(hDC);
        hOldPal2 = SelectPalette(hMemDC, hPal, TRUE); //FALSE
    }

    // Select bitmap into the memory DC

    hOldBitmap = SelectBitmap(hMemDC, hDDB);
    // new ++ by brad   5-1-97 see other stuff in CreateDIBPalette
    // this+other change causes color matching to screw up on preview
    // sort of fixed it                              5-1-97
    // revised other parts, but can't get 16 to paste right
    //  SetDIBColorTable(hMemDC, 0, 256, dRGBquad);	     // Palette indexes   5-1-97

    SetStretchBltMode(hDC, COLORONCOLOR);            // stretching mode best for color pictures

    // Determine whether to call StretchBlt() or BitBlt()
    // Don't require MagnifyGraphicTexture then

    SourceHeight = RECTHEIGHT(lpDDBRect);
    SourceWidth  = RECTWIDTH(lpDDBRect);
    TargetHeight = RECTHEIGHT(lpDCRect);
    TargetWidth  = RECTWIDTH(lpDCRect);

    // get larger of source dimensions and scale to max fit and not distort

    if (SourceHeight > SourceWidth)     // height is bigger
    {   ratio =  (float)TargetWidth/(float)SourceHeight;
    }
    else
    {   ratio = (float)TargetWidth/(float)SourceWidth;
    }

    Height = ratio * SourceHeight;
    Width  = ratio * SourceWidth;

    if (Height > TargetHeight)
    {   ratio = (float) TargetHeight/(float)Height;
        Height = TargetHeight;
        Width  *=ratio;
    }

    if (Width > TargetWidth)
    {   ratio  = (float) TargetWidth/(float)Width;
        Width  = TargetWidth;
        Height*= ratio;
    }

    fSuccess = StretchBlt(hDC, lpDCRect->left, lpDCRect->top,
                                Width, Height,
                                hMemDC, lpDDBRect->left, lpDDBRect->top,
                                SourceWidth, SourceHeight, SRCCOPY);

    // Clean up
    SelectBitmap(hMemDC, hOldBitmap);

    if (hOldPal1)  SelectPalette(hDC, hOldPal1, FALSE);

    if (hOldPal2)  SelectPalette(hMemDC, hOldPal2, FALSE);

    DeleteDC(hMemDC);

    //    if (PaletteBufr) FreeMemory(PaletteBufr);  //   3-1-97 temp

    // Return with success/fail flag
    return fSuccess;
}


//
//-------------------------------------------------------------------------
//  Show the user a dib error occured
//-------------------------------------------------------------------------
//

void DIBError(int nErrNo)
{
    char szBuffer[40];

    if ((nErrNo < ERR_MIN) || (nErrNo >= ERR_MAX))
        nErrNo = ERR_UNDEFINEDERROR;

    LoadString(gd.hInst, nErrNo, szBuffer, sizeof(szBuffer));
    MessageBox(NULL, szBuffer, szLoadString(IDS_APPNAME), MB_OK | MB_ICONHAND);
}

////////////////////////////////////////////////////////////////////////////////
//
//	FUNCTION:	DrawBitmap      (used by tips)
//
//	PURPOSE:	Draws a bitmap on the supplied DC using supplied Raster Operation
//
// Same as PaintBitmap but does not do Stretch
////////////////////////////////////////////////////////////////////////////////
void DrawBitmap(HDC hdc, HBITMAP Bitmap, int x, int y, DWORD dwRop)
{
 BITMAP bm;

 if (Bitmap)
 {
     HDC      BitmapDC = CreateCompatibleDC(hdc);
     HBITMAP  OldBitmap= SelectBitmap(BitmapDC, Bitmap);
     GetObject(Bitmap, sizeof(BITMAP), &bm);

     BitBlt(hdc, x, y, bm.bmWidth,abs(bm.bmHeight), BitmapDC, 0, 0, dwRop);
     // just playing for benchmark to see if this makes a difference
     //SetStretchBltMode(hdc,HALFTONE);                  //halftone required for color pictures COLORONCOLOR
     //StretchBlt(hdc, x, y, bm.bmWidth,abs(bm.bmHeight),
     //      BitmapDC, 0, 0,bm.bmWidth,abs(bm.bmHeight), dwRop);

     GdiFlush();
     SelectBitmap(BitmapDC, OldBitmap);
     DeleteDC(BitmapDC);
 }
}

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------

static int4 DIBNumColors(LPSTR lpDIB)
{
    WORD wBitCount;  // DIB bit count

    DWORD dwClrUsed;

    dwClrUsed = ( (LPBITMAPINFOHEADER)lpDIB)->biClrUsed;

    if (dwClrUsed) return (WORD)dwClrUsed;

    // Calculate the number of colors in the color table based on
    // the number of bits per pixel for the DIB.

    wBitCount = ((LPBITMAPINFOHEADER)lpDIB)->biBitCount;

    // return number of colors based on bits per pixel
    return (1<<wBitCount);
    /*
    switch (wBitCount)
    {
        case 1:
            return 2;
        case 4:
            return 16;
        case 8:
            return 256;

        case 16:                    // added for error detection
            return 65536;
        case 24:
            return 1677216;
        case 32:
            return 1677216;
        default:
            return 0;
    } */
}

//
//-------------------------------------------------------------------------
//  PURPOSE: Removes the current DIB section and resets global variables
//-------------------------------------------------------------------------
//
static void RemoveDIBSection(void)
{
    if (hViewDibInfo)                // typo's   2-10-97 bk
    {   // free old DIB header
        GlobalFree(hViewDibInfo);
        hViewDibInfo = NULL;
    }

    if (hViewBitmap)
    {   // remove existing DIB section
        DeleteBitmap(hViewBitmap);
        hViewBitmap = NULL;
        // bits are gone
        lpViewBits = NULL;
    }


    if (hViewPalette)
    {   // remove existing palette object
        DeletePalette(hViewPalette);
        hViewPalette = NULL;
    }
}


//------------------------------------------------------------------------------
// Make a Palette for a PCX picture and set to global hViewPalette
//------------------------------------------------------------------------------
Bool SetPCXPalette(HWND hwnd, byte * PCXPaletteBufr)
{
   int4 n;
	LPLOGPALETTE   lplp;			// pointer to our Logical Palette
	HANDLE 			hdpal;		// handle for memory allocation


   if(hViewPalette)                     // Get rid of old Palette
   {                                    // PCXPaletteBufr has already been
      DeletePalette(hViewPalette);      // clearing by calling routine
      hViewPalette = NULL;
   }

	hdpal	= GlobalAlloc(GHND, sizeof(LOGPALETTE)+(256*sizeof(PALETTEENTRY)));

   if (hdpal)  								// Got our Handles
   {   lplp = (LPLOGPALETTE)GlobalLock(hdpal);	// Get our Memory Pointers
       lplp->palVersion = PALVERSION;
	    lplp->palNumEntries = 256;
       // Initialize palette entries
		 for (n = 0; n < 256; n++)
	    {                               // supplied buffer
          lplp->palPalEntry[n].peRed   = PCXPaletteBufr[n*3+0];
		    lplp->palPalEntry[n].peGreen = PCXPaletteBufr[n*3+1];
	   	 lplp->palPalEntry[n].peBlue  = PCXPaletteBufr[n*3+2];
		    lplp->palPalEntry[n].peFlags = PC_NOCOLLAPSE;

	     	 // a copy of the PCX Palette for Global DIB access
	    	 //PCXrgbq[n].rgbRed   = lplp->palPalEntry[n].peRed;   //  5-1-97
		    //PCXrgbq[n].rgbGreen = lplp->palPalEntry[n].peGreen;
		    //PCXrgbq[n].rgbBlue  = lplp->palPalEntry[n].peBlue;
		    //PCXrgbq[n].rgbReserved = 0;
       }

		 // Create our PCX Palette and store Globally
   	 hViewPalette = CreatePalette(lplp);   // req'd

  		 GlobalUnlock(hdpal);
		 GlobalFree(hdpal);
       return TRUE;
   }
   else
   {
       OutOfMemoryMsg(hwnd);//NotifyMsg(hwnd, "Not enough memory for PCX palette");
       hViewPalette = NULL;
       return FALSE;
   }

}


////////////////////////       GRAPHIC EDITOR        ///////////////////////////
//                                                                            //
//                                                                            //


//============================    DEFINES     ==================================

#define BUT0        0x0400          // drawing
#define BUT1        0x0401          // convert TEXTURES
#define BUT2        0x0402          // convert FLAT
#define BUT3        0x0403          // open
#define BUT4        0x0404          // export lump
#define BUT5        0x0405          //  ..     texture
#define BUT6        0x0406          //  ..     flat
#define BUT7        0x0407          //  ..     sprites
#define BUT8        0x0408          // help
#define BUT9        0x0409          // exit

#define BUT10       0x040A          // 10- 19  Conversion Buttons
#define BUT11       0x040B
#define BUT12       0x040C
#define BUT13       0x040D
#define BUT14       0x040E
#define BUT15       0x040F
#define BUT16       0x0410          // help
#define BUT17       0x0411          // EXIT
#define BUT18       0x0412
#define BUT19       0x0413
                                    // 20 - 29  Drawing Control ID'S
#define BUT20       0x0414          // IMPORT
#define BUT21       0x0415          // EDIT
#define BUT22       0x0416          // UNDO
#define BUT23       0x0417
#define BUT24       0x0418
#define BUT25       0x0419
#define BUT26       0x041A
#define BUT27       0x041B
#define BUT28       0x041C
//#define BUT29       0x041D

#define BUT30       0x041E
#define BUT31       0x041F
#define BUT32       0x0420
#define BUT33       0x0421
#define BUT34       0x0422        //exit
#define BUT35       0x0423
#define BUT36       0x0424
#define BUT37       0x0425
#define BUT38       0x0426
#define BUT39       0x0427
#define BUT40       0x0428

#define MAIN        0x0001        // Our Modes was EXP
#define CVT         0x0002
#define DRAW        0x0003

//#define TOOLXSTART 50

#define TOOLXSTART ((BARWIDTH - (22*3) ) / 2)
#define TOOLYSTART 100

#define NOTMASTER  0  // DisplayGraphicTexture don't show graphics in master iwad
#define DOMASTER 1    // DisplayGraphicTexture show all master & files

//============================    EXTERNALS     ================================

///////////////    In ColorCvt.cpp  //////////////////////////
//extern int2     TextureConvertCount;
extern char **  TextureConvertList;
extern NameAreaList *listnames;     // patch definition
extern int4     ImageGetSize;
extern int4     TextureDirStart;

//extern byte*  RawImageFlag;       // raw image data to flag where been (STRIFE has dups) (ColorCvt)
//extern byte*  RawImageData;       // raw image data                                      (ColorCvt

extern WadFiles ConvertWad;          // the External File texture directory
extern int2     ConvertMagnify;
static int4     ConvertWadDirStart;  // offset in directory

//------------------- global ---------------------------------------------------

       char    StatMsg[200] = {0};   // also used in convert

// Modeless floating Tool Bar window
static BOOL    CALLBACK ToolDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);

//============================    GLOBALS     ==================================

static int4    Mode      = MAIN;     // Startup mode
static  Bool   TexturesLoaded= FALSE;
static Bool    EditDraw  = FALSE;    // Currently Drawing on Screen
static Bool    FlatLump;             // Drawing on FLAT's ?? TRUE/FALSE   5-1-97 lower case convention upper = defines
static Bool    EditActive= FALSE;    // new 8-1-97 show hEditWindow is active
static Bool    BMPtoPWAD = FALSE;

static HWND    hControl[16] = {0};   // List Box, Buttons etc.  0 - 13 (spares)
static HWND    hClrL;                // Left Button Color
static HWND    hClrT;                // Transparent Color
static HWND    hClrC;                // Current Color under cursor

static Bool    AutoCVT   = TRUE;
static Bool    ConvertOK = FALSE;

static char    ConvertName[10]={0};
static int2    DrawingXsize,DrawingYsize;
static char    DrawingName[16]={0};  // 16 so can get separator names

static int4    EditChange = FALSE;   // if name is being changed
static LRESULT EditNameIndex=1;
static char    EditName[10] = {0};   // EditText for name matching made

// Menu #1  = The Default on init                            4
#define BUTTONTEXTE  10
static char *ButtonTextE[] = {
                              "DRAWing mode",           // BUT0
                              "Convert Textures",       // BUT1
                              "Convert Flats",          // BUT2
                              "OPEN Texture/Flat",      // BUT3

                              "Export Lump",            // BUT4
                              "Export All Textures",    // BUT5
                              "Export All Flats",       // BUT6
                              "Export All Sprites",     // BUT7

                              "H E L P",                // BUT8
                              "Exit"};                  // BUT9

// Menu Color Conversion Buttons                            8
#define BUTTONTEXTC  8   // NOTE if go past 8 make sure list is moved from 9
static char *ButtonTextC[] = {"Convert (if not Auto)",  // BUT10
                              "Save",                   // BUT11
                              "Save as",                // BUT12
                              "Save all",               // BUT13
                              "Auto Convert",           // BUT14 AutoCheckBox
                              "MAIN Buttons",           // BUT15 Main Menu
                              "H E L P",                // BUT16
                              "Exit"};                  // BUT17

// Menu #2 Select Edit Texture & Flats Buttons             6
#define BUTTONTEXTD  9
static char *ButtonTextD[] = {"EDIT",                   // BUT20
                              "Zoom In",                // BUT21
                              "Zoom Out",               // BUT22
                              "BMP/PCX to PWAD",        // BUT23
                              "New Blank Texture",      // BUT24
                              "Change Size",            // BUT25
                              "MAIN Buttons",           // BUT26 3
                              "H E L P",                // BUT27 4
                              "Exit"};                  // BUT28 5

// Menu # 3 Actual Drawing on Texture & Flats Buttons
#define BUTTONTEXTG  11
static char *ButtonTextG[] = {"Import Bmp/Pcx/Wad",     // BUT35                  0
                              "Import Direct",          // BUT36                  1
                              "Save",                   // BUT37                  2
                              "Save as",                // BUT38                  3
                              "UNDO",                   // BUT39                  4
                              "Transparent Color",      // BUT40                  5

                              "Float Tools",            // BUT30 AutoCheckBox     6
                              "Auto Convert BMP",       // BUT31 AutoCheckBox     7
                              "MAIN Buttons",           // BUT32                  8
                              "H E L P",                // BUT33                  9
                              "Select Texture"};        // BUT34                  10

// List Box LineMode Strings and window handle
static char *szLineStr[] = {"Freehand", "Dots", "Lines"};
static HWND hLineMode = NULL;

static HBITMAP ToolBitmapsUp[6] = {0};
static HBITMAP ToolBitmapsDn[6] = {0};
//    0          1          2        3         4    5
// Magnify, Eye Dropper, Eraser, Flood Fill, Pen, Select
static int ToolActive[6] = {0};           // Which Button is DOWN ?  = 1

static char *ToolMsg[] = {"Left Click to Zoom in  -  Right to Zoom out",
                          "Left Click to choose the color under the cursor",
                          "Draw with the Transparent Color",
                          "Left Click to fill all areas with the Drawing Color",
                          "Left Click to Draw in any of 3 MODES",
                          "Hold and Drag the Left Button to Select an Area"};


                 // global for FreeMemory
static int4          DrawListCount= 0;
static char **       DrawList     = NULL;
static NameAreaList* DrawListNames= NULL;          // patches

// Flats
static int4          DrawFListCount=0;
static char **       DrawFList     = NULL;
static NameAreaList* DrawFListNames= NULL;         // flats

//static Bool PasteTranslate     = TRUE;             // to translate past stuff
static char CopyFileName[260]  = {0};              // keep for common paste file

static Bool ToolWindow   = FALSE;//TRUE;
static Bool LineMode     = 1;                       // freehand=1, dots=2, line = 3


//========================    Local Prototypes    ==============================

static Bool LoadDrawOnTextures(HWND hwnd);//, HWND hList);

static void ToggleTool(void);

//============================    END HEADER    ================================



//-------------------------------------------------------------------------
//  Handles drawing the Tool List Box for our Line Modes, could have owner Draw List Box !
//  with different graphics in it !
//
//  Called from GFX_OnLButtonDown  &  Tool_OnLButtonDown, and CreateMenuWindow3
//
//-------------------------------------------------------------------------

static void  DrawLineList(HWND hwnd, Bool On)
{
 int2 i;

  if(On)
  {   if (!hLineMode)        //   6-1-97 safety
      {   hLineMode = CreateWindow("listbox", "Line Mode",
                                   WS_CHILD | WS_VISIBLE | LBS_NOTIFY,
                                  ((BARWIDTH / 2) - 42),
                                   TOOLYSTART + 56,
                                   80, 60,
                                   hwnd,
                                   (HMENU)lst3,          // handle of child id
                                   gd.hInst, NULL);      // handle of app

          for(i = 0; i < 3; i ++)
          {   ListBox_AddString(hLineMode, szLineStr[i]); // Add the line modes
          }
      }
      ListBox_SetCurSel(hLineMode, LineMode -1);  // set for the current mode
  }
  else
  {
      DeePDestroyWindow(hLineMode);
      hLineMode = NULL;
      ToolActive[4] = 0;            // safety
  }
}

//
//-------------------------------------------------------------------------
// Draw the Status Bar at the ==bottom== of the client area
// also used by colorcvt
//-------------------------------------------------------------------------
//
#pragma argsused
void GFXDrawStatus(HWND hwnd)
{
   //RECT        rc;
   HPALETTE    hOldPal;
   HDC         hdc;

//==================================================

   hdc = GetDC(hGFXWindow);//hwnd);

  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);
	//GetClientRect(hGFXWindow, &rc);   // was hwnd
   DrawScreenBox3D(hdc, BARWIDTH, GfxRC.bottom - STATHEIGHT, GfxRC.right -1, GfxRC.bottom -1, gd.hBrushes[GRAY]);

   SetBkMode(hdc, TRANSPARENT);
   SetTextColor(hdc, PALETTEINDEX(GameColors[YELLOW]));   	// Yellowish

   TextOut(hdc, BARWIDTH + 20 , GfxRC.bottom - STATHEIGHT + 2, StatMsg, lstrlen(StatMsg));

   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hGFXWindow, hdc);
}


/////////////////////////////////////////////////////////////
//
//  Draw the 2 Color Box's, Drawing Color and Transparent
//
/////////////////////////////////////////////////////////////

static void GFXDrawColors(HWND hwnd)
{
   RECT        rc;
   HPALETTE    hOldPal;
   HDC         hdc;
   HBRUSH      hBrush;
   HFONT       hOldFont;

   char szTxt[12] = {0};

//==================================================

   // left button color
   hdc = GetDC(hClrL);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);
	GetClientRect(hClrL, &rc);
   hBrush = CreateSolidBrush(PALETTEINDEX(DrawingColor));
   FillRect(hdc, &rc, hBrush);
   DeleteBrush(hBrush);
   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hClrL, hdc);
   // right button color
   hdc = GetDC(hClrT);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);
	GetClientRect(hClrT, &rc);
   hBrush = CreateSolidBrush(PALETTEINDEX(Transparent));
   FillRect(hdc, &rc, hBrush);
   DeleteBrush(hBrush);
   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hClrT, hdc);

   // Now update the TEXT with the INDEX values

   hdc = GetDC(hwnd);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);

   SetTextColor(hdc, PALETTEINDEX(GameColors[BLACK]) );
   SetBkMode(hdc, OPAQUE);
   SetBkColor(hdc, PALETTEINDEX(GameColors[GRAY]));
   hOldFont = SelectFont(hdc, gd.hStockFont);//GetStockFont(ANSI_VAR_FONT) );

   // Left edge of the 2 centered color box's ( 60 pixels wide )

   sprintf(szTxt, "%03d", DrawingColor);
   TextOut(hdc, ((BARWIDTH /3) - (COLORBOXWIDTH+2)+16) , 5, szTxt, lstrlen(szTxt));
   wsprintf(szTxt, "%03d", Transparent);
   TextOut(hdc, ((BARWIDTH*2/3) - (COLORBOXWIDTH+5)+16) , 5, szTxt, lstrlen(szTxt));

   TextOut(hdc, ( (BARWIDTH /3) - (COLORBOXWIDTH+2)+8), 58, "Draw",   4 );
   TextOut(hdc, ( (BARWIDTH*2/3)- (COLORBOXWIDTH+5)  ), 58, "Transp", 6 );
   TextOut(hdc, ( (BARWIDTH)    - (COLORBOXWIDTH+5)  ), 58, "Picture",7 );

   SelectFont(hdc, hOldFont);
   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hwnd, hdc);
}

//============================================================================
// draw a box with current color under cursor
//============================================================================
static CurrentPictureColor;

static void GFXCurrentPictureColor(void)//HWND hwnd)
{
   RECT        rc;
   HPALETTE    hOldPal;
   HDC         hdc;
   HBRUSH      hBrush;
   HFONT       hOldFont;

   char szTxt[12] = {0};

   // right button color
   hdc = GetDC(hClrC);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);
	GetClientRect(hClrC, &rc);
   hBrush = CreateSolidBrush(PALETTEINDEX(CurrentPictureColor));
   FillRect(hdc, &rc, hBrush);

   DeleteBrush(hBrush);
   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hClrC, hdc);

   // Now update the TEXT with the INDEX values

   hdc = GetDC(hGFXWindow);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);

   SetTextColor(hdc, PALETTEINDEX(GameColors[BLACK]) );
   SetBkMode(hdc, OPAQUE);
   SetBkColor(hdc, PALETTEINDEX(GameColors[GRAY]));
   hOldFont = SelectFont(hdc, gd.hStockFont);//GetStockFont(ANSI_VAR_FONT) );

   wsprintf(szTxt, "%03d", CurrentPictureColor);
   TextOut(hdc, ((BARWIDTH) - (COLORBOXWIDTH+5)+8 ) , 5, szTxt, lstrlen(szTxt));

   SelectFont(hdc, hOldFont);
   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hGFXWindow, hdc);
}
//
//-------------------------------------------------------------------------
// Draw the Tools Bar when NOT floating around
//-------------------------------------------------------------------------
//

static void GFXDrawTools(HWND hwnd)
{
   HPALETTE    hOldPal;
   HDC         hdc;
   int2        i;

//----------------------------------------------------------

   if( Mode == DRAW
    && EditDraw
    && !ToolWindow )
   {
      hdc = GetDC(hwnd);
  	   hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	   RealizePalette(hdc);

      for(i = 0; i < 3; i ++)
      {
        if(!ToolActive[i])
           DrawBitmap(hdc, ToolBitmapsUp[i], (i*22) + TOOLXSTART, TOOLYSTART, SRCCOPY);
        else
           DrawBitmap(hdc, ToolBitmapsDn[i], (i*22) + TOOLXSTART, TOOLYSTART, SRCCOPY);
      }

      for(i = 3; i < 6; i ++)
      {
         if(!ToolActive[i])
            DrawBitmap(hdc, ToolBitmapsUp[i], ((i-3)*22) + TOOLXSTART, TOOLYSTART + 23, SRCCOPY);
         else
            DrawBitmap(hdc, ToolBitmapsDn[i], ((i-3)*22) + TOOLXSTART, TOOLYSTART + 23, SRCCOPY);
      }

      SelectPalette(hdc, hOldPal, FALSE);
      ReleaseDC(hwnd, hdc);
   }

}

//
//-------------------------------------------------------------------------
// Draw the Left Hand Side Bar and associated windows (hwnd always GFXWindow)
// call left with hwnd only for documentation purposes (easier to follow code)
//-------------------------------------------------------------------------
//
#pragma argsused
static void GFXLeftDrawBar(HWND hwnd)
{
   RECT        rc;
   HPALETTE    hOldPal;
   HDC         hdc;

//==================================================

   hdc     = GetDC(hGFXWindow);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);

   // clear/draw the left box

   DrawScreenBox3D(hdc, 0, 0, BARWIDTH -1, GfxRC.bottom -1, gd.hBrushes[GRAY]);
   rc.top   = 0;
   rc.left  = 0;
   rc.right = BARWIDTH-1;
   rc.bottom= GfxRC.bottom -1;
   ValidateRect(hGFXWindow, &rc);

   RefreshWindowButtons();           // Update window buttons

   if(Mode == DRAW && EditDraw)      // Drawing Active
   {  GFXDrawColors(hGFXWindow);     // Update the Top Color Box's
      GFXDrawTools(hGFXWindow);      // only draws if NOT Floating
      ShowCurrUnDo();
   }
   else
   {  EraseWindow(hEditWindow, MenuColor,-1,0,0,0);   //1-1-98
   }


   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hGFXWindow, hdc);
}

//-------------------------------------------------------------------------
// Refresh our Buttons - get them to redraw themselves if overwritten
// used only by GFXLeftDrawBar; have to do all this manually since Win95 doesn't
//
//   mode 1 = Main Window,  2 = Convert,  3 = Drawing
//-------------------------------------------------------------------------

static void RefreshWindowButtons(void)
{
 int4 i;

 //---------------------------------------

 switch(Mode)
 { // menu #1
   case MAIN:
     for(i = 0; i < BUTTONTEXTE; i ++)           // Main Menu buttons
     {
         InvalidateRect(hControl[i], NULL, TRUE);
         UpdateWindow(hControl[i]);
     }
     break;

   case CVT:                                       // Convert Buttones
     for(i = 0; i < 8; i ++)
     {
         InvalidateRect(hControl[i], NULL, TRUE);
         UpdateWindow(hControl[i]);
     }

     InvalidateRect(hControl[9], NULL, TRUE);       // List Box
     UpdateWindow(hControl[9]);
     break;

   case DRAW:                                       // Either Editing or NOT
    if(!EditDraw)
    {
       for(i = 0; i < BUTTONTEXTD; i ++)    //9
       {
          InvalidateRect(hControl[i], NULL, TRUE);
          UpdateWindow(hControl[i]);
       }

       InvalidateRect(hControl[14], NULL, FALSE);   // List Box name , don't do background
       UpdateWindow(hControl[14]);

       InvalidateRect(hControl[13], NULL, TRUE);    // List Box list of names, do background
       UpdateWindow(hControl[13]);
    }
    else
    {
      for(i = 0; i < BUTTONTEXTG ; i ++)   //11
      {
          InvalidateRect(hControl[i], NULL, TRUE);
          UpdateWindow(hControl[i]);
      }

      InvalidateRect(hClrL, NULL, TRUE);          // Color Box
      UpdateWindow(hClrL);
      InvalidateRect(hClrT, NULL, TRUE);          // Transparent Color Box
      UpdateWindow(hClrT);
      InvalidateRect(hClrC, NULL, TRUE);          // Current Color Box
      UpdateWindow(hClrC);

      if(ToolActive[4] == 1)                      // Pen mode
      {
         InvalidateRect(hLineMode, NULL, TRUE);   // LineMode List Box
         UpdateWindow(hLineMode);
      }
    }
    break;
  }
}

//
//-------------------------------------------------------------------------
//  Draw the Right side Client Area of The Graphic Editor
//  picture not redone if drawing
//-------------------------------------------------------------------------
//

static void GFXDrawPicture(void)
{
   //RECT        rcB;
   HPALETTE    hOldPal;
   int2        code;
   HDC         hdc;
   int2        xoffset,yoffset;      // dims of graphic

//==================================================

   if (!hEditWindow) return;  // 8-1-97

   hdc     = GetDC(hEditWindow);
  	hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	RealizePalette(hdc);

	//GetClientRect(hEditWindow, &rcB);

   switch(Mode)
   {
     //------------
     // menu# 1
     //------------
      case MAIN:
       if(GameVersion == 1 || GameVersion == 2)
         DisplayThingPic (hdc, int2(EditRC.left), int2(EditRC.top),int2(EditRC.right), int2(EditRC.bottom),"M_DOOM", 0,0, 2);
       else if(GameVersion == 5)
         DisplayThingPic (hdc, int2(EditRC.left), int2(EditRC.top),int2(EditRC.right), int2(EditRC.bottom),"M_STRIFE", 0,0, 2);
       else
         DisplayThingPic (hdc, int2(EditRC.left), int2(EditRC.top),int2(EditRC.right), int2(EditRC.bottom),"M_HTIC", 0,0, 2);
       break;

     //------------
     // menu  for convert from one palette to another
     //------------
     case CVT:
      if(ConvertOK)   // Paint OK to draw
      {
        if(AutoCVT)   // uses
        {  ColorShowTexture(hEditWindow, EditRC.left, EditRC.top, EditRC.right, EditRC.bottom, ConvertName,TRUE); //   hGFXWindow
        }
        else
        {  ColorShowTexture(hEditWindow, EditRC.left, EditRC.top, EditRC.right, EditRC.bottom, ConvertName,FALSE); //   hGFXWindow
        }

      }
      break;

     //------------
     // menu#2
     //------------
     case DRAW:
      if(!EditDraw)                     // not drawing
      {
        if (!BMPtoPWAD)                 // not making a PWAD from a BMP
        {  //static count;
           //Trace(hGFXWindow,10,20,"here %d x=%d,y=%d",count++,LastPicXloc,LastPicYloc);

           if(DrawingName[0] != '\0')  // Valid Selection from the List Box NOT separator
           {  code = DisplayGraphicTexture(hEditWindow, hdc, NOTMASTER,   // hGFXWindow 8-1-97
                                         EditRC.left, EditRC.top, EditRC.right, EditRC.bottom,
                                         NULL, NULL, NULL, NULL, NULL,
                                         DrawingName,
                                         DrawingMagnify,
                                         NOSHOWINFO,
                                         Transparent,
                                         SHOWIMAGE);

             if(code < 1) // usually -2 if not a valid graphic
             {   code = DisplayGraphicFlat(hEditWindow,hdc,NOTMASTER,    // hGFXWindow 8-1-97
                                          EditRC.left, EditRC.top, EditRC.right, EditRC.bottom,
                                          NULL,NULL,NULL,NULL,NULL,
                                          DrawingName,
                                          DrawingMagnify,
                                          NOSHOWINFO,
                                          Transparent,
                                          SHOWIMAGE);
                 DrawingXsize = 64;
                 DrawingYsize = 64;
             }
             else
             {
                 FindLumpPicSize(DrawingName,&DrawingXsize,&DrawingYsize,&xoffset,&yoffset);
             }

             if (code >= 1)
             {
                 DirPtr dir;

                 dir = FindMasterDir( MasterWad, DrawingName,FALSE);
                 if( dir )
                 {
                    SetActiveWad(&ConvertWad,dir->filenumber);
                    ConvertWadDirStart = dir->start;

                    wsprintf(StatMsg, "%4hd x %-4hd  %-8s  %-46.46s",
                                      DrawingXsize,DrawingYsize,DrawingName,ConvertWad.filename);

                    DrawingMagnify = DisplayMagnify;   // size actually used by displayflat/graphic
                    GFXDrawStatus(hGFXWindow);         // status bar on bottom
                 }
             }
           }
        }
        else
        {
            EraseWindow(hEditWindow,MenuColor,-1,0,0,0);  // 1-1-98 
        }
         //NotifyMsg(NULL,"x=%d,%d  y=%d,%d",EditRC.left, EditRC.top, EditRC.right, EditRC.bottom);
      }
      break;
   }

   if(!EditDraw)                             // not drawing (since no picture done then
   {
       ValidateRect(hEditWindow, &EditRC);
   }

   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hEditWindow, hdc);            // oops this WAS hEditwindow should have been hGFXwindow in old code
}


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Clear window handles and id's in preparation for the next batch of controls//
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
static void ClearButtons(void)
{
  int2 i;

//==============================================================================

  for(i = 0; i < 16; i ++)      // Button ID's and Handles
  {
     DeePDestroyWindow(hControl[i]);
     hControl[i] = NULL;        // All Window Handles NULL by default
  }

  if(Mode == DRAW && EditDraw)
  {
      DeePDestroyWindow(hClrL);     // left button color
      hClrL = NULL;
      DeePDestroyWindow(hClrT);     // right button color
      hClrT = NULL;
      DeePDestroyWindow(hClrC);     // current button color
      hClrC = NULL;
  }
}

//==============================================================================
//  set font to use for buttons (center to make it easy to change)
//==============================================================================
void SetButtonFont(HWND hButtonControl)
{
     SetWindowFont(hButtonControl,  gd.hStockFont,TRUE);//GetStockFont(ANSI_VAR_FONT), TRUE);       // gd.hTipsFont,TRUE);//
}

////////////////////////////////////////////////////////////////////////////////
//                                                                             /
// create Menu #1 This is first level button menu                              /
//                                                                             /
////////////////////////////////////////////////////////////////////////////////
static void CreateMenuWindow1(HWND hwnd)
{
  int4 i;

//==================

  Mode       = MAIN;
  EditDraw   = FALSE;
  EditActive = FALSE;     //++

  ClearButtons();                 // destroy any buttons (was reversed 8-1-97)

  for (i = 0; i < BUTTONTEXTE; i++)        // 10 Pushbuttons
  {
       hControl[i] = CreateWindow("button",                               // registered class name
                                  ButtonTextE[i],                         // window name
                                  WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                                  XBUTTONLOCATION,                        // horiz position
                                  (ScrMaxY-(10*BUTTONHEIGHT))/2 + (i * BUTTONHEIGHT),         // vert position  centered   5-1-97
                                  BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                                  hwnd,
                                 (HMENU)(BUT0+i),                         // handle of child id
                                  gd.hInst, NULL);                        // handle of app

     SetButtonFont(hControl[i]);  //   7-1-97
  }

  //GFXLeftDrawBar(hGFXWindow);      //now draw left side 1-1-98 not reqd
}

////////////////////////////////////////////////////////////////////////////////
// Menu for Color Conversion buttons                                          //
////////////////////////////////////////////////////////////////////////////////
static void CreateConvertWindows(HWND hwnd)
{
  int4 i;

//==================

  Mode      = CVT;
  EditDraw  = FALSE;
  EditActive= FALSE;  //++
  ConvertOK = FALSE;            // NOT OK to draw until PIC loaded

  ClearButtons();               // destroy any buttons (was reversed) 8-1-97

  for (i = 0; i < BUTTONTEXTC; i++)   // 8
  {
       if ( i != 4)  // 4 is autocheckbox
       {
            hControl[i] = CreateWindow("button",
                      ButtonTextC[i],  WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                      XBUTTONLOCATION,                        // horiz position
                      300 + (i * BUTTONHEIGHT),
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT10+i),                       // handle of child id
                      gd.hInst, NULL);

            SetButtonFont(hControl[i]);       //   7-1-97
       }
  }

  hControl[4] = CreateWindow("button",
                      ButtonTextC[4],  WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
                      XBUTTONLOCATION,                        // horiz position
                      300 + (4 * BUTTONHEIGHT),
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT14),                         // handle of child id
                      gd.hInst, NULL);


  SetButtonFont(hControl[4]);

  hControl[9] = CreateWindow("listbox",
                      NULL, WS_CHILD | WS_VISIBLE | LBS_STANDARD | WS_TABSTOP | DS_SETFONT,
                     (BARWIDTH - 100) / 2, 20,
                      100, 280,
                      hwnd,
                      (HMENU)(lst1),                           // handle of child id
                      gd.hInst, NULL);


  SetWindowFont(hControl[9], gd.hStockFont,TRUE);//GetStockFont(ANSI_VAR_FONT), TRUE);   // left separate in case button changes

  Button_SetCheck(hControl[4], TRUE);  // AutoCVT TRUE on start
  AutoCVT = TRUE;

  //GFXLeftDrawBar(hGFXWindow); 1-1-98 not reqd
}

////////////////////////////////////////////////////////////////////////////////
// Menu # 3  Actual drawing buttons + no list box                             //
//          (  5-1-97 no list box here)
////////////////////////////////////////////////////////////////////////////////
static void CreateMenuWindow3(HWND hwnd)
{
  int4 i;

//===================================================

  Mode = DRAW;
  ClearButtons();               // was reverse

  for (i = 0; i < 6; i++)       // (0 - 4) 5 Pushbuttons   BUT35 - BUT39  ALLTOGETHER
  {
     hControl[i] = CreateWindow("button", ButtonTextG[i],  WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                      XBUTTONLOCATION,                        // horiz position
                      220 + (i * BUTTONHEIGHT),
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT35+i),                       // handle of child id
                      gd.hInst, NULL); // *22

  }


  hControl[6] = CreateWindow("button", ButtonTextG[6],  WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
                      XBUTTONLOCATION,                        // horiz position
                      220 + (6 * BUTTONHEIGHT),
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT30),                         // handle of child id
                      gd.hInst, NULL);   // *22

  hControl[7] = CreateWindow("button", ButtonTextG[7],  WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
                      XBUTTONLOCATION,                        // horiz position
                      220 + (7 * BUTTONHEIGHT),
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT31),                        // handle of child id
                      gd.hInst, NULL);   // *22

  for (i = 8; i < 11; i++)       // (7 - 9) 3 Pushbuttons   BUT32  - BUT34 ALLTOGETHER
  hControl[i] = CreateWindow("button", ButtonTextG[i],  WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                      XBUTTONLOCATION,                        // horiz position
                      220 + (i * BUTTONHEIGHT),               //7
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT32+i-8),                     // handle of child id
                      gd.hInst, NULL);// *22

  for (i = 0; i < BUTTONTEXTG; i++)      // 11
  {    SetButtonFont(hControl[i]);       //   7-1-97
  }

  //GFXLeftDrawBar(hGFXWindow); 1-1-98 not reqd

  Button_SetCheck(hControl[6], ToolWindow);  // Tool Window floating or not
  if (!ToolWindow)
  {   GFXDrawTools(hwnd);                    // NOT Floating
  }

  Button_SetCheck(hControl[7], TRUE);  // AutoCVT TRUE on start

  PasteTranslate = TRUE;
  EditDraw = TRUE;                     // turn DRAWING is active on

  ToolActive[4]  = 1;                  // Initialize with Pen tool
  DrawLineList(hwnd, TRUE);            // Draw the Tool list box if ToolActive[4] = 1
}

////////////////////////////////////////////////////////////////////////////////
// Menu #2  Edit choice+others + list box  (we are not drawing yet)
////////////////////////////////////////////////////////////////////////////////
// 5-1-97   only thing I can't solve is if last active entry is past the
//            end of 1st box, scroll bars don't show
// 8-1-97   fixed, extra paints were occurring

static void CreateMenuWindow2(HWND hwnd)
{
  int4 i;

//==================

  Mode       = DRAW;
  EditDraw   = FALSE;
  EditActive = FALSE;  //++
  EditChange = FALSE;

  ClearButtons();             // was reverse

  //GFXLeftDrawBar(hGFXWindow); // 1-1-98 moved from below not req'd

  // name box

  hControl[14] = CreateWindow("edit",NULL,
                     ES_LEFT| ES_UPPERCASE | ES_WANTRETURN| WS_CHILD | WS_VISIBLE |WS_BORDER | WS_TABSTOP,
                    (BARWIDTH - 100) / 2,20,
                     100, 20,
                     hwnd,
                     (HMENU)(IDD_TXTREDIT),              // handle of child id
                     gd.hInst, NULL);

  // list of textures

  hControl[13] = CreateWindow("listbox", "Textures",
                     WS_CHILD | WS_VISIBLE | LBS_NOTIFY | WS_VSCROLL | WS_TABSTOP,
                    (BARWIDTH - 100) / 2,44,
                     100, 240,
                     hwnd,
                     (HMENU)(lst2),                       // handle of child id
                     gd.hInst, NULL);

  // 9 Pushbuttons  BUT20-BUT28
  for (i = 0; i < BUTTONTEXTD; i++)
  {    hControl[i] = CreateWindow("button", ButtonTextD[i],
                      WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
                      XBUTTONLOCATION,                        // horiz position
                      300 + (i * BUTTONHEIGHT),
                      BUTTONWIDTH,BUTTONHEIGHT2,              // width, height
                      hwnd,
                      (HMENU)(BUT20+i),                       // handle of child id
                      gd.hInst, NULL);

       SetButtonFont(hControl[i]);       //   7-1-97
  }


  SetWindowFont(hControl[13],gd.hStockFont,TRUE);// GetStockFont(ANSI_VAR_FONT), TRUE);
  SetWindowFont(hControl[14],gd.hStockFont,TRUE);// GetStockFont(ANSI_VAR_FONT), TRUE);

  //GFXLeftDrawBar(hGFXWindow); // 1-1-98 moved to top  not req'd

}

////////////////////////////////////////////////////////////////////////////////
//  Graphic Editor Paint Routine  (redone   5-1-97)
////////////////////////////////////////////////////////////////////////////////
static void GFX_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
// int4        Update;     // Region of Screen to Update

   //--------------------------------------------------------------------------
   //static int4 count;
   //Trace(hwnd  ,10,4,"GFX_OnPaint, %d",count++);  //  test what's being passed

   BeginPaint(hwnd, &ps);
   // 1-1-98 if help is in the center, no update paint for left side occurs
   //        but left side has the wrong color, so it's needed
   /*
   if(ps.rcPaint.left < BARWIDTH && ps.rcPaint.right < BARWIDTH)
   {
      Update = 1;          // LEFT PANE ONLY
   }
   //else
   if(ps.rcPaint.left > BARWIDTH)
   {
      Update = 2;          // CLIENT ONLY UPDATE REQUIRED
   }
   else
   if(ps.rcPaint.left < BARWIDTH && ps.rcPaint.right > BARWIDTH)
   {
      Update = 3;          // ALL THE SCREEN
   } */

   // clear the right drawing window

   //if (Update == 2 || Update == 3)  // graphic right side side
   if (hEditWindow
   &&  !EditActive)   // not editing
   {
       EraseWindow(hEditWindow, MenuColor,-1,0,0,0);   //1-1-98
       /*
       //if (hEditWindow)  // if editing use edit window (smaller)
       if (EditActive     // if editing use edit window (smaller) repl   8-1-97
       &&  hEditWindow)   // if editing use edit window (smaller)
       {   EraseWindow(hEditWindow, MenuColor,-1,0,0,0);     //   5-1-97 LIGHTGRAY
       }
       else
       if (hGFXWindow)
       {
           HPALETTE  hOldPal;
           RECT      rc;
           HDC       hdc;

           hdc     = GetDC(hGFXWindow);
           hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
           RealizePalette(hdc);

           GetClientRect(hGFXWindow, &rc);

           SetRect(&rc, BARWIDTH + 1, 0,
                    rc.right -1, rc.bottom - STATHEIGHT);

           FillRect(hdc, &rc, gd.hBrushes[MenuColor]);//GetStockBrush(BLACK_BRUSH));

           SelectPalette(hdc, hOldPal, FALSE);
           ReleaseDC(hGFXWindow, hdc);
       }*/
   }

   //if(Update == 1 || Update == 3)    // left or all  1-1-98
   {
       GFXLeftDrawBar(hGFXWindow);   // Draw Left Pane
   }

   GFXDrawPicture();              // Draw Picture on right (not if not drawing)
   GFXDrawStatus(hGFXWindow);     // Update the Status Bar on  bottom

   EndPaint(hwnd, &ps);
}

////////////////////////////////////////////////////////////////////////////////
// Called from Menu.cpp CM_GRAPHICEDITOR                                       /
//                                                                             /
//        this calls GFXWndProc                                                /
//                                                                             /
//             MENU # 1 =   Main Display our Graphic Editor  ==                /
//                                                                             /
////////////////////////////////////////////////////////////////////////////////
#define hGFXWindowId  NULL
#define hEditWindowId NULL

#pragma argsused
void InitGFXWindow(HWND hwnd) // our Parent, ah
{
   // window on the whole screen

   hGFXWindow = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW | WS_EX_DLGMODALFRAME,
                               szLoadString(IDS_GFXNAME),
                               szLoadString(IDS_GFXNAME),
                               WS_OVERLAPPEDWINDOW | WS_VISIBLE ,
                               0, 0,                              // origin
                               CW_USEDEFAULT,
                               CW_USEDEFAULT,
                               hwnd,
                               (HMENU)hGFXWindowId,//NULL,        // handle of child id
                               gd.hInst, NULL);

   ShowWindow(hGFXWindow, SW_MAXIMIZE);

   UpdateWindow(hGFXWindow);           //WM_PAINT

   // window on the right

   CreateEditWindow(hwnd);             // create this one now too   8-1-97 sticks around forever

   GetClientRect(hGFXWindow,&GfxRC);   // get relative coordinates /size now

   GFXLeftDrawBar(hGFXWindow);         // Draw Left Pane req'd here for some reason, need to look closer
}

//------------------------------------------------------------------------------
// 2nd window on the left
//------------------------------------------------------------------------------
#pragma argsused
static void CreateEditWindow(HWND hwnd)
{
   RECT rc;                           // has to be gotton now to get it to work??

   GetClientRect(hGFXWindow, &rc);

   hEditWindow = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW | WS_EX_DLGMODALFRAME,
                                szLoadString(IDS_GFXEDITNAME),
                                szLoadString(IDS_GFXEDITNAME),                 // GFXEditWndProc
 	                             WS_CHILD | WS_VISIBLE  | WS_HSCROLL |WS_VSCROLL,
                                BARWIDTH, 0,                      // origin
                                rc.right - BARWIDTH -1, rc.bottom - STATHEIGHT,// x,y size - caption,
                                hGFXWindow,
                                (HMENU)hEditWindowId,//NULL,      // handle of child id
                                gd.hInst, NULL);
                                
   SetScrollRange (hEditWindow, SB_VERT, 0, 0, TRUE);
   SetScrollRange (hEditWindow, SB_HORZ, 0, 0, TRUE);

   GetClientRect(hEditWindow,&EditRC);   // get relative coordinates /size now

   cxHorzScrollPos = cyVertScrollPos = 0;
}

//////////////////////////////////////////////////////////////////////////////
//  Help File For Graphic Editor
//////////////////////////////////////////////////////////////////////////////
static void DrawHelp(HWND hwnd)
{
//    HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"Graphics - Drawing Tool");


	  HelpMsg(hwnd, NOCENTER|HELPHELP, "Drawing",
     "Float the Tool Bar by clicking Float - then drag as desired.",
     "Select tools by left clicking on the button. The status bar gives a",
     "short description of current activity. Right click anywhere within the",
     "graphic for choosing colors.  Press HELP button for more info.",
     "",
     "            Keyboard Summary",
     "",
     "Backspace - Mouse and Keys move image left-right-up-down",
     "Ctrl-C\tMake a Copy for Instant UNDO",
     "Ctrl-Z\tInstant UNDO",
     "Ctrl-V\tPaste a copy of Select Tool or Imported Image",
     "B\tBlur Image",
     "C\tChange all pixels matching color under cursor to active color",
     "D\tDots Drawing Tool  ( PEN on Tool Window )",
     "F\tFreehand Tool  ( PEN on Tool Window )",
     "F\tFloodfill (WHEN PASTING) PASTE REGION with active color",
     "I\tImport Image PCX or BMP",
     "J\tChange all pixels matching color under cursor to random color",
     "L\tLine Drawing Tool ( PEN on Tool Window )",
     "N\tNew Transparent Color",
     "O\tCircle Draw solid",
     "P\tCircle Draw hollow",
     "R\tRectangle Tool ( Select on Tool Window )",
     "T\tTranslate (for Paste)",
     "U\tUNDO",
     "X\tFlip X Axis",
     "Y\tFlip Y Axis",
     "Z\tFuzzy Image",
     "\\\tShrink Image 1/2 size",
     "; '\tCycle ALL Color up/down",
     "[ ]\tCycle Colors under cursor up/down",
     "<>\tRotate",
     "+-\tMagnify/Shrink Image Display",
     NULL );

}

//
//-------------------------------------------------------------------------
//  Toggle The Palette Modeless Dlg. We create it on a R-click over
//  the drawing area initializing it to the proper color with lParam
//  After the user left clicks on a valid color, a WM_CHANGE
//  will be posted to the Parent window with lParam containing the
//  chosen index into the palette
//-------------------------------------------------------------------------
//
void TogglePal(HWND hwnd, int2 init)
{
   if(gd.hPalWindow)
   {
       DeePDestroyWindow(gd.hPalWindow);
       gd.hPalWindow = NULL;
   }
   else
   {
       gd.hParent = hwnd;        // for proper return of our color choice
       gd.hPalWindow = CreateDialogParam(gd.hInst, (LPSTR)"PalettePick", hwnd, PaletteDlgProc, (LPARAM)(int2)init);
   }
}

//
//-------------------------------------------------------------------------
//  Toggle The Palette Modeless Dlg. ToolWindow = TRUE at init time
//  but user can click off with check mark on screen
//-------------------------------------------------------------------------
//
static void ToggleTool(void)
{
   if(gd.hToolWindow)
   {
       DestroyWindow(gd.hToolWindow);
       gd.hToolWindow = NULL;
   }
   else
   {
       gd.hParent = hGFXWindow;        // for proper return of our color choice
       gd.hToolWindow = CreateDialogParam(gd.hInst, (LPSTR)"Tool", hGFXWindow, ToolDlgProc, 0);
       SetFocus(hGFXWindow);           // for Keyboard to work
   }
}

//-------------------------------------------------------------------------
// Free up Memory Allocations After Conversion
//-------------------------------------------------------------------------
static void FreeConvert(void)
{
//  FreeMemory(RawImageFlag);
//  RawImageFlag = NULL;
    FreeMemory(RawImageData);
    RawImageData = NULL;
    FreeMemory(listnames);
    FreeMemory(TextureConvertList);
    listnames          = NULL;
    TextureConvertList = NULL;
    //ConvertMagnify     = PicMagnify;
}

//-------------------------------------------------------------------------
// Free up List of names only
//-------------------------------------------------------------------------
static void FreeListofNames(void)
{
   TexturesLoaded= FALSE;

   FreeMemory(DrawListNames);
   FreeMemory(DrawList);
   DrawListNames  = NULL;
   DrawList       = NULL;
   DrawListCount  = 0;

   FreeMemory(DrawFListNames);
   FreeMemory(DrawFList);
   DrawFListNames  = NULL;
   DrawFList       = NULL;
   DrawFListCount  = 0;
}

//----------------------------------------------------------------------
// free's: memory for names allocated, bitmaps, windows
//----------------------------------------------------------------------

static void DrawFree(BOOL freelist)
{
    int2 i;

    if(freelist)
    {
       FreeListofNames();
    }

    for(i = 0; i < 6; i++ )
    {
       ToolActive[i] = 0;
       if (ToolBitmapsUp[i])
       {   DeleteBitmap(ToolBitmapsUp[i]);
       }
       ToolBitmapsUp[i] = NULL;
    }

    for(i = 0; i < 6; i++ )
    {
       if (ToolBitmapsDn[i])
       {   DeleteBitmap(ToolBitmapsDn[i]);
           ToolBitmapsDn[i] = NULL;
       }
    }

    DeePDestroyWindow(gd.hToolWindow);
    DeePDestroyWindow(gd.hPalWindow);
    gd.hToolWindow = NULL;
    gd.hPalWindow = NULL;

    DeePDestroyWindow(hClrL);              // And the Drawing Color & Transparent
    DeePDestroyWindow(hClrT);
    DeePDestroyWindow(hClrC);
    hClrL = NULL;
    hClrT = NULL;
    hClrC = NULL;

    DeePDestroyWindow(hLineMode);
    hLineMode = NULL;

//    DeePDestroyWindow(hEditWindow);     // The Drawing Window revised 8-1-97 keep till final exit
//    hEditWindow = NULL;
    if (hEditWindow)
    {  SetScrollRange (hEditWindow, SB_VERT, 0, 0, TRUE);
       SetScrollRange (hEditWindow, SB_HORZ, 0, 0, TRUE);
    }
    cxHorzScrollPos = cyVertScrollPos = 0;

	 FreeMemory(RawImageData);           // free data buffer
    RawImageSize = 0;
    RawImageData = NULL;

    FreeMemory(RawImageUndoBufr);       // free image buffer original raw data
    RawImageUndoBufr = NULL;
    FreeMemory(ImageUndoBufr);          // free image buffer
    ImageUndoBufr = NULL;

    FreeMemory(UnDoArray);       // free undo    4-1-97  check some more <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    UnDoArray = NULL;

    //FreeMemory( CopyImageBufr );
    // Don't DO HERE, May want to PASTE A different Image
    if (CursorDropper)
    {	  DestroyCursor(CursorDropper);
        CursorDropper = NULL;
    }
    if (CursorEraser)
    {   DestroyCursor(CursorEraser);
        CursorEraser = NULL;
    }
    if (CursorFill)
    {   DestroyCursor(CursorFill);
        CursorFill = NULL;
    }
    if (CursorMag)
    {   DestroyCursor(CursorMag);
        CursorMag = NULL;
    }
    if (CursorPen)
    {   DestroyCursor(CursorPen);
        CursorPen = NULL;
    }

    DidRotate  = FALSE;
    DidTransp  = 0;
}


//
//-------------------------------------------------------------------------
// Destroy Old Buttons, List Box, Load Bitmaps, set up new Windows,
// to get ready to draw on a graphic ( Edit )
//-------------------------------------------------------------------------
//
static void EditSetup(HWND hwnd)
{
  //RECT rc;

  //----------------------

  DeePDestroyWindow(hClrL);     // left button color
  DeePDestroyWindow(hClrT);     // right button color
  DeePDestroyWindow(hClrC);     // current picture color

  //left button color
  hClrL = CreateWindow("static", NULL,
                                 WS_CHILD | WS_VISIBLE | SS_NOTIFY | SS_SUNKEN,
                                 ( (BARWIDTH /3) - (COLORBOXWIDTH+2)  ), 22,   //+2 rounding errors instead of 5
                                 COLORBOXWIDTH, 30,
                                 hwnd,
                                 (HMENU)stc1,
                                 gd.hInst, NULL);                               // handle of child id
  // right button color
  hClrT = CreateWindow("static", NULL,
                                 WS_CHILD | WS_VISIBLE | SS_NOTIFY | SS_SUNKEN,
                                 ( (BARWIDTH*2 /3) - (COLORBOXWIDTH+5) ), 22,
                                 COLORBOXWIDTH, 30,
                                 hwnd,
                                 (HMENU)stc2,                             // handle of child id
                                 gd.hInst, NULL);

  // right button color
  hClrC = CreateWindow("static", NULL,
                                 WS_CHILD | WS_VISIBLE | SS_NOTIFY | SS_SUNKEN,
                                 ( (BARWIDTH) - (COLORBOXWIDTH+10) ), 22,
                                 COLORBOXWIDTH, 30,
                                 hwnd,
                                 (HMENU)stc3,                             // handle of child id
                                 gd.hInst, NULL);

  CreateMenuWindow3(hwnd);    // Destroys old Buttons, Creates New Default

  //GetClientRect(hwnd, &rc);

  //DestroyWindow(hControl[13]);   // List Box
  //hControl[13] = NULL;

  ToolBitmapsUp[0] = LoadBitmap(gd.hInst, "MAGUP");
  ToolBitmapsDn[0] = LoadBitmap(gd.hInst, "MAGDOWN");
  ToolBitmapsUp[1] = LoadBitmap(gd.hInst, "DROPUP");
  ToolBitmapsDn[1] = LoadBitmap(gd.hInst, "DROPDOWN");
  ToolBitmapsUp[2] = LoadBitmap(gd.hInst, "ERASEUP");
  ToolBitmapsDn[2] = LoadBitmap(gd.hInst, "ERASEDOWN");
  ToolBitmapsUp[3] = LoadBitmap(gd.hInst, "FILLUP");
  ToolBitmapsDn[3] = LoadBitmap(gd.hInst, "FILLDOWN");
  ToolBitmapsUp[4] = LoadBitmap(gd.hInst, "PENUP");
  ToolBitmapsDn[4] = LoadBitmap(gd.hInst, "PENDOWN");
  ToolBitmapsUp[5] = LoadBitmap(gd.hInst, "SELECTUP");
  ToolBitmapsDn[5] = LoadBitmap(gd.hInst, "SELECTDOWN");

  // just sample a few to make sure
  if (!ToolBitmapsDn[3] || !ToolBitmapsUp[5] || !ToolBitmapsUp[1] || !ToolBitmapsDn[2])
  {   ErrorMsg(hwnd,"LoadBitMap failure\n\nF A T A L   E R R O R");
  }

  CursorDropper  = LoadCursor(gd.hInst, "CURSOR_DROPPER");
  CursorEraser   = LoadCursor(gd.hInst, "CURSOR_ERASER");
  CursorFill     = LoadCursor(gd.hInst, "CURSOR_FILL");
  CursorMag      = LoadCursor(gd.hInst, "CURSOR_MAG");
  CursorPen      = LoadCursor(gd.hInst, "CURSOR_PEN");
}

////////////////////////////////////////////////////////////////////////////////
// After Exiting out of Drawing on a Graphic, just refresh the ListBox        //
// instead of re-reading all the texture names since they are still valid     //
////////////////////////////////////////////////////////////////////////////////
static char StartTexture[] = "= Start Text =";
static char EndOfTexture[] = "= End Text   =";
static char EndSeparate [] = "= ========= =";
static char StartFlat   [] = "= Start Flat =";
static char EndFlat     [] = "= End Flat   =";

static void ReInitList(HWND hList)
{
 int4 i;
 char name[9] = {0};

//----------------------------------------------

   if (DrawListCount > 0)
   {
       ListBox_AddString(hList, StartTexture);    //"= Start Tex =");
       for(i = 0; i < DrawListCount; i++)
       {   strncpy(name, DrawList[i], 8);
           ListBox_AddString(hList, name);
       }
       ListBox_AddString(hList, EndOfTexture);    //"= END-TEXTURE =");

       if (DrawFListCount > 0)
       {
           ListBox_AddString(hList, EndSeparate); //"= =========== =");
       }
   }

   if (DrawFListCount > 0)
   {
       ListBox_AddString(hList, StartFlat);        //"= "START-FLAT =");

       for(i = 0; i < DrawFListCount; i++)
       {   strncpy(name, DrawFList[i], 8);
           ListBox_AddString(hList, name);
       }
       ListBox_AddString(hList, EndFlat);          //"= "End FLAT =");
  }

//   ListBox_SetCurSel(hList, 1);                    // Select # 1 <<<<<current later
}


///////////////////////////////////////////////////////////////////////////
//     Find closest name to one entered
///////////////////////////////////////////////////////////////////////////

static void FindClosestEditMatch(void)
{
  int4 i;
  int4 CompareLen  = lstrlen(EditName);
  int4 EditListBoxIndex = -1;


  //Trace(hGFXWindow,20,44,"%s %s %d %hd",PasteLumpName,EdTxt,CompareLen,NameChange);

  if (EditChange)
  {
      EditNameIndex = 1;   // global skip = start

		DrawingName[CompareLen] = '\0';                            // for length of entry only

      for(i = 0; i < DrawListCount; i++,EditNameIndex++)
      {   strncpy(DrawingName, (char *)DrawList[i], CompareLen);        // for len of name entered

         if (!strcmp(DrawingName, EditName))
         {   EditListBoxIndex = EditNameIndex;
             break;
         }
      }

      if (EditListBoxIndex < 0)
      {  if (DrawListCount > 0)   // skip =end
         {   EditNameIndex +=1;
         }

         for(i = 0; i < DrawFListCount; i++,EditNameIndex++)
         {   strncpy(DrawingName, (char *)DrawFList[i], CompareLen);        // for len of name entered

             if (!strcmp(DrawingName, EditName))
             {   break;
             }
         }
      }
  }

  if (EditNameIndex <=0)  EditNameIndex = 1;                    // can't be 0

  ListBox_SetCurSel(hControl[13], EditNameIndex);
  ListBox_GetText(hControl[13], EditNameIndex, DrawingName);   // get lumpnmae

}
///////////////////////////////////////////////////////////////////////////////
//
//   WM_COMMAND msg's from our Push Buttons, List Box
//
//////////////////////////////////////////////////////////////////////////////
#pragma argsused
static void GFX_Command(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  Bool    Success;
//LRESULT EditNameIndex;
  HDC     hdc;
//  RECT    rc;
  LRESULT LIndex;
  HWND    hList;
//  HPALETTE hOldPal;
  int2    code;

  LPSTR   ValidName;
//  char    LumpName[10]    = {0};
//  DOOMGraphicHdr PicHeader;

//--------------------------------------------------------------
 //Trace(hwnd,10,2,"id=%d",id);
 //Trace(hEditWindow,10,10,"id=%d",id);

//  GetClientRect(hwnd, &rc);

  switch(id)
  {

//-----------------------------------------------------------------------------
//                                                                           //
// ======   Menu#1 select : Draw , Color Convert. Help, Exit   =======       //
//                                                                           //
//-----------------------------------------------------------------------------

   // create menu#2 (draw step 1)
   case BUT0:
    SoundBeep(SoundSwish2);

    CreateMenuWindow2(hGFXWindow);                 // create Edit, ZOOM, etc buttons

    // Draw ONLY if have EXTRA Graphics Loaded

    hList = hControl[13];

    if (!TexturesLoaded)
    {   TexturesLoaded = LoadDrawOnTextures(hwnd); // preload TEXture names
    }

    if( TexturesLoaded)                            // preload TEXture names
    {
       LastPicXloc = -1;                           // 1st time
       LastcxRange = -1;                           // force clear

       strcpy(StatMsg, "Drawing on Textures - Choose a Texture to EDIT" );

       ReInitList(hList);                          // build list

       GFX_OnPaint(hGFXWindow);                    // erase right side, does GFXDrawStatus too (REQD 3-1-98 or else scroll update bad past 1 page)

       Edit_SetText(hControl[14], DrawingName);    //  posts change and then that draws texture

       //SendMessage(hwnd, WM_COMMAND, MAKEWPARAM((WORD)GetWindowLong(hList, GWL_ID), LBN_SELCHANGE ), 0); // gets closest name
       SetFocus(hList);    // keyboard focus in List Box for easy choosing
    }
    break;

   // Convert Textures
   case BUT1:
    SoundBeep(SoundSwish2);
    if (DeePRegistered)
    {
        CreateConvertWindows(hwnd);    // all the buttons

        //if (ConvertWallTexture(hwnd,hControl[9], rc.left + BARWIDTH, 0, rc.right, rc.bottom - STATHEIGHT) ) //   5-1-97 so can clear
        if (ConvertWallTexture(hGFXWindow,hControl[9])) //, EditRC.left,EditRC.top,EditRC.right,EditRC.bottom) ) //   8-1-97 so can clear
        {
            ConvertOK = TRUE;
            strcpy(StatMsg, "Color Conversion for TEXTURES" );
            GFXDrawStatus(hGFXWindow);// status bar on  bottom
        }
        else
        {
            CreateMenuWindow1(hwnd);   // Destroys old Buttons, Creates New Default
            GFXDrawPicture();          // picture on right
        }
    }
    else
    {   OnlyRegConvertExport();
    }
    break;
   // Convert FLATS
   case BUT2:
    SoundBeep(SoundSwish2);

    if (DeePRegistered)
    {
        CreateConvertWindows(hwnd);    // all the buttons

        if (ConvertFlatTexture(hGFXWindow,hControl[9]))//, EditRC.left,EditRC.top,EditRC.right,EditRC.bottom) ) //   8-1-97 so can clear
        {
            ConvertOK = TRUE;
            strcpy(StatMsg, "Color Conversion for FLATS" );
            GFXDrawStatus(hGFXWindow); // status bar on  bottom
        }
        else
        {
            CreateMenuWindow1(hwnd);   // Destroys old Buttons, Creates New Default
            GFXDrawPicture();          // picture on right
        }
    }
    else
    {   OnlyRegConvertExport();
    }
    break;
   // Open Texture
   case BUT3:
    // see if any FF_START replacement already exists
    // since it can only be done once for now!
    // Also make sure does not have a map, since this code won't handle it right
    SoundBeep(SoundSwish2);

    if (GetWadName(hwnd, "Open PWAD",GFXWadName, 259, IDS_FILTERWAD, PatchDir) )
    {
        int4  code;

        if (!FindAnEntry(GFXWadName,"THINGS\0\0",&code)) // make sure this is not a level
        {
            if (code == 0)           // OK, not found, rest are bad file, etc
            {
               DirPtr dirFFs;
               dirFFs = FindMasterDir(MasterWad,"FF_START",FALSE);            // FF_START

               if (dirFFs)
               {   // see if this file has F_START or FF_START entries

                   if (!FindAnEntry(GFXWadName,"F_START\0",&code)
                   &&  code == 0
                   &&  !FindAnEntry(GFXWadName,"FF_START", &code)
                   &&  code == 0)
                   {
                   }
                   else
                   {
                       HelpMsg(hwnd,CENTER|HELPINFO, "FLATS",
                                      "Replacement FLATS already loaded",
                                      "",
                                      "Go to the File Menu and Reset Files",
                                      NULL);
                       code = 1;
                   }
               }

               if (code == 0)
               {
                   StorePwad (GFXWadName);

                   // reread, since file could have

                   ReadWTextureNames();
                   ReadFTextureNames();
                   gd.OKDraw = 1;      // reset to normal (set by StorePwad)

                   // free all prior textures since we need to reload

                   DrawFree(TRUE);
               }

            }
        }
        else
        {
            HelpMsg(hwnd,CENTER|HELPINFO, "PWAD has a level",
                           "This PWAD contains a level!",
                           "",
                           "Go to the File Menu and Open File.",
                           NULL);
        }
    }
    break;

   //==   E X P O R T   Buttons  ====

   // Save Any Lump
   case BUT4:
     DLG2OPTS d2;
     ZeroMemory(&d2, sizeof(d2));
     memset(d2.sz1, 0,8);                  // name start

     SoundBeep(SoundSwish2);

     if (HelpMsg(hwnd,CENTER|HELPYESNO, "Export",
                                       "Sort List?",
                                       NULL))
     //if (Confirm(hwnd,"Sort List"))
     {
         d2.b1 = 1;                       // sorted option
         DialogBoxParam( gd.hInst, "Lump", hwnd, LumpDlgProc, (LPARAM)(LPDLG2OPTS)&d2 );   // sorted  1
     }
     else
     {   d2.b1 = 1;                       // unsorted option
         DialogBoxParam( gd.hInst, "Lump", hwnd, LumpDlgProc, (LPARAM)(LPDLG2OPTS)&d2 );   // unsorted 0
     }
     break;
   // Export All Textures
   case BUT5:
    SoundBeep(SoundSwish2);
    if (DeePRegistered)
    {   ExportTexture(hwnd);
    }
    else
    {   OnlyRegConvertExport();
    }
    break;
   // Export All Flats
   case BUT6:
    SoundBeep(SoundSwish2);
    if (DeePRegistered)
    {   ExportFlats(hwnd);
    }
    else
    {   OnlyRegConvertExport();
    }
    break;
   // Export All Sprites
   case BUT7:
    SoundBeep(SoundSwish2);
    if (DeePRegistered)
    {   ExportSprites(hwnd);
    }
    else
    {   OnlyRegConvertExport();
    }
    break;

   // Help main menu #1
   case BUT8:
    SoundBeep(SoundDrip);
    HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"GRAPHICS");
    break;

   //------------------------------------------
   // EXIT (all of it)                        /
   //------------------------------------------
   case BUT9:
    SaveCFGStatus();                    // colors save now   12-2-96
    SoundBeep(SoundGame);

    ClearButtons();                     // get rid of sub buttons

    DrawFree(TRUE);                     // Free list, Memory Allocations and windows ++8-1-97

    RemoveDIBSection();                 // Delete Paste stuff CopyImageBufr
    DeePDestroyWindow(hGFXWindow);
    hGFXWindow = NULL;

    DeePDestroyWindow(hEditWindow);     // New at exit now   8-1-97
    hEditWindow = NULL;

    SetForegroundWindow(gd.hMainWindow);// to get +/-,etc. to work again
    SetFocus(gd.hMainWindow);           //
    break;


//-----------------------------------------------------------------------------
//                                                                           //
//=========        C O N V E R T       MENU Buttons    ========              //
//                                                                           //
//-----------------------------------------------------------------------------

   // CONVERT
   case BUT10:              // CONVERT
    SoundBeep(SoundClick3);
    ColorShowTexture(hEditWindow, EditRC.left,EditRC.top, EditRC.right,EditRC.bottom, ConvertName,TRUE);
    break;
   case BUT11:              // SAVE
    SoundBeep(SoundClick3);
	 SaveGraphic(hGFXWindow, 1, ConvertName, ConvertWad.filename, TextureDirStart,ImageGetSize);
    break;
   case BUT12:              // SAVE AS
    SoundBeep(SoundClick3);
    SaveGraphic(hGFXWindow, 2, ConvertName, ConvertWad.filename, TextureDirStart,ImageGetSize);
    break;
   case BUT13:              // SAVEALL
    SoundBeep(SoundClick3);
    SaveAllGraphics(hGFXWindow,hEditWindow, EditRC.left,EditRC.top, EditRC.right,EditRC.bottom);
    break;
   case BUT14:              // Automatically Convert
    SoundBeep(SoundClick3);
    if(Button_GetCheck(hwndCtl))
    {  AutoCVT = TRUE;
    }
    else
    {  AutoCVT = FALSE;
    }

    ColorShowTexture(hEditWindow, EditRC.left,EditRC.top, EditRC.right,EditRC.bottom, ConvertName,AutoCVT);
    break;

   // restore Main Buttons
   case BUT15:
    SoundBeep(SoundSwish2);
    FreeConvert();
    CreateMenuWindow1(hwnd);  // Destroys old Buttons, Creates New Default
    GFXDrawPicture();         // picture on right (erase)
    break;
   // help
   case BUT16:
    SoundBeep(SoundDrip);
    HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"Graphics - Convert");
    break;
   // Exit
   case BUT17:
    SoundBeep(SoundGame);
    FreeConvert();
    DeePDestroyWindow(hwnd);
    break;

   //==========================================================================

   case lst1:		          // Conversion List Box
     switch(codeNotify)
     {
	    case LBN_SELCHANGE:
         EditNameIndex = (LRESULT)SendMessage(hControl[9], LB_GETCURSEL, 0, 0);
         ListBox_GetText(hControl[9], EditNameIndex, ConvertName);
         ConvertOK = TRUE;
         GFXDrawPicture();    // picture on right
		   break;
     }
     break;

//-----------------------------------------------------------------------------
//                                                                           //
//=====    2nd MENU Edit,zoom,zoom,main,help,exit  Buttons     ========      //
//                                                                           //
//-----------------------------------------------------------------------------

   //------------
   // EDIT (draw on) a texture
   //------------
   case BUT20:
     if(DrawingName[0] == '\0')  break; // not valid Selection from the List Box

     DirPtr dir;

     dir = FindMasterDir( MasterWad, DrawingName,FALSE);
     if( (!dir ) || (dir->size <= 0) || (dir->filenumber ==0) )
     {
         ErrorMsg(hwnd, "? %s Not found or size = 0 ?",DrawingName);
         break;
     }

     EditActive= TRUE;          // new 8-1-97 show active

     SoundBeep(SoundPhaser);

     EditSetup(hwnd);           // create menu #3 drawing stuff

     SetActiveWad(&ConvertWad,dir->filenumber);
     ConvertWadDirStart = dir->start;

     wsprintf(StatMsg, "%4hd x %-4hd  %-8s  %-46.46s",
                         DrawingXsize,DrawingYsize,DrawingName,ConvertWad.filename);

     GFXDrawStatus(hGFXWindow); //status bar on  bottom

     //SetScrollRange (hEditWindow, SB_VERT, 0, 0, TRUE);
     //SetScrollRange (hEditWindow, SB_HORZ, 0, 0, TRUE);
     //
	  //cxHorzScrollPos = cyVertScrollPos = 0;

     // ImageX and YSize are global so we can use to init the BI structure
     // Set up a BITMAPINFO structure for use with CreateDIBSection()

     hdc = GetDC(hwnd);

     ZeroMemory(&DrawBi, sizeof(BITMAPINFO));

     DrawBi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
     DrawBi.bmiHeader.biPlanes    = 1;
     DrawBi.bmiHeader.biBitCount  = 8;
     DrawBi.bmiHeader.biWidth     =  ImageXSize;//   5-1-97((ImageXSize + 3 ) & ~3);  // global size
     DrawBi.bmiHeader.biHeight    = -ImageYSize;
	  DrawBi.bmiHeader.biSizeImage = ((ImageXSize + 3 ) & ~3) * ImageYSize;//ImageXSize   5-1-97
     DrawBi.bmiHeader.biCompression = BI_RGB;

     // Create a DC to hold our DIBSECTION
     
     hDrawBitmap = (HBITMAP)CreateDIBSection(hdc, &DrawBi, DIB_PAL_COLORS, (VOID **)&lpDrawBits, NULL, 0L);

     // Allocate our Buffer and our Bitmap

     if (hDrawBitmap)
     {
         code = DisplayGraphicTexture(hEditWindow, hdc, DOMASTER,  // 8-1-97 was hwnd
                                      EditRC.left, EditRC.top, EditRC.right, EditRC.bottom,
                                     &RawImageSize,       // size
                                     &RawImageData,       // raw data bufr
                                     &MainImageSize,      // image size
                                     (byte **)&lpDrawBits,// image buffer
                                      hDrawBitmap,        // handle dib
                                     (char *)DrawingName,
                                      1,
                                      NOSHOWINFO,
                                      Transparent,
                                      NOSHOWIMAGE);



         if (code < 1) // usually -2 if not a valid graphic
         {   DisplayGraphicFlat(hEditWindow,hdc, DOMASTER,
                                EditRC.left, EditRC.top, EditRC.right, EditRC.bottom,
                               &RawImageSize,             // size
                               &RawImageData,             // raw data bufr
                               &MainImageSize,            // image size
                               (byte **)&lpDrawBits,      // image buffer
                                hDrawBitmap,              // handle dib
                               (char *)DrawingName,
                                1,
                                NOSHOWINFO,
                                Transparent,
                                NOSHOWIMAGE);
             FlatLump = TRUE;
         }
         else
         {   FlatLump = FALSE;
         }
         //
         // req'd to set x-y box limits of this texture
         //
         MagnifyGraphicTexture(hEditWindow, EditRC.left,  EditRC.top,
                                            EditRC.right, EditRC.bottom,
                                           (byte *)&*lpDrawBits,
                                            hDrawBitmap,
                                            DrawingMagnify, Transparent);

         // This is our only way of knowing if FLAT or not

         DidTransp = 0;                     // drew on top of transparent
         UnDoActive= 0;
         DidRotate = FALSE;                 // no rotate done
         ImageHasHoles   = FALSE;

         UnDoArray        = (UnDoListPtr)malloc( UnDoMax * sizeof(struct UnDoList) );
         ImageUndoBufr    = (byte *)malloc(MainImageSize);
         RawImageUndoBufr = (byte *)malloc(RawImageSize);

         if (!UnDoArray || !ImageUndoBufr ||!RawImageUndoBufr)
         {
             FreeMemory(UnDoArray);           //   5-1-97
             FreeMemory(ImageUndoBufr);       //
             FreeMemory(RawImageUndoBufr);    //   8-1-97
             UnDoArray        = NULL;         //
             ImageUndoBufr    = NULL;         //
             RawImageUndoBufr = NULL;         //

             HelpMsg(hwnd,CENTER|HELPINFO,"Out of Memory",
	                                       "Not enough Memory for UNDO!",
                                           NULL);
         }

         // now check out the image for possible transparent problems
         // image is using pixel color that matches current transparent color


         if(!FlatLump)
         {
             // first check raw image for any active pixels that match transparent color

             CheckForTransParent(hwnd, ImageXSize,RawImageData,Transparent,TRUE);  // give message

             // now check to see if the bitmap image has "holes" = transparent color

             ImageHasHoles=CheckBitMapForHoles(ImageXSize,ImageYSize, (byte *)&*lpDrawBits,Transparent);
         }

         if (ImageUndoBufr)
         {   memcpy(ImageUndoBufr, &*lpDrawBits, MainImageSize);
             memcpy(RawImageUndoBufr, RawImageData,RawImageSize);    // copy raw data
         }

         if (ToolWindow) ToggleTool();         // make floating tool window
     }
     else
     {
        OutOfMemoryMsg(hwnd);       //   8-1-97
     }

     ReleaseDC(hwnd, hdc);
     break;

   //------------
   // Zoom IN
   //------------
   case BUT21:                             // Zoom IN
    if(DrawingName[0] != '\0')
    {
       SoundBeep(SoundClick3);
       if (DrawingMagnify < MAXZOOM)       //  6-1-97
       {   DrawingMagnify ++;
           GFXDrawPicture();               // picture on right
           //if(EditDraw)  ShowCurrUnDo();   // ?
       }
    }
    break;
   //------------
   // Zoom OUT
   //------------
   case BUT22:                             // Zoom OUT
    if(DrawingName[0] != '\0')
    {
       SoundBeep(SoundClick3);
       if (DrawingMagnify > 1)
       {   DrawingMagnify --;
           GFXDrawPicture();               // Draw Client Area picture on right
           //if(EditDraw)  ShowCurrUnDo();   // this can't be here ?? leave for now
       }
    }
    break;
   //---------------------------------------------------------------------------
   //  Import BMP or PCX and store in a PWAD
   //---------------------------------------------------------------------------
   case BUT23:
     BMPtoPWAD = TRUE;            // control refresh
     SoundBeep(SoundClick3);

     //RemoveDIBSection();          // Get rid of Previous Memory Allocation if any

     //GFXDrawStatus(hGFXWindow);   //status bar on bottom

     // get the file to import

     if (ImportBMPtoPWAD(hwnd))
     {   DrawFree(TRUE);                          // make it reload with new imported BMP
         SendMessage(hwnd, WM_COMMAND, BUT0, 0);  // simulate enter Drawing button pushed
     }
     //SetFocus(hGFXWindow);         // for Keyboard to work
     BMPtoPWAD = FALSE;
     break;

   //------------
   // Make Dummy Texture to paste on top of
   //------------

   case BUT24:
     SoundBeep(SoundSwish2);
     if ( MakeBlankTexture(hwnd))
     {
          DrawFree(TRUE);            // make it reload with dummy
          SendMessage(hwnd, WM_COMMAND, BUT0, 0);  // simulate enter Drawing button pushed
     }
     break;

   //------------
   // Change Patch texture size and offsets
   //------------
   case BUT25:
     SoundBeep(SoundSwish2);
     if (FlatLump)
     {
         HelpMsg(hwnd,CENTER|HELPINFO, "FLATS",
                                       "FLAT lumps are fixed in size",
                                       NULL);
     }
     else
     if (DrawingName[0] == '\0')
     {
         HelpMsg(hwnd,CENTER|HELPINFO, "Patch",
                                       "Please select a Patch name",
                                       NULL);
     }
     else
     {   if ( ChangePatchTextureSize(hwnd))
         {
              strcpy(StatMsg, "See HELP for how to use new patches" );
              GFXDrawStatus(hGFXWindow);// status bar on  bottom
              // NotifyMsg(hwnd,"Don't forget to use the GROUP command\n"
              //                "and combine your new patches to replace\n"
              //                "the current patches - AND - make sure that\n"
              //                "all texture names using this patch now have\n"
              //                "the correct size reflected -OR- DOOM will\n"
              //                "will complain with a R_GenerateLookUp error!");

              DrawFree(TRUE);            // make it reload with new specs
              SendMessage(hwnd, WM_COMMAND, BUT0, 0);  // simulate enter Drawing button pushed
        }
     }
     break;
   //------------
   // Restore Main Menu buttons
   //------------
   case BUT26:
     SoundBeep(SoundSwish2);

     CreateMenuWindow1(hwnd);            // Destroys old Buttons, Creates New Default
     GFXDrawPicture();                   // Draw Client Area picture on right (clear)
     break;
   //
   // Help (2nd menu, Edit,zoom,zoom,main,help,exit)
   //
   case BUT27:
     SoundBeep(SoundDrip);
     HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"GRAPHICS");
     break;

   //--------------------------------------------------------------------------
   // Exit (2nd menu) exit ALL of it
   //--------------------------------------------------------------------------
   case BUT28:                          // EXIT
     SendMessage(hwnd, WM_COMMAND, BUT9, 0);   // simulate Exit button pushed
     break;


//                                                                            //
//===================  D R A W I N G   E D I T I N G    ========================
//                                                                            //

   // IMPORT BMP/PCX

   case BUT35:
     SoundBeep(SoundClick3);

     RemoveDIBSection();     // Get rid of Previous Memory Allocation if Selecting
                                                                                                //FILTERSTRING
     Pasting = TRUE;
     ValidName = GetWadName(hGFXWindow,"Select Image to Import", CopyFileName, 259, IDS_FILTERDRAW, PatchDir );
     Pasting = FALSE;

     RemoveDIBSection();    // Get rid of Preview Memory Allocation from above since we may be calling with TRANSLATE TRUE

     GFX_OnPaint(hGFXWindow);       // guarantee refresh (need to see why, must be a mistake somewhere still)
     GFXEdit_OnPaint(hEditWindow);  // req'd 1st time somehow

     if (ValidName)
     {
       Pasting = TRUE;
       if (ImageType == PCXIMAGE)
       {   hViewBitmap = ReadImage (hEditWindow, PCXIMAGE, PasteTranslate, DrawingMagnify, NULL, NULL, CopyFileName, OPENMUST);
       }
       else
       if (ImageType == BMPIMAGE)
       {   hViewBitmap = ReadImage (hEditWindow, BMPIMAGE, PasteTranslate, DrawingMagnify, NULL, NULL, CopyFileName, OPENMUST);
       }
       else
       if (ImageType == WADIMAGEGRAPHIC)
       {
           //NotifyMsg(NULL,MapLevel);
           PasteWadLump(hwnd, CopyFileName, MapLevel);    // lump name from browsing   8-1-97
       }

       if (hViewBitmap)
       {
           PasteImage(hwnd,DrawingMagnify, Transparent);//, FALSE);

       }
       Pasting = FALSE;
     }
     SetFocus(hGFXWindow);   // for Keyboard to work
     break;
   //----------------------
   // Import direct from local PWAD textures
   //----------------------
   case BUT36:
     SoundBeep(SoundClick3);

     DialogBoxParam( gd.hInst, "PasteLump", hwnd, PasteLumpDlgProc, 1);   // sorted

     GFX_OnPaint(hGFXWindow);       // guarantee refresh (need to see why, must be a mistake somewhere still)
     GFXEdit_OnPaint(hEditWindow);  // req'd 1st time somehow

     Pasting = TRUE;

     if (hViewBitmap)
     {   PasteImage(hwnd,DrawingMagnify, Transparent);//, FALSE);
     }
     Pasting = FALSE;

     SetFocus(hGFXWindow);         // for Keyboard to work
     break;
   //------------------------------------------
   // SAVE on top of existing                 /
   //------------------------------------------
   case BUT37:
     SoundBeep(SoundClick3);

     // Recalculate Image from ImageBuffer IF:
     // 1. no transparent drawing done
     // 1. original had no transparent
     // 2. image now has no tranparent
     // OK to do
     RecalculateImage = FALSE;

     // something was done which did not update the raw image buffer

     if (DidRotate)
     {   // make sure not transparent drawn on bitmap
         if (!DidTransp)           // no transparent drawing done
         {   if (!ImageHasHoles)   // image did not have any holes
             {   DidTransp = CheckBitMapForHoles(ImageXSize,ImageYSize, (byte *)&*lpDrawBits,Transparent);
                 DidRotate = FALSE;
                 RecalculateImage = TRUE;
             }
         }
     }

     if (DidTransp || DidRotate && !FlatLump)
     {   // can't since size may change
         HelpMsg(hwnd,CENTER|HELPINFO, "SaveAs required",
                         "! SORRY !",
                         "",
                         "Tranparent drawing Done",
                         "",
                         "Use SaveAs to recalculate",
                         "graphic size.",
                         NULL);
     }
     else
     {   //
         // 1 = write right on top of existing direct from RawImage
         //
         if (!RecalculateImage)
         {
	           SaveDrawGraphic(hwnd,
                         1,
                         DrawingName,
                         ConvertWad.filename,                   //GraphicWad.filename,
                         ConvertWadDirStart,                    //GraphicDirstart,
                         &RawImageSize,  &RawImageData,         // raw data
                         &MainImageSize, (byte **)&lpDrawBits); // na  image data
         }
         else
         //
         // 4 = write right on top of existing ,but recalculate from lpDrawBits
         //
         {
	           SaveDrawGraphic(hwnd,
                         4,
                         DrawingName,
                         ConvertWad.filename,                   //GraphicWad.filename,
                         ConvertWadDirStart,                    //GraphicDirstart,
                         &RawImageSize,  &RawImageData,         // raw data verify size!
                         &MainImageSize, (byte **)&lpDrawBits); // USE image data
         }
     }
     SetFocus(hGFXWindow);   // for Keyboard to work
     break;

   //------------------------------------------
   // SAVE AS                                 /
   //------------------------------------------
   case BUT38:
     SoundBeep(SoundClick3);
     RecalculateImage = FALSE;

     if (DidRotate)
     {   // make sure not transparent drawn on bitmap
         if (!DidTransp)           // no transparent drawing done
         {   if (!ImageHasHoles)   // image did not have any holes
             {   DidTransp = CheckBitMapForHoles(ImageXSize,ImageYSize, (byte *)&*lpDrawBits,Transparent);
                 DidRotate = FALSE;
                 RecalculateImage = TRUE;
             }
         }
     }

     if ( DidTransp || DidRotate)// && !FlatLump)
     {   //
	      // recalc for all from lpdrawbits (FLATS just a copy)
         //
         Success = SaveDrawGraphic(hwnd,
                           3,
                           DrawingName,
                           GFXWadName,
                           ConvertWadDirStart,
                           &RawImageSize, &RawImageData,     // raw data
                           &MainImageSize,        // na
                           (byte **)&lpDrawBits); // used image data
	  }
     else
	  {   // no recalculate
         Success = SaveDrawGraphic(hwnd,
                           2,
                           DrawingName,
                           GFXWadName,
                           ConvertWadDirStart,
                           &RawImageSize,  &RawImageData,        // raw data
                           &MainImageSize,(byte **)&lpDrawBits); // na image data
     }

     // reload new graphic if changed

     if (Success)
     {
         StorePwad (GFXWadName);
         gd.OKDraw = 1;                // reset to normal (set by StorePwad)

         FreeListofNames();            // for name list reload
     }
     SetFocus(hGFXWindow);   // for Keyboard to workif (GetFocus()!= hGFXWindow)
     break;
   // UNDO
   case BUT39:
    SoundBeep(SoundClick3);
     UndoPixel();
	  if (oldActive != UnDoActive)
	  {   ShowCurrUnDo();
	      oldActive = UnDoActive;
     }
     SetFocus(hGFXWindow);   // for Keyboard to work
     break;
   // Transparent
   case BUT40:
     SoundBeep(SoundClick3);
     TogglePal(hEditWindow, Transparent);                 // return in WM_CHANGE Edit_Command
     SetFocus(hGFXWindow);   // for Keyboard to work
     break;
   // Tool window visible
   case BUT30:
     SoundBeep(SoundSwish2);
     if(Button_GetCheck(hwndCtl))
     {  ToolWindow = TRUE;
     }
     else
     {  ToolWindow = FALSE;
     }

     ToggleTool();           // Toggle The Floating Tool Window
     GFXLeftDrawBar(hGFXWindow);
     SetFocus(hGFXWindow);   // for Keyboard to work
     break;
   // Auto Convert Pasted Image
   case BUT31:
     SoundBeep(SoundClick3);
     PasteTranslate = !PasteTranslate;
     Button_SetCheck(hControl[7], PasteTranslate);  // AutoCVT value   5-1-97
     SetFocus(hGFXWindow);   // for Keyboard to work
     break;
   // restore MAIN Buttons  ( Main Menu )
   case BUT32:
     SoundBeep(SoundSwish2);
	  if ( ( (UnDoActive || DidRotate)
          && HelpMsg(hwnd,CENTER|HELPYESNO, "Changes made",
                                            "You have made changes, continue?",
                                             NULL))
  	    || (!UnDoActive && !DidRotate) )
     {
        EditActive= FALSE;                  // new 8-1-97 show active
        EditDraw  = FALSE;

        DrawFree(FALSE);                    // Keep texture list, Free windows

        DrawLineList(hwnd, FALSE);          // Get rid of freehand, dots, Tool Line mode
        CreateMenuWindow1(hwnd);            // Destroys old Buttons, Creates New Default
        GFXDrawPicture();                   // Draw Client Area picture on right (clear)
        SetFocus(hGFXWindow);               // for Keyboard to work
     }
     break;

   //  HELP
   case BUT33:
     SoundBeep(SoundDrip);
     HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"Graphics Editor Keys");
     SetFocus(hGFXWindow);                  // for Keyboard to work
     break;
     
   //---------------------------------------------------------------------------
   // Select New Texture (EXIT to menu#2)
   //---------------------------------------------------------------------------
   case BUT34:
     SoundBeep(SoundGame);
	  if ( ( (UnDoActive || DidRotate)
          && HelpMsg(hwnd,CENTER|HELPYESNO, "Changes made",
                                            "You have made changes, continue?",
                                            NULL))
		    || (!UnDoActive && !DidRotate) )
     {
        EditActive= FALSE;                  // new 8-1-97 show active
        EditDraw  = FALSE;

        DrawFree(FALSE);                    // keep list, free up Memory Allocations & windows
        SendMessage(hGFXWindow, WM_COMMAND, BUT0, 0);  // simulate Button select DRAW
     }
     break;


//==============================================================================
//                      D R A W I N G  ListBox, Static                         =
//==============================================================================

   case WM_CHANGE:                      // POSTED from our Palette Dlg
     DrawingColor = int(hwndCtl);       // lParam suppose to contain the window handle
     GFXDrawColors(hwnd);
     break;

   // Name box was chosen
   case IDD_TXTREDIT:
     //NotifyMsg(hGFXWindow,"name"); // prototype
     switch (codeNotify)
	  {
	     case EN_CHANGE:                          // name box choosen
           Edit_GetText(hControl[14],EditName,9);
           FindClosestEditMatch();
           if (strncmp(DrawingName,"= ",2) )    // NOT the separator (-ignore our own controls-)
           {
               //static count;
               //Trace(hGFXWindow,10,10,"enchange %d, lastx=%d lastcx=%d",count++,LastPicXloc,LastcxRange);
               GFXDrawPicture();                // Draw Client Area picture on right
           }
           else
           {   DrawingName[0] = '\0';
           }
	        break;

	     case EN_SETFOCUS:	// Must be IN the Edit Control to Find Match
			 EditChange=TRUE;
			 break;

		  case EN_KILLFOCUS:
		    EditChange=FALSE;
		    break;

		  default:
          // NotifyMsg(hGFXWindow,"msg = %d",codeNotify);
		    break;
     }
	  break;

   case lst2:		                     // Drawing List Box
     EditChange=FALSE;
     switch(codeNotify)
     {  // double click select
    	  case LBN_DBLCLK:
		     PostMessage(hGFXWindow,WM_COMMAND,
                       MAKEWPARAM(BUT20,(WORD) GetWindowLong(hGFXWindow,GWL_ID)),
                      (LPARAM)hGFXWindow);
 		   break;
       // change in list box
	    case LBN_SELCHANGE:
        EditNameIndex = (LRESULT)SendMessage(hControl[13], LB_GETCURSEL, 0, 0);

        // Make sure NOT  'TEXTURE END' or 'FLAT START'  or '---------------' Divider Selected

        ListBox_GetText(hControl[13], EditNameIndex, DrawingName);

        if (strncmp(DrawingName,"= ",2) )   // not the separator
        {   //GFXDrawPicture();                // Draw Client Area picture on right
            Edit_SetText(hControl[14], DrawingName); // and this posts en_change above to draw picture
        }
        else
        {  DrawingName[0] = '\0';
        }
		  break;
     }
     break;

   case lst3:
     switch(codeNotify)
     {
	    case LBN_SELCHANGE:               // Click on Line Mode change
	    case LBN_DBLCLK:                  // Double Click
         LIndex = ListBox_GetCurSel(hLineMode);
         LineMode = LIndex +1;
		   break;
     }
     break;

    // NOT USED YET But could read click status here for
    // some as for yet unforseen situation

   case stc1:		                       // Static Color Left Button clicked on
     switch(codeNotify)
     {
	    case STN_CLICKED:                 // Click on Drawing Color
	    case STN_DBLCLK:                  // Double Click on Drawing Color
         TogglePal(hwnd, DrawingColor);
         SetFocus(hGFXWindow);           // for Keyboard to work
	      break;
     }
     break;

   case stc2:		                       // Static Color Left Button clicked on
     switch(codeNotify)
     {
	    case STN_CLICKED:                 // Click on Transparent Color
	    case STN_DBLCLK:                  // Double Click on Transparent Color
         TogglePal(hEditWindow, Transparent);
         SetFocus(hGFXWindow);           // for Keyboard to work
	      break;
     }
     break;
  }         // ID

}

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//   WM_COMMAND msg's from our Editing Window                                //
//                                                                           //
// NOTE :  Picking a new Transparent color returns here after                //
// since gd.hParent is set to the EDIT Window NOT the GFX Window Parent      //
// The DrawingColor returns there, the Transparent here                      //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
#pragma argsused
static void GFXEdit_Command(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  //RECT rc;

//--------------------------------------------------------------


  switch(id)
  {
   case WM_CHANGE:                    // POSTED from our Palette Dlg
    Transparent = int(hwndCtl);       // lParam suppose to contain the window handle
    GFXDrawColors(GetParent(hwnd));   // Update our Color Box's
	 //GetClientRect(hEditWindow, &rc);  // Re-draw with new Transparent color & Check

    //NotifyMsg(hwnd,"gfx %d,%d %d,%d %d",rc.left, rc.top,rc.right,  rc.bottom,hDrawBitmap);
    /*
    MagnifyGraphicTexture(hwnd, rc.left, rc.top,
                                rc.right,  rc.bottom,
                                (byte *)&*lpDrawBits,
                                hDrawBitmap,
                                DrawingMagnify,
                                Transparent);

    if(!FLAT)
       CheckForTransParent(hwnd, DrawBi.bmiHeader.biWidth,RawImageData,Transparent,TRUE);
    */
    break;
  }
}

//-----------------------------------------------------------------------------
// see if coordinates inside the image
// modifies DrawX,DrawY to integral coordinates (still relative to screen window)
//-----------------------------------------------------------------------------


static Bool CheckIfInsideImage(int *DrawX, int *DrawY,int Width,int Height)
{
  if ( *DrawX >= OrgXstart-Width                 // org - width of paste image
    && *DrawX <= OrgXend+DrawingMagnify-1
   &&  *DrawY >= OrgYstart-Height                // org - height of paste image
    && *DrawY <= OrgYend+DrawingMagnify-1 )
  {
	   if (*DrawX >= OrgXend) // round to even number
	   {   *DrawX  = OrgXend;
	   }
	   else
	   {   *DrawX = OrgXstart+( (*DrawX-OrgXstart) / DrawingMagnify ) *DrawingMagnify;
	   }

	   if (*DrawY >= OrgYend)
	   {   *DrawY = OrgYend;
	   }
	   else
	   {   *DrawY = OrgYstart+( (*DrawY-OrgYstart) / DrawingMagnify ) *DrawingMagnify;
	   }

	   return TRUE;
  }

  return FALSE;
}

//------------------------------------------------------------------------------
//  find absolute offset in RAW buffer for the pixel
//  coordinates are relative to -0-
//------------------------------------------------------------------------------

static byte *RawOffset(int4 Xcoord,int4 Ycoord)
{
static   int4 CurrentX,n;

register y,i;

//------------------------------------------------------------------------------


   if (Xcoord < 0     ) return NULL;
   if (Xcoord >= DrawBi.bmiHeader.biWidth)       return NULL;

   if (Ycoord < 0     ) return NULL;
   if (Ycoord >= abs(DrawBi.bmiHeader.biHeight)) return NULL;

   //Trace(hEditWindow,10,4,"raw=%d,%d",XScroll,YScroll);

   if(FlatLump)
   {
      return (RawImageData + Ycoord * 64 + Xcoord);   // current data loc
   }

   if (DidTransp || DidRotate) return NULL;            // already drew over transparent, cant touch it

   // get address of RAW data buffer where disk data was stored

   Yoffsets = (int4 *)&RawImageData[0] + 2;            // 1st Y offset start

   // Do X first -----------------------------------------------------------------

   CurrentX = Xcoord;
   {

     Yorigin  = (byte *) &RawImageData[0] +  Yoffsets[CurrentX]; // start of column

     // we now have the needed (Y) column data - display it

     n            = 1;
     YcolumnStart = Yorigin[0];        // y origin start

     // display a line of pixels for height (xofs,yofs assembles composites)

     while (YcolumnStart != 255 )
     {
         // resync Y origin

	      y = YcolumnStart;              // set start of y

	      YdataBytes = Yorigin[n];       // # of vertical data bits (byte)

	      n += 2;                        // skip over 'null' pixel in data

	      PixelPtr = Yorigin+n;          // address 1st pixel color

	      // Y direction---------------------------------------------------------

	      for (i = 0; i < YdataBytes; i++, PixelPtr++, y++)
	      {
	           //if (y < 0) continue;               // can't happen
	           if (y == Ycoord)  return PixelPtr;   //<<<<<<<<<<<<<<<<<<<<<< this is location
	      }

	      n += YdataBytes + 1;          // skip over written pixels, and the 'null' one

	      YcolumnStart = Yorigin[n++];  // next y origin (+n skips unused byte)
     }

     if (YcolumnStart != 255)
     {   ErrorMsg(NULL,"Image Invalid, Ycolumn != ff?");
     }
   }

   return NULL;
}

//------------------------------------------------------------------------------
//  find absolute offset in Imagebuffer for the pixel
//  coordinates are relative to -0-
//------------------------------------------------------------------------------

static byte *ImageBufferOffset(int4 Xcoord,int4 Ycoord)
{
 static byte *WrkPtr;

//------------------------------------------------------------------------------

   if (Xcoord < 0     ) return NULL;
   if (Xcoord >= DrawBi.bmiHeader.biWidth)       return NULL;   // too big

   if (Ycoord < 0     ) return NULL;
   if (Ycoord >= abs(DrawBi.bmiHeader.biHeight) )return NULL;   // too big

   WrkPtr = (byte *)(VOID **)&*lpDrawBits;

   return (&WrkPtr[0] + Ycoord*((DrawBi.bmiHeader.biWidth + 3 ) & ~3)  +Xcoord); // current data loc
}

//------------------------------------------------------------------------------
//  check to make sure image does not have transparent color in use
//------------------------------------------------------------------------------

static Bool CheckForTransParent(HWND hwnd, int4 Xsize, byte*RawWrkData,int2 colortocheck,Bool TellMsg)
{

   int4  CurrentX;            // 2
   int4  i,n;

   // image is displayed for each Y column, then move to next X, etc.

   Yoffsets  = (int4 *)&RawWrkData[0] + 2;                 // 1st Y offset start

   // Do X first -----------------------------------------------------------------

   for (CurrentX = 0; CurrentX < Xsize ; CurrentX++)
   {
       Yorigin  = (byte *) &RawWrkData[0] +  Yoffsets[CurrentX]; // start of Y column data

       n            = 1;
       YcolumnStart = Yorigin[0];         // y origin start

       // Y is not continous, can skip for transparent areas

       while (YcolumnStart != 255 )
       {
	       YdataBytes = Yorigin[n];       // # of vertical data bits (byte)

	       n += 2;                        // skip over 'null' pixel in data

	       PixelPtr = Yorigin+n;          // address 1st pixel color

	       // Y direction---------------------------------------------

	       for (i = 0; i < YdataBytes; i++, PixelPtr++)
	       {
	            if (*PixelPtr== colortocheck)     // data location
               {
                   if (TellMsg)
                   {
                      HelpMsg(hwnd,CENTER|HELPINFO, "Transparent Color",
                                                    "!! Warning !!",
                                                    "",
                                                    "Current tranparent color",
                                                    "is in use by this Image.",
                                                     "Switch to a different Color",
                                                     NULL);
                       //NotifyMsg(hwnd,"!! Warning !!\n\n"
                       //"Current tranparent color\n"
                       //"is in use by this Image.\n"
                       //"Switch to a different Color");
                   }
                   return TRUE;
               }
	       }

	       n += YdataBytes + 1;          // skip over written pixels, and the 'null' one

          YcolumnStart = Yorigin[n++];  // next y origin
       }
   }
   return FALSE;
}


//------------------------------------------------------------------------------
//  check to make see if BitMAP image has transparent color (holes)
//------------------------------------------------------------------------------

static Bool CheckBitMapForHoles(int4 Width,
                                int4 Height,
                                byte* UserImageBufr,
                                int2 transparent)
{
   int4  x,y;

   int4  j;
   int4  OrgPixLoc;
   int4  PixLoc;

   int2  Xalign;               // row bytes aligned

//-----------------------------------------------------------------------------

   // get size of the buffer

   Xalign  = ( Width + 3) & ~3;                      // row bytes aligned

   Height = abs(Height);                             // make sure positive   7-1-97

   for (y = 0; y < Height;  y++)                     // Y
   {    OrgPixLoc = (y)*Xalign;
        for (j=Width-1,x=0; x< Width; j--, x++)      // X-------
	     {    PixLoc = OrgPixLoc+ j;                  // offset
             if (UserImageBufr[PixLoc] == transparent)
             {   return TRUE;
             }
        }  // end x
   }     // end y

   return FALSE;
}


//-------------------------------------------------------------------------
//  Track each address and the old pixel value
//--------------------------------------------------------------------------

static void TrackUndo(int2 xloc, int2 yloc, byte color)
{
UnDoListPtr TempUnDoArray1;
UnDoListPtr TempUnDoArray2;
int4        i;

 //Trace(hEditWindow,100,4,"undo=%hd,%hd",xloc,yloc);
 //Trace(hEditWindow,500,4,"scroll=%d,%d",cxHorzScrollPos,cyVertScrollPos);

 if (UnDoArray)   //   5-1-97
 {
     if (UnDoActive >= UnDoMax)                    // shift  old data
     {  TempUnDoArray1 = UnDoArray;                //
        TempUnDoArray2 = UnDoArray+UnDoMaxShift;   // shift from here

	     for (i=UnDoMaxShift; i<  UnDoMax; i++, TempUnDoArray1+=1,TempUnDoArray2+=1)
	     {    TempUnDoArray1->xloc  = TempUnDoArray2->xloc;
	          TempUnDoArray1->yloc  = TempUnDoArray2->yloc;
	          TempUnDoArray1->color = TempUnDoArray2->color;
	     }

	     UnDoActive = UnDoMax - UnDoMaxShift+1; // number active now
     }

     UnDoArray[UnDoActive].xloc  = xloc;
     UnDoArray[UnDoActive].yloc  = yloc;
     UnDoArray[UnDoActive].color = color;

     UnDoActive++;
 }

}


//-------------------------------------------------------------------------
// undo the drawing
//-------------------------------------------------------------------------
static void UndoPixel()
{  byte *RawLoc;
   int4  PixelX;  //2
   int4  PixelY;
   int4  i,k;
UnDoListPtr TempUnDoArray;
   HDC hdc;
   HPALETTE hOldPal;

//-------------------------------------------------------------------------

    if (!UnDoActive) return;   // nothing left ++ DidTransp ??
    if (!EditDraw )  return;
    if (!UnDoArray)  return;   //   8-1-97

    hdc = GetDC(hEditWindow);
    hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
    RealizePalette(hdc);

    UnDoActive --;
    TempUnDoArray = UnDoArray+UnDoActive;
    //Trace(hEditWindow,300,4,"undo=%hd,%hd",TempUnDoArray->xloc,TempUnDoArray->yloc);
    //Trace(hEditWindow,500,4,"scroll=%d,%d",cxHorzScrollPos,cyVertScrollPos);

    // now do regular image buffer

    RawLoc = ImageBufferOffset(TempUnDoArray->xloc,TempUnDoArray->yloc);

    if (RawLoc) // legitimate location
    {
       *RawLoc = TempUnDoArray->color;      // old color

        PixelX = OrgXstart + (TempUnDoArray->xloc *DrawingMagnify )- cxHorzScrollPos;
        PixelY = OrgYstart + (TempUnDoArray->yloc *DrawingMagnify )- cyVertScrollPos;
        //Trace(hEditWindow,300,24,"undo=%d,%d Org=%d,%d",PixelX,PixelY,OrgXstart,OrgYstart);

        for (i = 0; i < DrawingMagnify; i++, PixelX++ )     //x
        {
             for (k=0; k< DrawingMagnify; k++)              //y
	          {
                 SetPixelV(hdc, PixelX, PixelY+k,PALETTEINDEX(TempUnDoArray->color));
             }
        }

        // now undo the raw image buffer if no rotate or transparent

        if (!DidTransp && !DidRotate )
        {   RawLoc = RawOffset(TempUnDoArray->xloc,TempUnDoArray->yloc);
            if (RawLoc)
            {  *RawLoc = TempUnDoArray->color;    // old color
            }
        }
    }

    SelectPalette(hdc, hOldPal, FALSE);
    ReleaseDC(hEditWindow, hdc);
}

//--------------------------------------------------------------------------

static  int4  Xpixloc,Ypixloc;  // to speed this up

//--------------------------------------------------------------------------
// store a pixel in the actual image data buffer (from disk)
//--------------------------------------------------------------------------

static void StorePixel(void)
{
static byte *RawLoc;
static int4  PixelX;
static int4  PixelY;

  register i,k;

static HDC hdc;
static HPALETTE hOldPal;

//-------------------------------------------------------------------------

  hdc = GetDC(hEditWindow);
  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  RealizePalette(hdc);

  // address in imagebuffer that's magnified = 1

  RawLoc = ImageBufferOffset(Xpixloc,Ypixloc);

  if (RawLoc) // legitimate location
  {
      TrackUndo(Xpixloc, Ypixloc, *RawLoc);

      PixelX = OrgXstart +  Xpixloc * DrawingMagnify -cxHorzScrollPos;
      PixelY = OrgYstart +  Ypixloc * DrawingMagnify -cyVertScrollPos;

      *RawLoc = DrawingColor;                            // store new color

      for (i = 0; i < DrawingMagnify; i++, PixelX++ )    //x
      {
	       for (k=0; k< DrawingMagnify; k++)              //y
	       {    SetPixelV(hdc, PixelX, PixelY+k,PALETTEINDEX(DrawingColor));
	       }
      }

      // now raw address buffer modify
      if (FlatLump)
      {   RawLoc = RawOffset(Xpixloc,Ypixloc);
          if (RawLoc)                             // pixel did not exist
          {  *RawLoc = DrawingColor;              // store new color
          }
      }
      else
      if (!DidTransp && !DidRotate)
      {   RawLoc = RawOffset(Xpixloc,Ypixloc);
          if (DrawingColor == Transparent         // drawing with transparent color
          || !RawLoc)                             // pixel did not exist
          {   DidTransp++;
          }
          else
          {  *RawLoc = DrawingColor;              // store new color
          }
      }
  }

  SelectPalette(hdc, hOldPal, FALSE);
  ReleaseDC(hEditWindow, hdc);
}



//--------------------------------------------------------------------------
// draw a line in memory buffer
//---------------------------------------------------------------------------
static void DrawTheLine(int4 x0, int4 y0,   // from
                	 	   int4 x1, int4 y1)   // to
{
register  int4 lo,hi;

static double slope, b;
   ShowCursor(FALSE);   //   5-1-97

	if ( y0 == y1 )      // Horizontal line
	{
	     lo = (x0 < x1 ? x0 : x1);
	     hi = (x0 > x1 ? x0 : x1);

	     Ypixloc = y0;

	     for (Xpixloc=lo ; Xpixloc <= hi; Xpixloc++ )
	     {    StorePixel();
	     }
	}
	else
	if ( x0 == x1 )      // Vertical line
	{
	     lo = (y0 < y1 ? y0 : y1);
	     hi = (y0 > y1 ? y0 : y1);

	     Xpixloc = x0;

	     for (Ypixloc=lo ; Ypixloc <= hi; Ypixloc++ )
	     {    StorePixel();
	     }
	}
	else
	{   // Equation:  y = mx + b
	    slope =  (double)( y1 - y0 ) /(double)(x1 - x0);
	    b     =  (double)( y0 - slope*(double)x0 );

	    // slope > -1 < 0  or slope > 0 < 1

	    if ( ( (slope < 0) ? slope > -1 : slope < 1) )
	    {
		    lo = (x0 < x1 ? x0 : x1);
	       hi = (x0 > x1 ? x0 : x1);
					  //<
		    for ( Xpixloc=lo; Xpixloc <=hi; Xpixloc++ )
		    {
		       Ypixloc = (int4) (slope*(double) Xpixloc ) + b ;
		       StorePixel();
		    }
	    }
	    else
	    {
		    lo = (y0 < y1 ? y0 : y1);
		    hi = (y0 > y1 ? y0 : y1);

	       for ( Ypixloc=lo; Ypixloc <=hi; Ypixloc++ )
		    {
		       Xpixloc = (int4)(  ( (double)Ypixloc - b )/slope ) ;
		       StorePixel();
		    }
	    }
	}
   ShowCursor(TRUE);   //   5-1-97
}

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------

static void CircleExit(HPALETTE hOldPal, HDC hdc, HPEN hOldPen, int OldROP)
{
  SelectPalette(hdc, hOldPal, FALSE);
  SetROP2(hdc, OldROP);
  SelectPen(hdc, hOldPen);
  ReleaseDC(hEditWindow, hdc);
}

/////////////////////////////////////////////////////////////////
// Called from OnKey  'O'  key to Draw A circle on the screen  //
/////////////////////////////////////////////////////////////////

static void CircleDraw(HWND hwnd,Bool solid) //   5-1-97 added hwnd
{
  HPALETTE hOldPal;
  HPEN     hOldPen;

static  double   Radius   = 32;
static  double   oldRadius;

  UINT     KeyData;
  MSG      msg;
  HDC      hdc;
  int      OldROP;
  int2     oldButton;

//=====================================================

 hdc = GetDC(hEditWindow);
 hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
 RealizePalette(hdc);
 MouseButton = 0;

 oldRadius = Radius;

 OldROP = SetROP2(hdc, R2_XORPEN);   // write mode xor
 hOldPen= SelectPen(hdc, gd.hPens[WHITE] );

 // SetCapture(hEditWindow);  not req'd   tested
 oldDrawX = -1;

  while (MouseButton == 0)
  {
     WaitMessage();
     KeyData =GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);

     if(msg.message == WM_MOUSEMOVE || KeyData)
     {
       if ( CheckIfInsideImage(&DrawX,&DrawY,
                               ((DrawX + (Radius*2)) * DrawingMagnify),
                               ((DrawY + (Radius*2)) * DrawingMagnify)) )
       {
         if (KeyData)
         {
           switch(KeyData)
           {
             //case(VK_EQUAL):  // Radius +
             case('+'):
             case('='):
              Radius+=1.0;
		        break;
             //case(VK_MINUS):  // Radius -
             case('-'):
             case('_'):
              if (Radius > 1) Radius-=1.0;
		        break;
             case(VK_ESCAPE):
              MouseButton = 2;
              break;
           }// end switch(KeyData)

         }

         //if(msg.message == WM_MOUSEMOVE || KeyData)
         {
	        if (oldDrawX != DrawX
            || oldDrawY != DrawY
            || KeyData
            || Radius != oldRadius)
	        {
              //KeyData = 0;    // reset so will work   5-1-97

  	           //restore prior
	           if (oldDrawX != -1)
              {  DrawCircle(hdc, oldDrawX,oldDrawY,oldRadius);// erase old
	           }

	           // for next erase
	           oldDrawX = DrawX;
	           oldDrawY = DrawY;
              oldRadius= Radius;

              DrawCircle(hdc,DrawX,DrawY,Radius);

	        }
         }   // MouseMove
       }    // If InsideImage ?
     }     // MouseMove or KeyDown
  }       // while no Mouse Button

//  ReleaseCapture(); //   5-1-97

  oldButton = MouseButton;
  // now do actual copy---

  // restore prior
  if (oldDrawX != -1)
  {   DrawCircle(hdc, oldDrawX,oldDrawY,oldRadius);// erase old
  }

  //   nothing done       Right button or Esc

  if (  oldDrawX == -1
   ||  oldButton  == 2 )
  {
      //oldDrawX = -1;
      CircleExit(hOldPal, hdc, hOldPen, OldROP);
      return ;
  }

  FillCircle((byte *)&*lpDrawBits,RelativeXcoord,RelativeYcoord,Radius/DrawingMagnify,Transparent,DrawingColor, 1, solid);

  DidRotate = TRUE;            // data stored only in image buffer
  UnDoActive= 0;               // can't undo anymore

  SelectPalette(hdc, hOldPal, FALSE);
  SetROP2(hdc, OldROP);
  SelectPen(hdc, hOldPen);
  ReleaseDC(hEditWindow, hdc);

  MagnifyGraphicTexture(hEditWindow,
                        EditRC.left,  EditRC.top,      // origin
                        EditRC.right, EditRC.bottom,   // dimensions
                       (byte *)&*lpDrawBits,
                        hDrawBitmap,
                        DrawingMagnify,
                        Transparent);

   //oldDrawX = -1;
}

//
//-------------------------------------------------------------------------
// Set a pixel in our bitmap and on the screen
// input X and Y are actual coordinates
//-------------------------------------------------------------------------
//
void DrawPixel(HDC hdc, int4 x, int4 y, int4 relX, int4 relY)
{
 int2 i,k;

 //-----------------------------

 RawLoc = ImageBufferOffset(relX,relY);
 if (!RawLoc) return;

 TrackUndo(relX,relY,*RawLoc);        //AdjustedX,AdjustedY, *RawLoc);
 *RawLoc = DrawingColor;              // store new color

 RawRaw = RawOffset(relX,relY);       //
 if (RawRaw) *RawRaw = DrawingColor;
 //Trace(hEditWindow,100,24,"undo=%d,%d",x,y);

 for (i = 0; i < DrawingMagnify; i++, x++ )          //x
 {    for (k=0; k< DrawingMagnify; k++)              //y
      {
         SetPixelV(hdc, x, y+k,PALETTEINDEX(DrawingColor));
      }
 }

}

/////////////////////////////////////////////////////////////////////////////
// Create Window
/////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL GFX_OnCreate(HWND hwnd, CREATESTRUCT FAR* lpCreateStruct)
{
//==================================================================

  LastPicXloc = -1;            // first time
  LastcxRange = -1;            // force clear

  CreateMenuWindow1(hwnd);     // Init the Main Buttons

  if (Transparent == 0)        //   5-1-97 a way to reset to defaults later?
  {  // safety code
     if (GameVersion <=2)
     {   Transparent = 251;    // DOOM , DOOM2
     }
     if (GameVersion == 3)     // Heretic
     {   Transparent = 172;
     }
     if (GameVersion == 4)     // Hexen
     {   Transparent = 236;
     }
     if (GameVersion == 5)     // Strife
     {   Transparent = 151;
     }
  }
//  strcpy(StatMsg, "Graphic Editor Init Complete" );

  //if(GetFocus()!= hGFXWindow)  SetFocus(hGFXWindow);   // for Keyboard to work
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Create Window
/////////////////////////////////////////////////////////////////////////////
#pragma argsused
BOOL GFX_EditOnCreate(HWND hwnd, CREATESTRUCT FAR* lpCreateStruct)
{
//==================================================================

  LastPicXloc = -1;          // first time
  LastcxRange = -1;          // force clear

  return TRUE;
}
///////////////////////////////////////////////////////
// Generated by X button in top right hand corner    //
///////////////////////////////////////////////////////

static void GFX_OnClose(HWND hwnd)
{
   switch(Mode)
   {
     case MAIN:
      SendMessage(hwnd, WM_COMMAND, BUT9, 0);   // simulate Exit button pushed
      return;//break;

     case CVT:
      SendMessage(hwnd, WM_COMMAND, BUT17, 0);  // simulate Exit button pushed
      return;//break;

     case DRAW:
      SendMessage(hwnd, WM_COMMAND, BUT28, 0);  // simulate Exit button pushed 1-1-98 forgot to update
      return;//break;
   }

   FORWARD_WM_CLOSE(hwnd, Deep_DefProc);
}

////////////////////////////////////////////////////////////////////////////////
//  Get rid of our resources, if any   /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#pragma argsused
static void GFX_OnDestroy(HWND hwnd)
{
   DeePDestroyWindow(gd.hPalWindow);
   gd.hPalWindow = NULL;

   SetForegroundWindow(gd.hMainWindow);
   SetFocus(gd.hMainWindow);
}


/////////////////////////////////////////////////////////
// WM_RBUTTONDOWN  -  check click right and dbl click right
/////////////////////////////////////////////////////////
#pragma argsused
//void GFXEdit_OnRButtonUp(HWND hwnd, int x, int y, UINT flags)
static void GFX_OnRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
//MouseButton = 2;
   switch(Mode)
   {
     case MAIN:
      SendMessage(hwnd, WM_COMMAND, BUT9, 0);       // simulate Exit button pushed
      break;

     case CVT:
      SendMessage(hwnd, WM_COMMAND, BUT17, 0);      // simulate Exit button pushed
      break;

     case DRAW:
      if (!EditDraw)
      {   SendMessage(hwnd, WM_COMMAND, BUT25, 0);  // simulate Exit button pushed
      }
      else
      {   SendMessage(hwnd, WM_COMMAND, BUT34, 0);  // simulate Select Texture Buttons pushed
      }

      break;
   }

  //PostMessage(hGFXWindow,WM_COMMAND,
  //           MAKEWPARAM(BUT25,(WORD)GetWindowLong(hGFXWindow, GWL_ID)), 0);

  //SetFocus(hGFXWindow);   // for Keyboard to work
}

////////////////////////////////////////////////////////////////////////////////
// WM_LBUTTONDOWN -  This is the Main Left Button Down Handler                //
////////////////////////////////////////////////////////////////////////////////

#pragma argsused
static void GFX_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
  int Xvalue, Yvalue, i;

//---------------------------------------------------------

  // User Clicked on a Tool Button

  if( (Mode == DRAW)
   && (EditDraw)
   && !ToolWindow                //   5-1-97
   && (x >  TOOLXSTART)
   && (x < (TOOLXSTART + (22*3)))
   && (y >  TOOLYSTART)
   && (y < (TOOLYSTART + (22*2))))
  {
    Xvalue = ((x - TOOLXSTART) / 22);
    Yvalue = ((y - TOOLYSTART) / 22);

    if( ToolActive[ Xvalue + (Yvalue*3) ] == 1)  return;    // SAME BUTTON

    if( ToolActive[4] == 0 && ( Xvalue + (Yvalue*3))== 4 )  // Line Mode not active
    {   DrawLineList(hGFXWindow, TRUE);                     // This is the 3 line Tool List Box for Freehand, dots etc.
    }

    if( ToolActive[4] == 1 && ( Xvalue + (Yvalue*3))!= 4 )  // Line Mode Active already
    {   DrawLineList(hwnd, FALSE);
    }

    // clear all
    for(i = 0; i < 6; i++)
    {   ToolActive[i] = 0;
    }
    // calc location
    i =Xvalue + (Yvalue*3);
    if( i < 0 )  i = 0;
    if( i > 5 )  i = 5;

    // set this one active

    ToolActive[i] = 1;

    SoundBeep(SoundDrip);       // sound

    GFXDrawTools(hwnd);
    strcpy(StatMsg, ToolMsg[i]);
    GFXDrawStatus(hGFXWindow);   // status bar on bottom

    SetFocus(hGFXWindow);   // for Keyboard to work
    return;
   }

}

//-------------------------------------------------------------------------
// EyeDropper
//-------------------------------------------------------------------------
void EyeDropper(void)
{
//  HPALETTE hOldPal;
//  HDC      hdc;
//  COLORREF cr;
  POINT    p;

  //--------------------------------------------------------------

  GetCursorPos(&p);
  ScreenToClient(hEditWindow, &p);

  // Find OUR Window Position

  DrawX = p.x;
  DrawY = p.y;

  if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
  {   InsideImage = TRUE;
  }
  else
  {   InsideImage = FALSE;
  }

  if (InsideImage)
  {    /*
      hdc = GetDC(hEditWindow);
      hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
      RealizePalette(hdc);

      cr = GetPixel(hdc, DrawX, DrawY);    // Get Color Reference
      DrawingColor = GetNearestPaletteIndex(gd.hDoomPal, cr);

      SelectPalette(hdc, hOldPal, FALSE);
      ReleaseDC(hEditWindow, hdc);
      */
      DrawingColor = CurrentPictureColor;       // get raw color   6-1-97
      GFXDrawColors(hGFXWindow);
  }

  SetFocus(hGFXWindow);   // for Keyboard to work
}

//
//-------------------------------------------------------------------------
// Zoom in/out for Editing using Magnify GLASS tool (centers on cursor)
//-------------------------------------------------------------------------
//
void ZoomGraphic(BOOL in)
{
//  HPALETTE hOldPal;
//  HDC      hdc;
//  MSG      msg;
  RECT     rc;
  int      magnify;
//========================================================

	GetClientRect(hEditWindow, &rc);

//  SetCapture(hEditWindow);
   magnify    = DrawingMagnify;

//   msg.message = 0;
//   while(1)
   {
//      WaitMessage();
//      GetMouseCoords(hEditWindow,&msg,NULL,NULL,NULL,&MouseButton);  //     5-1-97

//      if (msg.message == WM_LBUTTONUP)  break;
//      if (msg.message == WM_RBUTTONUP)  break;
//      oldRelativeX = RelativeXcoord*DrawingMagnify;
//      oldRelativeY = RelativeYcoord*DrawingMagnify;

      if (!in)
      {   DrawingMagnify --;
          if(DrawingMagnify < 1)  DrawingMagnify = 1;
      }
      else
      {   DrawingMagnify ++;
          if(DrawingMagnify > MAXZOOM)  DrawingMagnify = MAXZOOM;     //   6-1-97
      }

      if (DrawingMagnify != magnify)
      {
          //hdc = GetDC(hEditWindow);          //   5-1-97
  	       //hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	       //RealizePalette(hdc);
          //FillRect(hdc, &rc, gd.hBrushes[MenuColor]);//BackThingColor]);
          //LastPicXloc = -1;

          // see where cursor is and center on that

          int4  xsize,ysize;//,maxPageX,maxPageY;
          int4  cxRange,cyRange;

          xsize = BitmapWidth (hDrawBitmap)* DrawingMagnify;
          ysize = BitmapHeight(hDrawBitmap)* DrawingMagnify;

          if (xsize > rc.right || ysize > rc.bottom)
          {
              cxHorzScrollPos = RelativeXcoord*DrawingMagnify/2;
              cyVertScrollPos = RelativeYcoord*DrawingMagnify/2;

              cxRange = xsize - rc.right;     // Graphic Xsize - Xsize of window
              cyRange = ysize - rc.bottom;    //         Ysize - Ysize of Window

              // reset global variable if a scrollbar is turned off
              if (cxRange <=0)
              {   cxRange = 0;
                  cxHorzScrollPos = 0;
              }
              if (cyRange <=0)
              {   cyRange = 0;
                  cyVertScrollPos = 0;
              }

              // Set the ranges we've calculated (0 to 0 means invisible scrollbar) (SetCrollinfo new 95 later)

              cxRange = ( (cxRange+DrawingMagnify-1)/DrawingMagnify )*DrawingMagnify;
              cyRange = ( (cyRange+DrawingMagnify-1)/DrawingMagnify )*DrawingMagnify;

              SetScrollRange(hEditWindow, SB_HORZ, 0, cxRange, TRUE);
              SetScrollRange(hEditWindow, SB_VERT, 0, cyRange, TRUE);

              if (cxHorzScrollPos > cxRange)
              {   cxHorzScrollPos = cxRange;
              }
              if (cyVertScrollPos > cyRange)
              {   cyVertScrollPos = cyRange;
              }

              if (cxHorzScrollPos < 0) cxHorzScrollPos = 0;
              if (cyVertScrollPos < 0) cyVertScrollPos = 0;

              //if (cxHorzScrollPos || cyVertScrollPos)
              {
                  SetScrollPos(hEditWindow, SB_HORZ,cxHorzScrollPos, TRUE);
                  SetScrollPos(hEditWindow, SB_VERT,cyVertScrollPos, TRUE);
              }
          }

          //Trace(hGFXWindow,10,100,"zoom xs=%d,ys=%d, pg %d,%d",cxHorzScrollPos,cyVertScrollPos,maxPageX,maxPageY);

          MagnifyGraphicTexture(hEditWindow, EditRC.left, EditRC.top,
                                             EditRC.right,EditRC.bottom,
                                            (byte *)&*lpDrawBits,
                                             hDrawBitmap,
                                             DrawingMagnify,
                                             Transparent);

          //SelectPalette(hdc, hOldPal, FALSE);
          //ReleaseDC(hEditWindow, hdc);

      }
//      WaitMessage();                                          //   5-1-97
   }

   if(EditDraw) ShowCurrUnDo();

   //   ReleaseCapture();

   SetFocus(hGFXWindow);   // for Keyboard to work
}

////////////////////////////////////////////////////////////////////////////////
// WM_LBUTTONDOWN -  This is the Main Left Button Down Handler                //
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
static void GFXEdit_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
  HPALETTE hOldPal, hOldPal1;
  HPEN     hOldPen;
  HBRUSH   hOldBrush, hDrawBrush;
  static   HPEN hPenRed;
  static   HPEN hPenDraw;
  int      OldROP;
//  int      MemX, MemY;  //PixelX, PixelY,
//  POINT    pt;
//  POINTS   p;
  MSG      msg;
  HDC      hdc, hdcMem;
  int      CopyWidth, CopyHeight;
  int2     OldColor;
  HBITMAP  hOldBitmap;
  COLORREF cr;

//UINT     TempColor;
//  int      devcaps;
//  Bool     flood;

//=============================================================

  MouseButton = 1;                                                //   5-1-97
  GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton); //   5-1-97 set relative

  if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
  {   InsideImage = TRUE;
  }
  else
  {   InsideImage = FALSE;
  }


  //    0          1          2         3       4      5       6         7
  // Magnify, Eye Dropper, Eraser, Flood Fill, Pen, Select, Spray Can, Text

  // magnify - zoom in
  if(ToolActive[0])                  // Magnify  -  Zoom In
  {
     ZoomGraphic(TRUE);              // True=IN
  }
  // Eye Dropper
  else
  if(ToolActive[1])                  // Eye Dropper
  {
     EyeDropper();
  }
  // eraser
  else
  if(ToolActive[2])                  // Eraser
  {
     if( Mode == DRAW && EditDraw && InsideImage)
     {
         SetCapture(hwnd);

         hdc = GetDC(hwnd);
         hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
         RealizePalette(hdc);

         OldColor     = DrawingColor;
         DrawingColor = Transparent;
         RawXlast     =-1;
   		oldRelativeXcoord = RelativeXcoord;
		   oldRelativeYcoord = RelativeYcoord;

         while(msg.message != WM_LBUTTONUP)
         {
              WaitMessage();
              GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);

              if(msg.message == WM_MOUSEMOVE)
              {
                   if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
                   {
                      if (RelativeXcoord != RawXlast || RelativeYcoord != RawYlast)
		                {
       		              RawXlast = RelativeXcoord;
		                    RawYlast = RelativeYcoord;

		                    DrawTheLine(oldRelativeXcoord,oldRelativeYcoord,   // from
		                                   RelativeXcoord,   RelativeYcoord);  // to

               		     oldRelativeXcoord = RelativeXcoord;
		                    oldRelativeYcoord = RelativeYcoord;

	                       if (oldActive != UnDoActive)
       	                 {   ShowCurrUnDo();
	                           oldActive = UnDoActive;
                          }
                      }
                   }
              }
         }

         DrawingColor = OldColor;
		   //oldRelativeXcoord = -1;
		   //oldRelativeYcoord = -1;

	      //RawXlast = -1;

         ReleaseCapture();

         SelectPalette(hdc, hOldPal, FALSE);
         ReleaseDC(hwnd, hdc);
     }
  }
  else
  // Flood fill=================================================================
  if(ToolActive[3])             // Flood Fill
  {
     if( InsideImage)//CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
     {
        WaitMessage();             // for release
        GetGFXMouseCoords(hEditWindow,&msg, NULL,&DrawX, &DrawY,NULL);

        hdc     = GetDC(hEditWindow);
        //devcaps = GetDeviceCaps(hdc, RASTERCAPS); //   5-1-97 don't know why I get bad value???
        //Trace(hEditWindow,10,4,"caps=%04x %04x",devcaps,devcaps & RC_FLOODFILL);
        //
        //if( !(devcaps & RC_FLOODFILL) ) //flood = TRUE;
        //{
        //   NotifyMsg(hEditWindow, "Sorry, you Video driver and/or Device Context\ndoes not seem to support Flood Fill (RASTERCAPS)");
        //   if(GetFocus()!= hGFXWindow)  SetFocus(hGFXWindow);   // for Keyboard to work
        //   return;
        //}

        hOldPal = SelectPalette(hdc, gd.hDoomPal, TRUE);
        RealizePalette(hdc);

        hDrawBrush = CreateSolidBrush(PALETTEINDEX(DrawingColor) );

        hOldBrush  = SelectBrush(hdc, hDrawBrush);

        //cr = GetPixel(hdc, DrawX, DrawY);    // Get Color Reference
        //TempColor = GetNearestPaletteIndex(gd.hDoomPal, cr);

        //TempColor = CurrentPictureColor;       // get color reference direct from raw global

        if(CurrentPictureColor == DrawingColor) // can't fill in same color as drawing - just ignore (nomsg)   5-1-97
        {
            SelectPalette(hdc, hOldPal, FALSE);
            SelectBrush  (hdc, hOldBrush);

            ReleaseDC  (hEditWindow, hdc);
            DeleteBrush(hDrawBrush);
            SetFocus(hGFXWindow);              // for Keyboard to work
            return;
        }

        cr = PALETTEINDEX(CurrentPictureColor);//TempColor);

        ExtFloodFill(hdc, DrawX, DrawY, cr, FLOODFILLSURFACE);

        hdcMem     = CreateCompatibleDC(hdc);
        hOldBitmap = SelectBitmap (hdcMem, hDrawBitmap);  // Original Bitmap
        hOldPal1   = SelectPalette(hdcMem, gd.hDoomPal, FALSE);

        SelectBrush  (hdc, hOldBrush);                   // restore brush

        // Update the Dib's BITS as well as the screen
        hOldBrush  = SelectBrush(hdcMem, hDrawBrush);    // brush for this one
        ExtFloodFill (hdcMem, RelativeXcoord, RelativeYcoord, cr, FLOODFILLSURFACE);

        SelectPalette(hdcMem, hOldPal1, FALSE);
        SelectBrush  (hdcMem, hOldBrush);
        SelectBitmap (hdcMem, hOldBitmap);
        DeleteDC(hdcMem);

        SelectPalette(hdc, hOldPal, FALSE);
        ReleaseDC(hEditWindow, hdc);

        DeleteBrush(hDrawBrush);

        MagnifyGraphicTexture(hEditWindow, EditRC.left, EditRC.top,
                                           EditRC.right,EditRC.bottom,
                                          (byte *)&*lpDrawBits,
                                           hDrawBitmap,
                                           DrawingMagnify,
                                           Transparent);

        DidRotate = TRUE;                        // data stored only in image buffer

        if (CurrentPictureColor == Transparent)  // have to recalculate
        {
            DidTransp++;
        }
     }
  }
  else
  // Line draw (pen)========================================================
  if(ToolActive[4])             // Line Draw ,(line,dots,free)
  {
     // If Draw Mode and Edit Button Pushed and Cursor inside Graphic Area

     if( Mode == DRAW && EditDraw && InsideImage)
     {
         SetCapture(hwnd);

         hdc = GetDC(hwnd);
         hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
         RealizePalette(hdc);

         RawXlast     =-1;
   		oldRelativeXcoord = RelativeXcoord;
		   oldRelativeYcoord = RelativeYcoord;

         switch(LineMode)
         {   // FreeHand=========
             case 1:
                while(msg.message != WM_LBUTTONUP)
                {
                     WaitMessage();
                     GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);  //   5-1-97

                     if(msg.message == WM_MOUSEMOVE)
                     {
                        if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
                        {
	                        if (RelativeXcoord != RawXlast || RelativeYcoord != RawYlast)
		                     {
		                        RawXlast = RelativeXcoord;
		                        RawYlast = RelativeYcoord;;

		                        DrawTheLine(oldRelativeXcoord,oldRelativeYcoord,   // from
		                                       RelativeXcoord,   RelativeYcoord);  // to

               		         oldRelativeXcoord = RelativeXcoord;
		                        oldRelativeYcoord = RelativeYcoord;

	                           if (oldActive != UnDoActive)
	                           {   ShowCurrUnDo();
	                               oldActive = UnDoActive;
                              }
                           }
                        }
                     }
                }
                break;
             // Dots ===================
             case 2:
                DrawPixel(hdc, DrawX,DrawY, RelativeXcoord,RelativeYcoord);                //PixelX, PixelY);
                break;
             // Line Mode===============
             case 3:
                MagnifyAdj = DrawingMagnify/2;

                hPenRed   = CreatePen(PS_SOLID, DrawingMagnify, PALETTEINDEX(GameColors[RED]));
                hPenDraw  = CreatePen(PS_SOLID, DrawingMagnify, PALETTEINDEX(DrawingColor));
                hOldPen   = SelectPen(hdc, hPenRed );
		          OldROP    = SetROP2(hdc, R2_XORPEN);   // write mode xor
                oldDrawX  = -1;
                prevDrawX = -1;

                while(msg.message != WM_LBUTTONUP)
                {
                   WaitMessage();                       //   5-1-97
                   GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);

                   if(msg.message == WM_MOUSEMOVE)
                   {
                       if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
                       {
		                     SetROP2(hdc, R2_XORPEN);   // write mode xor
                           SelectPen(hdc, hPenRed );

                           if (DrawX != prevDrawX || DrawY != prevDrawY)
		                     {
                              ShowCursor(FALSE);

                              if (oldDrawX != -1)
			                     {
                                 MoveToEx(hdc, oldDrawX+MagnifyAdj, oldDrawY+MagnifyAdj, NULL);
                                 LineTo  (hdc, prevDrawX+MagnifyAdj,prevDrawY+MagnifyAdj);
			                     }
			                     else // save start
			                     {  oldDrawX = DrawX;
			                        oldDrawY = DrawY;
                                 oldRelativeXcoord = RelativeXcoord;
                                 oldRelativeYcoord = RelativeYcoord;
			                     }

			                     // for next erase
			                     prevDrawX = DrawX;
			                     prevDrawY = DrawY;

                              MoveToEx(hdc, oldDrawX+MagnifyAdj, oldDrawY+MagnifyAdj, NULL);
                              LineTo  (hdc,    DrawX+MagnifyAdj,    DrawY+MagnifyAdj);

                              ShowCursor(TRUE);
                           }
                       }
                   }

                }

                if(oldDrawX != -1)  // in case just click and get UP msg right away
                {
                    ShowCursor(FALSE);
                    // erase old line
                    MoveToEx(hdc, oldDrawX+MagnifyAdj, oldDrawY+MagnifyAdj, NULL);
                    LineTo  (hdc, prevDrawX+MagnifyAdj,prevDrawY+MagnifyAdj);
                    ShowCursor(TRUE);

		              SetROP2(hdc, R2_COPYPEN);   // write mode COPY
		              SelectPen(hdc, hPenDraw );

                    // the actual line
	                 DrawTheLine(RelativeXcoord,RelativeYcoord,        // from
			                       oldRelativeXcoord, oldRelativeYcoord);// to
                }

		          SelectPen(hdc, hOldPen );
		          SetROP2(hdc, OldROP);
                DeletePen(hPenDraw);
                DeletePen(hPenRed);
                break;
             }

             //oldRelativeXcoord = -1;
		       //oldRelativeYcoord = -1;

	          //oldDrawX = -1;
	          //prevDrawX= -1;
	          //RawXlast = -1;

             ReleaseCapture();

             SelectPalette(hdc, hOldPal, FALSE);
             ReleaseDC(hwnd, hdc);
     }
  }
  else
  // Select====================================================================
  if(ToolActive[5])             // Select
  {
    if( Mode == DRAW && EditDraw && InsideImage)
    {
        SetCapture(hwnd);

        hdc = GetDC(hwnd);
        hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
        RealizePalette(hdc);

        MagnifyAdj = DrawingMagnify/2;

        //hPenRed   = CreatePen(PS_SOLID, DrawingMagnify, PALETTEINDEX(GameColors[RED]));
        LOGBRUSH   lb1;                                               //   5-1-97
        lb1.lbStyle = BS_SOLID;
        lb1.lbColor = PALETTEINDEX(GameColors[WHITE]);
        lb1.lbHatch = 0;

        hPenRed   = ExtCreatePen(PS_COSMETIC | PS_DOT,2,&lb1,0,NULL); //hDotPen
        hOldPen   = SelectPen(hdc, hPenRed );
		  OldROP    = SetROP2(hdc, R2_XORPEN);   // write mode xor
        oldDrawX  = -1;
        prevDrawX = -1;

        hOldBrush = SelectBrush(hdc, GetStockBrush(NULL_BRUSH));
        //
        // Select box - keep doing this while the Left button is pressed down
        //
        while(msg.message != WM_LBUTTONUP)
        {
             WaitMessage();
             GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);

             if(msg.message == WM_MOUSEMOVE)
             {
		          SetROP2(hdc, R2_XORPEN);   // write mode xor
                SelectPen(hdc, hPenRed );

                if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
                {
                    if (DrawX != prevDrawX || DrawY != prevDrawY)
		              {
                        ShowCursor(FALSE);

                        if (oldDrawX != -1)
			               {
	                         Rectangle(hdc,oldDrawX,oldDrawY,prevDrawX,prevDrawY); // erase old
			               }
			               else // save start
			               {   oldDrawX = DrawX;
			                   oldDrawY = DrawY;
			               }

			               // for next erase
			               prevDrawX = DrawX;
			               prevDrawY = DrawY;

                        Rectangle(hdc,oldDrawX,oldDrawY,prevDrawX,prevDrawY);    // draw
                        ShowCursor(TRUE);
                    }
                }
             }
        }
        //
        // get the area that was selected
        //
        if(oldDrawX != -1)  // in case just click and get UP msg right away
        {
           ShowCursor(FALSE);

	        Rectangle(hdc,oldDrawX,oldDrawY,prevDrawX,prevDrawY); // erase old

		     SetROP2(hdc, R2_COPYPEN);                             // write mode COPY
           if (oldDrawX != prevDrawX &&  oldDrawY != prevDrawY)
	        {
               if(hViewBitmap)
               {
                  DeleteBitmap(hViewBitmap);
                  hViewBitmap  = NULL;
                  lpViewBits   = NULL;
               }

               CopyWidth  = abs((prevDrawX - oldDrawX));
               CopyHeight = abs((prevDrawY - oldDrawY));

               if( oldDrawX > prevDrawX ) oldDrawX = prevDrawX;
               if( oldDrawY > prevDrawY ) oldDrawY = prevDrawY;

               ZeroMemory(&ViewBi, sizeof(BITMAPINFO));
               ViewBi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
               ViewBi.bmiHeader.biPlanes     = 1;
               ViewBi.bmiHeader.biBitCount   = 8;
               ViewBi.bmiHeader.biWidth      =  CopyWidth;                        //   5-1-97
               ViewBi.bmiHeader.biHeight     = -CopyHeight;
               ViewBi.bmiHeader.biSizeImage  =  ((CopyWidth+3) & ~3) * CopyHeight;//  5-1-97
               ViewBi.bmiHeader.biCompression= BI_RGB;

               hViewBitmap = (HBITMAP)CreateDIBSection(hdc, &ViewBi, DIB_PAL_COLORS, (VOID **)&lpViewBits, NULL, 0L);

               if (!hViewBitmap || !lpViewBits)    // not enough memory
               {   OutOfMemoryMsg(hwnd);
                   if (hViewBitmap)
                   {   DeleteObject(hViewBitmap);
                       hViewBitmap = NULL;
                   }
                   lpViewBits   = NULL;
                   ReleaseCapture();
                   ShowCursor(TRUE);

                   SelectPalette(hdc, hOldPal, FALSE);
                   SelectBrush(hdc, hOldBrush);
                   SelectPen(hdc, hOldPen );
                   SetROP2(hdc, OldROP);
                   ReleaseDC(hwnd, hdc);
                   DeletePen(hPenRed);
                   return;
               }

               hdcMem = CreateCompatibleDC(hdc);
               hOldBitmap = SelectBitmap(hdcMem, hViewBitmap);
               SetDIBColorTable(hdcMem, 0, 256, dRGBquad);			// Palette indexes

 	            BitBlt(hdcMem, 0, 0, CopyWidth, CopyHeight, hdc, oldDrawX, oldDrawY, SRCCOPY);

               SelectBitmap(hdcMem, hOldBitmap);

               DeleteDC(hdcMem);

               POINT pt;                        //   5-1-97

               pt.x = oldDrawX;
               pt.y = oldDrawY;
               ClientToScreen(hEditWindow, &pt); // convert coordinates to screen
		         SetCursorPos(pt.x,pt.y);          // position cursor at upper left corner

	        }

           ShowCursor(TRUE);                    //   5-1-97
        }

		  SelectPen(hdc, hOldPen );
		  SetROP2(hdc, OldROP);
        SelectPalette(hdc, hOldPal, FALSE);
        SelectBrush(hdc, hOldBrush);
        ReleaseDC(hwnd, hdc);

        DeletePen(hPenRed);
        ReleaseCapture();
    }

    //oldRelativeXcoord=-1;//
    //oldRelativeYcoord=-1;//
	 //oldDrawX = -1;
	 //prevDrawX= -1;
	 //RawXlast = -1;

    if (lpViewBits && hViewBitmap)         //   5-1-97
    {   PasteImage(hwnd,DrawingMagnify, Transparent);
    }
  }

  SetFocus(hGFXWindow);   // for Keyboard to work
}

////////////////////////////////////////////////////////////////////////////////
// WM_RBUTTONDOWN  -  check click right and dbl click right
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
//void GFXEdit_OnRButtonUp(HWND hwnd, int x, int y, UINT flags)
static void GFXEdit_OnRButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
  //NotifyMsg(hwnd,"rightup");

  MouseButton = 2;               //   5-1-97

  if(ToolActive[0] == 1)         // Magnify Mode   Zoom Out
  {
     ZoomGraphic(FALSE);         // Zoom(out)
  }
  else
  if( (Mode == DRAW)
   && (EditDraw)
   && (x > OrgXstart)
   && (x < OrgXend)
   && (y > OrgYstart) && (y < OrgYend))
  {
     TogglePal(hGFXWindow, DrawingColor);
  }

  SetFocus(hGFXWindow);   // for Keyboard to work
}
////////////////////////////////////////////////////////////////////////////////
// WM_MOUSEMOVE -  This is the Main Mouse Move  (GFX area)                    //
////////////////////////////////////////////////////////////////////////////////

#pragma argsused
static void GFX_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
  //if(GetFocus()!= hGFXWindow)  SetFocus(hGFXWindow);   // for Keyboard to work
  EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));      // to always get cursor back   6-1-97

}

////////////////////////////////////////////////////////////////////////////////
// WM_MOUSEMOVE -  This is the Edit Mouse Move  (for EDIT area)               //
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
static void GFXEdit_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{

  //Trace(hGFXWindow,10,10,"  %d,%d  range x=%d,%d y=%d,%d",x,y, OrgXstart,OrgXend,OrgYstart,OrgYend);

  // If Draw Mode and Editing a Texture and Cursor inside Graphic Area

  if(Mode == DRAW && EditDraw && x > OrgXstart && x < OrgXend
                              && y > OrgYstart && y < OrgYend)
  {

   // Depending on ToolActive[i] = 1  Change Cursor to shape
   // Five Cursor Images ( CursorDropper, CursorEraser, CursorFill, CursorPen, CursorMag )

   //    0          1          2        3         4    5         ToolActive[
   // Magnify, Eye Dropper, Eraser, Flood Fill, Pen, Select

    if(ToolActive[0])       // Magnify can be outside image too, BELOW
    {
       if(GetCursor() != CursorMag)         EnableCursor(hwnd, CursorMag);
    }
    else       //   6-1-97
    if(ToolActive[1])      // Dropper
    {
       if(GetCursor() != CursorDropper)     EnableCursor(hwnd, CursorDropper);
    }
    else
    if(ToolActive[2])       // Eraser
    {
       if(GetCursor() != CursorEraser)      EnableCursor(hwnd, CursorEraser);
    }
    else
    if(ToolActive[3])       // Fill
    {
       if(GetCursor() != CursorFill)        EnableCursor(hwnd, CursorFill);
    }
    else
    if(ToolActive[4])       // PEN
    {
       if(GetCursor() != CursorPen)         EnableCursor(hwnd, CursorPen);
    }
    else
    if(ToolActive[5])       // Select
    {
       if(GetCursor() != gd.hCursorSelect)  EnableCursor(hwnd, gd.hCursorSelect);
    }

    XY_Update(hGFXWindow, x, y);

  }
  else
  {
     if(ToolActive[0])      // Magnify can be outside image (but in edit box)
     {
        if(GetCursor() != CursorMag)       EnableCursor(hwnd, CursorMag);
     }
     else
     {
        EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));
     }
  }
}

////////////////////////////////////////////////////////////////////////////////
// Our global variable for gd.CtrlDown etc. need to be reset upon
// active if paste not active
////////////////////////////////////////////////////////////////////////////////

#pragma argsused
static void GFX_OnKeyUp(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags)
{
 //NotifyMsg(NULL,"GFX_OnKeyUp");

 // should make general routine for all ONKEY routines

 if(vk == VK_CONTROL) gd.CtrlDown  = 0;
 if(vk == VK_SHIFT  ) gd.ShiftDown = 0;
 if(vk == VK_MENU   ) gd.AltDown   = 0;
}

//-------------------------------------------------------------------------
// Key pressed
// active if paste not active
//-------------------------------------------------------------------------
//
#pragma argsused
static void GFX_OnKey(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags)
{
   int4     i;
   SHORT    keystate;
   RECT     rc;
   int2     TempColor;
//   COLORREF cr;
   HDC      hdc;
   HPALETTE hOldPal;
//   POINT    p;
   MSG      msg;
   UINT     KeyData;
   Bool     Rotate;
   Bool     redraw;
// POINTS   pt;

//==============================================================================

// NotifyMsg(hGFXWindow,"GFX_OnKey\nvk=%d\nfdown=%d\ncrepeat=%d\nflags=%d",
//                  vk, fDown, cRepeat,flags);
 GetClientRect(hEditWindow, &rc);
 redraw = FALSE;
// Trace(hGFXWindow,10,4,"vk=%d",vk);

 // should make general routine for all ONKEY routines

 if (vk == VK_SHIFT)
 {   gd.ShiftDown = 0x8000;
     return ;               //   5-1-97
 }

 if (vk == VK_CONTROL)
 {   gd.CtrlDown  = 0x8000;
     return ;
 }

 if (vk == VK_MENU)
 {   gd.AltDown  = 0x8000;
     return ;
 }

 if(gd.CtrlDown)
 {
    if(EditDraw)
    {
      switch(vk)
      {//ctrl+C
       case VK_C:
        if (ImageUndoBufr)   //   5-1-97
        {    memcpy(ImageUndoBufr, &*lpDrawBits, MainImageSize);
             memcpy(RawImageUndoBufr, RawImageData,RawImageSize);    // copy raw data
             strcpy(StatMsg, "Image Copied to Undo Buffer - Control-Z to RESTORE");
             DidCopy = TRUE;
        }
        GFXDrawStatus(hGFXWindow);  // Status Bar  bottom
        return;
       //ctrl+Z
       case VK_Z:
        if (ImageUndoBufr)          //   5-1-97
        {   memcpy(&*lpDrawBits,   ImageUndoBufr,   MainImageSize);
            memcpy(RawImageData,RawImageUndoBufr,RawImageSize);    // copy raw data
            UnDoActive= 0;
            ShowCurrUnDo();

            if (!DidCopy)          // did not make an image copy (original image copy)
            {   DidRotate = FALSE;
                DidTransp = 0;
            }
        }

        GFXEdit_OnPaint(hEditWindow); // redo right drawing side
        return;
       //ctrl+V
       case VK_V:
        PasteImage(hwnd,DrawingMagnify, Transparent);//, TRUE);
        return;
      }
    }
 }

 if (vk == 112)                     // F1 Help
 {
    SoundBeep(SoundDrip);
    if(EditDraw)                     // doing edit
    {  //HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"Graphics Editor Keys");
        DrawHelp(hwnd);
    }
    else                            // nothing yet on main 1st level buttons
    {   //HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"Graphics - Drawing Graphics Tutorial");
        HelpOpened = WinHelp(hwnd, HelpName, HELP_KEY,(DWORD)"GRAPHICS");
    }
 }

 // if drawing already

 if(EditDraw)
 {
   GetClientRect(hEditWindow, &rc); //   5-1-97

   switch(vk)
   {
    case VK_T:                      // T Autocvt BMP Paste Translate
      SendMessage(hGFXWindow, WM_COMMAND, BUT31, 0);  // simulate Button
      break;
    case VK_ESCAPE:                 // esc Exit
      SendMessage(hGFXWindow, WM_COMMAND, BUT34, 0);  // simulate Exit button pushed
      break;
    case VK_I:                      // I Import Image
      SendMessage(hGFXWindow, WM_COMMAND, BUT35, 0);  // simulate Button
       break;
    case VK_S:                      // S SAVE Image
      SendMessage(hGFXWindow, WM_COMMAND, BUT37, 0);  // simulate Button 9.3 had wrong button
      break;
    case VK_A:                      // A SAVE AS Image
      SendMessage(hGFXWindow, WM_COMMAND, BUT38, 0);  // simulate Button 9.3 had wrong button
      break;

    case VK_TAB:                    // TAB PickImage under cursor for Drawing
      EyeDropper();
      break;

    // cursor keys======================     6-1-97
	 case(VK_UP):    // Up arrow
      GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION, -DrawingMagnify);
  	   break;
	 case(VK_DOWN):  // Down arrow
      GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION,  DrawingMagnify);
  	   break;
	 case(VK_LEFT):  // Left arrow
      GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION, -DrawingMagnify);
      break;
	 case(VK_RIGHT): // Right arrow
      GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION,  DrawingMagnify);
      break;

	 case(VK_PRIOR): // Page up
      GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION, -DrawingMagnify*8);
      break;
	 case(VK_NEXT):  // Page down
      GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION,  DrawingMagnify*8);
	   break;
    case(VK_HOME):  // home
      GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION, -DrawingMagnify*8);
      break;
    case(VK_END):   // end
      GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION,  DrawingMagnify*8);
      break;
    //==================================

    case VK_O:                      // O Circle Draw solid Dos - '(' key
      CircleDraw(hwnd,TRUE);
      break;
    case VK_P:                      // P Circle Draw hollow Dos - '(' key
      CircleDraw(hwnd,FALSE);
      break;

    case VK_EQUAL:                  // =+, Zoom in
    case VK_PLUSN:                  // + on number pad
      ZoomGraphic(TRUE);            // True=IN    6-1-97
      break;
    case VK_MINUS:                  // -_, Zoom out
    case VK_MINUSN:                 // - on number pad
      ZoomGraphic(FALSE);           // FALSE=OUT   6-1-97
      break;

    case VK_U:                      // UNDO
      UndoPixel();
	   if (oldActive != UnDoActive)
	   {   ShowCurrUnDo();
	       oldActive = UnDoActive;
      }

      if(cRepeat > 1)
      {
        for(;;)
        {
          UndoPixel();
	       if (oldActive != UnDoActive)
	       {   ShowCurrUnDo();
	            oldActive = UnDoActive;
          }

          keystate = GetAsyncKeyState(VK_U);
          if(!(keystate & 0x8000)) break;
        }
        break;
      }
      break;

    //case (VK_G):                     // G= gamma someday
    //  FlipImage  (hwnd,(byte *)&*lpDrawBits, 6,Transparent, DrawBi.bmiHeader.biWidth, DrawBi.bmiHeader.biHeight);
    //  DidRotate=TRUE;
    //  UnDoActive= 0;                 // can't undo anymore
    //  redraw = TRUE;
    //  break;
    case VK_BSLASH:                 // \ shrink, keep bufsize
      FlipImage  (hwnd,(byte *)&*lpDrawBits, 5,Transparent, DrawBi.bmiHeader.biWidth, DrawBi.bmiHeader.biHeight);
      DidRotate = TRUE;
      UnDoActive= 0;                 // can't undo anymore
      redraw = TRUE;
      break;

    case VK_COMMA:                   // < Rotate
    case VK_PERIOD:                  // > Rotate
      FlipImage  (hwnd,(byte *)&*lpDrawBits, 2, 0 , DrawBi.bmiHeader.biWidth, DrawBi.bmiHeader.biHeight);
      DidRotate = TRUE;
      UnDoActive= 0;                 // can't undo anymore
      redraw    = TRUE;
      break;

    case VK_X:                       // Flip X Axis
      FlipImage  (hwnd,(byte *)&*lpDrawBits, 0, 0 , DrawBi.bmiHeader.biWidth,DrawBi.bmiHeader.biHeight);
      DidRotate=TRUE;
      UnDoActive= 0;                 // can't undo anymore
      redraw = TRUE;
      break;
    case VK_Y:                       // Flip Y Axis
      FlipImage  (hwnd,(byte *)&*lpDrawBits, 1, 0 , DrawBi.bmiHeader.biWidth, DrawBi.bmiHeader.biHeight);
      DidRotate=TRUE;
      UnDoActive= 0;                 // can't undo anymore
      redraw = TRUE;
      break;

    case VK_N:                       // TRANSPARENT   return in WM_CHANGE Edit_Command - therefore hEditWindow
      if(!FlatLump)
      {   TogglePal(hEditWindow, Transparent);
      }
      else
      {
        HelpMsg(hEditWindow,CENTER|HELPINFO, "Not FLATS",
                                    "FLATs have no transparency",
                                    NULL);
      }
      break;

    // Match Pixel Routines b,z,j,c,;,\,[,]

    case VK_B:                      // BLUR
	   MatchPixels((byte *)&*lpDrawBits,Transparent, 0 ,7, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight));
      DidRotate= TRUE;  //++
      UnDoActive= 0;                 // can't undo anymore
      redraw   = TRUE;
      //SetFocus(hGFXWindow);   // for Keyboard to work
      break;
    case VK_Z:                      // FUZZ
	   MatchPixels((byte *)&*lpDrawBits,Transparent, 0 ,6, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight));
      DidRotate= TRUE;  //++
      UnDoActive= 0;                 // can't undo anymore
      redraw = TRUE;
      //SetFocus(hGFXWindow);   // for Keyboard to work
      break;

    case VK_J:                      // Change these Pixels to random color
    case VK_C:                      // Change these Pixels to Current color
    case VK_SEMI:                   // cycle all colors up
    //case VK_QUOTE:                // cycle all colors down
    //case (':'):                   // cycle all colors down
    case VK_LSQUARE:                // cycle all matching colors up ( under cur )
    case VK_RSQUARE:                // cycle all matching colors dn ( under cur )
      WaitMessage();
      GetGFXMouseCoords(hEditWindow,&msg, NULL,&DrawX, &DrawY,NULL);

      if( CheckIfInsideImage(&DrawX, &DrawY, 0,0) )
      {   InsideImage = TRUE;
      }
      else
      {   InsideImage = FALSE;
      }

      if (InsideImage)
      {
        DidRotate= TRUE;  //++
        UnDoActive= 0;                 // can't undo anymore
        redraw   = TRUE;

        hdc = GetDC(hEditWindow);
        hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
        RealizePalette(hdc);

        //cr = GetPixel(hdc, DrawX, DrawY);    // Get Color Reference
        //TempColor = GetNearestPaletteIndex(gd.hDoomPal, cr);
        TempColor = CurrentPictureColor;

        SelectPalette(hdc, hOldPal, FALSE);
        ReleaseDC(hEditWindow, hdc);

        switch(vk)
        {
          case VK_C:                      // Change these Pixels to Current color
           MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 0, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // c
           break;
          case VK_J:                      // Change these Pixels to random color
           MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 1, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // j
           break;
          case VK_SEMI:                   // cycle all colors up
           if (gd.ShiftDown)
           {   MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 2, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // ;
           }
           else  // :                     // cycle all colors down
           {   MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 3, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // '
           }
           break;
          // case VK_QUOTE:                 // cycle all colors down
          //case(':'):                       // cycle all colors down
          // MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 3, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // '
          // break;
          case VK_LSQUARE:                // cycle all matching colors up ( under cur )
           MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 4, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // [
           break;
          case VK_RSQUARE:                // cycle all matching colors dn ( under cur )
           MatchPixels((byte *)&*lpDrawBits, TempColor, DrawingColor, 5, DrawBi.bmiHeader.biWidth, abs(DrawBi.bmiHeader.biHeight)); // ]
           break;
        }
      }
      else
      {  for (i=0; i<2; i++)
         {   SoundBeep(SoundPhaser);
             Sleep(100);
         }
         //strcpy(StatMsg,"Please move cursor over a valid color INSIDE the image")
         //GFXDrawStatus(hwnd);

         //NotifyMsg(hEditWindow, "Please move cursor over a valid color INSIDE the image first ");
      }
      break;


    case VK_R:
      if(ToolActive[4] == 1)
      {  DrawLineList(hGFXWindow, FALSE);// Bye Bye
      }

      if(ToolActive[5] != 1)             // Select TOOL
      {

        for(i = 0; i < 6; i++)           // Get Rid of old Active Mode
            ToolActive[i] = 0;

        ToolActive[5] = 1;               // Make Pen Active for Freehand

        SoundBeep(SoundDrip);

        if(gd.hToolWindow)
        {
           InvalidateRect(gd.hToolWindow, NULL, TRUE);
           UpdateWindow(gd.hToolWindow);
        }
        else
        {
           GFXDrawTools(hGFXWindow);     // Tool Window BMP if NO ToolWnd
           strcpy(StatMsg, ToolMsg[5]);
           GFXDrawStatus(hGFXWindow);    // Status Bar  bottom
        }
      }
      break;

    case VK_F:                           // FREEHAND
    case VK_D:                           // DOT MODE
    case VK_L:                           // LINE MODE
      if(ToolActive[4] != 1)             // Pen NOT Selected
      {

        for(i = 0; i < 6; i++)           // Get Rid of old Active Mode
            ToolActive[i] = 0;

        ToolActive[4] = 1;               // Make Pen Active for Freehand

        SoundBeep(SoundDrip);

        if(!hLineMode)
           DrawLineList(hGFXWindow, TRUE);// Draw the Tool list box if NO ToolWindow

        if(gd.hToolWindow)
        {
          InvalidateRect(gd.hToolWindow, NULL, TRUE);
          UpdateWindow(gd.hToolWindow);
        }
        else
        {
          GFXDrawTools(hGFXWindow);      // Tool Window BMP if NO ToolWnd
          strcpy(StatMsg, ToolMsg[4]);
          GFXDrawStatus(hGFXWindow);     // Status Bar bottom
        }
      }

      if(vk == VK_F)   LineMode = 1;
      if(vk == VK_D)   LineMode = 2;
      if(vk == VK_L)   LineMode = 3;

      ListBox_SetCurSel(hLineMode, LineMode -1);  // set for the current mode
      break;

    case VK_BACK:                        // BACKSPACE - ROTATE MODE
      //Trace(hGFXWindow,10,4,"vk=%d",vk);
      WaitMessage();
      MouseButton = 0;
      GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);

      RotateX = DrawX;
      RotateY = DrawY;

      Rotate = TRUE;

      while(Rotate)
      {
          WaitMessage();
          KeyData =GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);

          if (CheckEscape(&msg,KeyData))    //escape
          {   // Rotate = FALSE;
              break;
          }

          if (MouseButton == 1 || MouseButton == 2) break;       // Mouse Button Breaks

          //switch(msg.message)
          //{
          //    default:
          //      break;
          if (KeyData)
          {
              //case(WM_MOUSEMOVE):
              //  break;

              //case(WM_KEYDOWN):
                switch(KeyData)
                {
                   case VK_UP:        //up arrow  Y
                     RotateY +=1;
		               break;
                   case VK_DOWN:      //down arrow Y
                     RotateY -=1;
		               break;
                   case VK_LEFT:      //left arrow X
                     RotateX +=1;
		               break;
                   case VK_RIGHT:     // right arrow X
                     RotateX -=1;
		               break;
                   case VK_BACK:     // BREAK OUT AGAIN
                     Rotate = FALSE;
		               break;
                }
                //break;
          }

          if (RotateX != DrawX ||  RotateY != DrawY)
          {
              RotateImage((byte *)&*lpDrawBits,(DrawX-RotateX),(DrawY-RotateY));
              RotateX  = DrawX;
              RotateY  = DrawY;
              DidRotate= TRUE;
              UnDoActive= 0;                    // can't undo anymore

              MagnifyGraphicTexture(hEditWindow,
                                    EditRC.left, EditRC.top,
                                    EditRC.right,EditRC.bottom,
                                   (byte *)&*lpDrawBits,
                                    hDrawBitmap,
                                    DrawingMagnify, Transparent);
          }
      } // end while rotate

      //SetFocus(hGFXWindow);   // for Keyboard to work
      break;

    }       // end switch(vk)
  }   // end if(Editing)
  else
  { // main #1 menu   5-1-97
    switch(vk)
    {
     case VK_D:
       SendMessage(hGFXWindow, WM_COMMAND, BUT0, 0);  // simulate Draw button pushed
       break;
     /*
     case VK_T:
       SendMessage(hGFXWindow, WM_COMMAND, BUT1, 0);  // simulate Convert Text button pushed
       break;
     case VK_F:
       SendMessage(hGFXWindow, WM_COMMAND, BUT2, 0);  // simulate Convert Flat button pushed
       break;
     case VK_O:
       SendMessage(hGFXWindow, WM_COMMAND, BUT3, 0);  // simulate Open button pushed
       break;
     */
     case VK_H:
       SendMessage(hGFXWindow, WM_COMMAND, BUT8, 0);  // simulate Help button pushed
       break;
     case VK_X:
     case VK_ESCAPE:                 // esc Exit
       SendMessage(hGFXWindow, WM_COMMAND, BUT9, 0);  // simulate Exit button pushed
       return;//break;  // Send kills window 9-1-97
    }
  }

  if(redraw)
  {
     LastPicXloc = -1;      // first time
     LastcxRange = -1;      // force clear

     MagnifyGraphicTexture(hEditWindow,
                           EditRC.left, EditRC.top,
                           EditRC.right,EditRC.bottom,
                          (byte *)&*lpDrawBits,
                           hDrawBitmap,
                           DrawingMagnify, Transparent);

     ShowCurrUnDo();
  }

  SetFocus(hGFXWindow);   // for Keyboard to work   5-1-97
}
////////////////////////////////////////////////////////////////////////////////
//  The Deep Graphic Editor Window        IDS_GFXNAME
//
//  Left side menu buttons
//
//  Buttons are  1: Edit, convert, Help,exit        (initial)
//               2: Edit ,zoom,zoom,main,help,exit  (drawing selected)
//               3. Import, ...etc                  (Actual drawing selected)
//
//  Handles Exporting of Texture, Conversion
// and drawing on Txtr's, Flats etc.
//
// Creates a Child Window, hEditWindow to conduct the actual Editing Process
//
////////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK GFXWndProc(WINDOWS_PARAMS)
{
  //Trace(hwnd  ,10,4,"GFXWndProc msg=%d %d %d", msg, wParam, lParam);  //  test what's being passed

  switch(msg)
  {
	 HANDLE_MSG(hwnd, WM_CREATE, 			GFX_OnCreate);
	 HANDLE_MSG(hwnd, WM_DESTROY, 		GFX_OnDestroy);
	 HANDLE_MSG(hwnd, WM_COMMAND, 		GFX_Command);
    HANDLE_MSG(hwnd, WM_CLOSE, 	      GFX_OnClose);
    HANDLE_MSG(hwnd, WM_PAINT,         GFX_OnPaint);
	 HANDLE_MSG(hwnd, WM_LBUTTONDOWN, 	GFX_OnLButtonDown);
	 HANDLE_MSG(hwnd, WM_RBUTTONDOWN, 	GFX_OnRButtonDown);

	 HANDLE_MSG(hwnd, WM_KEYDOWN, 		GFX_OnKey);
	 HANDLE_MSG(hwnd, WM_KEYUP, 		   GFX_OnKeyUp);
	 HANDLE_MSG(hwnd, WM_MOUSEMOVE, 		GFX_OnMouseMove);

	 default:
		return Deep_DefProc(hwnd, msg, wParam, lParam);
  }
}

///////////////////////////////////////////////////////////////////////////////
//  Graphic Editor Paint Routine
///////////////////////////////////////////////////////////////////////////////
#pragma argsused
static void GFXEdit_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   //RECT rc;

//---------------------------------------------------

   //static int4 count;
   //Trace(hwnd  ,10,24,"GFX_OnPaint, %d",count++);  //  test what's being passed
   //Trace(hGFXWindow  ,10,10,"GFXEdit_OnPaint");  //  test what's being passed

   //BeginPaint(hwnd, &ps);
   //GetClientRect(hwnd, &rc);
   //
   //LastPicXloc = -1;
   //LastcxRange = -1;     // clears area
   //
   //MagnifyGraphicTexture( hwnd,
   //                       rc.left, rc.top,
   //                       rc.right,rc.bottom,
   //                       (byte *)&*lpDrawBits,
   //                       hDrawBitmap,
   //                       DrawingMagnify, Transparent);
   //EndPaint(hwnd, &ps);

   BeginPaint(hEditWindow, &ps);

   //GetClientRect(hEditWindow, &rc);

   if (hDrawBitmap && lpDrawBits && EditDraw)  // have data and are drawing
   {
       LastPicXloc = -1;     // first time
       LastcxRange = -1;     // force clear

       MagnifyGraphicTexture( hEditWindow,
                              EditRC.left, EditRC.top,
                              EditRC.right,EditRC.bottom,
                             (byte *)&*lpDrawBits,
                              hDrawBitmap,
                              DrawingMagnify, Transparent);

       ValidateRect(hEditWindow, &EditRC);
   }
   else
   {
       //GFXDrawPicture();
   }

   EndPaint(hEditWindow, &ps);
}

////////////////////////////////////////////////////////////////////////////////
//  Get rid of the bitmap              //
////////////////////////////////////////////////////////////////////////////////
#pragma argsused
static void GFXEdit_OnDestroy(HWND hwnd)
{
  if(hDrawBitmap)
  {
      DeleteBitmap(hDrawBitmap);          // Our Dib Section
      lpDrawBits = NULL;
      hDrawBitmap = NULL;
  }
}
//
//-------------------------------------------------------------------------
//
//  The Actual Editor Window that handles drawing  =  IDS_GFXEDITNAME
//  (this is the Big Right Side Window)
//-------------------------------------------------------------------------
//
LRESULT CALLBACK GFXEditWndProc(WINDOWS_PARAMS)
{
  //Trace(hwnd  ,10,4,"GFXEditWndProc msg=%d %d %d", msg, wParam, lParam);  //  test what's being passed
  switch(msg)
  {
	 HANDLE_MSG(hwnd, WM_DESTROY, 		GFXEdit_OnDestroy);
    HANDLE_MSG(hwnd, WM_PAINT,         GFXEdit_OnPaint);
	 HANDLE_MSG(hwnd, WM_COMMAND, 		GFXEdit_Command);

	 HANDLE_MSG(hwnd, WM_RBUTTONDOWN,   GFXEdit_OnRButtonDown);
	 HANDLE_MSG(hwnd, WM_LBUTTONDOWN, 	GFXEdit_OnLButtonDown);
	 HANDLE_MSG(hwnd, WM_MOUSEMOVE, 		GFXEdit_OnMouseMove);

	 HANDLE_MSG(hwnd, WM_HSCROLL, 		GFXEdit_OnHScroll);
	 HANDLE_MSG(hwnd, WM_VSCROLL, 		GFXEdit_OnVScroll);

	 HANDLE_MSG(hwnd, WM_KEYDOWN, 		GFX_OnKey);
	 HANDLE_MSG(hwnd, WM_KEYUP, 		   GFX_OnKeyUp);

	 HANDLE_MSG(hwnd, WM_CREATE, 			GFX_EditOnCreate);

	 default:
		return Deep_DefProc(hwnd, msg, wParam, lParam);
  }
}

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// Loads the List of Add on Textures etc. into a supplied list box  - hList   //
// Global :  listcount, *listnames  and char **list                           //
//                                                                            //
// Calling App must free memory for the 2 lists                               //
//                                                                            //
// Called from Draw On Wall Texture BUTTON to setup data                      //
////////////////////////////////////////////////////////////////////////////////

static Bool CheckForFlat(DirPtr dir)
{
   int4 i;

   if (dir->filenumber >0         // file number
    && (   dir->size == 4096      // should be this size (ignores F_SKY in heretic = 4
        || dir->size == 4160      // mistakes in HERETIC 4160
        || dir->size == 8192) )   // mistakes in HEXEN   8192
    {
       for(i = 0; i < DrawListCount; i++)
       {   if (!strncmp(DrawList[i], dir->name,8))
           {    return FALSE;     // it's in TEXTURE list already
           }
       }
       return TRUE;               // made it
   }
   return FALSE;
}

//------------------------------------------------------------------------------

static Bool LoadDrawOnTextures(HWND hwnd)//, HWND hList)
{
   DirPtr dir;
   DirPtr TempDir;
   DOOMGraphicHdr PicHeader;

   DirPtr dirFs;
   DirPtr dirFe;
   DirPtr dirFFs;
   DirPtr dirFFe;
   int4   listmain;     //# between regular F_START - F_END
   int4   listflat=0;   //                  FF_START- FF_END

   int4   n;
   int2   listotal;
   int2   f;

   NameAreaList *listnamwrk;

   WadFiles AnyWad;             // the texture directory (usually MasterWad)

   char  name[9]={0};

//-------------------------------------------------------------------------

   //                                                                         //
   //         TEXTURES, SPRITES, and ANYTHING that looks like it              //
   //                                                                         //

   // count the names first to optimize getmemory       //
   // allocate space for all the names all at once     //

   dir = MasterWad->directory;                          // dir buffer
   listotal = 0;
   DrawListCount = 0;                                   // number of entries

   for (n=0; n < MasterWad->dirnum ; n++, dir+=1)
   {
     f = dir->filenumber;                               // file number

     if (f > 0 )
     {
   	 TempDir  = dir+1;
	    if ( n+1 < MasterWad->dirnum
	    &&  !strncmp(TempDir->name,"THINGS",6) )         // THINGS
	    {
	      if (WadFileList[f].maptype == 1 )              // which type
	      {   n  +=10;                                   // DOOM-HERETIC
		       dir+=10;
	      }
	      else
	      {   n  +=11;                                   // HEXEN
		       dir+=11;
	      }
	    }
	    else
	    if ( dir->size > 28)                             // was -0-
	    {    listotal++;
	    }
     }
   }

   if (listotal > 0)
   {
       DrawList      = (char **)       GetMemory( (int4)listotal * sizeof( char *) );
       DrawListNames = (NameAreaList *)GetMemory( (int4)listotal * sizeof(NameAreaList) );    // 8 name is 8
       listnamwrk= DrawListNames;

       SetCursor(LoadCursor(NULL, IDC_WAIT));

       // Update Status Bar
       wsprintf(StatMsg, "Loading %4hd Textures",listotal);
       GFXDrawStatus(hGFXWindow);                             // status bar bottom

       PercentObject(hwnd,StatMsg,SAVE,listotal);

       //                                                                        //
       //  read the names from master dir that belong to external files (and check)
       //                                                                        //


       dir = MasterWad->directory;                            // dir buffer

       for (n = 0; n < MasterWad->dirnum; n++,dir+=1)
       {
           f = dir->filenumber;                               // file number

           if (f > 0 )
           {  strncpy( name, dir->name, 8);                  // name
	           TempDir  = dir+1;
	           if ( n+1 < MasterWad->dirnum	 &&  !strncmp(TempDir->name,"THINGS",6) )       // THINGS
	           {   if (WadFileList[f].maptype == 1 )          // which type
	               {   n  +=10;                               // DOOM-HERETIC
		                dir+=10;
	               }
	               else
	               {   n  +=11;                               // HEXEN
		                dir+=11;
	               }
	           }
	           else
	           if( strncmp(dir->name,"PNAMES",6)              // not PNAMES
	           &&  strncmp(dir->name,"TEXTURE",7)             // not TEXTURE1 /2
	           &&  dir->size > 28)                            // and has size was 0
              {
	               if (WadFileList[f].type[0] == 'C' )            // is closed
	               {  WadFileList[f].fileinfo = FileOpen(WadFileList[f].filename,"rb",OPENMUST);
	                  WadFileList[f].type[0] = 'O';              // leave open
	               }

	               CopyLevelInfo(&AnyWad, &WadFileList[f]);       // assign to texturewad

                  WadSeek( AnyWad.fileinfo, dir->start);         // seek start of entry
	               WadRead( AnyWad.fileinfo, &PicHeader,12);      // x width, yheight +offsets

                  // see if image is within for 320x200?)
                  if (DoomGraphicVerify(&PicHeader,dir->size) )
	               {
                     DrawList[DrawListCount] = (char *) listnamwrk;

	                  strncpy( (char *)DrawList[DrawListCount], name, 8);  // store name

	                  DrawListCount++;
	                  listnamwrk = listnamwrk + 1;

                     // Update Status Bar
                     PercentDone(hwnd,DrawListCount,listotal);
                     //wsprintf(StatMsg, "LOADING -  %8s  %4hd  ", name,DrawListCount);
                     //GFXDrawStatus(hwnd);
                  }
	           }
           }
       }

       PercentDone(hwnd,100,100);
       // sort the Texture list

       if (DrawListCount > 0)                    // Yes, we have textures to Draw on
       {   qsort( DrawList, DrawListCount, sizeof( char *), QsortName);
       }
       else
       // free allocated list
       {
           FreeMemory(DrawListNames);
           FreeMemory(DrawList);
           DrawListNames = NULL;
           DrawList      = NULL;
       }
   }

   //                                                                         //
   //                           == FLATS ==                                   //
   //                                                                         //

   wsprintf(StatMsg, "SEARCHING for External FLATS . . .");
   GFXDrawStatus(hGFXWindow); // update status bar  bottom

   //DrawingMagnify = PicMagnify;
   LastPicXloc = -1;                 // 1st time
   LastcxRange = -1;                 // force celar

   DrawFListCount=0;
   listotal=0;

   // first get all the Main names --------------------------------------

   dirFs = FindMasterDir(MasterWad,"F_START",TRUE);  // normal FLAT start
   dirFe = FindMasterDir(MasterWad,"F_END",  TRUE);  // ..     FLAT end

   listmain = dirFe - dirFs-1;      // ignore start, end
   dir = dirFs+1;

   // this gets replacements of regular FLATs

   for (n=0; n < listmain; n++, dir+=1)
   {
       if (CheckForFlat(dir) )                 // not in texture list
       {   listotal++;
       }
   }

   // maximum is number of external file names

   dirFFs = FindMasterDir(MasterWad,"FF_START",FALSE);            // FF_START
   if (dirFFs)
   {   dirFFe = FindMasterDir( MasterWad, "FF_END", FALSE);
       if (dirFFe)
       {
          listflat = dirFFe - dirFFs-1;      // ignore start, end

          dir = dirFFs+1;
          for (n=0; n < listflat; n++, dir+=1)
          {
             if (CheckForFlat(dir) )        // not in texture list
             {   listotal++;
             }
          }
       }
   }

   if (listotal > 0)
   {
       DrawFList      = (char **)GetMemory( listotal * sizeof( char *)      );
       DrawFListNames = (NameAreaList *)GetMemory( listotal * sizeof(NameAreaList) );   // 8 name is 8

       // Update Status Bar
       wsprintf(StatMsg, "Loading %4hd FLAT names",listotal);
       GFXDrawStatus(hGFXWindow);// status bar  bottom
       PercentObject(hwnd,StatMsg,SAVE,listotal);

       //--------------------------------------------------------------------------
       //  read the names from master dir that belong to external files (and check)
       //--------------------------------------------------------------------------

       listnamwrk= DrawFListNames;                                      // first loc store

       dir = dirFs+1;                        // skip  F_START

       for (n=0; n < listmain; n++, dir+=1)  // number regular FLATS
       {
            if (CheckForFlat(dir) )
	         {
                DrawFList[DrawFListCount] = (char *) listnamwrk;

	             strncpy( (char *)DrawFList[DrawFListCount], dir->name, 8);  // store name

	             DrawFListCount++;
	             listnamwrk = listnamwrk + 1;

                PercentDone(hwnd,DrawFListCount,listotal);
            }
       }

       dir = dirFFs+1;                        // skipt FF_START

       for (n=0; n < listflat; n++, dir+=1)
       {
            if (CheckForFlat(dir) )
	         {
	             DrawFList[DrawFListCount] = (char *) listnamwrk;

	             strncpy( (char *)DrawFList[DrawFListCount], dir->name, 8);  // store name

	             DrawFListCount++;
	             listnamwrk = listnamwrk + 1;

                PercentDone(hwnd,DrawFListCount,listotal);
	         }
       }

       PercentDone(hwnd,100,100);

       if (DrawFListCount >0)
       {   qsort( DrawFList, DrawFListCount, sizeof( char *), QsortName);
       }
       else
       // free allocated list
       {
           FreeMemory(DrawFListNames);
           FreeMemory(DrawFList);
           DrawFListNames = NULL;
           DrawFList      = NULL;
       }
   }

   SetCursor(LoadCursor(NULL, IDC_ARROW));              // Back to normal Cursor

   if (DrawListCount <=0 && DrawFListCount <= 0)
   {
        HelpMsg(hwnd,CENTER|HELPINFO, "No graphics found",
                       "No external PWAD files found!",
                       "",
                       "You must have loaded a PATCH WAD",
                       "Containing Addon Textures or FLATS",
                       "to use the Graphic Editor to draw",
                       "on Flats, Wall Textures or Sprites.",
                       "",
                       "See the help file for more information",
                       "About Exporting Textures and Saving Lumps",
                       NULL);
       return FALSE;
   }

   return TRUE;
}


/////////////////////////////////////////////////////////////////////
//  The Deep Modeless ToolBar Window
//  Contains custom window class "Tool Window", 8 Bitmaps depicting
//  the tools available
//////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK ToolDlgProc(DIALOG_PARAMS)
{
  RECT rc, rc1;

  ScreenSaverTimer = 0;               // reset screensaver timer   8-1-97

  switch(msg)
  {
    case 160:                         //WM_NCLMOUSEMOVE, couldn't get LBUTTONUP for some reason
      GetWindowRect(hDlg, &rc);       // current window (GFX or Window)

      if(rc.left < (BARWIDTH - 30) )  // Dock the Floating toolbar
      {
         ToolWindow = FALSE;
         ToggleTool();                //5
         Button_SetCheck(hControl[6], FALSE); // no tool
      }
      break;

	 case WM_INITDIALOG:
      GetWindowRect(hEditWindow, &rc);
      GetWindowRect(hDlg, &rc1);
      MoveWindow(hDlg, rc.left + 40, rc.top + 40, rc1.right - rc1.left, rc1.bottom - rc1.top, TRUE);

      SetFocus(hGFXWindow);             // for Keyboard to work
	   return TRUE;                      // or FALSE if we set our own focus
  }
  return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  This is the Main Left Button Down Handler for the Tool Window             //
//    5-1-97 not used anywhere??                                            //
////////////////////////////////////////////////////////////////////////////////

#pragma argsused
static void Tool_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
  int Xvalue, Yvalue, i;

//------------------------------------------------------

  Xvalue = (x / 22);
  Yvalue = (y / 22);

  if( ToolActive[ Xvalue + (Yvalue*3) ] == 1)  return;

  if(ToolActive[4] == 0 && ( Xvalue + (Yvalue*3) ) == 4 )    // Line Mode
     DrawLineList(hGFXWindow, TRUE);

  if(ToolActive[4] == 1 && ( Xvalue + (Yvalue*3) ) != 4 )    // Line Mode Active already
     DrawLineList(hGFXWindow, FALSE);                        // Draw Valid Lines

  for(i = 0; i < 6; i++)                  // Negate all Active Tools
      ToolActive[i] = 0;

  ToolActive[ Xvalue + (Yvalue*3) ] = 1;  // Only 1 tool Active [1]

  SoundBeep(SoundDrip);

  InvalidateRect(hwnd, NULL, TRUE);       // Redraw the buttons up or down
  UpdateWindow(hwnd);

  SetFocus(hGFXWindow);   // for Keyboard to work

}

//////////////////////////////////////////////////////
//  Graphic Editor Paint Routine
//////////////////////////////////////////////////////
static void Tool_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   HPALETTE hOldPal;
   int2     i;

//--------------------------------------------------------------------------

   BeginPaint(hwnd, &ps);
   hOldPal = SelectPalette(ps.hdc, gd.hDoomPal, FALSE);
   RealizePalette(ps.hdc);

   for(i = 0; i < 3; i++)
    {
      if(!ToolActive[i])
         DrawBitmap(ps.hdc, ToolBitmapsUp[i], i*22, 0, SRCCOPY);
      else
         DrawBitmap(ps.hdc, ToolBitmapsDn[i], i*22, 0, SRCCOPY);
    }
   for(i = 3; i < 6; i++)
    {
      if(!ToolActive[i])
         DrawBitmap(ps.hdc, ToolBitmapsUp[i], (i-3)*22, 22, SRCCOPY);
      else
         DrawBitmap(ps.hdc, ToolBitmapsDn[i], (i-3)*22, 22, SRCCOPY);
    }

   SelectPalette(ps.hdc, hOldPal, FALSE);
   EndPaint(hwnd, &ps);
}

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------
#pragma argsused
static void Tool_OnMouseMove(HWND hwnd, int x, int y, UINT keyFlags)
{
  int Xvalue, Yvalue, i;

//------------------------------------------------------

  Xvalue = (x / 22);
  Yvalue = (y / 22);
  i =  Xvalue + (Yvalue*3) ;

  if( i >= 0 && i < 6)          // safety - was screwing up before
   strcpy(StatMsg, ToolMsg[i]);

  // Our parent is the Dlg. and it's parent is the Main Graphic Editor Window

  GFXDrawStatus(hGFXWindow);   // status bar  bottom
}
//
//-------------------------------------------------------------------------
//  The Deep Graphic Editor Tool Window  IDS_TOOLNAME
//-------------------------------------------------------------------------
//

LRESULT CALLBACK ToolWndProc(WINDOWS_PARAMS)
{
  switch(msg)
  {
    HANDLE_MSG(hwnd, WM_PAINT,         Tool_OnPaint);
	 HANDLE_MSG(hwnd, WM_LBUTTONDOWN, 	Tool_OnLButtonDown);
	 HANDLE_MSG(hwnd, WM_MOUSEMOVE, 		Tool_OnMouseMove);
	 default:
		return Deep_DefProc(hwnd, msg, wParam, lParam);
  }
}


///////////////////////////////////////////////////////////////////////////
//	This updates the X and Y coordinates embedded in the menu bar !
///////////////////////////////////////////////////////////////////////////
static void XY_Update(HWND hwnd, int x, int y)
{
static char   szBuf4[18];
//static int    RelativeXcoord, RelativeYcoord, oldXcoord, oldYcoord;
static HDC    hdc;
static HFONT  hOldFont;
static RECT   rc;			   // Rectangle to paint
static HBRUSH FillBrush;
static int4   xOffset;
static int4   yOffset;
static byte  *RawLocColor;

   //

   if (PasteActive)
   {   xOffset = PasteWidth;
       yOffset = PasteHeight;
   }
   else
   {   xOffset = 0;
       yOffset = 0;
   }

	// show graphic picture coordinates on the screen (relative to graphic)
   if( x > OrgXstart - xOffset && x < OrgXend  //   5-1-97
    && y > OrgYstart - yOffset && y < OrgYend )
   {

	RelativeXcoord = (x-OrgXstart +cxHorzScrollPos)/DrawingMagnify;
	RelativeYcoord = (y-OrgYstart +cyVertScrollPos)/DrawingMagnify;

	if (RelativeXcoord != oldXcoord ||  RelativeYcoord != oldYcoord)
   {
       // get color directly from BITMAP

       RawLocColor = ImageBufferOffset(RelativeXcoord,RelativeYcoord);
       if (RawLocColor)
       {
           CurrentPictureColor = *RawLocColor;//old color
           GFXCurrentPictureColor();
       }

	    sprintf(szBuf4, "x=%-4d  y=%-4d", RelativeXcoord, RelativeYcoord);// format the text

       hdc     = GetWindowDC(hwnd);                            // Get a DC for Window
       hOldFont= SelectFont(hdc,gd.hStockFont);//GetStockFont(ANSI_VAR_FONT));  // Select a font

       rc.top   = 5;                                           // Top of Text Rect.
       rc.left  = gd.rcClient.right - gd.rcClient.left - 200;  // Left edge of text
       rc.right = rc.left+100;
       rc.bottom= rc.top+16;

       //SetBkMode(hdc, OPAQUE);             // This fixes NT menus not disappearing, so leave it
       SetBkMode(hdc, TRANSPARENT);         // 1-1-98 since font changes screwed up everything????

       FillBrush = GetStockBrush(WHITE_BRUSH);

       DrawEdge(hdc, &rc, EDGE_ETCHED, BF_RECT | BF_SOFT);

       rc.left   += 2;
       rc.right  -= 2;
       rc.bottom -= 2;
       rc.top    += 1;
       FillRect(hdc, &rc, FillBrush);                             //   1-1-98
       TextOut(hdc,rc.left+3,rc.top-1,szBuf4, lstrlen(szBuf4));   //    1-1-98 top-1
       //ExtTextOut(hdc,  rc.left+4, rc.top, ETO_OPAQUE, &rc, szBuf4, lstrlen(szBuf4), NULL);

       SelectFont(hdc, hOldFont);
       ReleaseDC(hwnd, hdc);
       oldXcoord = RelativeXcoord;
       oldYcoord = RelativeYcoord;
	}

	if (oldActive != UnDoActive)
	{   ShowCurrUnDo();
	    oldActive = UnDoActive;
	}
   }
}


//------------------------------------------------------------------------------
// Import BMP or PCX and store in a PWAD
//------------------------------------------------------------------------------
#pragma argsused
Bool ImportBMPtoPWAD(HWND hwnd)
{
  Bool           Success = FALSE;
  LPSTR          ValidName;
  DOOMGraphicHdr PicHeader;
  char           LumpName[10] = {0};

  int2        oldXsize;          // original BMP/PCX size
  int2        oldYsize;

  int2        NewXsize;
  int2        NewYsize;
  int2        NewXoff;
  int2        NewYoff;
  char        msgText[80];

  HPALETTE    hOldPal1;          //
  HPALETTE    hOldPal2;          //
  HBITMAP     hOldBitmap1;       // Handle to previous bitmap
  HBITMAP     hOldBitmap2;       // Handle to previous bitmap

  HDC         hdcSource;
  BITMAPINFO  BitmapInfoSource;  // DIB Bitmap Info Struct. for original image
  HBITMAP     hBitmapSource;     // Handle to new bitmap
  VOID **     lpBitsSource;      // Pointer to bits in DIBSECTION

  HDC         hdcTarget;         // Handle to memory DC
  BITMAPINFO  BitmapInfoTarget;  // DIB Bitmap Info Struct. for original image
  HBITMAP     hBitmapTarget;     // Handle to new bitmap
  VOID **     lpBitsTarget;      // Pointer to bits in DIBSECTION

  //============================================================================

  ValidName = GetWadName(hGFXWindow,"Select Image to Put in a PWAD", CopyFileName, 259, IDS_FILTERSTRING, PatchDir );

  RemoveDIBSection();           // Get rid of Preview Memory Allocation from above since we may be calling with TRANSLATE TRUE

  if (ValidName
  && (ImageType == PCXIMAGE || ImageType == BMPIMAGE) )
  {
       if (ImageType == PCXIMAGE)
       {   hViewBitmap = ReadImage (hEditWindow, PCXIMAGE, PasteTranslate, 1, NULL, NULL, CopyFileName, OPENMUST);
       }
       else
       if (ImageType == BMPIMAGE)
       {   hViewBitmap = ReadImage (hEditWindow, BMPIMAGE, PasteTranslate, 1, NULL, NULL, CopyFileName, OPENMUST);
       }
       //if (ImageType == WADIMAGEGRAPHIC)
       //{   // not supported
       //    PasteWadLump(hwnd, CopyFileName, MapLevel);    // lump name from browsing   8-1-97
       //}

       if (hViewBitmap)
       {
           FlatLump = FALSE;

           NewXsize=    ViewBi.bmiHeader.biWidth;
           NewYsize=abs(ViewBi.bmiHeader.biHeight);
           NewXoff =0;
           NewYoff =0;

           oldXsize = NewXsize;
           oldYsize = NewYsize;

           sprintf(msgText,"BMP/PCX Size/Offsets");

           if(!Input4Numbers(hwnd, msgText,
                            "X size",      1,1024,&NewXsize,
                            "Y size",      1, 255,&NewYsize,
                            "X offset",-1023,1023,&NewXoff,
                            "Y offset", -254, 254,&NewYoff)) return Success;

           // Create Source bitmap (original)
           memcpy( &BitmapInfoSource, &ViewBi, sizeof (BITMAPINFO) );
           hBitmapSource = hViewBitmap;
           lpBitsSource  = lpViewBits;

           hdcSource = GetDC(hEditWindow);                  // window we are interested in
           hOldPal1  = SelectPalette(hdcSource, gd.hDoomPal, TRUE ); //FALSE    TRUE matches colors for us
           RealizePalette(hdcSource);
           MagnifyGraphicTexture(hEditWindow, 0,0,
                                              oldXsize-1,oldYsize-1,
                                             (byte *)&*lpBitsSource,
                                              hBitmapSource,
                                              1, Transparent);

           //CheckActivity(NULL, WAIT);
           // Create a memory DC to copy to
           hdcTarget = CreateCompatibleDC(hdcSource);
           if (!hdcTarget)
           {   ReleaseDC   (hwnd, hdcSource);
               DeleteBitmap(hBitmapSource);
               return FALSE;
           }
           hOldPal2 = SelectPalette(hdcTarget, gd.hDoomPal, TRUE ); //FALSE
           RealizePalette(hdcTarget);

           //---
           // create bitmap copy to copy into   (or createcompatible bitmap ?)
           // --
           ZeroMemory(&BitmapInfoTarget,     sizeof(BITMAPINFO));
           BitmapInfoTarget.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
           BitmapInfoTarget.bmiHeader.biPlanes    = 1;
           BitmapInfoTarget.bmiHeader.biBitCount  = 8;
           BitmapInfoTarget.bmiHeader.biWidth     =  NewXsize;
           BitmapInfoTarget.bmiHeader.biHeight    = -NewYsize;
           BitmapInfoTarget.bmiHeader.biSizeImage = ((NewXsize + 3 ) & ~3) * NewYsize;
           BitmapInfoTarget.bmiHeader.biCompression = BI_RGB;
           hBitmapTarget= (HBITMAP)CreateDIBSection(hdcTarget, &BitmapInfoTarget, DIB_PAL_COLORS, (VOID **)&lpBitsTarget, NULL, 0L);
           if (!hBitmapTarget)
           {   ReleaseDC(hwnd, hdcSource);
               DeleteDC(hdcTarget);
               DeleteBitmap(hBitmapSource);         // Our Dib Section
               OutOfMemoryMsg(hwnd);
               return Success;
           }


            hOldBitmap1 = SelectBitmap(hdcSource,hBitmapSource);   // selectobject
            SetDIBColorTable(hdcSource, 0, 256, dRGBquad);         // set Palette indexes
            hOldBitmap2 = SelectBitmap(hdcTarget,hBitmapTarget);   // selectobject
            SetDIBColorTable(hdcTarget, 0, 256, dRGBquad);         // set Palette indexes

            // copy contents of current bitmap screen  =

            SetStretchBltMode(hdcTarget,HALFTONE);                  //halftone required for color pictures
            SetBrushOrgEx (hdcTarget,0,0,NULL);

            StretchBlt(hdcTarget, 0,0, NewXsize, NewYsize,
                       hdcSource, 0,0, oldXsize, oldYsize, SRCCOPY);
            GdiFlush();

            // DEBUG -start - show what image will look like
            //MagnifyGraphicTexture(hEditWindow, EditRC.left,  EditRC.top,
            //                                   EditRC.right, EditRC.bottom,
            //                                   (byte *)&*lpBitsTarget,
            //                                   hBitmapTarget,
            //                                   1, Transparent);
            //CheckActivity(NULL, WAIT);
            // DEBUG - end -


            //------
            SelectPalette(hdcSource,hOldPal1, FALSE);   // probably don't need this check later
            SelectPalette(hdcTarget,hOldPal2, FALSE);   // probably don't need this check later
            SelectBitmap (hdcSource, hOldBitmap1);      // selectobject
            SelectBitmap (hdcTarget, hOldBitmap2);      // selectobject
            DeleteDC(hdcTarget);                        // delete hdc
            DeleteBitmap(hBitmapSource);                // delete original bitmap

            PicHeader.xsize= NewXsize;                       // 2 width in pixels
            PicHeader.ysize= NewYsize;                       // 2 height in pixels
            PicHeader.xoffs= NewXoff;                        // 2 initial x offset
            PicHeader.yoffs= NewYoff;                        // 2 initial y offset
            //PicHeader.ystart;                              // 4 offset 1st Y column start (there are X of these)
            byte *PicHeaderData = (byte*)&PicHeader;

            // DrawBI is implicitly used here -- must be for now --

            DrawBi.bmiHeader.biWidth = NewXsize;  // global size
            DrawBi.bmiHeader.biHeight=-NewYsize;

            if (SaveDrawGraphic(hwnd,
                                3,                 // saveas, converts raw to DOOM format again
                                LumpName,          // patch name
                                GFXWadName,        // filename,
                                0,                 // n/a GraphicDirstart,
                               &RawImageSize,      // n/a
                               &PicHeaderData,     // header 8
                               &MainImageSize,     // n/a
                               (byte **)&lpBitsTarget))
            {
                 StorePwad (GFXWadName);     // store
                 gd.OKDraw = 1;              // reset to normal (set by StorePwad)
                 Success = TRUE;
            }

            DeleteBitmap(hBitmapTarget);     // created by createdibsection above

            FreeMemory(RawImageData);
            RawImageData = NULL;

            ReleaseDC(hwnd, hdcSource);

       } // new size or offsets
       /////////////////////////////////////////////////////////////////////////

       RemoveDIBSection();
   }
   else
   if (ValidName)
   {
       HelpMsg(hwnd,CENTER|HELPINFO, "",
                                     "Please select PCX or BMP images",
                                     NULL);
       //ErrorMsg(hwnd,"Please select PCX or BMP images");
       Success = FALSE;
   }
   return Success;
}

//------------------------------------------------------------------------------
// change X Y size of graphic or offsets - has to be a TEXTURE lump format
//------------------------------------------------------------------------------

static Bool MakeBlankTexture(HWND hwnd)
{
  HDC      hdc;
  Bool     Success = FALSE;
  int2     NewXsize=128;
  int2     NewYsize=128;
  int2     NewXoff =0;
  int2     NewYoff =0;
  char     msgText[80];

  byte    *WrkImageBufr;          // buffer containing image
  DOOMGraphicHdr PicHeader;

//=============================================================================

  sprintf(msgText,"Dummy Lump %s Size/Offsets",DrawingName);
  if(!Input4Numbers(hwnd, msgText,
                            "X size",      1,1024,&NewXsize,
                            "Y size",      1, 255,&NewYsize,
                            "X offset",-1023,1023,&NewXoff,
                            "Y offset", -254, 254,&NewYoff)) return Success;

  // Set up a BITMAPINFO structure for use with CreateDIBSection()

  sprintf(DrawingName,"%hdx%hd",NewXsize,NewYsize);             // init as this name

  hdc = GetDC(hwnd);

  ZeroMemory(&DrawBi, sizeof(BITMAPINFO));

  DrawBi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
  DrawBi.bmiHeader.biPlanes    = 1;
  DrawBi.bmiHeader.biBitCount  = 8;
  DrawBi.bmiHeader.biWidth     =  NewXsize;
  DrawBi.bmiHeader.biHeight    = -NewYsize;
  DrawBi.bmiHeader.biSizeImage = ((NewXsize + 3 ) & ~3) * NewYsize;
  DrawBi.bmiHeader.biCompression = BI_RGB;

  // Create a DC to hold our DIBSECTION
  hDrawBitmap = (HBITMAP)CreateDIBSection(hdc, &DrawBi, DIB_PAL_COLORS, (VOID **)&lpDrawBits, NULL, 0L);

  // Allocate our Buffer and our Bitmap

  if (!hDrawBitmap)
  {
      ReleaseDC(hwnd, hdc);
      OutOfMemoryMsg(hwnd);
      return Success;
  }

  WrkImageBufr = (byte *)&*lpDrawBits;                  // Grab pointer to the bits of our DIBSection

  FillMemory(WrkImageBufr, DrawBi.bmiHeader.biSizeImage, Transparent); // fill with background

  PicHeader.xsize= NewXsize;                       // 2 width in pixels
  PicHeader.ysize= NewYsize;                       // 2 height in pixels
  PicHeader.xoffs= NewXoff;                        // 2 initial x offset
  PicHeader.yoffs= NewYoff;                        // 2 initial y offset
  //PicHeader.ystart;                              // 4 offset 1st Y column start (there are X of these)
  byte *PicHeaderData = (byte*)&PicHeader;

  // DrawBi is implicitly used here -- must be for now --

  if (SaveDrawGraphic(hwnd, 3,     // saveas, converts raw to DOOM format again
                            DrawingName,       // patch name
                            GFXWadName,        // filename,
                             0,                 // n/a GraphicDirstart,
                            &RawImageSize,      // n/a
                            &PicHeaderData,     // header 8
                            &MainImageSize,     // n/a
                            (byte **)&lpDrawBits))
  {
      StorePwad (GFXWadName);
      gd.OKDraw = 1;                // reset to normal (set by StorePwad)
      Success = TRUE;
  }

  DeleteBitmap(hDrawBitmap);                       // created by createdibsection above
  lpDrawBits  = NULL;
  hDrawBitmap = NULL;

  FreeMemory(RawImageData);
  RawImageData = NULL;

  ReleaseDC(hwnd, hdc);
  return Success;
}

//------------------------------------------------------------------------------
// change X Y size of graphic or offsets - has to be a TEXTURE lump format
//------------------------------------------------------------------------------

static Bool ChangePatchTextureSize(HWND hwnd)
{
  DirPtr   dir;
  Bool     Success = FALSE;
  int2     code;
  int2     oldXsize;
  int2     oldYsize;
  int2     oldXoff;
  int2     oldYoff;
  int2     NewXsize;
  int2     NewYsize;
  int2     NewXoff;
  int2     NewYoff;
  char     msgText[80];
  WadFiles PicWad;               // the sprite file

  DOOMGraphicHdr PicHeader;


  HPALETTE    hOldPal1;          //
  HPALETTE    hOldPal2;          //
//HBITMAP     hOldBitmap1;       // Handle to previous bitmap
  HBITMAP     hOldBitmap2;       // Handle to previous bitmap

  HDC         hdcSource;
  BITMAPINFO  BitmapInfoSource;  // DIB Bitmap Info Struct. for original image
  HBITMAP     hBitmapSource;     // Handle to new bitmap
  VOID **     lpBitsSource;      // Pointer to bits in DIBSECTION

  HDC         hdcTarget;         // Handle to memory DC
  BITMAPINFO  BitmapInfoTarget;  // DIB Bitmap Info Struct. for original image
  HBITMAP     hBitmapTarget;     // Handle to new bitmap
  VOID **     lpBitsTarget;      // Pointer to bits in DIBSECTION

//=============================================================================

  // get lump

  dir = FindMasterDir( MasterWad, DrawingName,FALSE);

  if (!dir)
  {
      ErrorMsg(hwnd,"Patch %s not found",DrawingName);
      return Success;
  }
  if (dir->filenumber != 0)
  {   CloseWadFiles('O',FREEDIRNO,FREENAMENO);  // close all
  }

  SetActiveWad(&PicWad,dir->filenumber);

  FindLumpPicSize(DrawingName,&NewXsize,&NewYsize,&NewXoff,&NewYoff);

  oldXsize = NewXsize;
  oldYsize = NewYsize;
  oldXoff  = NewXoff ;
  oldYoff  = NewYoff ;

  sprintf(msgText,"Change Lump %s Size/Offsets",DrawingName);

  if(!Input4Numbers(hwnd, msgText,
                            "X size",      1,1024,&NewXsize,
                            "Y size",      1, 255,&NewYsize,
                            "X offset",-1023,1023,&NewXoff,
                            "Y offset", -254, 254,&NewYoff)) return Success;

   if ( NewXsize != oldXsize || NewYsize != oldYsize
     || NewXoff  != oldXoff  || NewYoff  != oldYoff)
   {

       // if only offsets change, can write back as is immediate change

       if (NewXsize == oldXsize && NewYsize == oldYsize)
       {
	        SeekBytes( PicWad.fileinfo, dir->start   );     // seek start of entry
           ReadBytes( PicWad.fileinfo, &PicHeader,12);     // x width , y height, xoff,yoff
	        PicHeader.xsize = NewXsize;
	        PicHeader.ysize = NewYsize;
	        PicHeader.xoffs = NewXoff;
           PicHeader.yoffs = NewYoff;
	        SeekBytes ( PicWad.fileinfo, dir->start   );    // seek start of entry
           WriteBytes( PicWad.fileinfo, &PicHeader,12);    // x width , y height, xoff,yoff
       }
       else
       {
           // Create Source bitmap (original)

           hdcSource = GetDC(hEditWindow);                  // window we are interested in
           hOldPal1 = SelectPalette(hdcSource, gd.hDoomPal, TRUE ); //FALSE    TRUE matches colors for us
           RealizePalette(hdcSource);

           ZeroMemory(&BitmapInfoSource, sizeof(BITMAPINFO));
           BitmapInfoSource.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
           BitmapInfoSource.bmiHeader.biPlanes    = 1;
           BitmapInfoSource.bmiHeader.biBitCount  = 8;
           BitmapInfoSource.bmiHeader.biWidth     =  oldXsize;
           BitmapInfoSource.bmiHeader.biHeight    = -oldYsize;
	        BitmapInfoSource.bmiHeader.biSizeImage = ((oldXsize + 3 ) & ~3) * oldYsize;
           BitmapInfoSource.bmiHeader.biCompression = BI_RGB;
           hBitmapSource = (HBITMAP)CreateDIBSection(hdcSource, &BitmapInfoSource, DIB_PAL_COLORS, (VOID **)&lpBitsSource, NULL, 0L);
           if (!hBitmapSource)
           {   ReleaseDC(hwnd, hdcSource);
               OutOfMemoryMsg(hwnd);
               return Success;
           }

           // Create a memory DC to copy to
           hdcTarget = CreateCompatibleDC(hdcSource);
           if (!hdcTarget)
           {   ReleaseDC   (hwnd, hdcSource);
               DeleteBitmap(hBitmapSource);
               return FALSE;
           }
           hOldPal2 = SelectPalette(hdcTarget, gd.hDoomPal, TRUE ); //FALSE
           RealizePalette(hdcTarget);

           //---
           // create bitmap copy to copy into   (or createcompatible bitmap ?)
           // --
           ZeroMemory(&BitmapInfoTarget,     sizeof(BITMAPINFO));
           BitmapInfoTarget.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
           BitmapInfoTarget.bmiHeader.biPlanes    = 1;
           BitmapInfoTarget.bmiHeader.biBitCount  = 8;
           BitmapInfoTarget.bmiHeader.biWidth     =  NewXsize;
           BitmapInfoTarget.bmiHeader.biHeight    = -NewYsize;
           BitmapInfoTarget.bmiHeader.biSizeImage = ((NewXsize + 3 ) & ~3) * NewYsize;
           BitmapInfoTarget.bmiHeader.biCompression = BI_RGB;
           hBitmapTarget= (HBITMAP)CreateDIBSection(hdcTarget, &BitmapInfoTarget, DIB_PAL_COLORS, (VOID **)&lpBitsTarget, NULL, 0L);
           if (!hBitmapTarget)
           {   ReleaseDC(hwnd, hdcSource);
               DeleteDC(hdcTarget);
               DeleteBitmap(hBitmapSource);         // Our Dib Section
               OutOfMemoryMsg(hwnd);
               return Success;
           }

           // Create original bitmap in hdcSource

           code = DisplayGraphicTexture(hEditWindow, hdcSource, DOMASTER,
                                        0,0,oldXsize-1,oldYsize-1,                       // whatever fits
                                       &RawImageSize,         // size
                                       &RawImageData,         // raw data bufr
                                       &MainImageSize,        // image size
                                       (byte **)&lpBitsSource,// image buffer
                                        hBitmapSource,        // handle dib
                                       (char *)DrawingName,
                                        1,
                                        NOSHOWINFO,
                                        Transparent,
                                        SHOWIMAGE);           // show it with old size

           ValidateRect(hEditWindow,NULL);                    // if nothing shown not reqd

           //CheckActivity(NULL, WAIT);     // debug only, look at it

           if (code >=1 ) // usually -2 if not a valid graphic but should not happen
           {
               //hOldBitmap1 = SelectBitmap(hdcSource,hBitmapSource);   // selectobject
               //SetDIBColorTable(hdcSource, 0, 256, dRGBquad);		    // set Palette indexes

               hOldBitmap2 = SelectBitmap(hdcTarget,hBitmapTarget);   // selectobject
               SetDIBColorTable(hdcTarget, 0, 256, dRGBquad);         // set Palette indexes

               // copy contents of current bitmap screen  =

               SetStretchBltMode(hdcTarget,HALFTONE);                  //halftone required for color pictures
               SetBrushOrgEx (hdcTarget,0,0,NULL);

               StretchBlt(hdcTarget, 0,0, NewXsize, NewYsize,
                          hdcSource, 0,0, oldXsize, oldYsize, SRCCOPY);
               GdiFlush();

               // DEBUG -start - show what image will look like
               //MagnifyGraphicTexture(hEditWindow, EditRC.left,  EditRC.top,
               //                                   EditRC.right, EditRC.bottom,
               //                                   (byte *)&*lpBitsTarget,
               //                                   hBitmapTarget,
               //                                   1, Transparent);
               //CheckActivity(NULL, WAIT);
               // DEBUG - end -


               //------
               SelectPalette(hdcSource,hOldPal1, FALSE);   // probably don't need this check later
               SelectPalette(hdcTarget,hOldPal2, FALSE);   // probably don't need this check later
               //SelectBitmap (hdcSource, hOldBitmap1);      // selectobject
               SelectBitmap (hdcTarget, hOldBitmap2);      // selectobject
               DeleteDC(hdcTarget);                        // delete hdc
               DeleteBitmap(hBitmapSource);                // delete original bitmap

               PicHeader.xsize= NewXsize;                       // 2 width in pixels
               PicHeader.ysize= NewYsize;                       // 2 height in pixels
               PicHeader.xoffs= NewXoff;                        // 2 initial x offset
               PicHeader.yoffs= NewYoff;                        // 2 initial y offset
               //PicHeader.ystart;                              // 4 offset 1st Y column start (there are X of these)
               byte *PicHeaderData = (byte*)&PicHeader;

               // DrawBI is implicitly used here -- must be for now --

               DrawBi.bmiHeader.biWidth = NewXsize;  // global size
               DrawBi.bmiHeader.biHeight=-NewYsize;

               if (SaveDrawGraphic(hwnd,
                             3,                 // saveas, converts raw to DOOM format again
                             DrawingName,       // patch name
                             GFXWadName,        // filename,
                             0,                 // n/a GraphicDirstart,
                            &RawImageSize,      // n/a
                            &PicHeaderData,     // header 8
                            &MainImageSize,     // n/a
                            (byte **)&lpBitsTarget))
               {
                    StorePwad (GFXWadName);
                    gd.OKDraw = 1;              // reset to normal (set by StorePwad)
                    Success = TRUE;
               }

               DeleteBitmap(hBitmapTarget);     // created by createdibsection above
           }
           else
           {
               DeleteDC(hdcTarget);                       // delete

               DeleteBitmap(hBitmapSource);
               DeleteBitmap(hBitmapTarget);
               NotifyMsg(hwnd,"Display Failed due to invalid image?");
           }


           FreeMemory(RawImageData);
           RawImageData = NULL;

           ReleaseDC(hwnd, hdcSource);

       } // new size or offsets
   }
   return Success;
}

               /*
               GetDIBits(hdcSource,       	    // handle of device context
                         hBitmapSource,          // handle of bitmap
                         0,	                   // first scan line to set in destination bitmap
                         oldYsize,               // number of scan lines to copy
                         (byte **)&lpBitsSource, //  address of array for bitmap bits      (Target)
                         &BitmapInfoSource,	    // address of structure with bitmap data  (Target)
                         DIB_PAL_COLORS);        // RGB or palette index

               */
/*
                /////////////////////////////////////////////////////////////////////////////////
                // examples of scrolling a bitmap (help reference find "bitmap resize"
static HDC hdcWin;           //* window DC
static HDC hdcScreen;        //* DC for entire screen
static HDC hdcScreenCompat;  //* memory DC for screen
static HBITMAP hbmpCompat;   //* bitmap handle for old DC
static BITMAP bmp;           //* bitmap data structure



        // * Create a normal DC and a memory DC for the entire
        // * screen. The normal DC provides a snapshot of the
        // * screen contents. The memory DC keeps a copy of this
        // * snapshot in the associated bitmap.

        hdcScreen = CreateDC("DISPLAY", (LPCSTR) NULL,
                              (LPCSTR) NULL, (CONST DEVMODE *) NULL);
        hdcScreenCompat = CreateCompatibleDC(hdcScreen);

        //
        // * Retrieve the metrics for the bitmap associated with the
        // * regular device context.
        //

        bmp.bmBitsPixel =
            (BYTE) GetDeviceCaps(hdcScreen, BITSPIXEL);
        bmp.bmPlanes = (BYTE) GetDeviceCaps(hdcScreen, PLANES);

        bmp.bmWidth = GetDeviceCaps(hdcScreen, HORZRES);
        bmp.bmHeight = GetDeviceCaps(hdcScreen, VERTRES);

        //* The width must be byte-aligned.

        bmp.bmWidthBytes = ((bmp.bmWidth + 15) &~15)/8;

        //* Create a bitmap for the compatible DC.

        hbmpCompat = CreateBitmap(bmp.bmWidth, bmp.bmHeight,
                     bmp.bmPlanes, bmp.bmBitsPixel, (CONST VOID *) NULL);

        // Select the bitmap for the compatible DC.


        SelectObject(hdcScreenCompat, hbmpCompat);

                //
                //  Copy the contents of the current screen
                //  into the compatible DC.

                BitBlt(hdcScreenCompat, 0, 0, bmp.bmWidth,
                    bmp.bmHeight, hdcScreen, 0, 0, SRCCOPY);

                //
                //  Copy the compatible DC to the client area.
                //
                hdcWin = GetDC(hwnd);

                BitBlt(hdcWin, 0, 0, bmp.bmWidth, bmp.bmHeight,
                    hdcScreenCompat, 0, 0, SRCCOPY);

                ReleaseDC(hwnd, hdcWin);
                */

//------------------------------------------------------------------------------
// see how many lumps are using the same file as this LumpName
// answer is number of lumps -1 (self is not counted)
// file 0 (main iwad) is not counted
//------------------------------------------------------------------------------

int4 FindLumpFilesUsed(char*LumpName)
{
   //LevelPtr level=&LevelWad.level;
   DirPtr   dir;
   int4     count = 1;
   int4     d;
   int2     ActiveFileNumber;

   dir = FindMasterDir( MasterWad, LumpName,FALSE);
   if( dir )
   {   ActiveFileNumber = dir->filenumber;

       if (ActiveFileNumber > 0)               // ignore master
       {
           dir = MasterWad->directory;
           for (d=0; d < MasterWad->dirnum ; d++)
           {    if (dir->filenumber == ActiveFileNumber) count++;  // count self too
           }
       }
   }
   count --;            // subtract self out of answer
   return count;
}

//------------------------------------------------------------------------------
// save the image
//
//  DoSaveAs :
//             1: old file name    - save from RawDataBufr on top of existing
//             4: old file name    - recalculate from image buffer make sure same size
//
//             2: New file name    - save from raw unless FLAT then from image
//             3: new file name    - recalculate from image buffer
//
//------------------------------------------------------------------------------

#pragma argsused
static Bool SaveDrawGraphic(HWND   hwnd,
                            int4   DoSaveAs,
                            char*  picname,           // picname
                            char*  outfilename,
                            int4   diroffset,
                            int4*  ActualRawDataSize, byte** ActualRawDataBufr,
                            int4*  ActualImageSize,   byte** ActualImageBufr)
{
   Bool  Success = FALSE;

   int4  ActiveCount;
   char  LumpName[10] = {0};
   char  tempname[260];
   FILE *FileANY;
   char  msg1[80];

   DirPtr dir;

   DOOMImagePtr YoffImage=NULL;    // allocate X of these
   DOOMImagePtr YoffWrk;
   DOOMImagePtr YoffNext;

   int4  Xwidth;
   register x;
   int4  FlatSize = 4096;                        // make it 4096 always

   int4  HdrDirEntries  = 1;
   int4  HdrDirStart = 12;
   int4  totalsize;

   byte  Hex00 = 0x00;
   byte  HexFF = 0xff;

//------------------------------------------------------------------------------

// NotifyMsg(NULL,"save = %hd Flat = %hd",DoSaveAs,FlatLump);

   Xwidth = DrawBi.bmiHeader.biWidth;   // global size

   //
   //  3 = new filename, save modified, recalculate now (find out any memory probs right away)
   //  4 = old filename, save modified, recalculate now (find out any memory probs right away)
   //
   if ( DoSaveAs == 3
     || DoSaveAs == 4
     && !FlatLump)                                      // NOT FLATS
   {
       if (!ImageToRaw( *ActualImageBufr,&YoffImage) )  // gets image dimensions from Drawbi buffer
       {
            OutOfMemoryMsg(hwnd);

            return Success;
       }
   }

   // fileany open/close is required to actually write the data!!

   //--------------------------------------------------------
   // 2= New file name - Save As from RAW data              /
   // 3= New file name - Recalculate from image buffer      /
   //--------------------------------------------------------

   if (DoSaveAs == 2
   ||  DoSaveAs == 3)
   {
       strcpy (tempname,picname);
       strcat (tempname,".WAD");

       GetSaveName(hwnd,"Graphic PWAD", tempname,256, IDS_FILTERWAD );

       if(tempname[0] != '\0')  // cancel ?
       {
	       strcpy(LumpName,picname);

          // get lumpname first so we can check to see if file is in use here

	       if (!InputUserText (hwnd,"8 Characters MAX", 8, "Graphic Lump Name", LumpName,UPPERCASE)
          ||  LumpName[0] == '\0')
	       {   DoSaveAs = 0;
	       }
	       else
          {   // find lump name  (if it duplicates existing)
              if (!strnicmp(LumpName,picname,8))                // same name
              {
                  ActiveCount = FindLumpFilesUsed(LumpName);    // more than one?

                  if (ActiveCount)
                  {
                      // if multiple use, can't use same file name (different name is ok)

                      if ( FileIsInUse(tempname,TRUE,FALSE) )   // msg + no set outfile to '\0'
	                   {
		                    DoSaveAs = 0;
	                   }
                  }
                  else
                  {   // only used once -self- get rid of it
                      // by giving a  so FreeNotUsedFiles gets rid of it
                      dir = FindMasterDir( MasterWad, picname,FALSE);
                      if( dir )
                      {   if (dir->filenumber > 0)
                          {   dir->filenumber = 0;
                          }
                      }
                  }
              }
              else
	           if ( FileIsInUse(tempname,TRUE,FALSE) )   // msg + no set outfile to '\0'
	           {    DoSaveAs = 0;
	           }
	       }
       }
       else
       {   DoSaveAs = 0;
       }
   }

   // make sure still active-----------------------------------------------

   if (!DoSaveAs) return Success;  // exit here to keep indents less, that's all

   // 1 = old file name - save from RawDataBufr on top of existing
   // 4 = old file name - save modified, recalculate now (find out any memory probs right away)

   if (DoSaveAs == 1
   ||  DoSaveAs == 4 )
   {
       // extra safety stuff make sure we don't destroy something

       dir = FindMasterDir( MasterWad, picname,FALSE);
       if( !dir )
       {    ErrorMsg(hGFXWindow,"SaveGraphicName Error: %s was not found?",picname);
            return Success;
       }
       else
       {   if (dir->filenumber == 0
           ||  dir->start != diroffset
           ||  dir->size  != *ActualRawDataSize)
           {
               ErrorMsg(hGFXWindow,"SaveGraphicName Error: %s logical mismatch?",picname);
               return Success;
           }
       }

       FileANY = FileOpen(outfilename, "r+b",OPENMISS);                     // old open

       if (FileANY)
	    {   // 1= oldfile name- copy rawdata
           if (DoSaveAs == 1)
           {
	            WadSeek   ( FileANY, diroffset);
	            WriteBytes( FileANY,*ActualRawDataBufr,*ActualRawDataSize);  // modified data
	            fclose(FileANY);
               Success = TRUE;
           }
           else
           // 4 = oldfilename, save modified from ImageBufr
           if (FlatLump)  // ONLY FOR FLATS write imagebuffer
           {
               WadSeek   ( FileANY,  diroffset);
	            WriteBytes( FileANY, *ActualImageBufr,FlatSize);             // modified data in imagebuffer
	            fclose(FileANY);
               Success = TRUE;
           }
           else
           // not a Flat
           {
               int4  actualsize;                                            // size of data all of it
               int4  Offset;
               int4  YoffsetSize = 0;                                       // cumulative
               int4 *YoffsetOrg    =(int4*) malloc( (Xwidth+1)* 4 );        // max X offsets
               int4 *YoffsetStorage= YoffsetOrg;

               if  (!YoffsetOrg)
               {    OutOfMemoryMsg(hwnd);
                    fclose(FileANY);
                    return Success;
               }

               int4  YdataMax  = 127996;
               int4  YdataSize = 0;                        // cumulative count
               byte *YdataOrg    =(byte*) malloc(128000);  // start with 128kb takes care of almost all
               byte *YdataStorage= YdataOrg;

               if  (!YdataOrg)
               {    OutOfMemoryMsg(hwnd);
                    FreeMemory(YoffsetOrg);
                    fclose(FileANY);
                    return Success;
               }

               // get total size of all data

               Offset  = 8 + 4*Xwidth;                     // loc of first data

               YoffWrk = YoffImage;

               // create X offsets pointing to Y columns

               for (x=0; x < Xwidth; x++, YoffWrk+=1)      // X
               {
                   *YoffsetStorage = Offset;
                    YoffsetStorage++;                      // next storage
                    YoffsetSize +=4;                       // count size

                    Offset += YoffWrk->ybytes +4;          // size +control 4
                    for (YoffNext=YoffWrk->next; YoffNext; YoffNext = YoffNext->next)
                    {    if (YoffNext->ybytes > 0)
                         {   Offset +=YoffNext->ybytes+4;  // control+data
                         }
                    }
                    Offset++;   // and last FF (just here to make this clear)
               }

               YoffWrk = YoffImage;

               for (x=0; x < Xwidth; x++, YoffWrk+=1)      // X
               {
                    if ( (YdataSize+YoffWrk->ybytes+4) > YdataMax)
                    {
                         YdataOrg = (byte*) realloc(YdataOrg,YdataMax+10000);
                         YdataStorage = YdataOrg+ YdataSize;       // new org + amount stored
                         YdataMax +=10000;
                         if (!YdataOrg)
                         {   OutOfMemoryMsg(hwnd);
                             FreeMemory(YoffsetOrg);
                             fclose(FileANY);
                             return Success;
                         }
                    }

                    *YdataStorage++ = YoffWrk->yorigin;                      // 1 Yoffset
	                 *YdataStorage++ = YoffWrk->ybytes;                       // 2 #bytes
	                 *YdataStorage++ = Hex00;                                 // 3 n/a
	                  memcpy(YdataStorage,YoffWrk->ydata,YoffWrk->ybytes);    // data
                     YdataStorage  += YoffWrk->ybytes;
	                 *YdataStorage++ = Hex00;                                 // 4 n/a

                    YdataSize +=YoffWrk->ybytes+4;                     // control+data

                    for (YoffNext=YoffWrk->next; YoffNext; YoffNext = YoffNext->next)
                    {
                         if (YoffNext->ybytes > 0)
                         {
                             if ( (YdataSize+YoffNext->ybytes+4) > YdataMax)
                             {     YdataOrg     = (byte*)realloc(YdataOrg,YdataMax+10000);
                                   YdataStorage = YdataOrg+ YdataSize;       // new org + amount stored
                                   YdataMax +=10000;
                                   if (!YdataOrg)
                                   {   OutOfMemoryMsg(hwnd);
                                       FreeMemory(YoffsetOrg);
                                       fclose(FileANY);
                                       return Success;
                                   }
                             }
                         }

                         *YdataStorage++ = YoffNext->yorigin;                      // 1 Yoffset
          	             *YdataStorage++ = YoffNext->ybytes;                       // 2 #bytes
	                      *YdataStorage++ = Hex00;                                  // 3 n/a
	                       memcpy(YdataStorage,YoffNext->ydata,YoffNext->ybytes);   // data
                          YdataStorage  += YoffNext->ybytes;
	                      *YdataStorage++ = Hex00;                                  // 4 n/a

                          YdataSize += YoffNext->ybytes+4;              // control+data
                    }
	                 *YdataStorage = HexFF;                             // 0xff end
                     YdataStorage++;
                     YdataSize++;
               }

               actualsize = YoffsetSize+ YdataSize +8; // 8 header, + X-Y offsets, + data

               if (actualsize == *ActualRawDataSize)
               {
      	            WadSeek   ( FileANY, diroffset);
                     WriteBytes( FileANY, *ActualRawDataBufr, 8);       // header only X ,Y size x,y offsets
                     WriteBytes( FileANY, YoffsetOrg, YoffsetSize); //
                     WriteBytes( FileANY, YdataOrg,   YdataSize);   //
                     Success = TRUE;
               }
               else
               {    sprintf(msg1,"Saved size %d not same as calculated size %d",actualsize,*ActualRawDataSize);
                    HelpMsg(hwnd,CENTER|HELPINFO, "Save Image",
                                                  msg1,
                                                  "",
                                                  "Original Image is not stored correctly",
                                                  "Use SaveAS to save",
                                                  "",
                                                  "Data not written",
                                                  NULL);
               }

               FreeMemory(YdataOrg);
               FreeMemory(YoffsetOrg);
               fclose(FileANY);
           }

           if (Success)
           {   ShowStatus(hwnd,"Graphic Saved Successfully !");
           }

	    } // end of FileAny
   }
   else
   //   2 = New file name - save from raw                 unless FLAT then from image
   //   3 = new file name - recalculate from image buffer unless FLAT ...............
   if ( DoSaveAs == 2
   ||   DoSaveAs == 3 )
   {
       FileANY = FileOpen(tempname, "w+b",OPENMISS);                  // new open

       if (FileANY)
       {
           strcpy(outfilename,tempname);                 // store name back   8-1-97

           if(FlatLump)  // ONLY FOR FLATS
           {
	           WriteBytes( FileANY, "PWAD",          4);        // PWAD id
	           WriteBytes( FileANY, &HdrDirEntries , 4);        //+4   1 direntry entry
	           WriteBytes( FileANY, &HdrDirStart   , 4);        //+8  12 offset = start of Dir
              totalsize = 12;

	           WriteBytes( FileANY,* ActualImageBufr,FlatSize);   // modified data in imagebuffer

              // write directory
	           WriteBytes( FileANY, &totalsize, 4);         // start of data
	           WriteBytes( FileANY, &FlatSize, 4);          // size of file 4096
	           WriteBytes( FileANY, LumpName , 8);          // name

              totalsize +=FlatSize;

              // write directory start
              SeekBytes ( FileANY, 8);
	           WriteBytes( FileANY, &totalsize  , 4);        //+8 offset = start of Dir

	           fclose(FileANY);

              ShowStatus(hwnd,"Graphic Saved Successfully !");
              Success = TRUE;
           }
           else
           // not a Flat
           {
              int4  actualsize= 0;                         // size of data
              int4  Offset;

	           WriteBytes( FileANY, "PWAD",          4);        // PWAD id
	           WriteBytes( FileANY, &HdrDirEntries , 4);        //+4   1 direntry entry
	           WriteBytes( FileANY, &HdrDirStart   , 4);        //+8  12 offset = start of Dir
              totalsize = 12;

              if (DoSaveAs == 2)
              {
	              WriteBytes( FileANY, *ActualRawDataBufr,*ActualRawDataSize); // modified data
                 actualsize = *ActualRawDataSize;
              }
              else
              {
	              WriteBytes( FileANY, *ActualRawDataBufr, 8); // header only X ,Y size x,y offsets
                 actualsize +=8;

                 // get total size of all data

                 Offset = 8 + 4*Xwidth;                  // loc of first data

                 YoffWrk = YoffImage;

                 for (x=0; x < Xwidth; x++, YoffWrk+=1)  // X
                 {
	                   WriteBytes( FileANY, &Offset, 4);    // offset
                      actualsize +=4;

                      Offset += YoffWrk->ybytes +4;        // size +control 4
                      for (YoffNext=YoffWrk->next; YoffNext; YoffNext = YoffNext->next)
                      {    if (YoffNext->ybytes > 0)
                           {   Offset +=YoffNext->ybytes+4;  // control+data
                           }
                      }
                      Offset++;   // and last FF (just here to make this clear)
                 }

                 YoffWrk = YoffImage;
                 for (x=0; x < Xwidth; x++, YoffWrk+=1)  // X
                 {
	                   WriteBytes( FileANY,&YoffWrk->yorigin, 1);            // Yoffset
	                   WriteBytes( FileANY,&YoffWrk->ybytes,  1);            // #bytes
	                   WriteBytes( FileANY,&Hex00            ,1);            // n/a
	                   WriteBytes( FileANY, YoffWrk->ydata,YoffWrk->ybytes); // data
	                   WriteBytes( FileANY,&Hex00            ,1);            // n/a
                      actualsize +=YoffWrk->ybytes+4;  // control+data

                      for (YoffNext=YoffWrk->next; YoffNext; YoffNext = YoffNext->next)
                      {    if (YoffNext->ybytes > 0)
                           {   WriteBytes( FileANY,&YoffNext->yorigin, 1);             // Yoffset
	                            WriteBytes( FileANY,&YoffNext->ybytes,  1);             // #bytes
	                            WriteBytes( FileANY,&Hex00             ,1);             // n/a
	                            WriteBytes( FileANY, YoffNext->ydata,YoffNext->ybytes); // data
	                            WriteBytes( FileANY,&Hex00             ,1);             // n/a
                               actualsize +=YoffNext->ybytes+4;  // control+data
                           }
                      }
	                   WriteBytes( FileANY,&HexFF ,1);            // 0xff end
                      actualsize++;
                 }

              }

              // write directory
	           WriteBytes( FileANY, &totalsize,  4);      // start of data
              WriteBytes( FileANY, &actualsize, 4);      // size of file
              WriteBytes( FileANY, LumpName, 8);         // name

              totalsize +=actualsize;

              WadSeek( FileANY, 8 );                     // seek dir start
              WriteBytes( FileANY, &totalsize, 4);       // start of dir

	           fclose(FileANY);

              ShowStatus(hwnd,"Graphic Saved Successfully !");
              Success = TRUE;
           }  // NOT FLAT

	    }  //end FileANY
   }

   if (YoffImage)
   {   YoffWrk = YoffImage;

        for (x=0; x < Xwidth; x++,YoffWrk+=1) // X
        {
             FreeDoomGraphic(&YoffWrk);   // free all chained info
        }
        FreeMemory(YoffImage);
        YoffImage = NULL;
   }

   UnDoActive = 0;
   DidRotate  = FALSE;
   DidCopy    = FALSE;

   return Success;
}

//----------------------------------------------------------------------------
// convert image format to DOOM format
//
// Data HAS to be in DrawBi for now (easy to change)
//
//  DOOM Format:  (output routine formats it this way, not here ,just raw data)
//       xsize      2
//       ysize      2
//       xoffset    2
//       yoffset    2
//       dataoffset 4 offset from start of file to Y column data (starts with yorigin)
//        ...       4
//        ...       4 there is one offset for ever X value (ie, X offsets)
//
//   I   yorigin    1 Y start of this column at X=0
//       ybytes     1 Y number of data bytes
//       notused    1
//       ydata..... variable = ybytes of course
//       notused    1
//       yorigin2   1 next Y start, the last one is FF, meaning end of column
//       ybytes2
//       ..etc..
//
//  II   yorigin    1 next Y start for column at X=1
//       ..etc..
//
//----------------------------------------------------------------------------

static Bool ImageToRaw(byte *ImageBufr, DOOMImagePtr *DoomBufr)
{
   register x,y;

   int2  lastX;

   int2  Xwidth =     DrawBi.bmiHeader.biWidth;  // global size
   int2  Yheight= abs(DrawBi.bmiHeader.biHeight);
   int2  Align  = 0;
   int2  Xalign = (Xwidth+3)&~3;  //   5-1-97 added +3~3


   Bool  Success = TRUE;          // alloc went ok
   byte *WrkByte;                 // wrk ptr to structure
   byte  color;

   DOOMImagePtr Yoffsets;         // allocate X of these
   DOOMImagePtr YoffWrk;          //
   DOOMImagePtr YoffNext;         //

   int2  LastColor;                // for switching

   //------------------------------------------------------------------------------

   Yoffsets = (DOOMImagePtr)malloc(Xwidth * sizeof(struct DoomImage )); // one for each X

   if (!Yoffsets)  return FALSE;
   if (!ImageBufr) return FALSE;

   YoffWrk = Yoffsets;

   for (x=0; x < Xwidth; x++,YoffWrk+=1) // X-------
   {
        YoffWrk->ydata   = (byte *)malloc (Yheight); // data
        if (!YoffWrk->ydata)
        {   Success = FALSE;
            lastX = x-1;
            break;
        }

        LastColor = Transparent;            // for first time switching

        YoffWrk->yorigin = 0;               // yorg = 0 for now
        YoffWrk->ybytes  = 0;               // #of Y bytes
        YoffWrk->next    = NULL;            // next if transparent

        WrkByte  = YoffWrk->ydata;          // where to store data
        YoffNext = YoffWrk;

        for (y=0; y< Yheight; y++)       // Y-------
        {
             color  = *&ImageBufr[ y*Xalign+x +Align];

             if (color !=Transparent)           // good data
             {  if (LastColor == Transparent)   // start of column data
                {    YoffNext->yorigin = y;     // starting Y location
                }
                YoffNext->ybytes ++;
                *WrkByte++ = color;             // store data
             }
             else
             // transparent color
             if (LastColor != Transparent)      // prior was not transparent
             {
                 if ( (Yheight -y) <= 1) break;   // last y -> done

                 // chain this one to any new one
                 YoffNext->next   = (DOOMImagePtr)malloc(sizeof(struct DoomImage));

                 if (!YoffNext->next)                  // alloc failed
                 {   Success = FALSE;
                     OutOfMemoryMsg(hEditWindow);
                     break;
                 }

                 YoffNext = YoffNext->next;              // switch to new one
                 YoffNext->ydata   = (byte *)malloc (Yheight-y); // data storage
                 YoffNext->yorigin = 0;               // yorg = 0 for now
                 YoffNext->ybytes  = 0;               // #of Y bytes
                 YoffNext->next    = NULL;            // next if transparent

                 WrkByte = YoffNext->ydata;           // where to store data

                 if (!YoffNext->ydata)                // alloc failed
                 {   Success = FALSE;
                     OutOfMemoryMsg(hEditWindow);
                     break;
                 }
             }

             LastColor = color;
        } // end of y

        if (!Success) break;     // allocate failed in Y

   } // end of x

   // if failure on allocation

   if (!Success)
   {
       YoffWrk = Yoffsets;

       for (x=0; x < lastX; x++,YoffWrk+=1) // X
       {
            FreeDoomGraphic(&YoffWrk);      // free all chained info
       }
       FreeMemory(Yoffsets);
       return FALSE;
   }


   *DoomBufr = Yoffsets;   // pass back to caller
   return TRUE;
}

//----------------------------------------------------------------------//
//   free the Doom Graphic list chain
//----------------------------------------------------------------------//

static void FreeDoomGraphic( DOOMImagePtr *list)
{
   DOOMImagePtr cur,prev;

   cur = *list;

   cur = cur->next;
   while (cur)
   {     prev = cur;
         cur  = cur->next;   // chain to next now
         free( prev->ydata); // free column data
         free( prev);        // free this block
   }
}

//////////////////////////////////////////////////////////////////////////////////
///
///   image manipulation
///
//////////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------------
// Rotates image left or right in imagebuffer
//
//--------------------------------------------------------------------------------
static void RotateImage(byte *UserImageBufr,int2 xoffset, int2 yoffset)
{
   int4  Xalign;               // was 2 row bytes aligned

   int4  x,y;                  // was 2
   int4  j;
   int4  WrkImageSize;
   byte *WrkImageBufr;          // buffer containing image
   byte *baseYcolor;            // start of Y data
   byte  color;
   int4  PixLoc;
//   int4  align;                // was 2
   LONG  xsize, ysize;

//-----------------------------------------------------------------------------

   xsize   =     DrawBi.bmiHeader.biWidth;
   ysize   = abs(DrawBi.bmiHeader.biHeight);

// align   = 0;
   Xalign  = ( xsize + 3) & ~3;      // row bytes aligned

   WrkImageSize = Xalign * ysize;   //   5-1-97

   WrkImageBufr = (byte *)malloc( WrkImageSize);       // allocate memory needed
   if (!WrkImageBufr) return;                          // no memory msg++++

   memcpy(WrkImageBufr,UserImageBufr,WrkImageSize);

   // copy mag 1 image buffer into this one
   if (xoffset < 0) xoffset = xsize + xoffset;
   if (yoffset < 0) yoffset = ysize + yoffset;

   xoffset = xoffset%xsize;
   yoffset = yoffset%ysize;

   // first do x

   if (xoffset > 0)
   {
     for (y = 0; y < ysize;  y++)
     {
        baseYcolor = &WrkImageBufr[ y*Xalign];  // source
	     // first 1/2 of shift
	     for (j=0,x=xoffset; x< xsize; j++, x++)          // X-------
        {
	         color  = *(baseYcolor+x);           // source image

	         PixLoc = (y)*Xalign + j;//+ align;     // target offset

	         UserImageBufr[PixLoc] = color;

	     }  // end x 1

        // second 1/2 of shift
        for (j=xsize-xoffset,x=0; x< xoffset; j++, x++)          // X-------
	     {
	        color  = *(baseYcolor+x);           // source image

	        PixLoc = (y)*Xalign + j;//+ align;     // target offset

	        UserImageBufr[PixLoc] = color;

	     }   // end x 2
     } //end y
   }


   // now y

   if (yoffset > 0)
   {
     for (x=0; x< xsize; x++)                      // X-------
     {
        // first 1/2 of shift
	     for (j=0,y =yoffset; y < ysize; j++, y++)
        {
	         baseYcolor = &WrkImageBufr[ y*Xalign];//+align];  // source

	         color  = *(baseYcolor+x);              // source image

	         PixLoc = (j)*Xalign + x;//+align;         // target offset

	         UserImageBufr[PixLoc] = color;

	     }  // end y 1

        // second 1/2 of shift
	     for (j=ysize-yoffset,y=0; y< yoffset; j++, y++)          // X-------
	     {
	         baseYcolor = &WrkImageBufr[ y*Xalign ];//+align];  // source

	         color  = *(baseYcolor+x);              // source image

	         PixLoc = (j)*Xalign + x;//+ align;        // target offset

	         UserImageBufr[PixLoc] = color;

	     }   // end x 2
     } //end y
   }

   FreeMemory(WrkImageBufr);
}

//-------------------------------------------------------------------------------
// rotate
//-------------------------------------------------------------------------------
#define xPI  3.141592654        // for some reason I have to have this here ?? and can't use global PI
#define D90  xPI/2.0
#define D180 xPI
#define D270 3.0 * xPI/2.0
#define D360 xPI * 2.0

static void RotateXY( double *x, double *y, double angleChange)//angledelta)
{                                                // sqrt pi =0.0177245385
   // radians to degrees : radian = deg*pi/180 = .017453292

   long double r;                                // 3.141592654 /2 = 1.570796327 = 90degrees
   long double newangle;                         // 0.0174533
   long double bigX        = *x;                 // increase for accuracy
   long double bigY        = *y;
   long double newX;
   long double newY;
// long double angleChange = (angledelta * PI)/180.0;    // 0.017453292;    // convert to radians
   // documentation for me:
   // atan = y/x ;
   // hypot = sqrt (x2+y2);
   {

      // rotated position

      if (bigX != 0.0)
      {   newangle = atan2l( bigY, bigX )+ angleChange; // old angle+new angle
      }
      else
      {
          if (bigY < 0.0 )   // x = 0           <
	  {   newangle = D90+ angleChange;            // D90 at 0 degrees
	  }
	  else
	  if (bigY > 0.0)    // x = 0           >
	  {   //newangle = D270 +angleChange;          //D270  at 180 degrees
              return;
	  }
	  //else  //
	  //if (bigX > 0.0)    // y = 0           >
	  //{   newangle = angleChange ;               // at 90 degrees
	  //}
	  //else
	  //if (bigX < 0.0)    //  y=0            <
	  //{   newangle = D180+ angleChange;         // at 270.00
	  //}
	  else  //
	  {   return;                                // at the center
	  }
      }

       //if (newangle > D360) newangle-=D360;

       r = hypotl( bigX,bigY );              // radius

       newX = r* cosl ( newangle );// +.5;
       newY = r* sinl ( newangle );// +.5;

       *x   = newX;
       *y   = newY;

   }

}

//--------------------------------------------------------------------------------
// Flips image X or Y imagebuffer
// fliptype
//   0 = x,
//   1 = Y
//   2=  rotate left
//   3 = rotate right
//   4 = shrink + change x/y sizes
//   5 = shrink same bufr, x/y size same
//   6 = gamma correct
//--------------------------------------------------------------------------------
static void FlipImage(HWND hwnd,byte *UserImageBufr,int2 fliptype, int2 transparent, int4 Width, int4 Height)
{
   int4  x,y;                  // was 2  8-1-97  
   int4  sourceX,sourceY;      // ..
   int4  targetX,targetY;      // ..

   int4  j;
   int4  WrkImageSize;
   byte *WrkImageBufr;          // buffer containing image
   byte *baseYcolor;            // start of Y data
   byte  color;
   int4  PixLoc;

   int4  Xalign;               // row bytes aligned  was 2

   int4  TargetXsize;          //
   int4  TargetYsize;          //
   int4  TargetXalign;         //

static int2 Angle = 45;
static int2 Gamma = 0;
   double Fangle;
   double FCenterX,FCenterY;
   double Fdx,Fdy;
   int4   Rx,Ry;
   int4   oldRx = -1;
   int4   oldRy = -1;
   double Fxsize,Fysize;
   double Fx,Fy;
   int    cur,max;

//-----------------------------------------------------------------------------

   // get size of the buffer

   Xalign  = ( Width + 3) & ~3;                      // row bytes aligned

   Height = abs(Height);                             // make sure positive   7-1-97

   WrkImageSize = Xalign * Height;                   //   5-1-97

   WrkImageBufr = (byte *)malloc( WrkImageSize);     // allocate memory needed
   if (!WrkImageBufr) return;                        // no memory msg++++

   memcpy(WrkImageBufr,UserImageBufr,WrkImageSize);  // copy it

   //  x  flip
   switch(fliptype)
   {
     case(0):
       for (y = 0; y < Height;  y++)                     // Y
       {  baseYcolor = &WrkImageBufr[ y*Xalign];// +align];// source
          for (j=Width-1,x=0; x< Width; j--, x++)       // X-------
	       { color  = *(baseYcolor+x);                   // source image
            PixLoc = (y)*Xalign + j;//+ align;            // target offset
            UserImageBufr[PixLoc] = color;
          }  // end x
       }     // end y
       break;
     // flip Y
     case(1):
       for (x=0; x< Width; x++)                            // X-------
       { for (j=Height-1,y =0; y < Height; j--, y++)      // Y
	       { baseYcolor = &WrkImageBufr[ y*Xalign];// +align];// source
            color  = *(baseYcolor+x);                     // source image
            PixLoc = (j)*Xalign + x;//+ align;              // target offset
            UserImageBufr[PixLoc] = color;
          }  // end y 1
       }     // end x
       break;
     //
     // rotate +/-  ----------------------
     //
     case(2):
       if (!InputNumbers(hEditWindow, "Angle in Deg.", 1 ,359,&Angle, 50, 50) )
       {   return;
       }

       if (Angle == 0) return;

       Fxsize   = Width;
       Fysize   = Height;

       FCenterX = (Fxsize)/2.0;
       FCenterY = (Fysize)/2.0;
       Fangle   = ((double)Angle * xPI)/180.0;          //convert to radians

       //SetCursorImage(3);

       cur = 0;
       max = ((Fxsize * 2) * (Fysize * 2));
       PercentObject(hwnd,"Please Wait - Rotating Image",SAVE, max);

       for (Fx=0.0; Fx < Fxsize; Fx+=.50)              // X-------
       {
	      x = Fx;   	  // Y shift
         for (Fy =0.0; Fy < Fysize; Fy+=.50)  // Y
	      {
           cur ++;
           PercentDone(hwnd,cur, max);

           Fdx = Fx - FCenterX;
	        Fdy = Fy - FCenterY;
           RotateXY( &Fdx, &Fdy, Fangle );

           // hack till I find out what's going on
           if (Angle <=90)
           {
                if (Fdy > 0)       // in upper quads   1,2
                {   Fdy +=.5;
                    Fdx +=.5;
                }
                else               // lower quads      3,4
                {   Fdy +=.5;
                    Fdx -=.5;
                }
           }
           else
           if (Angle <=180)
           {
                if (Fdx > 0)       //  2,3
                {   Fdy -=.5;
                    Fdx -=.5;
                }
                else               // 1,4
                {   Fdy +=.5;
                    Fdx -=.5;
                }
           }
           else
           if (Angle <=270)
           {
                if (Fdy > 0)       // in upper quads   1,2
                {   Fdy +=.5;
                    Fdx +=.5;
                }
                else               // lower quads      3,4
                {   Fdy +=.5;
                    Fdx -=.5;
                }
           }

	        Rx = FCenterX+ Fdx;   // int4
	        Ry = FCenterY+ Fdy;

           if (oldRx != Rx || Ry != oldRy)
	        {  oldRx = Rx;
		        oldRy = Ry;

		        if (Rx >=0 && Rx < Width &&  Ry >=0 && Ry < Height)
		        {
		          y = Fy;
                baseYcolor = &WrkImageBufr[ y*Xalign];// +align];  // source
                color  = *(baseYcolor+x);                 // source image
		          PixLoc = Ry*Xalign + Rx;//+ align;       // target offset
                UserImageBufr[PixLoc] = color;
		        }
	        }

	       }  // end y 1
        }    //end x
       PercentDone(hwnd,100, 100);
       break;
     //
     // circle  n/a
     //
     case(3):
       break;
     //
     // -shrink image--              (also reduces original buffer?? not required   5-1-97)
     //
     case(4):
       TargetXsize  = Width/2;
       TargetYsize  = Height/2;
       TargetXalign = (TargetXsize+3)& ~3;

       if (TargetXsize <=0 || TargetYsize <=0) break;

       //ViewBi.bmiHeader.biWidth      =  TargetXsize;  maybe later global??
       //ViewBi.bmiHeader.biHeight     = -TargetYsize;
       //ViewBi.bmiHeader.biSizeImage  =  TargetXalign * TargetYsize;//   5-1-97

    //
    // shrink but keep buffer size same
    //
    case(5):
       memset(UserImageBufr,transparent, WrkImageSize);  // set transparent

       if (fliptype == 4)       // reduced buffer
       {   targetX      = 0;
       }
       else
       {   targetX      = Width/4;
	        TargetXalign = Xalign;
       }

       for (sourceX=0; sourceX< Width; sourceX+=2)           // X-------
       {
	       // Y shift

          if (fliptype == 4)     // reduced buffer
	       {   targetY = 0;
	       }
	       else
	       {   targetY = Height/4;
	       }

	       for (sourceY =0; sourceY < Height; sourceY+=2)  // Y
	       {
	         baseYcolor = &WrkImageBufr[sourceY*Xalign]  ; // source

	         color  = *(baseYcolor+sourceX);               // source image

	         PixLoc = targetY*TargetXalign + targetX;      //target offset

	         UserImageBufr[PixLoc] = color;

	         targetY++;
	       }  // end y 1
	       targetX++;
       } //end x
       break;
    //
    // gamma correct not yet- requires more code by far
    //
    case(6):
       if (!InputNumbers(hEditWindow, "Gamma Value", 1 ,254,&Gamma, 50, 50)
       || !Gamma)
       {   return;
       }
       //for (y = 0; y < WrkImageSize;  y++)                     // Y
       //{  baseYcolor = &WrkImageBufr[y] &0xff;// source
       //}
      break;
   }

   FreeMemory(WrkImageBufr);
}

//--------------------------------------------------------------------------------
//  draw a circle
//--------------------------------------------------------------------------------

static void DrawCircle (HDC hdc, int4 centerX, int4 centerY, double radius)
{
   int4 x,y;

   double Fangle;

   if (radius <= 0.0) return;

   for (Fangle = 0.0; Fangle < D360; Fangle+=.01)
   {
      x = centerX + radius * cos ( Fangle);
      y = centerY + radius * sin ( Fangle);

      SetPixelV(hdc, x, y, PALETTEINDEX(DrawingColor));

      // MoveToEx(hdc, x, y, NULL);
      // LineTo  (hdc, x, y );
   }
}

//----------------------------------------------------------------------------
// fill buffer with the circle
//----------------------------------------------------------------------------

static void FillCircle(byte *UserImageBufr,
                       int4 CenterX,int4 CenterY,
                       double radius,
                       int2 fillcolor,
                       int2 newcolor,
                       int2 type,Bool solid)
{
   int4   x,y;
   byte  *baseYcolor;            // start of Y data

   int4   Xalign;
   static int4 CX, CY;

   double Fangle;
   double R;
   BITMAP bm;
   int4   xsize, ysize;

//-----------------------------------------------------------------------------


   CX = CenterX;// + ( cxHorzScrollPos / DrawingMagnify );
   CY = CenterY;// + ( cyVertScrollPos / DrawingMagnify );

   // get size of the buffer

   GetObject(hDrawBitmap, sizeof(BITMAP), &bm);

   xsize   =     bm.bmWidth;
   ysize   = abs(bm.bmHeight);    //   7-1-97

   Xalign = (xsize + 3) & ~3;      // row bytes aligned

   // if from paste, clear background to transparent

   if (type == 2)
   {    memset(UserImageBufr,fillcolor,ysize*Xalign);    // fill with
   }

   if (radius <= 0.0) return;

   if (solid)
   {
      for (Fangle = 0.0; Fangle < D360; Fangle+=.01)
      {
        for (R=0; R < radius; R+=.5)
        {
          x = R * cos ( Fangle);
          y = R * sin ( Fangle);

          x += CX;
          y += CY;

          if ( y >=0 && y < ysize
          &&   x >=0 && x < xsize )
          {
             baseYcolor = &UserImageBufr[ y*Xalign];  // source
             *(baseYcolor+x) = newcolor;              // target color
          }
        }
      }
   }
   else
   {  for (Fangle = 0.0; Fangle < D360; Fangle+=.01)
      {
         x = CenterX + radius * cos ( Fangle);
         y = CenterY + radius * sin ( Fangle);

         if ( y >=0 && y < ysize
         &&   x >=0 && x < xsize )
         {
              baseYcolor = &UserImageBufr[ y*Xalign]; // source
            *(baseYcolor+x) = newcolor;               // target color
         }
      }
   }

}

//--------------------------------------------------------------------------------
// Finds all colors matching oldcolor and makes them into new color
// type = true for random matching
//        0 = change all old to new
//        1 = change all old to random
//        2 = change ALL     cycle +
//        3 = change ALL     cycle -
//        4 = change all old cycle +
//        5 = change all old cycle -
//        6 = change ALL     Blur switch random pixels in image
//        7 = change ALL ... Fuzzy
//--------------------------------------------------------------------------------

static void MatchPixels(byte *UserImageBufr,
                        int2  oldcolor,
                        int2  newcolor,
                        int2  type,
                        int2  width, int2 height)
{
static int2 BlurValue   = 2;

   int2  x,y;
   byte *baseYcolor;            // start of Y data
   byte  color;
// int2  align;
   int2  Xalign;
   int2  newx,newy;
   int2  odd = 0;
   int2  offset;

   //-----------------------------------------------------------------------------

   RandomInitial();

   // get size of the buffer


   //align   = ((uint2 *) UserImageBufr)[4];
   // align   = 0;
   Xalign = (width + 3) & ~3;      // row bytes aligned

   if (type == 7 )
   {  if(!InputNumbers(hEditWindow, "Blur Value", 1, height, &BlurValue, 50, 50))
      {  return;
      }
   }
   if (type == 6 )
   {   if(!InputNumbers(hEditWindow, "Fuzz Value", 1, height, &BlurValue, 50, 50))
       {  return;
       }
   }
   //-------------------------------------------------------------------------

   for (y = 0; y < height;  y++)
   {
      baseYcolor = &UserImageBufr[ y*Xalign];//+align];  // source

      for (x=0; x< width; x++)                         // X-------
	   {
	     color  = *(baseYcolor+x);                 // source color
	     if (type == 2 || type == 3 || type == 6 || type == 7)
	     {
		    if (type == 2 )
	       {   newcolor = color+1;
	       }
		    else
		    if (type == 3)
	       {   newcolor = color-1;
		    }
		    else
		    if (type == 6)
		    {   if (color == oldcolor) continue;   // this should be trans

		        offset = rand()%BlurValue;

		        if (odd == 3)
		        {   odd = -3;
		        }
		        else
		        {   odd = 3;
		        }

		        offset *=odd;
		        newx = x+offset;
		        newy = y+offset;

		        if (newx >= 0 && newx < width && newy >=0 && newy < height)
		        {
		           newcolor = *&UserImageBufr[ newy*Xalign+newx];//+align];  // source
			        if (newcolor != oldcolor)      // not tranparent
			        {   *(baseYcolor +x ) = newcolor;
		           }
		        }
		        continue;
		    }
		    else
	       if (type == 7)
	       {   if (color == oldcolor) continue;   // this should be trans

		        offset = rand()%BlurValue;         // blurr amount

		        if (odd == 1)
		        {   odd = -1;
		        }
		        else
		        {   odd = 1;
		        }

		        offset *=odd;
		        newx = x+offset;
		        newy = y+offset;

		        if (newx >= 0 && newx < width && newy >=0 && newy < height)
		        {
		           newcolor = *&UserImageBufr[ newy*Xalign+newx];//+align];  // source
			        if (newcolor != oldcolor)  // not tranparent
		           {   *(baseYcolor +x ) = newcolor;
			        }
		        }
		        continue;
		    }
		    *(baseYcolor +x ) = newcolor;
	     }
	     else
	     if (color == oldcolor)
	     {  if (type==1)
		     {   newcolor = rand()%256;
		     }
		     else
		     if (type == 4)
		     {   newcolor = color+1;
	        }
		     else
		     if (type == 5)
		     {   newcolor = color-1;
		     }
	        // else 0, use newcolor passed by user
		     *(baseYcolor +x ) = newcolor;
	     }

	   }  // end x 1
   }
}

//-------------------------------------------------------------------------
//  Exit from PasteImage freeing all associated resources
//-------------------------------------------------------------------------
static void PasteExit(HPALETTE hOldPal, HDC hdc, HDC hdcMem, HBITMAP hBmp, HBITMAP hOldBmp)//, byte *BMPImageBufr)
{
   PasteActive = FALSE;
   SelectBitmap(hdcMem, hOldBmp);
   DeleteDC(hdcMem);
   DeleteBitmap(hBmp);

   SelectPalette(hdc, hOldPal, FALSE);
   ReleaseDC(hEditWindow, hdc);

   wsprintf(StatMsg, "%4hd x %-4hd  %-8s  %-46.46s",
                      DrawingXsize,DrawingYsize,DrawingName,ConvertWad.filename);
   GFXDrawStatus(hGFXWindow);    // status bar  bottom

   EatMouseData(hGFXWindow);         // Don't let the keys/buttons pile up
   EatMouseData(hEditWindow);        // Don't let the keys/buttons pile up
}

//-----------------------------------------------------------------------------
// past image on clipboard
//-----------------------------------------------------------------------------
#pragma argsused
static Bool PasteImage(HWND hwnd, int2  magnify, int2  transparentfill)//, Bool fromSelect)    // fill background duke
{
  int4 i,j;
//    int4 x,k,l,y;
//  byte color;
//  int4 PixLoc;
  int2 TempColor;

  int2 CopyWidth;
  int2 CopyHeight;
  int2 CopyWidthAlign;
//  int2 CopyShrink = 0;

//  int2 Width,Height;
//  int2 InputWidth;
//  int2 InputHeight;
//  int2 InputWidthAlign;

  int4 RawXcoord;
  int4 RawYcoord;

  byte NewByte;

  int  oldButton;

  int  priorDrawX;
  int  priorDrawY = -1;

  byte *baseYcolor;            // start of Y data
  byte *RawLoc;
  byte *RawRaw;

  char  JustFileName[260];

  Bool  AllowTransparent;

  MSG   msg;

  UINT       KeyData;
  HDC        hdc;
  HPALETTE   hOldPal;
  LOGBRUSH   lb1;
  HPEN       hDotPen, hOldPen;
  HBRUSH     hOldBrush;
  HDC 	    hdcMem;
  HBITMAP    hOldBmp, hBmp;
  BITMAP     bm;
//COLORREF   cr;
//  RECT       rc; //, rcBm;
  byte *     CopyImageBufr;
//  POINTS     pt;
//byte       *BMPImageBufr = NULL; // Mag buffer if a BMP


//----------------------------------------------------------------------------

  if (!lpViewBits || !hViewBitmap)
  {    //OnlyRegPaste();                           // msg "nothing to paste" or regmsg
       HelpMsg(hwnd,CENTER|HELPINFO, "Paste",
                                     "Nothing to Paste!",
                                     NULL);
       return FALSE;
  }

  GetObject(hViewBitmap, sizeof(BITMAP), &bm);

  CopyWidth      =     bm.bmWidth;
  CopyHeight     = abs(bm.bmHeight);
  CopyWidthAlign = (( CopyWidth + 3) & ~3);

//  Width          = CopyWidth;
//  Height         = CopyHeight;
//  InputWidth     = CopyWidth;
//  InputWidthAlign= CopyWidthAlign;
//  InputHeight    = CopyHeight;

  if (CopyWidth <= 0 ||  CopyHeight<= 0)    return FALSE;

  CopyImageBufr = (byte *) &*lpViewBits;

  // old code here was moved

//GetWindowRect(hEditWindow, &rc);

  hdc     = GetDC(hEditWindow);
  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE); //   5-1-97 hViewPalette
  RealizePalette(hdc);

  hBmp = CreateCompatibleBitmap(hdc, CopyWidth, CopyHeight);

  AllowTransparent = TRUE;

  // if pasting a transparent, then ok too

  if (DrawingColor != Transparent)
  {   for (j=0; j < CopyHeight; j++)
      {  baseYcolor = &CopyImageBufr[j*CopyWidthAlign];// +CopyAlign];  // source
         for (i=0; i < CopyWidth; i++, baseYcolor++)
         {  if (*baseYcolor != transparentfill)
            {   AllowTransparent = FALSE;
                break;
            }
         }
      }
  }

  GetJustTheName(CopyFileName,JustFileName);

  strcpy(StatMsg, "Move graphic to location and click Left button" );
  GFXDrawStatus(hGFXWindow);                //Draw the Status Bar  bottom

  hdcMem  = CreateCompatibleDC(hdc);
  hOldBmp = SelectBitmap(hdcMem, hBmp);     // our Copy Image

  SetFocus(hEditWindow);                    //   5-1-97 for Keyboard to work
//SetCapture(hEditWindow);                  //   5-1-97 guess I don't need it
  PasteActive = TRUE;

  for (;;)
  {
    MouseButton = 0;
    priorDrawX = -1;

    while (MouseButton == 0)// && !bioskey(1))
    {
      PasteWidth = CopyWidth;   //   5-1-97 a bit of a hack for now
      PasteHeight= CopyHeight;

      WaitMessage();
      KeyData =GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);
      oldButton = MouseButton;

      // for pasting we want this cursor (doesn't stick if done before ??)

      if(GetCursor() != gd.hCursorSelect)  EnableCursor(hwnd, gd.hCursorSelect);

      //Trace(hEditWindow,10,10,"key=%d coord=%d,%d",KeyData,DrawX,DrawY); // old mouse below

      if ( CheckIfInsideImage(&DrawX,&DrawY,CopyWidth,CopyHeight)
      ||   KeyData == 0x001B )
      {
          if (KeyData)
          {
            switch(KeyData)
            {
            case('F')://VK_F):
             for (i=0; i < CopyWidth; i++)
             {  for (j=0; j < CopyHeight; j++)
                {
                  *( CopyImageBufr+ j*CopyWidthAlign + i)=DrawingColor;//+CopyAlign )=DrawingColor;
                }
             }
             break;
             // blur
            case('B')://VK_B):
		       MatchPixels(CopyImageBufr,transparentfill,0 ,7, CopyWidth, CopyHeight);
		       break;
            // fuzz
            case('Z')://VK_Z):
		       MatchPixels(CopyImageBufr,transparentfill, 0 ,6, CopyWidth, CopyHeight);
		       break;
            // random, current, cycle+/-/etc
            case ('J'): //VK_J:         // Change these Pixels to random color
            case ('C'): //VK_C:         // Change these Pixels to Current color
            case (';'): //VK_SEMI:      // cycle all colors up
            //case ('\'')://VK_QUOTE:     // cycle all colors down  +++same as left cursor bummer +++
            case (':'):// cycle all colors down
            case ('['): //VK_LSQUARE:   // cycle all matching colors up ( under cur )
            case (']'): //VK_RSQUARE:   // cycle all matching colors dn ( under cur )
             DidRotate=TRUE;
             //cr = GetPixel(hdc, DrawX, DrawY);    // Get Color Reference
             //TempColor = GetNearestPaletteIndex(gd.hDoomPal, cr);
             TempColor = CurrentPictureColor;
             switch(KeyData)
             {
               case ('C'):// Change these Pixels to Current color
                MatchPixels(CopyImageBufr, TempColor, DrawingColor, 0, CopyWidth, CopyHeight); // c
                break;
               case ('J'):// Change these Pixels to random color
                MatchPixels(CopyImageBufr, TempColor, DrawingColor, 1, CopyWidth, CopyHeight); // j
                break;
               case (';'):// cycle all colors up
                MatchPixels(CopyImageBufr, TempColor, DrawingColor, 2, CopyWidth, CopyHeight); // ;
                break;
               //case ('\'')://VK_QUOTE:                 // cycle all colors down
               case (':'):// cycle all colors down
                MatchPixels(CopyImageBufr, TempColor, DrawingColor, 3, CopyWidth, CopyHeight); // '
                break;
               case ('['):// cycle all matching colors up ( under cur )
                MatchPixels(CopyImageBufr, TempColor, DrawingColor, 4, CopyWidth, CopyHeight); // [
                break;
               case (']'):// cycle all matching colors dn ( under cur )
                MatchPixels(CopyImageBufr, TempColor, DrawingColor, 5, CopyWidth, CopyHeight); // ]
                break;
             }
             break;
             // / = shrink
            case('\\'):     //(VK_BSLASH):
  	          //restore prior
             if (CopyWidth > 1 && CopyHeight >1)
             {
	              if (priorDrawX != -1)
	              {   SelectBitmap(hdcMem, hBmp);               // Old Screen Data
                     BitBlt(hdc, priorDrawX, priorDrawY, CopyWidth, CopyHeight, hdcMem, 0, 0, SRCCOPY);
                     priorDrawX = -1;
	              }
   	           FlipImage  (hwnd,CopyImageBufr, 4, transparentfill, CopyWidth, CopyHeight ); //shrink copyimage bufr //++++++++

                 CopyWidth      /= 2;              // width
                 CopyHeight     /= 2;              // height
                 CopyWidthAlign = (CopyWidth+3) & ~3;

                 HDC        Bhdc;
                 HBITMAP    BhViewBitmap;   // Handle to the bitmap BMP
                 VOID **    BlpViewBits ;   // Pointer to the actual bits
                 BITMAPINFO BViewBi;        // Our DIB Bitmap Info Struct.

                 //GetObject(hViewBitmap, sizeof(BITMAP), &bm);

                 // Set up a BITMAPINFO structure for use with CreateDIBSection()
                 ZeroMemory(&BViewBi, sizeof(BITMAPINFO));
                 BViewBi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
                 BViewBi.bmiHeader.biPlanes     = 1;
                 BViewBi.bmiHeader.biBitCount   = 8;
                 BViewBi.bmiHeader.biWidth      =  CopyWidth;
                 BViewBi.bmiHeader.biHeight     = -CopyHeight;
     	           BViewBi.bmiHeader.biSizeImage  =  CopyWidthAlign * CopyHeight; //   5-1-97 was width
                 BViewBi.bmiHeader.biCompression= BI_RGB;

                 // Create a DC to hold our DIBSECTION
                 Bhdc = GetDC(hwnd);                           // use DIB_PAL instead of RGB
                 BhViewBitmap = (HBITMAP)CreateDIBSection(Bhdc, &BViewBi, DIB_PAL_COLORS, (VOID **)&BlpViewBits, NULL, 0L);
                 ReleaseDC(hwnd, Bhdc);

                 if (!BhViewBitmap || !BlpViewBits)                            // not enough memory
                 {    OutOfMemoryMsg(hwnd);
                      DIBError(ERR_CREATEDIBSECTION);

                      if (BhViewBitmap)
                      {   DeleteObject(BhViewBitmap);
                      }
                      MouseButton = 2;                // to exit
                      break;
                 }
                 // now replace bitmap with shrunken one
                 memcpy((byte *) &*BlpViewBits, (byte *) &*lpViewBits, (CopyWidthAlign * CopyHeight));

                 DeleteObject(hViewBitmap);                    // delete old one
                 hViewBitmap = BhViewBitmap;                   // new BMP
                 lpViewBits  = BlpViewBits;                    // new actual BMP
                 CopyImageBufr = (byte *) &*lpViewBits;        // new actual BMP bits
             }
             break;
            // Rotate Angle
	         case(','):   //(VK_COMMA):                    //  Rotate angle
	         case('.'):   //(VK_PERIOD):                   //  Rotate angle
		       FlipImage  (hwnd,CopyImageBufr, 2, 0 , CopyWidth, CopyHeight);   // rotate
		       break;
            // flip x
            case('X'):   //(VK_X):
             FlipImage  (hwnd,CopyImageBufr, 0, 0 , CopyWidth, CopyHeight);   // flip X
             break;
            // flip y
            case('Y'):   //(VK_Y):
             FlipImage  (hwnd,CopyImageBufr, 1, 0 , CopyWidth, CopyHeight);   // flip Y
             break;
            case(0x001B)://(VK_ESCAPE):
             MouseButton = 2;
             break;
            // cursor keys======================     6-1-97
	         case(VK_UP):    // Up arrow
             if (GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION, -DrawingMagnify))
	              priorDrawX=-1; // redraw done
  	          break;
	         case(VK_DOWN):  // Down arrow
             if (GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION,  DrawingMagnify))
	              priorDrawX=-1; // redraw done
  	          break;
	         case(VK_LEFT):  // Left arrow
             if (GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION, -DrawingMagnify))
	              priorDrawX=-1; // redraw done
             break;
	         case(VK_RIGHT): // Right arrow
             if (GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION,  DrawingMagnify))
	              priorDrawX=-1; // redraw done
             break;

	         case(VK_PRIOR): // Page up
             if (GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION, -DrawingMagnify*8))
	              priorDrawX=-1; // redraw done
             break;
	         case(VK_NEXT):  // Page down
             if (GFXScroll(hEditWindow, SB_VERT, DRAWINGTHUMBPOSITION,  DrawingMagnify*8))
	              priorDrawX=-1; // redraw done
	          break;
            case(VK_HOME):  // home
             if (GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION, -DrawingMagnify*8))
	              priorDrawX=-1; // redraw done
             break;
            case(VK_END):   // end
             if (GFXScroll(hEditWindow, SB_HORZ, DRAWINGTHUMBPOSITION,  DrawingMagnify*8))
	              priorDrawX=-1; // redraw done
             break;
             //==================================
           }// end switch(KeyData)
          } // end if keydata

          if(MouseButton != 2                          // not if cancelling
          && (msg.message == WM_MOUSEMOVE || KeyData) )
          {
	         if (priorDrawX != DrawX || priorDrawY != DrawY || KeyData )
	         {
              KeyData = 0;    // reset so will work

  	           //restore prior
	           if (priorDrawX != -1)
	           {   SelectBitmap(hdcMem, hBmp);               // Old Screen Data
                  BitBlt(hdc, priorDrawX, priorDrawY, CopyWidth, CopyHeight, hdcMem, 0, 0, SRCCOPY);
	           }

	           // save what is there now
              SelectBitmap(hdcMem, hBmp);                    // our Screen Data
 	           BitBlt(hdcMem, 0, 0, CopyWidth, CopyHeight, hdc, DrawX, DrawY, SRCCOPY);

              // Copy ImageBufr
              SelectBitmap(hdcMem, hViewBitmap);             // our Copy Image
              SetDIBColorTable(hdcMem, 0, 256, dRGBquad);

              //SetStretchBltMode(hdc, COLORONCOLOR);          // stretching mode best for color pictures   5-1-97
              //StretchBlt(hdc, DrawX, DrawY, CopyWidth, CopyHeight, hdcMem, 0, 0, Width, Height, SRCCOPY); //   5-1-97
              BitBlt(hdc, DrawX, DrawY, CopyWidth, CopyHeight, hdcMem, 0, 0, SRCCOPY);

              GdiFlush();

	           priorDrawX  = DrawX;
	           priorDrawY  = DrawY;

              lb1.lbStyle = BS_SOLID;
              lb1.lbColor = PALETTEINDEX(GameColors[WHITE]);
              lb1.lbHatch = 0;

              hDotPen   = ExtCreatePen(PS_COSMETIC | PS_DOT,2,&lb1,0,NULL);
              hOldPen   = SelectPen(hdc, hDotPen);
              hOldBrush = SelectBrush(hdc, GetStockBrush(NULL_BRUSH) );
              Rectangle(hdc,DrawX,DrawY,DrawX+CopyWidth,DrawY+CopyHeight);// box it

              SelectPen(hdc, hOldPen);
              SelectBrush(hdc, hOldBrush);
              DeletePen(hDotPen);
	         }
          }  // MouseMove
      }  // if(InsideImage)

    }  // while no Mouse Button

    MouseButton = oldButton;              // real button

    // now do actual copy------------------------------------------------------

    // restore prior image
    if ( priorDrawX != -1)
    {    SelectBitmap(hdcMem, hBmp);           // our Copy Image
         BitBlt(hdc, priorDrawX, priorDrawY, CopyWidth, CopyHeight, hdcMem, 0, 0, SRCCOPY);
    }

    if ( KeyData                               // don't do if some key
      || priorDrawX  == -1                     // nothing done
      || MouseButton == 2 )                    // Right button or Esc (was oldButton)
    {
        //ReleaseCapture();                           //   5-1-97 matches above comment
        PasteExit(hOldPal, hdc, hdcMem, hBmp, hOldBmp);//, BMPImageBufr);
        return FALSE;
    }

    //
    //-----------------     paste it      ----------------------------------------
    //

    GetGFXMouseCoords(hwnd,&msg, NULL,&DrawX, &DrawY,&MouseButton);  // relative mouse to window
    XY_Update(hGFXWindow, DrawX,DrawY);                              // relative to graphics

    for (i=0; i < CopyWidth; i+=DrawingMagnify)
    {
         //RawXcoord = (DrawX-OrgXstart +i +cxHorzScrollPos ) / DrawingMagnify ;   // current loc
         RawXcoord = RelativeXcoord +i/DrawingMagnify;

         for (j=0; j < CopyHeight; j+=DrawingMagnify)           // Y--------------
         {
	          // RawYcoord = (DrawY-OrgYstart + j +cyVertScrollPos  ) / DrawingMagnify ;
             RawYcoord = RelativeYcoord +j/DrawingMagnify;

             RawLoc    = ImageBufferOffset(RawXcoord,RawYcoord);

             if (RawLoc) // legitimate location
	          {
                 TrackUndo(RawXcoord, RawYcoord, *RawLoc);
                 NewByte =*( CopyImageBufr+ j*CopyWidthAlign + i);// +CopyAlign);

                 if (!AllowTransparent)
                 {   if (NewByte == transparentfill) continue;    // do not copy tranparent
                 }

                 *RawLoc = NewByte;
                 if (FlatLump)                                    // FLATS always
                 {
   	               RawRaw  = RawOffset(RawXcoord,RawYcoord);     // drawing with transparent color
                     if (RawRaw) *RawRaw = NewByte;
                 }
                 else
                 if (!DidTransp && !DidRotate)
   	           {   RawRaw = RawOffset(RawXcoord,RawYcoord);     // drawing with transparent color
                     if ( NewByte == transparentfill || !RawRaw)  // pixel did not exist
                     {   DidTransp++;
                     }
                     else
                     {   *RawRaw = NewByte;
                     }
                 }
	          }
         }
    }

    // PASTE OUR IMAGE ON THE SCREEN ONCE AND FOR ALL, Then, keep going . . .

    //Trace(hGFXWindow,10,10,"%d,%d  %d,%d", OrgXstart,OrgXend,OrgYstart,OrgYend);

	 //GetClientRect(hEditWindow, &rc);  // Re-draw with new Transparent color & Check

    MagnifyGraphicTexture(hEditWindow,
                          EditRC.left, EditRC.top,
                          EditRC.right,EditRC.bottom,
                         (byte *)&*lpDrawBits,
                          hDrawBitmap,
                          DrawingMagnify, Transparent);

    EatMouseData(hGFXWindow);         // Don't let the keys/buttons pile up
    EatMouseData(hEditWindow);        // Don't let the keys/buttons pile up

  }  // end for(;;)   exit is always above in pasteexit

}

////////////////////////////////////////////////////////////////////////////////
//  get any non flat image from IWAD or PWAD
////////////////////////////////////////////////////////////////////////////////
#define FLATTYPE    1            // which type of graphic was displayed
#define TEXTURETYPE 2            //  ...

static int4         PasteListCount,PasteListTotal;
static char         **PasteList;
static NameAreaList *PasteListNames;
static NameAreaList *Pastelistnamwrk;

static char         EdTxt[10]        = {0};   // EditText for name matching made
static char         PasteLumpName[10]={0};
static int2         PasteDisplayLumpFirst= TRUE;
static Bool         PasteDidShow         = FALSE;
static int4         PasteCode;
static int4         PasteImageXSize;
static int4         PasteImageYSize;

//////////////////////////////////////////////////////////////////////////////

static HWND 	     hEdit;              // easy to grab handles for controls
static HWND 		  hList;
static HWND         hStatic;
static HWND 	     hTxtrPrev;

static Bool         NameChange = FALSE; // inside of Name box
static LRESULT 	  ListBoxIndex;       // index for grabbing List Box entries


////////////////////////////////////////////////////////////////////////////////
//
//  Generic New List Box entry chosen
//  Global INDEX returns the index of chosen
//  Global hList is the Handle of the ListBox upon entry
//  Global hStatic if text displayed for Sub-Texture
////////////////////////////////////////////////////////////////////////////////
static void PasteListChange(void)
{
	ListBoxIndex = (LRESULT)ListBox_GetCurSel(hList);      // get index

   ListBox_GetText(hList, ListBoxIndex, PasteLumpName);   // get lumpnmae

   InvalidateRect(hTxtrPrev, NULL, TRUE);                 // req'd to get paint to redraw
}

///////////////////////////////////////////////////////////////////////////
//   find closest name so far for pasting
///////////////////////////////////////////////////////////////////////////

static void FindClosestPasteMatch(void)
{
  int4 i;
  int4 CompareLen  = lstrlen(EdTxt);

  //Trace(hGFXWindow,20,44,"%s %s %d %hd",PasteLumpName,EdTxt,CompareLen,NameChange);

  if (NameChange)
  {
      PasteDidShow = FALSE;
		PasteLumpName[CompareLen] = '\0';                            // for length of entry only

      //ListBoxIndex = 0;   // if not found?

	   for(i=0; i < PasteListCount; i++)
      {
         strncpy (PasteLumpName, (char *) PasteList[i],CompareLen);// name entered

         if (!strcmp(PasteLumpName, EdTxt))
         {   ListBoxIndex = i;
             //Trace(hGFXWindow,20,64,"%s hit",PasteLumpName);
             break;
         }
      }
      //if (i >=PasteListCount)
      //Trace(hGFXWindow,20,64,"%s miss",PasteLumpName);
  }

  ListBox_SetCurSel(hList, ListBoxIndex);
  PasteListChange();
}

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// on paint and the list box use this == this uses PasteLumpName ==           //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
void PasteDrawLump(HWND hwnd, HDC hdc, LPRECT rc)
{
   DirPtr   dir;
   int4     n;
   int2     filenum;
   char     TexName[9]={0};
   WadFiles AnyWad;      // the texture directory (usually MasterWad)

   DOOMGraphicHdr PicHeader;

   int4     oldImageXSize;
   int4     oldImageYSize;
   int2     PasteMagnify = 3;

//==========================================================================

   oldImageXSize = ImageXSize;  // save current drawing size since this destroys it
   oldImageYSize = ImageYSize;

   PasteDidShow = FALSE;
   PasteCode    = 0;    //  

   dir = FindMasterDir( MasterWad, PasteLumpName,FALSE);

   if (dir)
   {
       filenum = dir->filenumber;                      // which file its in

       // see if this is a flat
       for (n=0; n< NumFTexture; n++)
       {
          if (!strncmp (PasteLumpName,FTexture[n],8) )
          {   PasteCode = DisplayGraphicFlat(hwnd,hdc,DOMASTER,
                                             rc->left,rc->top,rc->right,rc->bottom, //1-1-97 bk
                                             NULL,NULL,NULL,NULL,NULL,//&RawImage,size,imagesize,&ImageBufr,handle hdib
                                             PasteLumpName,
                                             PasteMagnify,
                                             SHOWINFO,
                                             GameColors[MenuColor],
                                             SHOWIMAGE);

              if (PasteCode > 0)
	           {   PasteDisplayLumpFirst = TRUE;
                  PasteDidShow          = FLATTYPE;
              }
	       }
       }

       // if nothing, try for texture
       if (!PasteDidShow)
       {
	        SetActiveWad(&AnyWad,filenum);                  // where patch is
           WadSeek( AnyWad.fileinfo, dir->start);          // seek start of entry
	        WadRead( AnyWad.fileinfo, &PicHeader,12);//&xsize, 12);          // x width, etc

	        if (filenum != 0)
	        {   CloseWadFiles('O',FREEDIRNO,FREENAMENO);    // close all
	        }

           if (DoomGraphicVerify(&PicHeader,dir->size) )
	        {
               PasteCode = DisplayGraphicTexture(hwnd, hdc, DOMASTER,
                                                 rc->left, rc->top, rc->right, rc->bottom,
                                                 NULL,NULL,NULL,NULL,NULL,// rawsize ,&RawImage,Imagesize,&ImageBufr
                                                 PasteLumpName,
                                                 PasteMagnify,
                                                 SHOWINFO,
                                                 GameColors[MenuColor],    //   5-1-97
                                                 SHOWIMAGE);

               if (PasteCode > 0)
               {   PasteDisplayLumpFirst = TRUE;
                   PasteDidShow = TEXTURETYPE;
               }

           }
      }
   }
   //
   //-------------show DOOM logo if nothing done-------------------------------
   //
   if (!PasteDidShow  && PasteDisplayLumpFirst)          //did not display last time
   {
       PasteDisplayLumpFirst = FALSE;

       char * GameTitleGraphics[5]= {"M_DOOM\0\0",       //  1  DOOM
                                     "M_DOOM\0\0",       //  2  DOOM 2
                                     "M_HTIC\0\0",       //  3  HERETIC
                                     "M_HTIC\0\0",       //  4  HEXEN
                                     "M_STRIFE\0\0"};    //  5  STRIFE

       DisplayGraphicTexture(hwnd, hdc, DOMASTER,
                                 rc->left, rc->top, rc->right, rc->bottom,
                                 NULL,NULL,NULL,NULL,NULL,// Imagesize//&RawImage,//size//&ImageBufr,
                                 GameTitleGraphics[GameVersion -1],
                                 2,
                                 NOSHOWINFO,
                                 GameColors[MenuColor],
                                 SHOWIMAGE);  //   5-1-97
   }
   else
   {   PasteImageXSize = ImageXSize;            // set size for Paste
       PasteImageYSize = ImageYSize;
   }

   if (!PasteDidShow &&  dir)
   {   strncpy(TexName,PasteLumpName,8);
       wsprintf(LViewMsg,"%8s not a graphic %-64.64s",TexName,WadFileList[filenum].filename);
       PostMessage(hwnd, WM_COMMAND, WM_CHANGE , 0);
   }

   if (PasteCode < 0)
   {   OutOfMemoryMsg(hwnd);
   }


//   ValidateRect(hwnd,NULL);  // reqd prevent repaint   6-1-97
   ValidateRect(hwnd,&EditRC);  // reqd prevent repaint   6-1-97

   //   restore original sizes

   ImageXSize = oldImageXSize;
   ImageYSize = oldImageYSize;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Display Paste Any graphic from resident graphic names dialog
//
//////////////////////////////////////////////////////////////////////////////
#pragma argsused
static BOOL CALLBACK PasteLumpDlgProc(DIALOG_PARAMS)
{
 static char  name[9]={0};

 DirPtr  dir;
 int2    MapType,n;

 RECT    rc;
 HDC     hdc;

 static    HPALETTE     hOldPal;
 static    int2         oldDrawingMagnify;
 static    BITMAPINFO   PasteLumpbi;               // Our DIB Bitmap Info Struct.

//-------------------------------------------------------------------------

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

 switch( msg )
  {
    case WM_INITDIALOG:
     hEdit 		= GetDlgItem( hDlg, IDD_TXTREDIT );
     hList 		= GetDlgItem( hDlg, lst1);          //IDD_TXTRLIST );
     hStatic 	= GetDlgItem( hDlg, stc1);          //IDD_STATICTEXT );
     hTxtrPrev = GetDlgItem( hDlg, rct1);          //IDD_CHOOSETXTRPREVIEW );

     LastPicXloc = -1;                             // first time (so don't clear prior)

     oldDrawingMagnify = DrawingMagnify;

	  SetWindowLong(hDlg, DWL_USER, lParam);	      // store our Pointer
     GetWindowRect(hDlg, &rc);

     // allocate space for all the names all at once     //

     PasteListTotal = MasterWad->dirnum ;

     PasteList      = (char **)GetMemory( PasteListTotal * sizeof( char *) );
     PasteListNames = (NameAreaList *)GetMemory(PasteListTotal * sizeof(NameAreaList) );    // 8 name is 8

     //------------------------------------------------
     // TEXTURE + FLATS store names (eliminate level entries)
     //------------------------------------------------

     dir = MasterWad->directory;                             // dir buffer

     Pastelistnamwrk= PasteListNames;
     PasteListCount = 0;                                     // number of entries

     for (n = 0; n < MasterWad->dirnum; n++)
     {
          MapType = CheckIfLevel (&dir[n],n, PasteListTotal );  // is it MAP level id
          // skip all levels
          if (MapType)                                          // this is a level
          {
              if (MapType == 1)
	           {   n += 10;
	           }
	           else
	           {   n += 11;
	           }
              // +1 at end of this loop
	           continue;
          }

          if (dir[n].size < 28) continue;                             // cant be graphic

          strncpy(name,dir[n].name,8);

          PasteList[PasteListCount] = (char *) Pastelistnamwrk;

          strncpy( (char *)PasteList[PasteListCount], name, 8);       // store name

          PasteListCount++;
          Pastelistnamwrk +=1;
     }

     // (Sorted) always
     qsort( PasteList, PasteListCount, sizeof( char *), SortListName8);

     // if no name yet
     if (!PasteLumpName[0])
     {   strncpy ( PasteLumpName,(char *) PasteList[0],8);         // first one
     }

     for(n = 0; n < PasteListCount; n++)
     {
        strncpy ( name, (char *) PasteList[n],8);
        ListBox_AddString(hList, name);  // listbox1
     }

     //-----------------------------------------------------------------------------

     PasteDisplayLumpFirst = TRUE;

     // set the focus to our list box and then select item 0 or our Current Map
     //SetFocus(hList);               //GetDlgItem(hDlg, lst1));
     //SendMessage(GetDlgItem(hDlg, lst1), LB_SETCURSEL, 0, 0);
     //SendMessage(hList, LB_SETCURSEL, 0, 0);

     strncpy(EdTxt,PasteLumpName, 8);
     //Trace(hGFXWindow,20,20,"%s",EdTxt);

     ListBoxIndex  = 0;                         // if not found 1st time
     NameChange    = TRUE;
     FindClosestPasteMatch();
     NameChange    = FALSE;

     //Trace(hGFXWindow,20,44,"%s",PasteLumpName);

     Edit_SetText( hEdit, PasteLumpName);
///  SendMessage(hList, LB_SETCURSEL, ListBoxIndex, 0);
     SetCursorPos(rc.left + 118, rc.top + 40);  // position it
     //SetFocus(hList);
     // If we don't set focus and return TRUE the focus is set by
     // the order of the controls in the .rc file
     return TRUE;//FALSE; we've set our own focus

    case WM_RBUTTONUP:
      PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
      break;
    //   6-1-97           changed mind 7-1-97
    //case WM_LBUTTONUP:
    //  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
    //  break;

    case WM_COMMAND:

       switch LOWORD(wParam) 						    // handle of the control
       {
         case WM_CHANGE:
          Static_SetText(hStatic,LViewMsg);      //GetDlgItem(hDlg, stc1), LViewMsg);
          break;

       	// Name box was chosen
         case IDD_TXTREDIT:
            switch HIWORD(wParam)
			   {
			      case EN_CHANGE:
                 Edit_GetText(hEdit,EdTxt,9);    //PasteLumpName, 9);
                 FindClosestPasteMatch();
				     break;

				   case EN_SETFOCUS:	                // Must be IN the Edit Control to Find Match
				     NameChange=TRUE;
				     break;

				  case EN_KILLFOCUS:
				     NameChange=FALSE;
					  break;

			     default:
				     return FALSE;
            }
		      break;

         //listbox1
         case lst1:		// Out list box was chosen in some fashion
 		      NameChange = FALSE;
            switch HIWORD(wParam)
			   { // double click = Post IDOK to Dialog ( use MAKEWPARAM MACRO )
				  case LBN_DBLCLK:
				   PostMessage(hDlg,WM_COMMAND,
                          MAKEWPARAM(IDOK,(WORD)GetWindowLong(hTxtrPrev,GWL_ID)), //GetDlgItem(hDlg,rct1), GWL_ID)),
                          (LPARAM)hTxtrPrev);//GetDlgItem(hDlg, rct1));
				   break;

				  case LBN_SELCHANGE:
            	ListBoxIndex = (LRESULT)ListBox_GetCurSel(hList);      // get index
               ListBox_GetText(hList, ListBoxIndex, PasteLumpName);   // get lumpnmae
               Edit_SetText( hEdit, PasteLumpName);

               hdc = GetDC(hTxtrPrev);      // our picture rect
               hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  	            RealizePalette(hdc);
               //
	            GetClientRect(hTxtrPrev,&rc);
               PasteDrawLump(hDlg, hdc, &rc);
               //
               SelectPalette(hdc, hOldPal, FALSE);
               ReleaseDC(hTxtrPrev,hdc);
				   break;

				  default:
				   return FALSE;
            }
		      break;

         case IDCANCEL:
            LastPicXloc = -1;  // first time
            LastcxRange = -1;  // force clear

            DrawingMagnify = oldDrawingMagnify;
            FreeMemory(PasteListNames);
            FreeMemory(PasteList);
            EndDialog( hDlg, FALSE );
            return TRUE;

         //psh1 = Import   || double click = OK  || enter
         case IDOK:
         case psh1:
            int Xadjust = PasteImageXSize*oldDrawingMagnify;       // global size for now
            int Yadjust = PasteImageYSize*oldDrawingMagnify;
            int Xadjust1=(Xadjust+3) & ~3;     // DWORD Aligned

            //NotifyMsg(NULL,"OK  %s",PasteLumpName);

       	   ListBoxIndex = (LRESULT)SendMessage(hList, LB_GETCURSEL, 0, 0);
            ListBox_GetText(hList, ListBoxIndex, PasteLumpName);

            if (!PasteDidShow)// || !FindMasterDir( MasterWad, PasteLumpName,FALSE) )
            {
                HelpMsg(hDlg,CENTER|HELPINFO, "No graphics found",
                                              "Not a graphics file",
                                              "",
                                              "Happens for PWADS with multiple",
                                              "lumps, but you didn't select one",
                                              "Try again",
                                              NULL);
                break;
            }

            RemoveDIBSection();

            hdc = GetDC(hTxtrPrev);      //GetDlgItem(hDlg, rct1));
	         GetClientRect(hTxtrPrev,&rc);//GetDlgItem(hDlg, rct1), &rc);

    	      // Set up a BITMAPINFO structure for use with CreateDIBSection()
            ZeroMemory(&PasteLumpbi, sizeof(BITMAPINFO));

            PasteLumpbi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
            PasteLumpbi.bmiHeader.biPlanes    = 1;
            PasteLumpbi.bmiHeader.biBitCount  = 8;
            PasteLumpbi.bmiHeader.biWidth     =  Xadjust;
    	      PasteLumpbi.bmiHeader.biHeight    = -Yadjust;
	         PasteLumpbi.bmiHeader.biSizeImage =  Xadjust1 * Yadjust;
    	      PasteLumpbi.bmiHeader.biCompression = BI_RGB;

            // Create a DC to hold our DIBSECTION
            hViewBitmap = (HBITMAP)CreateDIBSection(hdc, &PasteLumpbi, DIB_PAL_COLORS, (VOID **)&lpViewBits, NULL, 0L);

            if (hViewBitmap)    //   8-1-97
            {
                if (PasteDidShow == TEXTURETYPE)
                {
                    PasteCode = DisplayGraphicTexture(hDlg, hdc, DOMASTER,
                                         rc.left, rc.top, rc.right, rc.bottom,
                                         NULL,NULL,NULL,          //&RawImage,size,imagesize
                                         (byte **)&lpViewBits,    //,&ImageBufr
                                         hViewBitmap,             // handle hdib
                                         (char *)PasteLumpName,
                                         oldDrawingMagnify,
                                         NOSHOWINFO,
                                         Transparent,             // for paste transparent
                                         NOSHOWIMAGE);            //
                }
                else
                {  PasteCode = DisplayGraphicFlat(hDlg,hdc,DOMASTER,
                                         rc.left,rc.top,rc.right,rc.bottom,
                                         NULL,NULL,NULL,          //&RawImage,size,imagesize
                                         (byte **)&lpViewBits,    //,&ImageBufr
                                         hViewBitmap,             // handle hdib
                                        (char *)PasteLumpName,
                                         oldDrawingMagnify,
                                         NOSHOWINFO,
                                         BLACK,            // doesn't matter  for flats
                                         NOSHOWIMAGE);
                }
            }
            else
            {   PasteCode = -1;
            }

            // *lpViewBits has image
            ReleaseDC(hDlg, hdc);

            // now exit
            PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
            break;
		 }

       break;
  }
  return FALSE;
}

////////////////////////////////////////////////////////////////////////////////
//	Paste Lump Window Preview Paint
////////////////////////////////////////////////////////////////////////////////
void PasteLump_OnPaint(HWND hwnd)
{
   PAINTSTRUCT ps;
   RECT rc;
   HPALETTE hOldPal;

//======================================

   BeginPaint(hwnd, &ps);

  	hOldPal = SelectPalette(ps.hdc, gd.hDoomPal, FALSE);
  	RealizePalette(ps.hdc);
	GetClientRect(hwnd, &rc);

   // only fill first time and on redraws !

   FillRect(ps.hdc, &rc, gd.hBrushes[MenuColor]);//BackThingColor]);//GetStockBrush(BLACK_BRUSH));1-1-98

   PasteDrawLump(hwnd, ps.hdc, &rc);

   if (hOldPal)  SelectPalette(ps.hdc, hOldPal, FALSE);
   EndPaint(hwnd, &ps);
}

////////////////////////////////////////////////////////////////////////////////
//  The Deep Lump Preview Window Procedure
////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK PasteLumpWndProc(WINDOWS_PARAMS)
{
  switch(msg)
  { HANDLE_MSG(hwnd, WM_PAINT, PasteLump_OnPaint);
	 default:
		return PasteLump_DefProc(hwnd, msg, wParam, lParam);
  }
}




////////////////////////////////////////////////////////////////////////////////
//                                                                             /
/////////////////  PWAD graphics Pasting - direct from file ////////////////////
//                                                                             /
////////////////////////////////////////////////////////////////////////////////


int4 PasteWadLump(HWND hwnd, char* PasteWadFileName, char* PasteWadLumpName)
{
   HDC      hdc;
   WadFiles FileDrawLevelWad ;

   RECT     rc;

   int4     Num;

   Directory      BufferDir;
   DOOMGraphicHdr PicHeader;
   BITMAPINFO     PasteLumpbi;       // Our DIB Bitmap Info Struct.

   int4     oldImageXSize;
   int4     oldImageYSize;
   byte*    RawBuffer=NULL;

   char     msg1[80];
//==========================================================================

   GetClientRect(hwnd, &rc);
   oldImageXSize = ImageXSize;  // save current drawing size since this destroys it
   oldImageYSize = ImageYSize;

   FileDrawLevelWad.filename = PasteWadFileName;     // name for COPY Paste
   FileDrawLevelWad.fileinfo = fopen(PasteWadFileName, "rb");             // and open

   // read in the WAD directory info------------------------------

   WadSeek( FileDrawLevelWad.fileinfo, 4);                            // skip PWAD id
   WadRead( FileDrawLevelWad.fileinfo, &FileDrawLevelWad.dirnum,  4); // #entries
   WadRead( FileDrawLevelWad.fileinfo, &FileDrawLevelWad.dirstart,4); // offset

   if (FileDrawLevelWad.dirnum <=0 )                                  // not enough
   {   fclose(FileDrawLevelWad.fileinfo);
       return -2;                                                     // bad file
   }

   // read in each entry

   PasteDidShow = FALSE;
   PasteCode    = 0;

   WadSeek( FileDrawLevelWad.fileinfo, FileDrawLevelWad.dirstart);    // start of dir

   for (Num=0; Num < FileDrawLevelWad.dirnum; Num++)
   {
        WadRead( FileDrawLevelWad.fileinfo,&BufferDir,16 );           // Start,End,Name
        if (!strnicmp( BufferDir.name,PasteWadLumpName,8))            // find the lump
        {
            break;
        }
   }

   if (Num >=FileDrawLevelWad.dirnum) return 0;                       // not found
   if (BufferDir.size < 28)           return 0;                       // not big enough

   hdc = GetDC(hwnd);

   WadSeek( FileDrawLevelWad.fileinfo, BufferDir.start);              // seek start of entry
   WadRead( FileDrawLevelWad.fileinfo, &PicHeader,12);                // x width, yheight +offsets

   // see if image is within for 320x200?)

   if (DoomGraphicVerify(&PicHeader,BufferDir.size) )
   {
       RawBuffer =(byte*) malloc(BufferDir.size);
       if (!RawBuffer)
       {   PasteCode = -1;                                 // no memory
       }
       else // if viewbitmap passed back instead, we don't need to do it this way
       {
           WadSeek( FileDrawLevelWad.fileinfo, BufferDir.start);           // seek start of entry
           WadRead( FileDrawLevelWad.fileinfo, RawBuffer,BufferDir.size);  //

           PasteCode = DisplayFileGraphicTexture(hwnd, hdc,
                                 rc.left, rc.top, rc.right, rc.bottom,
                                 RawBuffer,                   // raw data of image
                                 NULL,NULL,NULL,
                                 1,                           // magnify,
                                 NOSHOWINFO,
                                 GameColors[MenuColor],
                                 NOSHOWIMAGE);
       }

       if (PasteCode > 0)
       {   PasteDidShow = TEXTURETYPE;
       }
   }
   else
   {
      if (BufferDir.size == 4096)   // only exact ones this time, since can't tell really
      {
          RawBuffer =(byte*) malloc(BufferDir.size);
          if (!RawBuffer)
          {   PasteCode = -1;                                 // no memory
          }
          else
          {
              WadSeek( FileDrawLevelWad.fileinfo, BufferDir.start);           // seek start of entry
              WadRead( FileDrawLevelWad.fileinfo, RawBuffer,BufferDir.size);  //

              PasteCode = DisplayFileGraphicFlat(hwnd, hdc,
                                 rc.left, rc.top, rc.right, rc.bottom,
                                 RawBuffer,                               // raw data of image
                                 NULL,NULL,NULL,
                                 1,                                       // magnify,
                                 NOSHOWINFO,
                                 GameColors[MenuColor],
                                 NOSHOWIMAGE);
          }

          if (PasteCode > 0)
	       {   PasteDidShow = FLATTYPE;
          }
      }
   }

   fclose(FileDrawLevelWad.fileinfo);

   //
   //-------------show DOOM logo if nothing done-------------------------------
   //

   if (PasteCode < 0)
   {   OutOfMemoryMsg(hwnd);
   }
   else
   if (PasteDidShow)
   {
       int Xadjust = ImageXSize*DrawingMagnify;       // global size for now
       int Yadjust = ImageYSize*DrawingMagnify;
       int Xadjust1=(Xadjust+3) & ~3;                 // DWORD Aligned

       // Set up a BITMAPINFO structure for use with CreateDIBSection()
       ZeroMemory(&PasteLumpbi, sizeof(BITMAPINFO));

       PasteLumpbi.bmiHeader.biSize      = sizeof(BITMAPINFOHEADER);
       PasteLumpbi.bmiHeader.biPlanes    = 1;
       PasteLumpbi.bmiHeader.biBitCount  = 8;
       PasteLumpbi.bmiHeader.biWidth     =  Xadjust;
       PasteLumpbi.bmiHeader.biHeight    = -Yadjust;
	    PasteLumpbi.bmiHeader.biSizeImage =  Xadjust1 * Yadjust;
       PasteLumpbi.bmiHeader.biCompression = BI_RGB;

       // Create a DC to hold our DIBSECTION
       hViewBitmap = (HBITMAP)CreateDIBSection(hdc, &PasteLumpbi, DIB_PAL_COLORS, (VOID **)&lpViewBits, NULL, 0L);

       if (hViewBitmap)
       {
           if (PasteDidShow == TEXTURETYPE)
           {
               PasteCode = DisplayFileGraphicTexture(hwnd, hdc,
                                         rc.left, rc.top, rc.right, rc.bottom,
                                         RawBuffer,               // &RawImage
                                         (byte **)&lpViewBits,    // &ImageBufr
                                         hViewBitmap,             // handle hdib
                                         (char *)PasteLumpName,
                                         DrawingMagnify,
                                         NOSHOWINFO,
                                         Transparent,             // for paste transparent
                                         NOSHOWIMAGE);            //
           }
           else
           {  PasteCode = DisplayFileGraphicFlat(hwnd,hdc,
                                         rc.left,rc.top,rc.right,rc.bottom,
                                         RawBuffer,
                                         (byte **)&lpViewBits,    //,&ImageBufr
                                         hViewBitmap,             // handle hdib
                                        (char *)PasteLumpName,
                                         DrawingMagnify,
                                         NOSHOWINFO,
                                         BLACK,            // doesn't matter  for flats
                                         NOSHOWIMAGE);
           }

           // *lpViewBits has image

           if (PasteCode > 0)
           {   PasteImageXSize = Xadjust;                   // set size for Paste
               PasteImageYSize = Yadjust;
           }
           else
           // free stuff
           {
              DeleteObject(hViewBitmap);
              hViewBitmap = NULL;
           }
       }
   }
   else
   {   sprintf(msg1,"%8s is not a graphic",PasteWadLumpName);
       HelpMsg(hwnd,CENTER|HELPINFO,"Not a Graphic",
                                    msg1,
                                    "",
                                    CopyFileName,
                                    NULL);
   }

   if (RawBuffer)  FreeMemory(RawBuffer);

   ReleaseDC(hwnd, hdc);

   //   restore original sizes

   ImageXSize = oldImageXSize;
   ImageYSize = oldImageYSize;

   return PasteCode;

}


///////////////////////////////   end   //////////////////////////////////////




