//////////////////////////////////////
//  Program Name: Tools.CPP
//////////////////////////////////////
#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include "deep.h"
#include "windeep.h"
#pragma hdrstop

#include <dlgs.h>
#include <math.h>

//#pragma warning (disable: 4068)
/* VK_0 thru VK_9 are the same as ASCII '0' thru '9' (0x30 - 0x39) */
/* VK_A thru VK_Z are the same as ASCII 'A' thru 'Z' (0x41 - 0x5A) */

/*

  StretchTool
  StretchTool
  DragTool
  RectangleTool
  PolygonTool
  StairTool
  DrawingTool
  RotateAndScaleTool
  MergeTool
  RotateThings
  AutoTag
  MakePoolPedLight

*/

//---------------global ----------------------------------------------------

Bool DrawingVertex  =  FALSE;      // not drawing

//--------------------------------------------------------------------------

static int2 MoveOldPointerX, MoveOldPointerY;

extern HBITMAP hVertexBmp;			// Loaded in Deep.cpp

static  TypeDecode ToolOptions[8] =
   {         // 12345678
	{   0, "   Plain"  },
	{   1, "    Pool"  },
	{   2, "Pedestal"  },
	{   3, "   Light"  },
	{   4, "  Window"  },
	{   5, "TeleExit"  },
	{   6, "TeleEntr"  },
	{   7, "    Door"  },
   };

static void CreateDoor(HWND hwnd, LevelPtr level,int2 sector);


double WindowZoomScale    = 0.0;    // what Zoom-to-Window got us
int4   WindowZoomOrigX    = 0;      //
int4   WindowZoomOrigY    = 0;      //

double oldWindowZoomScale = 0.0;    // before Zoom-to-Window
int4   oldWindowZoomOrigX = 0;      //
int4   oldWindowZoomOrigY = 0;      //

static int2 VertexObject=-1;
static int2 NewObject   =-1;


//---------------------------------------------------------------------------------

//---------------------------------------------------------------------------------
// global access to routines below
//----------------------------------------------------------------------------------

static int4 PolygonX[440];  // Doom engine can't really handle this many at once
static int4 PolygonY[440];
static int4 PolyLength;

///////////////////////////////////////////////////////////////////////////////
Bool CheckEscape(MSG *msg, UINT Key)
{
    if (msg->message == WM_CHAR
    &&  Key == VK_ESCAPE) return TRUE;

    return FALSE;
}
///////////////////////////////////////////////////////////////////////////////
Bool CheckF1(MSG *msg,UINT Key)
{
    if (msg->message == WM_KEYDOWN
    &&  Key == VK_F1 ) return TRUE;  //   6-1-97
    //if ( Key == VK_F1 ) return TRUE;
    return FALSE;
}

//----------------------------------------------------------------------
//  automatically fix sector references
//----------------------------------------------------------------------

static void AutoFix(SelPtr NewSelected, int2 side)
{
   LevelPtr level=&LevelWad.level;
   SelPtr cur;
   int2 line;
   int2 opposite=-1;
   int2 sd1;
   int2 sd2;
   int2 sector1=-1;
   int2 sector2=-1;


   if ( SmartDraw  && NewSelected )               // SmartDraw   turned on
   {    // first see that all sectors are the same

        cur = NewSelected;
        line = cur->objnum;                      // this line

        if (side == 1)
        {   sd1 =  level->LineDefs[line].sidedef1;
            if (sd1 >=0)
            {   sector1 = level->SideDefs[sd1].sector;
            }
        }
        else
        {   sd2 =  level->LineDefs[line].sidedef2;
            if (sd2 >=0)
            {   sector2 = level->SideDefs[sd2].sector;
            }
        }

        // now see if all the same to start with

        for (cur = NewSelected; cur; cur = cur->next)
        {   line = cur->objnum;                      // this line
            if (side == 1)
            {   sd1 =  level->LineDefs[line].sidedef1;
                if (sd1 < 0) return;
                if (sector1 != level->SideDefs[sd1].sector) return;
            }
            else
            {   sd2 =  level->LineDefs[line].sidedef2;
                if (sd2 < 0) return;
                if (sector2 != level->SideDefs[sd2].sector) return;
            }
        }

        // set all sectors to -1

        for (cur = NewSelected; cur; cur = cur->next)
        {   line = cur->objnum;                      // this line
            if (side == 1)
            {   sd1 =  level->LineDefs[line].sidedef1;
                level->SideDefs[sd1].sector = -1;
            }
            else
            {   sd2 =  level->LineDefs[line].sidedef2;
                level->SideDefs[sd2].sector = -1;
            }
        }

        for (cur = NewSelected; cur; cur = cur->next)
        {   line = cur->objnum;                      // this line
            if (side == 1)
            {
               sd1 =  level->LineDefs[line].sidedef1;
               if (sd1 >=0)
               {   opposite = GetOppositeSector( line, 1 ); // for sidedef1
                   if (opposite >=0) break;
               }
            }
            else
            {
               sd2 =  level->LineDefs[line].sidedef2;
               if (sd2 >=0)
               {   opposite = GetOppositeSector( line, 2 ); // for sidedef1
                   if (opposite >=0) break;
               }
            }
        }

        // if opposite not found, reset them all back

        if (opposite < 0)
        {
            for (cur = NewSelected; cur; cur = cur->next)
            {   line = cur->objnum;                      // this line
                if (side == 1)
                {   sd1 =  level->LineDefs[line].sidedef1;
                    level->SideDefs[sd1].sector = sector1;
                }
                else
                {   sd2 =  level->LineDefs[line].sidedef2;
                    level->SideDefs[sd2].sector = sector2;
                }
            }
            return;
        }


        for (cur = NewSelected; cur; cur = cur->next)
        {   line = cur->objnum;                      // this line
            if (side == 1)
            {
               sd1 =  level->LineDefs[line].sidedef1;
               level->SideDefs[sd1].sector = opposite;   // replace
            }
            else
            {
               sd2 =  level->LineDefs[line].sidedef2;
               level->SideDefs[sd2].sector = opposite;   // replace
            }
        }
   }
}

//----------------------------------------------------------------------
//  make a pool from linedefs selected
//----------------------------------------------------------------------

void  MakePoolPedLight(int2 sectordepth,int2 savednum,int2 PoolMode)
{
   LevelPtr level=&LevelWad.level;

   LDPtr  TempLineDef;
   TPtr   TempThing;

   int2   sd1,sd2;
// int2   newsd;
   int2   sd1sector;
   int2   newsector;
   int2   CeilHeight;
   int2   FloorHeight;
   int2   i;
   int2   SetFlag;

   int2   centerx,centery;
   double accx=0.0, accy=0.0;
   double num = 0.0;

//------------------------------------------------------------------------------

   // Create the new sector for the pool - pedestal

   InsertObject(level, OBJ_SECTORS, -1, 0, 0);

   newsector = int2(level->NumSectors - 1);

   SetFlag = 0x0014;  // 2-sided=04, lower unpeg=10

   // teleport exit need linedef teleport

   if (PoolMode == 5)
   {
       TempLineDef = &level->LineDefs[savednum];

       for (i = savednum; i < level->NumLineDefs; i++, TempLineDef+=1)
       {
	    if (GameVersion == 4)
	    {   TempLineDef->type = 70;     // W? teleport
           SetFlag           =  0x0214;// repeat
	    }
	    else
	    {   TempLineDef->type = 97;     // WR teleport, OK STRIFE
	    }
       }
   }

   // teleport entrance need a teleport thing

   if (PoolMode == 6)
   {   TempLineDef = &level->LineDefs[savednum];

       for (i = savednum; i < level->NumLineDefs; i++, TempLineDef+=1)
       {
           accx += (double) level->Vertexes[TempLineDef->start].x;
           accy += (double) level->Vertexes[TempLineDef->start].y;
           num++;
       }
       // center coord
       centerx = accx  / num;
       centery = accy  / num;

       InsertObject(level, OBJ_THINGS, -1, 0, 0);
       TempThing = &level->Things[level->NumThings-1];
       TempThing->xpos = centerx;
       TempThing->ypos = centery;
       TempThing->type = THING_TELEPORT;    // same DOOM/HEXEN = 14
   }

   TempLineDef = &level->LineDefs[savednum];

   for (i = savednum; i < level->NumLineDefs; i++, TempLineDef+=1)
   {
       InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);          //+1 sidedef
       sd2 = int2(level->NumSideDefs-1);

       level->SideDefs[sd2].sector = newsector;              // assign sector

       sd1 = TempLineDef->sidedef1;          // sidedef

       TempLineDef->sidedef2 = sd2;          // new sidedef2
       TempLineDef->flags    = SetFlag;      //0x0014, 2-sided=04, lower unpeg=10

       // sd1 should always be there (but who knows)

      if (sd1 >= 0)                                   // better be there!
	   {
	     sd1sector = level->SideDefs[ sd1 ].sector;
	     if (sd1sector >= 0)
	     {
	       FloorHeight = level->Sectors[sd1sector].floorh;
	       CeilHeight  = level->Sectors[sd1sector].ceilh;
	       switch (PoolMode)
	       {
	       // sunken Pool
	       case(1):
            level->Sectors[ newsector].floorh = int2(FloorHeight-sectordepth);   //sink it
            level->Sectors[ newsector].ceilh  = CeilHeight;

            strncpy( level->Sectors[ newsector].ceilt,  level->Sectors[sd1sector].ceilt, 8); // ceiling textsame
            strncpy( level->Sectors[newsector].floort, DefaultPoolTexture, 8);               // floor text

            strncpy( level->SideDefs[ sd2].tex2, DefaultPoolSideTexture, 8);                 // lower
		      break;
	       // pedestal
	       case(2):
            level->Sectors[ newsector].floorh = int2(FloorHeight+sectordepth);   //raise it
            level->Sectors[ newsector].ceilh  = CeilHeight;

            strncpy( level->Sectors[ newsector].ceilt, level->Sectors[sd1sector].ceilt, 8); // ceiling textsame
            strncpy( level->Sectors[newsector].floort, DefaultPedestalTexture, 8);          // floor text

            strncpy( level->SideDefs[ sd1].tex2, DefaultPedestalSideTexture, 8);            // lower
		      break;
	       // light
	       case(3):
            level->Sectors[ newsector].floorh = int2(FloorHeight+sectordepth);         //raise it
            level->Sectors[ newsector].ceilh  = int2(CeilHeight -sectordepth);         //lower it

            strncpy( level->Sectors[newsector].floort, DefaultLightTexture, 8);  // floor         //light text
            strncpy( level->Sectors[ newsector].ceilt, DefaultLightTexture, 8);  // ceiling textsame

            strncpy( level->SideDefs[ sd1].tex2, DefaultPedestalSideTexture, 8); // lower
            strncpy( level->SideDefs[ sd1].tex1, DefaultPedestalSideTexture, 8); // upper
            break;
	       // Window
	       case(4):
            level->Sectors[ newsector].floorh = int2(FloorHeight+24);                  //raise it
            if (sectordepth < 4) sectordepth = 4;
                level->Sectors[ newsector].ceilh  = int2(FloorHeight+24+sectordepth);      //lower it

            strncpy( level->Sectors[newsector].floort, DefaultWindowTexture, 8);  // floor         //light text
            strncpy( level->Sectors[ newsector].ceilt, DefaultWindowTexture, 8);  // ceiling textsame

            strncpy( level->SideDefs[ sd1].tex2, DefaultPedestalSideTexture, 8); // lower
            strncpy( level->SideDefs[ sd1].tex1, DefaultPedestalSideTexture, 8); // upper
            break;

	       // teleport exit
	       case(5):
            level->Sectors[ newsector].floorh = int2(FloorHeight+sectordepth);   //raise it
            level->Sectors[ newsector].ceilh  = CeilHeight;

            //  if F_SKY
            if ( !strncmp(level->Sectors[sd1sector].ceilt,SkyTexture, 5) )
            {   strncpy( level->Sectors[ newsector].ceilt, level->Sectors[sd1sector].ceilt, 8); // ceiling textsame
            }
            else
            {   strncpy( level->Sectors[ newsector].ceilt, DefaultTeleportExitCeilTx,8 );
            }

            strncpy( level->Sectors[newsector].floort, DefaultTeleportExitFloorTx,8);// floor text

            strncpy( level->SideDefs[ sd1].tex2, DefaultTeleportSideTx, 8);          // lower
            break;
	       // teleport entrance
	       case(6):
            level->Sectors[ newsector].floorh = int2(FloorHeight+sectordepth);             //raise it
            level->Sectors[ newsector].ceilh  = CeilHeight;

            // if F_SKY
            if ( !strncmp(level->Sectors[sd1sector].ceilt,SkyTexture, 5) )
            {   strncpy( level->Sectors[ newsector].ceilt, level->Sectors[sd1sector].ceilt, 8); // ceiling textsame
            }
            else
            {   strncpy( level->Sectors[ newsector].ceilt, DefaultTeleportEntranceCeilTx,8);
            }

            strncpy( level->Sectors[newsector].floort, DefaultTeleportEntranceFloorTx, 8);  // floor text

            strncpy( level->SideDefs[ sd1].tex2, DefaultTeleportSideTx, 8);                 // lower
            break;
 	       }

	       level->Sectors[ newsector].light  = level->Sectors[ sd1sector].light;

	       if (level->Sectors[newsector].floorh > level->Sectors[newsector].ceilh)
	       {
              HelpMsg(gd.hMainWindow,CENTER|HELPINFO, "Tool Error",
                                                       "The floor is higher than the ceiling. Please fix!",
                                                       NULL);
              //ErrorMsg(gd.hMainWindow, "The floor is higher than the ceiling. Please fix!");
	       }
	     }

	     strncpy( level->SideDefs[ sd2].tex3, NoTexture, 8);  // normal
	     strncpy( level->SideDefs[ sd1].tex3, NoTexture, 8);  // normal
	   }
   }
}

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//       Routines for Drawing a Select Box on the Screen with                 //
//       optional text display for line length                                //
//                                                                            //

//////////////////////////////////////////////////////////////////////////////
// Make sure in right order for drawing
//////////////////////////////////////////////////////////////////////////////

void NormalizeRect(LPRECT rc) // LPRECT = RECT *
{
   if (rc->right  < rc->left) SWAP(rc->right,  rc->left);
   if (rc->bottom < rc->top)  SWAP(rc->bottom, rc->top);
}

static void DrawStretch(HDC hdc, RECT *lprClip)
{
   char     sz[80];
   int      x, y, len, dx, dy;
   HDC      hdcTxt;
   HBITMAP  hbm;
   RECT     rcClip, rc;
   SIZE     size;
   HPALETTE hOldPal;
   HPEN     hOldPen;

   SetROP2(hdc, R2_XORPEN);

   rcClip = *lprClip;

   NormalizeRect(&rcClip);

   if(EditMode == OBJ_THINGS && ShowThingPic)
   {  hOldPen   = SelectPen(hdc, gd.hSelectPenT);   //   12-13-96
   }
   else
   {  hOldPen   = SelectPen(hdc, gd.hSelectPen);    // gd.hSelectPen from MakePens utils.cpp
   }

   if (!IsRectEmpty(&rcClip))
   {
      MoveToEx(hdc, rcClip.left,  rcClip.top, NULL);
      LineTo  (hdc, rcClip.right, rcClip.top);
      LineTo  (hdc, rcClip.right, rcClip.bottom);
      LineTo  (hdc, rcClip.left,  rcClip.bottom);
      LineTo  (hdc, rcClip.left,  rcClip.top);

      // Format the dimensions string ..
      wsprintf(sz, "%dx%d", rcClip.right - rcClip.left, rcClip.bottom - rcClip.top);
      len = lstrlen(sz);

      // ... and center it in the rectangle
      GetTextExtentPoint32(hdc, sz, len, &size);

      dx = size.cx;
      dy = size.cy;
      x = (rcClip.right  + rcClip.left - dx) / 2;
      y = (rcClip.bottom + rcClip.top - dy) / 2;

      hdcTxt = CreateCompatibleDC(hdc);

      // Output the text to the DC
      hbm = CreateCompatibleBitmap(hdc, dx, dy);
      rc.left  = rc.top = 0;
      rc.right = dx;
      rc.bottom= dy;

      if(hbm)
      {
         hbm = SelectBitmap(hdcTxt, hbm);
         hOldPal = SelectPalette(hdcTxt, gd.hDoomPal, FALSE);
         RealizePalette(hdcTxt);                                             //   12-13-96 was missing?
         FillRect(hdcTxt, &rc, GetStockBrush(BLACK_BRUSH) );

         SetTextColor(hdcTxt, PALETTEINDEX(GameColors[WHITE]) );
  	      SetBkColor(hdcTxt, RGB(0,0,0));
         SetBkMode (hdcTxt, TRANSPARENT);
         TextOut(hdcTxt, 0, 0, sz, len);
         BitBlt(hdc, x, y, dx, dy, hdcTxt, 0, 0, SRCINVERT);
         hbm = SelectBitmap(hdcTxt, hbm);
         DeleteObject(hbm);
         SelectPalette(hdcTxt, hOldPal, FALSE);      //   11-7-96 was below
      }

      DeleteDC(hdcTxt);
   }

   SetROP2(hdc, R2_COPYPEN);                        //   12-13-96
   SelectPen  ( hdc, hOldPen);
}

//////////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////////
void StretchHelp(HWND hwnd)
{
	  HelpMsg(hwnd, NOCENTER|HELPHELP, "Select Tool",
                   "\tMouse Summary",
                   "",
                   "Left Button\t= Start Select Draw",
                   "Move Mouse\t= Draw Select",
                   "",
                   "\tKey Summary",
                   "",
                   "ESC Key    \t= Exit Select Mode",
                   "Z Key      \t= Zoom to Window",NULL );
}
//////////////////////////////////////////////////////////////////////////
void RectHelp(HWND hwnd)
{
	  HelpMsg(hwnd,NOCENTER|HELPHELP,"Rectangle Tool",
                      " \t\tMouse Summary",
                      "",
                      "Left Button\t\t= Start Rectangle Draw",
                      "Move Mouse\t\t= Draw Rectangle",
                      "Left Button\t\t= Finish Drawing 1 Rectangle",
                      "Right Button\t\t= Drag Rectangle",
                      "            \tRelease to stop dragging",
                      "Right Button\t\t= Exit Rectangle Mode",
                      "            \tBefore drawing starts",
                      "",
                      "\t\tKey Summary",
                      "",
                      "ESC Key   \t\t= Exit Rectangle Mode",
                      "PgUp/PgDn \t\t= Change Width",
                      "Home/End   \t\t= Change Height",
                      "Ins/Del    \t\t= Change Depth",
                      "Space Bar  \t\t= Toggle Angle/Size Increment",
                      "Up/Down Arrow\t= Change width and height",
                      "> / <      \t\t= Rotate Object",
                      "+ / -      \t\t= Change width and height",
                      "shift + /- \t\t= Zoom Map",NULL);
}
///////////////////////////////////////////////////////////////////////////
void PolyHelp(HWND hwnd)
{

	  HelpMsg(hwnd,NOCENTER|HELPHELP,"POLYGON Tool",
                      "\t\tMouse Summary",
                      "",
                      "Left Button\t\t= Start Polygon Draw",
                      "Move Mouse\t\t= Change Polygon Dimensions",
                      "Left Button\t\t= Finish Drawing 1 Polygon",
                      "Right Button\t\t= Drag Polygon",
                      "             \tRelease to stop dragging",
                      "Right Button\t\t= Exit Polygon Mode",
                      "             \tBefore drawing starts",
                      "",
                      "\t\tKey Summary",
                      "",
                      "ESC Key      \t\t= Exit Polygon Mode",
                      "Right/Left Cur\t= Change # sides",
                      "Up/Down Cursor\t= Change Radius",
                      "> / < Key    \t\t= Rotate",
                      "+ / -        \t\t= Change Radius",
                      "shift + /-   \t\t= Zoom Map",
                      "Space Bar    \t\t= Toggle Angle/Size Increment",NULL);
}
//////////////////////////////////////////////////////////////////////////////
void StairHelp(HWND hwnd)
{
	  HelpMsg(hwnd,NOCENTER|HELPHELP,"Stair Tool",
                      "\t\tMouse Summary",
                      "",
                      "Left Button\t\t= Start Stair Draw",
                      "Move Mouse\t\t= Move Stairs to Location",
                      "Left Button\t\t= Finish Drawing 1 Stair",
                      "Right Button\t\t= Exit Stair Mode",
                      "             \tBefore drawing starts",
                      "",
                      "\t\tKey Summary",
                      "",
                      "ESC Key   \t\t= Exit Stair Mode",
                      "Home / End\t\t= Change Height",
                      "Ins / Del \t\t= Change Depth",
                      "PgUp / PgDn\t= Change Width",
                      "Tab / BkSpc\t\t= Rotate Angle",
                      "Right/Left Curs\t= Radius Swirl",
                      "Up/Down Cursor\t= Change #steps",      //radius?
                      "> / <       \t\t= Rotate",
                      "+ / -       \t\t= Change radius",
                      "Space Bar   \t\t= Toggle Angle/Size Increment",
                      "shift + / - \t\t= Zoom Map",NULL);
}
//=================================
void DragHelp(HWND hwnd)
{
     HelpMsg(hwnd,NOCENTER,"Drag Tool",
                      "\tMouse Summary",
                      "",
                      "Move Mouse\t= Drag Objects",
                      "Right Button\t= Release to Exit Drag Mode",
                      "",
                      "\tKey Summary",
                      "",
                      "ESC Key    \t= Exit Drag Mode",
                      "+ / - Key  \t= Zoom Map",NULL);
}

///////////////////////////////////////////////////////////////////////////
void TagHelp(HWND hwnd)
{
     HelpMsg(hwnd,NOCENTER|HELPHELP, "Set Tag",
                    "\tMouse Buttons",
                    "",
                    " Right button\t= Cancel",
                    " Left  button\t= Set Tag",
                    ""
                    " Move the mouse to select",
                    " a Sector or LineDef",
                    "",
                    " \tKeyboard Keys",
                    "",
                    "  + -       \t= Map size",
                    " Enter      \t= Set Tag",
                    " Esc        \t= Cancel/Exit",NULL);
}

/////////////////////////////////////////////////////////////////////////////
void DrawingHelp(HWND hwnd)
{
     HelpMsg(hwnd,NOCENTER|HELPHELP,"LineDraw Tool",
                      "\tMouse Summary",
                      "",
                      "Left Button\t= Insert Vertex",
                      "Right Button\t= Close Sector",
                      "Right Button\t= Exit LineDraw",
                      "\t(when not drawing)",
                      "",
                      "\tKey Summary",
                      "",
                      "ESC Key    \t= Exit Select Mode",
                      "Insert Key \t= Close Sector",
                      "Cursor Keys\t= Move Map",NULL);
}

////////////////////////////////////////////////////////////////////////////

void RotateHelp(HWND hwnd)
{
     HelpMsg(hwnd,NOCENTER|HELPHELP,"Rotate Tool",
                      "\t\tMouse Summary",
                      "",
                      "Left Button\t\t= Finish Rotate Mode",
                      "Move Mouse \t= Change Rotation",
                      "Right Button\t\t= Drag Map",
                      "            \t\tRelease to stop dragging",
                      "",
                      "\t\tKey Summary",
                      "",
                      "ESC Key  \t\t= Exit Rotate Mode",
                      "Space Bar\t\t= Toggle Angle/Size Increment",
                      "Up/Down Arrow\t= Change Width and Height",
                      "> / < Key \t\t= Rotate Object",
                      "+ / - Key \t\t= Size Object",NULL );
}


//////////////////////////////////////////////////////////////////////////////
//   zoom to window for stretch below
//////////////////////////////////////////////////////////////////////////////

static void ZoomToWindow(RECT *rc, int4 mapOrigX, int4 mapOrigY, int4 mapEndX, int4 mapEndY)
{
  int      width, height;
  double   NewScale =1.0;
  double   NewScale2=1.0;

//-----------------------------------------------

  width = abs(rc->right  - rc->left);
  height= abs(rc->bottom - rc->top);

  if (width)
  {   NewScale  =(double)(ScrMaxX-10) /(double)abs(MAPX(rc->left) - MAPX(rc->right));
  }

  if (height)
  {   NewScale2 =(double)(ScrMaxY-50) /(double)abs(MAPY(rc->top) - MAPY(rc->bottom));
  }

  if (NewScale2 < NewScale) NewScale = NewScale2;

  // save prior location for shift+Z rezoom

  oldWindowZoomScale = Scale;
  oldWindowZoomOrigX = OrigX;
  oldWindowZoomOrigY = OrigY;

  Scale = NewScale;
  if (Scale < 0.01) Scale = 0.01;
  //if (Scale > 8.0 ) Scale = 8.0;    //anything over 8  blows up?? why??
  if (Scale > 16.0 ) Scale = 16.0;    //   3-1-97 fixed in grid made int4

  CenterMapAroundCoords(  ( mapOrigX + mapEndX )/2,
                          ( mapOrigY + mapEndY )/2);//- (16/Scale)) );
  // current values
  WindowZoomScale = Scale;
  WindowZoomOrigX = OrigX;
  WindowZoomOrigY = OrigY;
}

//static void StretchExit(HWND hwnd, HDC hdcSelect, HPEN hOldPen, HPALETTE hOldPal)
static void StretchExit(HWND hwnd, HDC hdcSelect, HPALETTE hOldPal)
{
  Selecting = FALSE;

//SelectPen    ( hdcSelect, hOldPen);
  SelectPalette( hdcSelect, hOldPal, FALSE);
  SetROP2      ( hdcSelect, R2_COPYPEN);       //   12-13-96 always default to this
  ReleaseDC    ( hwnd, hdcSelect);

  EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));
  InstructionsOff();

  RedrawMap(NULL, &gd.rcClient, ERASEYES, FALSE);
  XYUpdate(hwnd, PointerX,PointerY); // update our position

  ReleaseCapture();
}

////////////////////////////////////////////////////////////////////////////
//        The Main Routine                                                //
//                                                                        //
//  PURPOSE: Draws a SelectBox                                            //
//                                                                        //
// currently only supports the Esc key to exit                            //
////////////////////////////////////////////////////////////////////////////
 #pragma argsused
void	StretchTool(HWND hwnd, int x, int y, UINT keyFlags)
{
 int4     ptOriginX;
 int4     ptOriginY;

 LevelPtr level = &LevelWad.level;

 SelPtr   oldsel = NULL;
 int4     mapEndX,  mapEndY;
 int4     mapOrigX, mapOrigY;
// double   NewScale =1.0;
// double   NewScale2=1.0;


 HDC      hdcSelect;
//HPEN     hOldPen;
 HPALETTE hOldPal;
 RECT     rc;
 MSG      msg;
 int      width, height;
 UINT     KeyData;
 DWORD    future;

//-----------------------------------------------------------------------

   hdcSelect = GetDC(hwnd);
   hOldPal   = SelectPalette(hdcSelect, gd.hDoomPal, FALSE);//   12-13-96 was TRUE?????
   RealizePalette(hdcSelect);                               // also move up here from below trying anything
   SetBkMode (hdcSelect, TRANSPARENT);                      //   12-13-96 DOES THIS have to be RESTORED??????
   SetBkColor(hdcSelect, PALETTEINDEX(BLACK));              // 2-1-97 bk  No.

   Instructions("Select Tool - F1=Help");

   SetCapture(hwnd);                                        // Capture mouse input
 	EnableCursor(hwnd, gd.hCursorSelect);
   Selecting = TRUE;

   ptOriginX = x;
   ptOriginY = y;

   mapOrigX = MAPX(x);
   mapOrigY = MAPY(y);

   rc.left = rc.right  = ptOriginX; //ptOrigin.x;
   rc.top  = rc.bottom = ptOriginY; //ptOrigin.y;
   // dragging cause pen changes so did in drawstretch instead
//   if(EditMode == OBJ_THINGS && ShowThingPic)
//   {  hOldPen   = SelectPen(hdcSelect, gd.hSelectPenT);   //   12-13-96
//   }
//   else
//   {  hOldPen   = SelectPen(hdcSelect, gd.hSelectPen);    // gd.hSelectPen from MakePens utils.cpp
//   }

   DrawStretch(hdcSelect, &rc);                     // draw starting

   MouseButton = 1;                                 //start fresh

   for (;;)
   {
      WaitMessage();
      KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
      if ( CheckEscape(&msg,KeyData) ) break;
      if ( CheckF1(&msg,KeyData))                   // HELP this is not there
      {    StretchHelp(hwnd);
           continue;
      }

      if (msg.message == WM_MOUSEMOVE)
      {
          XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList
          // erase
          DrawStretch(hdcSelect, &rc);

          rc.left   = ptOriginX;
          rc.top    = ptOriginY;
          rc.right  = PointerX;
          rc.bottom = PointerY;

          mapEndX = MAPX(abs(rc.right ));      // Store our position in Map Coords
          mapEndY = MAPY(abs(rc.bottom));      // before Normalize gets it

          if (CheckMapAndScroll(hwnd, hdcSelect, level, FALSE, FALSE))// drag is not active
          {   // Update our Origin since it  May have scrolled out of sight

              ptOriginX = SCREENX(mapOrigX);    // new org
              ptOriginY = SCREENY(mapOrigY);

              rc.left   = ptOriginX;            // Set up RECT for Rectangle
              rc.top    = ptOriginY;
              rc.right  = PointerX;
              rc.bottom = PointerY;

          }
          // draw
          DrawStretch(hdcSelect, &rc);
      }
      else
      {
          if(msg.message == WM_CHAR)
          {
            switch(KeyData)
            {
	           case('Z'):              //  ZOOM this area
               ZoomToWindow(&rc, mapOrigX, mapOrigY, mapEndX, mapEndY);
               StretchExit(hwnd, hdcSelect, hOldPal);           // Clean UP
               //StretchExit(hwnd, hdcSelect, hOldPen, hOldPal);// Clean UP   12-13-96
		         return;

              default:
               break;
            }
          }
      }

      ///////////////////////////////////////////////
	   // if 2nd mouse pressed, drag select box around
      ///////////////////////////////////////////////
	   if ( (MouseButton & 2) )
	   {
            width = abs(rc.right - rc.left);
            height= abs(rc.bottom - rc.top);
            future = GetTickCount() + ButtonDelay;   // add dblclk time

            while ( (MouseButton & 2) )      // Wait for R-BUTTON UP
            {
                 WaitMessage();

                 // erase
                 DrawStretch(hdcSelect, &rc);

                 KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
                 if ( CheckEscape(&msg,KeyData) ) break;

                 // zoom to window if right click right away

                 if (msg.message == WM_RBUTTONUP                //   8-1-97
                 ||  msg.message == WM_RBUTTONDBLCLK)           //
                 {
                     if( future >= GetTickCount() )             // within time frame for double click
                     {
                          ZoomToWindow(&rc, mapOrigX, mapOrigY, mapEndX, mapEndY);
                          StretchExit(hwnd, hdcSelect, hOldPal);           // Clean UP
                          return;
                     }
                 }
                 future = 0;                                   // disable timer if anything else is going on

                 if (msg.message == WM_MOUSEMOVE)
                 {

                    XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList

                    ptOriginX = PointerX-width;   // new location
                    ptOriginY = PointerY-height;

                    // if map moved, get new orgs
                    if (CheckMapAndScroll(hwnd, hdcSelect, level, FALSE, FALSE) )
                    {
                       ptOriginX = SCREENX(mapOrigX);
                       ptOriginY = SCREENY(mapOrigY);
                    }

                 }

                 rc.left   = ptOriginX;   // Set up RECT for Rectangle
                 rc.top    = ptOriginY;
                 rc.right  = rc.left+width;
                 rc.bottom = rc.top+height;

                 if (msg.message == WM_CHAR
                 ||  msg.message == WM_KEYDOWN)
                 {
                      if (ProcessMapSizeCommands(hwnd,level,
                                                   msg.message,
                                                   KeyData,
                                                   FALSE, FALSE)) // no goto, nodrag    8-1-97

                      {    ptOriginX = SCREENX(mapOrigX);
                           ptOriginY = SCREENY(mapOrigY);
                           rc.left   = ptOriginX;   // Set up RECT for Rectangle
                           rc.top    = ptOriginY;
                           rc.right  = PointerX;
                           rc.bottom = PointerY;
                     }
                 }
                 // draw
                 DrawStretch(hdcSelect, &rc);
	         }

            mapEndX = MAPX(abs(rc.right ));      // 2-1-97 bk Wouldn't select properly
            mapEndY = MAPY(abs(rc.bottom));      // if dragged on RButton

            mapOrigX = int2(MAPX(PointerX - width));
            mapOrigY = int2(MAPY(PointerY - height));

	   }  // end(MouseButton == 2)

      // if no more buttons, exit
      if (MouseButton == 0 )          // no buttons
      {
          // additive selection box or not ?
          if (AdditiveSelBox == FALSE)        // ignore
          {   ForgetSelection( &SelectList);
          }
          else
          {   oldsel = SelectList;
          }

          Instructions("Selecting Objects");

          // select the objects inside the box
          SelectList = SelectObjectsInBox(EditMode, mapOrigX, mapOrigY, mapEndX,  mapEndY );

          if (AdditiveSelBox == TRUE)
          {   while (oldsel != NULL)
              {
	                 if (!IsSelected( SelectList, oldsel->objnum))
		                  SelectObject( &SelectList, oldsel->objnum);

	                 UnSelectObject( &oldsel, oldsel->objnum);
              }
          }

          if (SelectList)
          {   CurObject = SelectList->objnum;
              SoundBeep(SoundClick1);
          }
          else
          {   CurObject = -1;
          }
          break;

      }
   } // end  for(;;)

   StretchExit(hwnd, hdcSelect, hOldPal);           // Clean UP
   //StretchExit(hwnd, hdcSelect, hOldPen, hOldPal);// Clean UP   12-13-96
}

//---------------------------------------------------------------------------//
//                                                                           //
//  DragMap  ctrl+ left mouse button   11-5-96                             //
//                                                                           //
//---------------------------------------------------------------------------//
#define MAPSPEED 20      //   8-1-97 (50)
void DragMap(HWND hwnd, int2 Button)
{
  Bool     DidScale;
  int4     xnew,ynew;
  int4     oldx,oldy;
  int4     dragx,dragy;
  int4     edgelimit=48;

  HDC      hdc;
  HPALETTE hOldPal;
  MSG      msg;

  int      ElapsedTime;                              //   3-1-97
  //--------------------------------------------------------------------------

  hdc = GetDC(hwnd);
  SetCapture(hwnd);                           // Capture mouse input

  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);  // TRUE??   12-13-96 need to follow who's who
  RealizePalette(hdc);

  EnableCursor(hwnd, gd.hCursorPan);//gd.hCursorMove);

  DrawBigMap( PARTIALYES,hdc, TRUE);

  // put this first to eat any mouse stuff hanging back   11-10-96
  GetMouseCoords(hwnd,&msg,NULL, &oldx,  &oldy, NULL);

  xnew = oldx;     // origin
  ynew = oldy;

  MouseButton = Button;                   // original button status

  // have to have a button down

  while (MouseButton)
  {
        WaitMessage();                    // put back   3-1-97 make mouse move to zoom
        GetMouseCoords(hwnd,&msg,NULL, &xnew, &ynew, &MouseButton);

        dragx = oldx - xnew;              //xorg
        dragy = oldy - ynew;              //yorg

        if (dragx || dragy)
        {
	        oldx  = xnew;
	        oldy  = ynew;

	        OrigX +=dragx/Scale;
	        OrigY -=dragy/Scale;

           // boundary scrolling
	        // -TOP-- Y limits ----------------------------
	        if (ynew <= ScrollMapBorder )
	        {   ynew = ScrollMapBorder+edgelimit;
	        }
	        // --BOTM --Y  near the bottom of screen
	        if (ynew >= (gd.rcClient.bottom - ScrollMapBorder))
	        {   ynew = (gd.rcClient.bottom - ScrollMapBorder) -edgelimit;
	        }
	        // LEFT X axis -- are we near the left side
	        if (xnew <= ScrollMapBorder)
	        {   xnew = ScrollMapBorder +edgelimit;
	        }
	        // RIGHT X axis -- are we near the right side
	        if (xnew >= (gd.rcClient.right - ScrollMapBorder))
	        {   xnew = (gd.rcClient.right - ScrollMapBorder)-edgelimit;
	        }
	        if (oldx != xnew || oldy != ynew)
	        {   SetCursorPos(xnew,ynew+ gd.CaptionHeight); // set back    11-7-96
	            oldx  = xnew;
	            oldy  = ynew;
	        }
           DrawBigMap( PARTIALYES,hdc,TRUE);
        }
        else
        if (Button == 3              // started with both
         && MouseButton              // have a button
         && MouseButton !=Button)    // not both
        {
            //  ZoomMap===============================

            if (MouseButton == 1)         // increase
            {   DidScale = SetScale(1, FALSE);
	             if (AutoCenter == 1
                && (EditMode == OBJ_VERTEXES || EditMode == OBJ_THINGS) ) //   12-2-96 only vertex or things
	             {  if (CurObject >=0 )                                     //   12-8-96
		             {  GoToObject( EditMode, CurObject, FALSE);             // don't force zoom
                   }
		          }
            }
            else
            if (MouseButton == 2)         // decrease
            {   DidScale = SetScale(-1, FALSE);
            }

            if (DidScale)                 // notmaxed out
            {
              ElapsedTime = GetTickCount();           //   12-2-96 clock screen draw
              DrawBigMap( PARTIALYES,hdc,TRUE);
              ElapsedTime = GetTickCount() - ElapsedTime;
              if (ElapsedTime < MAPSPEED) Sleep(MAPSPEED-ElapsedTime);  //   12-8-96 slow it down 8-1-97
              EatMouseData(hwnd);                           //   12-8-96 don't let the keys/buttons pile up
            }
        }
  }
  EatMouseData(hwnd);

  RedrawMap(hdc, &gd.rcClient, ERASEYES, TRUE);

  ReleaseCapture();
  SelectPalette(hdc, hOldPal, FALSE);
  ReleaseDC(hwnd, hdc);

  EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));
}

//---------------------------------------------------------------------------
//  ZoomMap  entered shift + a mouse button //   11-5-96
//           shift + left  = ZOOM +
//           shift + right = ZOOM -
//---------------------------------------------------------------------------

void ZoomMap(HWND hwnd, int2 Button)
{
  Bool     DidScale;
  int4     xnew,ynew;
  int4     oldx,oldy;
  int4     dragx,dragy;
  int4     edgelimit=48;

  HDC      hdc;
  HPALETTE hOldPal;
  MSG      msg;

  int      ElapsedTime;                              //   12-8-96
  //--------------------------------------------------------------------------

  hdc = GetDC(hwnd);
  SetCapture(hwnd);                           // Capture mouse input

  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);//TRUE);   12-13-96
  RealizePalette(hdc);

  EnableCursor(hwnd, gd.hCursorMove);

  DrawBigMap( PARTIALYES,hdc, TRUE);

  // put this first to eat any mouse stuff hanging back   11-10-96
  GetMouseCoords(hwnd,&msg,NULL, &oldx,  &oldy, NULL);

  MouseButton = Button;

  xnew = oldx;     // origin
  ynew = oldy;

  while (MouseButton)
  {
        WaitMessage();                   // ok now   3-1-97
        GetMouseCoords(hwnd,&msg,NULL, &xnew, &ynew, &MouseButton);

        dragx = oldx - xnew;              //xorg
        dragy = oldy - ynew;              //yorg

        // see if moved

        if (dragx || dragy)
        {
	        oldx  = xnew;
	        oldy  = ynew;

	        OrigX +=dragx/Scale;
	        OrigY -=dragy/Scale;

           // boundary scrolling
	        // -TOP-- Y limits ----------------------------
	        if (ynew <= ScrollMapBorder )
	        {   ynew = ScrollMapBorder+edgelimit;
	        }
	        // --BOTM --Y  near the bottom of screen
	        if (ynew >= (gd.rcClient.bottom - ScrollMapBorder))
	        {   ynew = (gd.rcClient.bottom - ScrollMapBorder) -edgelimit;
	        }
	        // LEFT X axis -- are we near the left side
	        if (xnew <= ScrollMapBorder)
	        {   xnew = ScrollMapBorder +edgelimit;
	        }
	        // RIGHT X axis -- are we near the right side
	        if (xnew >= (gd.rcClient.right - ScrollMapBorder))
	        {   xnew = (gd.rcClient.right - ScrollMapBorder)-edgelimit;
	        }
	        if (oldx != xnew || oldy != ynew)
	        {   SetCursorPos(xnew,ynew+ gd.CaptionHeight); // set back    11-7-96
	        }

      }
      else
      if (MouseButton == 1)         // increase
      {   DidScale = SetScale(1, FALSE);
	       if (AutoCenter == 1
           && (EditMode == OBJ_VERTEXES || EditMode == OBJ_THINGS) ) //   12-2-96 only vertex or things
	       {  if (CurObject >=0 )                                     //   12-8-96
		       {  GoToObject( EditMode, CurObject, FALSE);             // don't force zoom
             }
		    }
      }
      else
      if (MouseButton == 2)         // decrease
      {   DidScale = SetScale(-1, FALSE);
      }

      if (!DidScale) break;         // maxed out

      ElapsedTime = GetTickCount();           //   12-2-96 clock screen draw
      DrawBigMap( PARTIALYES,hdc,TRUE);
      ElapsedTime = GetTickCount() - ElapsedTime;
      if (ElapsedTime < MAPSPEED) Sleep(MAPSPEED-ElapsedTime);  //  12-8-96 slow it down 8-1-97
      EatMouseData(hwnd);                     //   12-8-96 don't let the keys/buttons pile up
  }

  CheckKeyState();

  RedrawMap(hdc, &gd.rcClient, ERASEYES, TRUE);
  StatBarUpdate4(Scale);                      //   3-1-96

  ReleaseCapture();
  SelectPalette(hdc, hOldPal, FALSE);
  ReleaseDC(hwnd, hdc);

  EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));
}


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//       Routines for Inserting Objects into the map                          //
//  1. Pool, Ped, Light, Basic Rectangle                                      //
//     Rectangle Ctrl-R                                                       //
//  2. Polygon   Ctrl-P                                                       //
//  3. Stairs    Ctrl-A                                                       //
//  4. Door                                                                   //
//  5. LineDraw  Ctrl-D                                                       //
//                                                                            //
//                                                                            //

//-------------------------------------------------------------------------------
// 1. Check that none of the vertices about to be made are the same coordinates
// 2. Check that no other objects are already inside of area to be made
//-------------------------------------------------------------------------------

static Bool PolyVertexVerify(HWND hwnd, LevelPtr level, int2 numsides,int2 angle, Bool intersect)
{
   int4 n,m;
   int4 Xwork,Ywork;

   int4 minX =  32767;
   int4 minY =  32767;
   int4 maxX = -32767;
   int4 maxY = -32767;

   LDPtr TempLineDef;

   Bool cross = FALSE;
//------------------------------------------------------------------------------

   for (n=0; n< numsides; n++)
   {    Xwork = PolygonX[n];
	     Ywork = PolygonY[n];

        // get min/max for boundary check later
        if (Xwork < minX ) minX = Xwork;
        if (Ywork < minY ) minY = Ywork;
        if (Xwork > maxX ) maxX = Xwork;
        if (Ywork > maxY ) maxY = Ywork;

        for (m=0; m < numsides; m++)
        {  if (m != n)
	        {   if (Xwork == PolygonX[m] &&  Ywork == PolygonY[m])
  		         {
                   HelpMsg(hwnd,CENTER|HELPINFO, "Tool Error",
		                             "2 or more vertices occupy the same location!",
                                   " ",
		                             "You made the size too small or the grid is",
		                             "forcing them to the same coordinates!",
                                   NULL);

		            return FALSE;
		         }
	        }
	     }
   }

   // now see that nothing is in area
   if (!intersect) return TRUE;      // don't check for intersection
   if (angle)      return TRUE;      // don't check if drawn at an angle

   TempLineDef = level->LineDefs;

   for (n = 0; n < level->NumLineDefs; n++, TempLineDef +=1)
   {
	   // the two ends of the line must be in the box
	   m     = TempLineDef->start;
	   Xwork = level->Vertexes[m].x;
	   Ywork = level->Vertexes[m].y;

	   if ( (Xwork >= minX && Xwork <= maxX) && (Ywork >= minY && Ywork <= maxY) )
	   {  cross = TRUE;
	   }
	   else
	   {
	      m     = TempLineDef->end;
	      Xwork = level->Vertexes[m].x;
	      Ywork = level->Vertexes[m].y;

	      if ( (Xwork >= minX && Xwork <= maxX) && (Ywork >= minY && Ywork <= maxY) )
	      { cross = TRUE;
	      }
	   }

	   // possible problem
	   if (cross)
      {  if (SmartDraw)
         {
            HelpMsg(hwnd, CENTER|HELPINFO, "Tool Error",
 	                     "Area will be adjusted",
                        " ",
                        "You have drawn an area that encloses",
                	      "existing areas! Sector References have been",
             		      "adjusted. Please verify by checking each SideDef",
                        "and changing manually if not correct !",
                        NULL);
         }
         else
     	   {
            HelpMsg(hwnd, CENTER|HELPINFO, "Possible Error",
                      "You have drawn an area that encloses existing areas!",
                      "",
                      "",
                      "You must change the sector references of the existing area(s)",
                      "to match the new area sector number just created.",
                      NULL);
         }
	      break;
	   }
   }

   return TRUE;
}
////////////////////////   RECTANGLE   TOOL     ////////////////////////////////
// Make a rectangle
////////////////////////////////////////////////////////////////////////////////
void  RectangleDraw(HDC hdc, int4 xorg, int4 yorg, int4 xlast,int4 ylast, int2 anglemore )
{
   int4 n;
   int4 Xwork,Ywork;

   double dx,dy;
   double centerx, centery;
   double accx=0.0, accy=0.0;

   int4   width,length;

//--------------------------------------------------------------------------------


  // flip orgs as required

  if (xorg > xlast)
  {   n    = xorg;
      xorg = xlast;
      xlast= n;
  }
  // y is greater going up
  if (yorg < ylast)
  {   n    = yorg;
      yorg = ylast;
      ylast= n;
  }

  width  = (xlast-xorg);
  length = (yorg-ylast);

  PolygonX[0] = xorg;          // upper left
  PolygonY[0] = yorg;
  PolygonX[1] = xlast;         // upper right
  PolygonY[1] = yorg;

  PolygonX[2] = xlast;         // lower right
  PolygonY[2] = ylast;
  PolygonX[3] = xorg;          // lower left
  PolygonY[3] = ylast;

  PolygonX[4] = xorg;          // for drawing later
  PolygonY[4] = yorg;

  // rotated position

  if (anglemore )
  {    // get geometric center
       for (n = 0; n < 4; n++)
       {   accx += (double) PolygonX[n];
	        accy += (double) PolygonY[n];
       }

       centerx = accx  / 4;
       centery = accy  / 4;

       for (n = 0; n < 4; n++)
       {
           Xwork = PolygonX[n];
           Ywork = PolygonY[n];

           dx    = Xwork - centerx;       // x offset from center
           dy    = Ywork - centery;       // y offset from center

           RotateAndScaleCoords( &dx, &dy, -anglemore, 1.0);

           Xwork = centerx + dx;
           Ywork = centery + dy;

           PolygonX[n] = Xwork;
           PolygonY[n] = Ywork;
       }
   }

   // distance apart they are

   if (GridScale
   &&  GridSnap
   &&  width  >= GridScale/2        // exceeds grid size
   &&  length >= GridScale/2)
   {   for (n = 0; n < 4; n++)
       {
           Xwork = PolygonX[n];
           Ywork = PolygonY[n];

           if (Xwork < 0)
           {   Xwork= (Xwork - GridScale/2)/GridScale * GridScale;
           }
           else
           {   Xwork= (Xwork + GridScale/2)/GridScale * GridScale;
           }
      	  if (Ywork < 0)
           {   Ywork= (Ywork - GridScale/2)/GridScale * GridScale;
           }
           else
           {   Ywork= (Ywork + GridScale/2)/GridScale * GridScale;
           }
           // see at end for old code
           PolygonX[n] =  Xwork;
           PolygonY[n] =  Ywork;
       }
   }

   // last line for drawing routine next

   PolygonX[4] = PolygonX[0];
   PolygonY[4] = PolygonY[0];

   for (n = 0; n < 4; n++)
   {
       MoveToEx(hdc, SCREENX( PolygonX[n] ),   SCREENY( PolygonY[n] ), NULL );
       LineTo  (hdc, SCREENX( PolygonX[n+1] ), SCREENY( PolygonY[n+1] ) );
   }
}

///////////////////////////////////////////////
// Called From RectangleTool                 //
///////////////////////////////////////////////
static void DrawRect(HDC hdc, int4 xorg, int4 yorg, int4 xlast, int4 ylast, int2 angle)
{
    ShowCursor(FALSE);
	 SetROP2(hdc, R2_XORPEN);                   // write mode xor
    SelectPen(hdc, gd.hDrawPen);
	 RectangleDraw(hdc, xorg,yorg, xlast,ylast,angle);
	 SetROP2(hdc, R2_COPYPEN);
    ShowCursor(TRUE);
}

//////////////////////////////////////////////////
// Just a wrapper around all the clean up stuff //
//////////////////////////////////////////////////
static void ToolExit(HWND hwnd, HDC hdc, Bool oldsnap, int2 oldscale, HPALETTE hOldPal, HPEN hOldPen, HFONT hOldFont)
{                                       //int OldRop,
  GridSnap  = oldsnap;
  GridScale = oldscale;
  //SetROP2(hdc, OldRop);          //   3-1-97
  SetROP2(hdc, R2_COPYPEN);        // set writing mode for line drawing
  SelectPalette(hdc, hOldPal, FALSE);
  SelectFont(hdc, hOldFont);
  SelectPen(hdc, hOldPen);

  ReleaseDC(hwnd, hdc);
  InstructionsOff();
  ReleaseCapture();
}

//---------------------------------------------------------------------------------
// draw the Rectangle box and stay here until mouse  click
//---------------------------------------------------------------------------------
#pragma argsused
void RectangleTool(HWND hwnd, LevelPtr level,
			  int2  Stype,
			  int2 *Swidth,
			  int2 *Slength,
			  int2 *Sdepth,
			  int2 *Sangle )

{
  int4  n;

  int4  xorg,yorg;
  int4  xlast,ylast;

  int2  sector;
  int2  oldsector = 0;

  int4  Ldx,Ldy;
  int2  TypeOfTool;
  // rectangle
  int4  width = 128;
  int4  length= 128;
  int2  depth =   0;   // Pool

  int4  dx,dy;

  int2  sizeinc  =8;
  int2  angleinc =1;

  int2  angle    = 0;

  SelPtr NewSelected=NULL;
  SelPtr cur;
  int4   savedLnum;
  int4   savedSnum;

  int2   sd1,sd2;
  int2   newsector;

  int2   oldscale = GridScale;
  Bool   oldsnap  = GridSnap;
  Bool   FirstTime = TRUE;

  // in case user changes it in a map cmd
  int2     priorscale;
  Bool     priorsnap ;
  Bool     Redraw;
  Bool     processCmd;

  MSG      msg;
  HDC      hdc;
  HPALETTE hOldPal;
//  int      OldRop;
  HPEN     hOldPen;
  HFONT    hOldFont;
  UINT     KeyData;

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------


 hdc = GetDC(hwnd);

 hOldPal  = SelectPalette(hdc, gd.hDoomPal, FALSE);
 RealizePalette(hdc);

 SetROP2(hdc, R2_XORPEN);   //OldRop
 hOldPen  = SelectPen(hdc, GetStockPen(WHITE_PEN));
 hOldFont = SelectFont(hdc, gd.hSmallFont);

 SetCapture(hwnd);                           // Capture mouse input

 ResetMapResident(hwnd, FALSE, TRUE, TRUE);// Pics, Object, List
 MouseButton = 0;

for (;;)
{
   if( !ToolContinue && !FirstTime)
   {   ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
	    return;
   }

   SetTextType(hdc, 5);

   if (Stype == 6|| Stype == 7)  // teleports
   {   Instructions("Teleport Tool +-=size, <>=rotate, F1=Help, Grid=64");
   }
   else
   {   Instructions("Rectangle Tool +-=Zoom Up/Dn=size <>=rotate PgUp/PgDn=width F1=Help");
   }

   MouseButton = 0;              // ctrl+R or not 1st time

   MouseMove();                  // sets PointerX/Y

   FirstTime = FALSE;
   //Redraw = FALSE;

   // Wait for Left Button ------------------------------------------------

   while (MouseButton != 1)         // Wait for L-BUTTON DOWN
   {
      WaitMessage();
      KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
      if ( CheckEscape(&msg,KeyData) )
      {   ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
          return;
      }
      if (CheckF1(&msg,KeyData))
      {   RectHelp(hwnd);
          continue;
      }

      if (MouseButton == 1) break;

      if (MouseButton & 2 )
      {   ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
		    return;
      }

      if (msg.message == WM_MOUSEMOVE)
      {   CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE);  //was comment   11-7-96
          XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
      }
  }

  //================

  TypeOfTool = Stype;

  if ( Stype > 0 )
  {
      TypeOfTool--;
      if (Stype == 6|| Stype == 7)  // teleports
      {   width    = 64;
	       length   = 64;
	       depth    =  8;
	       GridScale= 64;
	       GridSnap = TRUE;
      }
      else
      {   width = *Swidth;
	       length= *Slength;
	       depth = *Sdepth;

          if (Stype == 8)         // door
	       {   angle = *Sangle;
	       }
      }
  }

  // set origin current position

  xorg = MAPX( PointerX );                             // anchor point
  yorg = MAPY( PointerY );

  xlast= xorg + width;
  ylast= yorg - length;

  // adjust so screen won't scroll off
  if ( SCREENX(xlast)+8 >= ScrMaxX-8 )
  {    xlast = MAPX(ScrMaxX);        //-16
       xorg  = xlast - width;
  }

  if ( SCREENY(ylast) >= ScrMaxY )   //+16 -16
  {    ylast = MAPY(ScrMaxY);        //-32
       yorg  = ylast + length;
  }

  // Move the cursor to the correct position at the width/height

  SetCursorPos( SCREENX(xlast), ((SCREENY(ylast)) + gd.CaptionHeight) );
  // draw
  DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // SET initial Rect position

  savedLnum = level->NumLineDefs;
  savedSnum = level->NumSectors;

  // make sure button is gone

  while (MouseButton & 1)
  {     WaitMessage();
        GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
  }

  // wait for mouse 1 click=====

  while ( MouseButton != 1 )
  {
      WaitMessage();
      KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
      Redraw  = TRUE;  //   3-1-97

      if (CheckEscape(&msg,KeyData))    //escape
      {    ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
		     return;
      }
      if  (CheckF1(&msg,KeyData) )       //f1
      {    RectHelp(hwnd);
           continue;
      }

      if (MouseButton == 1) break;

      switch(msg.message)
      {
        default:
          break;
        case(WM_MOUSEMOVE):
          XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList
          //erase
          DrawRect(hdc, xorg, yorg, xlast, ylast, angle);

          CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE) ;

          xlast    = MAPX(PointerX);  // new origin
          ylast    = MAPY(PointerY);
          //draw
          DrawRect(hdc, xorg, yorg, xlast, ylast, angle);
          break;
        case(WM_CHAR):
          // erase
          DrawRect(hdc, xorg, yorg, xlast, ylast, angle);

          priorscale = GridScale;
          priorsnap  = GridSnap;
          Redraw     = TRUE;
          processCmd = FALSE;

          switch(KeyData)
          {
              case VK_RETURN:     //enter
                MouseButton = 1;
		          break;
              case VK_SPACE:     //space
                if (sizeinc==1 )  sizeinc = 8;
		          else              sizeinc = 1;
                if (angleinc==1 ) angleinc= 6;
		          else              angleinc= 1;
		          break;
              case ('>'):
              case ('.'):
                angle +=angleinc;
		          break;
              case ('<'):
              case (','):
                angle -=angleinc;
		          break;
              case('+'):
              case('='):
                if (!gd.ShiftDown)
                {   xorg -= sizeinc;
                    yorg += sizeinc;
                    xlast+= sizeinc;
                    ylast-= sizeinc;
                }
                else
                {   processCmd= TRUE;
                }
                break;
              case('-'):
              case('_'):
                if (!gd.ShiftDown)
                {   xorg  += sizeinc;
                    yorg  -= sizeinc;
                    xlast -= sizeinc;
                    ylast += sizeinc;
                }
                else
                {   processCmd= TRUE;
                }
                break;
              default:
                processCmd = TRUE;
                break;

          } // end switch keychar

          // process any leftover cmd
          if (processCmd)
          {
               ProcessMapSizeCommands(hwnd,level,
                                      msg.message,
                                      KeyData,
                                      FALSE, FALSE);  // no goto, nodrag   8-1-97
          }
          if (angle >  360) angle -=  360;
          if (angle < -360) angle +=  360;

          //draw
          DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // new position
          SetCursorPos( SCREENX(xlast), ((SCREENY(ylast)) + gd.CaptionHeight) );
          // eat data now
          GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
          break;

        case(WM_KEYDOWN):
          // erase
          DrawRect(hdc, xorg, yorg, xlast, ylast, angle);
          priorscale = GridScale;
          priorsnap  = GridSnap;
          // any special cmds ?
          ProcessMapSizeCommands(hwnd,level,
                                 msg.message,
                                 KeyData,
                                 FALSE, FALSE);  // no goto, nodrag   8-1-97

          Redraw = TRUE;

          switch(KeyData)
          {
              case VK_PRIOR:      // pgup
                xorg -= sizeinc;
                break;
              case VK_NEXT:       //pgdown
                xorg += sizeinc;
                break;
               case VK_HOME:       //home
                yorg -= sizeinc;
                break;
               case VK_END:        //end
                yorg += sizeinc;
                break;
               case VK_INSERT:     //ins
                depth++;
  		          break;
               case VK_DELETE:     //del
                depth--;
		          break;
               case VK_UP:        //up arrow  Y
                xorg -= sizeinc;
                yorg += sizeinc;
                xlast+= sizeinc;
                ylast-= sizeinc;
		          break;
               case VK_DOWN:      //down arrow Y
                xorg  += sizeinc;
                yorg  -= sizeinc;
                xlast -= sizeinc;
                ylast += sizeinc;
		          break;
               case VK_LEFT:      //left arrow X
                xorg -= sizeinc;
                yorg += sizeinc;
                xlast+= sizeinc;
                ylast-= sizeinc;
		          break;
               case VK_RIGHT:     // right arrow X
                xorg  += sizeinc;
                yorg  -= sizeinc;
                xlast -= sizeinc;
                ylast += sizeinc;
		          break;
          } // end switch keydown

          if (angle >  360) angle -=  360;
          if (angle < -360) angle +=  360;

          //draw
          DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // new position
          SetCursorPos(SCREENX(xlast), SCREENY(ylast)+ gd.CaptionHeight);
          // eat data now
          GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
          break;
      }


	   // user just changed it , save for exit so it stays the same

	   if ( priorscale != GridScale) oldscale = GridScale;
	   if ( priorsnap  != GridSnap ) oldsnap  = GridSnap ;

  	   if (MouseButton == 1) break;           // set by 0x000d

      //////////////////////////////////////////
	   // if 2nd mouse pressed, drag rectangle //
      //////////////////////////////////////////

	   if ( MouseButton & 2)
	   {
           while (MouseButton &2)
           {
               WaitMessage();
               KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
               if (CheckEscape(&msg,KeyData))
               {   ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
                   return;
               }
               if (CheckF1(&msg,KeyData))
               {   DragHelp(hwnd);
                   continue;
               }

               if (msg.message == WM_CHAR
               ||  msg.message == WM_KEYDOWN)
               {
                  //erase
                  DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // Adjust our position

                  ProcessMapSizeCommands(hwnd,level,
                                         msg.message,
                                         KeyData,                 // The Char Key '1' etc.
                                         FALSE, TRUE) ;           // no goto, drag

                  //draw
                  DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // Adjust our position
               }

               if(msg.message == WM_MOUSEMOVE)
               {
                   XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
                   if (!CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE) )
                   {    //erase
                        DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // Old position
                   }

	                dx = xlast - MAPX(PointerX);
	                dy = ylast - MAPY(PointerY);
                   xorg  -=dx;
	                yorg  -=dy;
	                xlast -=dx;
	                ylast -=dy;
                   //draw
                   DrawRect(hdc, xorg, yorg, xlast, ylast, angle); // New position
	            }
           } // end while
	   }   // if (MouseButton == 2)


	   // grab 2 vertexes and see what distance apart they are
	   Ldx = (PolygonX[0] - PolygonX[1]);
	   Ldy = (PolygonY[0] - PolygonY[1]);

	   width =  hypot (Ldx,Ldy);                  // width of a line
	   Ldx   = (PolygonX[1] - PolygonX[2]);
	   Ldy   = (PolygonY[1] - PolygonY[2]);

	   length= hypot (Ldx,Ldy);                   // length of a line

	   sector= GetCurObject(NOCONVERT, OBJ_SECTORS, xorg, yorg, xlast, ylast,THINGNOT);

	   // if width/length or angle, show it

	   if (Redraw)
	   {
            //Redraw  = FALSE;
            Sleep(50);                 // keep flicker down
            SetTextType(hdc, 3);
	         if (sector >=0)
	         {   DrawScreenTextOutline(hdc,10,  1,20,MenuInfoColor," %s Sector %4hd ",
						                    ToolOptions[TypeOfTool].typeofname,sector);
	         }
	         else
	         {   DrawScreenTextOutline(hdc, 10 , 1,20,MenuInfoColor,"     %s Room    ",
						                    ToolOptions[TypeOfTool].typeofname);
	         }

	         DrawScreenTextOutline(hdc,160 ,1,24,MenuInfoColor,"%3hd   L=%-4d W=%-4d Grid=%d ",angle,length,width,GridScale);

	         if (Stype > 0)
	         {   DrawScreenTextOutline(hdc,160,16,24,MenuInfoColor,"     D=%-4hd        ",depth);
	         }
      }
  }   // END while( MouseButton != 1)


  if ( ! PolyVertexVerify(hwnd,level,4,angle, TRUE) )
  {
     ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
     return;
  }

  // final draw
  SelectPen(hdc, gd.hPens[VertexColor]);               // blue
  RectangleDraw(hdc, xorg,yorg, xlast,ylast,angle);    // draw it

  oldsector = GetCurObject(NOCONVERT, OBJ_SECTORS, xorg, yorg, xlast, ylast,THINGNOT);

  if (Stype >= 6 && Stype <= 8 && oldsector < 0)
  {
      HelpMsg(hwnd, CENTER|HELPINFO, "Tool Error",
		   "Teleports and Doors have to be made inside a sector.",
         "",
		   "Please restart and keep the object inside a sector",
		   "when you create it.",
         NULL);
     ToolExit(hwnd, hdc, oldsnap, oldscale, hOldPal, hOldPen, hOldFont);
     return;
  }


  CheckPointLevel(UndoBuffer, level); // ------ CHECKPOINT ---

  for (n = 3; n >=0; n--)
  {    InsertObject(level, OBJ_VERTEXES, -1, PolygonX[n],PolygonY[n]);//xorg , yorg);
  }

  // see if door
  if (Stype == 8 )
  {
      CreateDoor(hwnd, level, oldsector);
  }
  else
  {   // if not in a sectors yet, create a new one

    sector = oldsector;

    if (sector < 0)
    {   InsertObject(level, OBJ_SECTORS, -1, 0, 0);       // create a sector
	     sector = int2(level->NumSectors - 1);             // new sector number
    }

    // create Linedefs/Sidedefs

    for (n = 0; n < 4; n++)
    {
        InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);             // create Linedef
        InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);             // create sidedef (for prior)

        level->LineDefs[ level->NumLineDefs - 1].sidedef1 = int2(level->NumSideDefs-1); // assign sidedef#
        level->SideDefs[ level->NumSideDefs - 1].sector   = sector;        // sector ref
    }

    // if in a sector vertexes sequence for 2nd linedef (point2 out)
    // make an object in a room
    if  (oldsector>=0) //choice == 2)
    {
      level->LineDefs[ level->NumLineDefs - 4].start = int2(level->NumVertexes - 4);
      level->LineDefs[ level->NumLineDefs - 4].end   = int2(level->NumVertexes - 3);

      level->LineDefs[ level->NumLineDefs - 3].start = int2(level->NumVertexes - 3);
      level->LineDefs[ level->NumLineDefs - 3].end   = int2(level->NumVertexes - 2);

      level->LineDefs[ level->NumLineDefs - 2].start = int2(level->NumVertexes - 2);
      level->LineDefs[ level->NumLineDefs - 2].end   = int2(level->NumVertexes - 1);

      level->LineDefs[ level->NumLineDefs - 1].start = int2(level->NumVertexes - 1);
      level->LineDefs[ level->NumLineDefs - 1].end   = int2(level->NumVertexes - 4);
    }      // make a room
    else   // outside a sector vertex sequence for 1st linedef (pointin)
    {
      level->LineDefs[ level->NumLineDefs - 4].start = int2(level->NumVertexes - 1);
      level->LineDefs[ level->NumLineDefs - 4].end   = int2(level->NumVertexes - 2);

      level->LineDefs[ level->NumLineDefs - 3].start = int2(level->NumVertexes - 2);
      level->LineDefs[ level->NumLineDefs - 3].end   = int2(level->NumVertexes - 3);

      level->LineDefs[ level->NumLineDefs - 2].start = int2(level->NumVertexes - 3);
      level->LineDefs[ level->NumLineDefs - 2].end   = int2(level->NumVertexes - 4);

      level->LineDefs[ level->NumLineDefs - 1].start = int2(level->NumVertexes - 4);
      level->LineDefs[ level->NumLineDefs - 1].end   = int2(level->NumVertexes - 1);
    }

    // 1. Select all the linedefs added and make 2-sided with no textures
    // 2. Or make a pool etc as required
    //------------------------------------------------------------------------

    // this gets all the linedefs created

    for (n = savedLnum; n < level->NumLineDefs; n++)
    {    SelectObject( &NewSelected, n);
    }

    if (oldsector >= 0)    // 2-sided
    {
      if (Stype > 0)       // from menu, do only if a special type
      {   if (Stype > 1)  MakePoolPedLight(depth,savedLnum,int2( Stype-1) );
      }
      else
      {
         // Create the new sector for inside

         InsertObject(level, OBJ_SECTORS, oldsector, 0, 0);        // copy existing sector
         newsector = int2(level->NumSectors - 1);

         for (cur = NewSelected; cur; cur = cur->next)
         {
           InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);           //+1 sidedef
           sd2 = int2(level->NumSideDefs-1);

           level->SideDefs[sd2].sector = newsector;               // assign sector

           sd1 = level->LineDefs[ cur->objnum].sidedef1;          // sidedef

           level->LineDefs[ cur->objnum].sidedef2 = sd2;          // new sidedef2
           level->LineDefs[ cur->objnum].flags    = 4;

           strncpy( level->SideDefs[ sd2].tex3, NoTexture, 8);    // normal
           strncpy( level->SideDefs[ sd1].tex3, NoTexture, 8);    // normal
         }
      }
    }
  }  // if SectorType == Door


  // 4-12-96 --- auto fix

  if (Stype <= 0)       // not menu special type
  {   AutoFix(NewSelected,1);
      if (oldsector >= 0)    // 2-sided
      {
         AutoFix(NewSelected,2);
      }
  }

  ForgetSelection( &NewSelected);               // unselect object

  //--------------------------------------------------------------------------

  // only for Sector or LineDefs change the selectlist

  if (EditMode == OBJ_LINEDEFS )
  {   ForgetSelection(&SelectList);

      for (n = savedLnum; n < level->NumLineDefs; n++)
	        SelectObject( &SelectList,(int2)n);
  }
  else
  if (EditMode == OBJ_SECTORS)
  {   ForgetSelection(&SelectList);

      for (n = savedSnum; n < level->NumSectors; n++)
	        SelectObject( &SelectList,(int2)n);
  }

  CalculateMinMax( level);            //   3-1-97
  
} // end for(;;)
}


//--------------------------------------------------------------------------
// DOOR
//--------------------------------------------------------------------------
static void CreateDoor(HWND hwnd, LevelPtr level, int2 sector)
{
  int4   n;
  int4   PriorNumLineDefs = level->NumLineDefs;
  SelPtr NewLines = NULL;

  // create Linedefs/Sidedefs

   for (n = 0; n < 4; n++)
   {
	InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);               // create 1 Linedef
	InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);               // create 1 sidedef (for prior)

	level->LineDefs[ level->NumLineDefs - 1].sidedef1 = int2(level->NumSideDefs-1); // assign sidedef#
	level->SideDefs[ level->NumSideDefs - 1].sector   = sector;        // sector ref
   }

   // make an object in a room
   level->LineDefs[ level->NumLineDefs - 4].start = int2(level->NumVertexes - 3);   //  1 in
   level->LineDefs[ level->NumLineDefs - 4].end   = int2(level->NumVertexes - 4);   //

   level->LineDefs[ level->NumLineDefs - 3].start = int2(level->NumVertexes - 3);   //  2 out
   level->LineDefs[ level->NumLineDefs - 3].end   = int2(level->NumVertexes - 2);   //

   level->LineDefs[ level->NumLineDefs - 2].start = int2(level->NumVertexes - 1);   //  3 in
   level->LineDefs[ level->NumLineDefs - 2].end   = int2(level->NumVertexes - 2);   //

   level->LineDefs[ level->NumLineDefs - 1].start = int2(level->NumVertexes - 1);   //  4 out
   level->LineDefs[ level->NumLineDefs - 1].end   = int2(level->NumVertexes - 4);   //

   // selected all the linedefs added for Door routine

   for (n = int2(PriorNumLineDefs); n < level->NumLineDefs; n++)
   {        SelectObject( &NewLines, n);
   }

   MakeDoorFromLineDefs(hwnd, NewLines);

   ForgetSelection( &NewLines);               // unselect object

   CalculateMinMax( level);            //   3-1-97
}


////////////////////////   POLYGON   TOOL     //////////////////////////////////
//                                                                            //



//----------------------------------------------------------------------------------
//  draw the actual polygon on screen
//----------------------------------------------------------------------------------
static void  DrawPolygon(HDC hdc, int4  centerx, int4 centery,
                                  int2 numsides,int2 radius, int2 anglemore )
{

   int4    n;
   double angle;
   int4   Xwork,Ywork;

   double dx,dy;

   int4   Ldx,Ldy;

   // radians to degrees = 180*radians/PI = 57.2957795

//--------------------------------------------------------------------------------


   // first create without snapping any of them

   for (n = 0; n < numsides; n++)
   {
       angle = ( 6.2832 * (double) n ) / (double) numsides ;  // angle between each x was 6.28 only

       Xwork =  (centerx + (int) ( (double) radius * cos( angle) ));
       Ywork =  (centery + (int) ( (double) radius * sin( angle) ));

       // rotated position
       if (anglemore )
       {
          dx   =  Xwork - centerx;       // x offset from center
          dy   =  Ywork - centery;       // y offset from center

          RotateAndScaleCoords( &dx, &dy, -anglemore, 1.0);

          Xwork = ( centerx + dx);// + 4) & ~7;  12-11-94
          Ywork = ( centery + dy);// + 4) & ~7;

       }

       PolygonX[n] =Xwork;
       PolygonY[n] =Ywork;
   }

   // grab 2 vertexes and see what distance apart they are

   Ldx = (PolygonX[0] - PolygonX[1]);
   Ldy = (PolygonY[0] - PolygonY[1]);

   PolyLength  = hypot (Ldx,Ldy);             // length of a line

   if (PolyLength >= GridScale/2              // exceeds grid size
    && GridScale
    && GridSnap)
   {
       for (n = 0; n < numsides; n++)
       {  // original coordinates
          Xwork = PolygonX[n];
          Ywork = PolygonY[n];

          if (Xwork < 0)
          {   Xwork= ((Xwork - GridScale/2)/GridScale * GridScale);
          }
          else
          {   Xwork= ((Xwork + GridScale/2)/GridScale * GridScale);
          }
          if (Ywork < 0)
          {   Ywork= ((Ywork - GridScale/2)/GridScale * GridScale);
          }
          else
          {   Ywork= ((Ywork + GridScale/2)/GridScale * GridScale);
          }

          // see at end for old code

          PolygonX[n] =  Xwork;
          PolygonY[n] =  Ywork;
       }
   }

   // last line for drawing routing next (line thickness always thin)

   PolygonX[numsides] = PolygonX[0];
   PolygonY[numsides] = PolygonY[0];

   for (n = 0; n < numsides; n++)
   {
        MoveToEx(hdc,  SCREENX( PolygonX[n] ),  SCREENY( PolygonY[n] ), NULL);
        LineTo  (hdc,  SCREENX( PolygonX[n+1]), SCREENY( PolygonY[n+1]) );

   }

   Ldx = (PolygonX[0] - PolygonX[1]);
   Ldy = (PolygonY[0] - PolygonY[1]);

   PolyLength  = hypot (Ldx,Ldy);              // length of a line

}

//////////////////////////////////////////////////////////////////////////////
// Called From PolygonTool, Calls DrawPolygon above with proper numbers     //
//////////////////////////////////////////////////////////////////////////////
void DrawPoly(HDC hdc, int4 centerx, int4 centery, int2 numsides, int2 radius, int2 angle)
{
    ShowCursor(FALSE);
	 SetROP2(hdc, R2_XORPEN);                   // write mode xor
    SelectPen(hdc, gd.hDrawPen);
    DrawPolygon(hdc, centerx, centery, numsides,radius, angle );  // draw
	 SetROP2(hdc, R2_COPYPEN);
    ShowCursor(TRUE);
}
//---------------------------------------------------------------------------------
// draw the Polygon box and stay here until mouse  click
//---------------------------------------------------------------------------------
#pragma argsused
void PolygonTool(HWND hwnd, LevelPtr level, int2  Stype,
                 int2 *Snumsides, int2 *Sradius, int2 *Sdepth )

{
  int4 n;

  int2   sector;
  int2   oldsector;

  int2   TypeOfTool;

  int2   radiusinc =8;
  int2   angleinc  =1;

  // polygon--------------//
  int2   numsides = 5;
  int2   depth    = 0;      // pool
  int2   radius   = 128;
  int2   radiusmore;
  int2   angle    = 0;      // initial 45degree adjust so square is square
  int4   centerx    ,centery;

  SelPtr NewSelected=NULL;
  SelPtr cur;
  int4   savedSnum;
  int4   savedLnum;
  int2   sd1,sd2;
  int2   newsector;
  int4   xnew, ynew, xold, yold;

  Bool     FirstTime = TRUE;
  Bool     Redraw;
  Bool     processCmd;

  MSG      msg;
  HDC      hdc;
  HPALETTE hOldPal;
//  int      OldRop;
  HPEN     hOldPen;
  HFONT    hOldFont;

  UINT     KeyData;

//-----------------------------------------------------------------------------

 hdc = GetDC(hwnd);

 hOldPal  = SelectPalette(hdc, gd.hDoomPal, FALSE);
 RealizePalette(hdc);

 SetROP2(hdc, R2_XORPEN);  //OldRop
 hOldPen  = SelectPen(hdc, GetStockPen(WHITE_PEN));
 hOldFont = SelectFont(hdc, gd.hSmallFont);

 SetCapture(hwnd);                           // Capture mouse input

 ResetMapResident(hwnd, FALSE, TRUE, TRUE);// Pics, Object, List

 Instructions("Polygon Tool  +-=Zoom, <>=rotate, Up/Down=size, Right/Left=sides, F1=Help");

 for (;;)
 {
    if(!ToolContinue && !FirstTime)
    {    ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
	      return;
    }

    MouseButton = 0;

    SetTextType(hdc, 5);

    //Redraw   = FALSE;
    FirstTime = FALSE;
    MouseMove();                     // set coordinates

    while (MouseButton != 1)
    {
       WaitMessage();
       KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);

       if (CheckEscape(&msg,KeyData))
       {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
           return;
       }
       if (CheckF1(&msg,KeyData))
       {   PolyHelp(hwnd);
           continue;
       }

       if (MouseButton == 1) break;

       if (MouseButton & 2 )
       {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
		     return;
       }

       if (msg.message == WM_MOUSEMOVE)
       {   CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE);
           XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
       }

    }

    TypeOfTool = Stype;

    if (Stype > 0)
    {   TypeOfTool--;
        numsides   = *Snumsides;
        radius     = *Sradius;
        depth      = *Sdepth;
    }

    // center of polygon
    centerx = MAPX(PointerX);
    centery = MAPY(PointerY);

    xnew = xold = PointerX;
    ynew = yold = PointerY;

    // Move the cursor to the correct position at the width/height

    SetCursorPos( SCREENX(centerx), ( (SCREENY(centery))+ gd.CaptionHeight ) );
    //draw
    DrawPoly(hdc, centerx, centery, numsides,radius, angle );  // draw

    savedLnum = level->NumLineDefs;
    savedSnum = level->NumSectors;

    // SetCursorPos generates WM_MOUSEMOVE
    // make sure button is gone

    while (MouseButton & 1)
    {     WaitMessage();
          GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
          XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
    }

    while ( MouseButton != 1 )
    {
       WaitMessage();
       KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
       Redraw  = TRUE;  //   3-1-97

       if (CheckEscape(&msg,KeyData))
       {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
		     return;
       }
       if (CheckF1(&msg,KeyData))
       {  PolyHelp(hwnd);
          continue;
       }

       radiusmore = 0;             // for fall thru on any key pressed

       switch(msg.message)
       {

        case(WM_MOUSEMOVE):
            XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList
            //erase
            DrawPoly(hdc, centerx, centery, numsides,radius, angle );

            if (CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE))
            {   xold = PointerX;
                yold = PointerY;
                centerx = MAPX(PointerX);
                centery = MAPY(PointerY);
            }

            xnew   = PointerX;
            ynew   = PointerY;
            Redraw = TRUE;
            //draw now so next erase is ok
            DrawPoly(hdc, centerx, centery, numsides,radius, angle );  // draw
            break;

        case(WM_KEYDOWN):
            //erase
            DrawPoly(hdc, centerx, centery, numsides,radius, angle );

            ProcessMapSizeCommands(hwnd,level,
                                  msg.message,
                                  KeyData,            // The Char Key '1' etc.
                                  FALSE, FALSE);      // no goto, nodrag   8-1-97

            Redraw = TRUE;

            switch(KeyData)
            {
              case VK_RIGHT:
		          numsides++;
		          if (numsides > 200) numsides= 200;
                break;
              case VK_LEFT:
		          numsides--;
		          if (numsides <  3) numsides = 3;
                break;
              case VK_UP:                        //   +  case 0x00BB:
		          radiusmore = radiusinc;
		          break;
              case VK_DOWN:                      //   -  case 0x00BD
		          radiusmore = -radiusinc;
		          break;
            }
            //draw
            DrawPoly(hdc, centerx, centery, numsides,radius, angle );
            break;

        case(WM_CHAR):
            //erase
            DrawPoly(hdc, centerx, centery, numsides,radius, angle );

            Redraw = TRUE;
            //radiusmore = 0;             // for fall thru on any key pressed
            processCmd = FALSE;

            switch(KeyData)
            {
              case VK_RETURN:
                MouseButton = 1;
		          break;
              case VK_SPACE:
		          if (radiusinc==1 )  radiusinc= 8 ;
		          else                radiusinc= 1;

		          if (angleinc ==1  ) angleinc = 6 ;
		          else                angleinc = 1;
		          break;
              case ('>'):     //   >
              case ('.'):
                angle +=angleinc;
		          break;
              case ('<'):     //  <
              case (','):
                angle -=angleinc;
		          break;
              case('+'):
              case('='):
                if (!gd.ShiftDown)
          		 {   radiusmore =+radiusinc;
                }
                else
                {   processCmd= TRUE;
                }
                break;
              case('-'):
              case('_'):
                if (!gd.ShiftDown)
          		 {   radiusmore =-radiusinc;
                }
                else
                {   processCmd= TRUE;
                }
                break;
              default:
                processCmd = TRUE;
                break;
            }

            if (processCmd)
            {
               ProcessMapSizeCommands(hwnd,level,
                                      msg.message,
                                      KeyData,
                                      FALSE, FALSE);  // no goto, nodrag   8-1-97
            }

	         if (angle >  360) angle -=  int2(360);
            if (angle < -360) angle +=  int2(360);

            //draw
            DrawPoly(hdc, centerx, centery, numsides,radius, angle );

       }  // End basic switch


        if     (xnew > xold)    radiusmore =  radiusinc;
        else if(xnew < xold)    radiusmore = -radiusinc;
        else if(ynew > yold)    radiusmore =  radiusinc;
        else if(ynew < yold)    radiusmore = -radiusinc;

        xold = xnew;
        yold = ynew;

  	     if (MouseButton == 1) break;           // set by 0x000d

        //////////////////////////////////////////
	     // if 2nd mouse pressed, drag Polygon   //
        //////////////////////////////////////////
        if ( MouseButton & 2 )
        {
            while( MouseButton & 2 )
            {
               WaitMessage();
               KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
               if(CheckEscape(&msg,KeyData))
               {  ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
		            return;
               }
               if (CheckF1(&msg,KeyData))
               {   DragHelp(hwnd);
                   continue;
               }

               if (msg.message == WM_CHAR
               ||  msg.message == WM_KEYDOWN)
               {
                  //erase
                  DrawPoly(hdc, centerx, centery, numsides,radius, angle );
                  ProcessMapSizeCommands(hwnd,level,
                                         msg.message,
                                         KeyData,                        // The Char Key '1' etc.
                                         FALSE, FALSE);                  // no goto, nodrag   8-1-97
                  //draw
                  DrawPoly(hdc, centerx, centery, numsides,radius, angle );
               }

               if (msg.message == WM_MOUSEMOVE)
               {
                   XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
                   if (!CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE) )
                   {   //erase
                       DrawPoly(hdc, centerx, centery, numsides,radius, angle );  // draw
                   }
	                centerx = MAPX(PointerX);
	                centery = MAPY(PointerY);
                   //draw
                   DrawPoly(hdc, centerx, centery, numsides,radius, angle );

                   xnew = PointerX;
                   ynew = PointerY;
                   radiusmore = 0;
	            }
            } //end while
	     }  // END if (MouseButton == 2)


	     // if width/length or angle, show it
	     if (Redraw)
	     {
            //Redraw = FALSE;
            sector = GetCurObject(NOCONVERT, OBJ_SECTORS, centerx,centery, centerx,centery,THINGNOT);
            Sleep(50);

            //erase
            DrawPoly(hdc, centerx, centery, numsides,radius,angle );

	         radius  +=radiusmore;
	         if (radius < 4 ) radius = 4;

            SetTextType(hdc, 3);
	         if (sector >=0)                        //12345678
	         {   DrawScreenTextOutline(hdc, 10, 1,20,MenuInfoColor,"%s Object Sector %4hd",
						                    ToolOptions[TypeOfTool].typeofname,sector);
	         }
	         else
	         {   DrawScreenTextOutline(hdc, 10, 1,20,MenuInfoColor,"       %s Room       ",
						                    ToolOptions[TypeOfTool].typeofname);
	         }

	         DrawScreenTextOutline(hdc, 160 ,1,24,MenuInfoColor,"#%hd %3hd   L=%-4hd R%-4hd ",
						                numsides,angle,PolyLength,radius);

	         if (Stype > 0)
	         {  DrawScreenTextOutline(hdc, 160, 16,24,MenuInfoColor,"        D=%-4hd       ",depth);
	         }

            //draw
            DrawPoly(hdc, centerx,centery, numsides,radius, angle );
        }
    } // END while( MouseButton != 1)


    if (!PolyVertexVerify(hwnd, level, numsides,angle, FALSE) )
    {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
        return;
    }

    //final draw
    SelectPen(hdc, gd.hPens[VertexColor]);                       // blue
    DrawPolygon(hdc, centerx,centery, numsides,radius, angle );  // draw

    CheckPointLevel(UndoBuffer, level); // ------ CHECKPOINT ---

    oldsector = GetCurObject(NOCONVERT, OBJ_SECTORS,centerx,centery, centerx,centery,THINGNOT);

    // create vertices

    for (n=0; n<numsides; n++)
    {
       InsertObject(level, OBJ_VERTEXES, -1, PolygonX[n],PolygonY[n] );
    }

    // create sector if an object
    sector = oldsector;

    if (sector < 0)
    {  InsertObject(level, OBJ_SECTORS, -1, 0, 0);
       sector = int2(level->NumSectors - 1);
    }


    for (n = 0; n < numsides; n++)
    {
       InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);     // linedef
       InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);     // sidedef1

       level->LineDefs[ level->NumLineDefs - 1].sidedef1 = int2(level->NumSideDefs-1);
       level->SideDefs[ level->NumSideDefs - 1].sector   = sector;
    }

    if (oldsector >= 0) //choice == 4)
    {
      level->LineDefs[ level->NumLineDefs - 1].start = int2(level->NumVertexes - 1);
      level->LineDefs[ level->NumLineDefs - 1].end   = int2(level->NumVertexes - numsides);

      for (n = 2; n <= numsides; n++)
      {
	   level->LineDefs[ level->NumLineDefs - n].start = int2(level->NumVertexes - n);
	   level->LineDefs[ level->NumLineDefs - n].end   = int2(level->NumVertexes - n + 1);
      }
    }
    else
    {
      level->LineDefs[ level->NumLineDefs - 1].start = int2(level->NumVertexes - numsides);
      level->LineDefs[ level->NumLineDefs - 1].end   = int2(level->NumVertexes - 1);

      for (n = 2; n <= numsides; n++)
      {
          level->LineDefs[ level->NumLineDefs - n].start = int2(level->NumVertexes - n + 1);
          level->LineDefs[ level->NumLineDefs - n].end   = int2(level->NumVertexes - n);
      }
    }

    // selected all the linedefs added and make 2-sided with no textures
    // this gets all the linedefs created
    for (n = int2(savedLnum); n < level->NumLineDefs; n++)
    {    SelectObject( &NewSelected, n);
    }

    if (oldsector >= 0)    // 2-sided
    {
      if (Stype > 0)       // from menu, do only if a special type
      {   if (Stype > 1)  MakePoolPedLight(depth,int2(savedLnum),int2(Stype-1) );
      }
      else
      {
        // Create the new sector for inside

        InsertObject(level, OBJ_SECTORS, oldsector, 0, 0);      // copy existing sector
        newsector = int2(level->NumSectors - 1);

        for (cur = NewSelected; cur; cur = cur->next)
        {
          InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);           //+1 sidedef
          sd2 = int2(level->NumSideDefs-1);

          level->SideDefs[sd2].sector = newsector;               // assign sector

          sd1 = level->LineDefs[ cur->objnum].sidedef1;          // sidedef

          level->LineDefs[ cur->objnum].sidedef2 = sd2;          // new sidedef2
          level->LineDefs[ cur->objnum].flags    = 4;

          strncpy( level->SideDefs[ sd2].tex3, NoTexture, 8);    // normal
          strncpy( level->SideDefs[ sd1].tex3, NoTexture, 8);    // normal
        }

      }
    }

    // 4-12-96 --- auto fix

    if (Stype <= 0)       // not menu special type
    {   AutoFix(NewSelected,1);
        if (oldsector >= 0)    // 2-sided
        {
           AutoFix(NewSelected,2);
        }
    }

    ForgetSelection( &NewSelected);               // unselect object

    // only for Sector or LineDefs change the selectlist

    if (EditMode == OBJ_LINEDEFS )
    {   ForgetSelection(&SelectList);
        for (n = savedLnum; n < level->NumLineDefs; n++)
        {    SelectObject( &SelectList, n);
        }
    }
    else
    if (EditMode == OBJ_SECTORS)
    {   ForgetSelection(&SelectList);

        for (n = savedSnum; n < level->NumSectors; n++)
        {    SelectObject( &SelectList, n);
        }
    }

    CalculateMinMax( level);            //   3-1-97
  } // end for(;;)
}



////////////////////////     STAIR     TOOL     ////////////////////////////////
//                                                                            //

//-----------------------------------------------------------------------------
// create vertices for stairs Rotating
//-----------------------------------------------------------------------------
void DrawTheStairs( HDC hdc,
                 int2 centerx,int2 centery,int2 stepnum,
					  int2 stepwidth,
					  int2 stepdepth,
					  int2 radius,
					  int2 stepradinc,
					  int2 rotateangle,
					  int2 stepangle )

{
  int4   i,n;
  double astep,astart;
  int4   x1,y1,x2,y2;
  int4   Ldx,Ldy;

  int4   Xwork,Ywork;

  double dx,dy;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

  astart = 0.0;
  astep  = (double) stepangle * .01743292 / (double) stepnum;

  n = 0; //2

  for (i = 0; i < stepnum+2; i++)
  {  // create the next 2 vertexes

     x1 = (centerx + (int)(cos(astart) * (double) radius ));
     y1 = (centery + (int)(sin(astart) * (double) radius ));

     x2 = (centerx + (int)(cos(astart) * (double) (radius+stepwidth) ));
     y2 = (centery + (int)(sin(astart) * (double) (radius+stepwidth) ));

     if (rotateangle)
     {
         dx   =  x1 - centerx;       // x offset from center
         dy   =  y1 - centery;       // y offset from center

         RotateAndScaleCoords( &dx, &dy, -rotateangle, 1.0);

         x1 = centerx + dx;
         y1 = centery + dy;

         dx =  x2 - centerx;        // x offset from center
         dy =  y2 - centery;        // y offset from center


         RotateAndScaleCoords( &dx, &dy, -rotateangle, 1.0);

         x2 =((int)( centerx + dx));// + 4) & ~7;  12-11-94
         y2 =((int)( centery + dy));// + 4) & ~7;

     }

     astart  += astep;
     radius  += stepradinc;
     centery += stepdepth;

     PolygonX[n] = (x1);
     PolygonY[n] = (y1);   //
     n++;
     PolygonX[n] = (x2);
     PolygonY[n] = (y2);   //
     n++;
  }

   Ldx = (PolygonX[0] - PolygonX[1]);
   Ldy = (PolygonY[0] - PolygonY[1]);
   PolyLength  = hypot (Ldx,Ldy);             // length of a line

   if (PolyLength >= GridScale/2              // exceeds grid size
    && GridScale
    && GridSnap)
   {   for (n = 0; n < stepnum*2+2; n++)
       {
            Xwork = PolygonX[n];
            Ywork = PolygonY[n];

            if (Xwork < 0)
            {   Xwork= ((Xwork - GridScale/2)/GridScale * GridScale);
            }
            else
            {   Xwork= ((Xwork + GridScale/2)/GridScale * GridScale);
            }
            if (Ywork < 0)
            {   Ywork= ((Ywork - GridScale/2)/GridScale * GridScale);
            }
            else
            {   Ywork= ((Ywork + GridScale/2)/GridScale * GridScale);
            }

            // see at end for old code

            PolygonX[n] =  (Xwork);
            PolygonY[n] =  (Ywork);
       }
   }


   // last line for drawing routing next

   PolygonX[stepnum*2+3] = PolygonX[0];
   PolygonY[stepnum*2+4] = PolygonY[1];

   // draw the bottom step as thick dashed

   //setlinestyle(USERBIT_LINE, 0x0F0F, 1);    // dotted line for the bottom

   MoveToEx(hdc,  SCREENX( PolygonX[0] ), SCREENY( PolygonY[0] ), NULL);    // 0
	LineTo  (hdc,  SCREENX( PolygonX[1] ), SCREENY( PolygonY[1] ));   // 1

   MoveToEx(hdc,  SCREENX( PolygonX[0] ), SCREENY( PolygonY[0] ), NULL);    // 0
	LineTo  (hdc,  SCREENX( PolygonX[2] ), SCREENY( PolygonY[2] ));   // 2

   MoveToEx(hdc,  SCREENX( PolygonX[1] ), SCREENY( PolygonY[1] ), NULL);    // 1
	LineTo  (hdc,  SCREENX( PolygonX[3] ), SCREENY( PolygonY[3] ));   // 3

   //setlinestyle(SOLID_LINE,  NULL, 1);    // solid line

   n = 2;
   for (i = 2; i < stepnum+1; i++)
   {
        MoveToEx(hdc,  SCREENX( PolygonX[n] ),   SCREENY( PolygonY[n] ), NULL);      // 0
        LineTo  (hdc,  SCREENX( PolygonX[n+1] ), SCREENY( PolygonY[n+1] ));   // 1

        MoveToEx(hdc,  SCREENX( PolygonX[n] ),   SCREENY( PolygonY[n] ), NULL);      // 0
        LineTo  (hdc,  SCREENX( PolygonX[n+2] ), SCREENY( PolygonY[n+2] ));   // 2

        MoveToEx(hdc,  SCREENX( PolygonX[n+1] ), SCREENY( PolygonY[n+1] ), NULL);    // 1
        LineTo  (hdc,  SCREENX( PolygonX[n+3] ), SCREENY( PolygonY[n+3] ));   // 3
        n +=2;
   }

   MoveToEx(hdc,  SCREENX( PolygonX[n] ),   SCREENY( PolygonY[n] ), NULL);      // 0
	LineTo  (hdc,  SCREENX( PolygonX[n+1] ), SCREENY( PolygonY[n+1] ));   // 1

   Ldx = (PolygonX[0] - PolygonX[1]);
   Ldy = (PolygonY[0] - PolygonY[1]);

   PolyLength  = hypot (Ldx,Ldy);              // length of a line

}

//////////////////////////////////////////////////////////////////////////////

void DrawStairs( HDC hdc, int2 centerx,int2 centery,int2 stepnum, int2 stepwidth,
					  int2 stepdepth, int2 radius, int2 stepradinc,int2 rotateangle, int2 angle )
{
    ShowCursor(FALSE);
	 SetROP2(hdc, R2_XORPEN);                   // write mode xor
    SelectPen(hdc, gd.hDrawPen); // can be bigger line width etc.
	 DrawTheStairs(hdc, centerx, centery,stepnum,stepwidth,stepdepth,
	                 radius, stepradinc, rotateangle, angle );
	 SetROP2(hdc, R2_COPYPEN);
    ShowCursor(TRUE);

    //Trace(gd.hMainWindow, 100, 6,
    //"COUNT = %d, CX=%d, CY=%d, num=%d, width=%d, depth=%d, rad=%d, inc=%d, roang=%d, ang=%d",
    //cnt, centerx, centery,stepnum,stepwidth,stepdepth, radius, stepradinc, rotateangle, angle);
}

//---------------------------------------------------------------------------------
// draw the STAIRS and stay here until mouse  click
//---------------------------------------------------------------------------------

#pragma argsused
void StairTool(HWND hwnd, LevelPtr level,int2 Stype,
	       int2 * Sfloorh,   int2 * Sceilh,
	       int2 * Sstepnum,  int2 * Sstepheight,
	       int2 * Sstepwidth,int2 * Sstepdepth)
{

  int4  stairtype;              // 2-1-97 bk Move here from global
  int4  i,l,n;
  int4  xnew,ynew, xold, yold;

  int2  radius   ;
  int2  radiusmore;
  int2  angle    ;      // initial 45degree adjust so square is square
  int2  centerx,centery;

  int4  savedSnum;
  int4  savedLnum;

  int2  startingsector;  // sector when we started
  int2  oldsector;       // sector where we are now
  int2  newsector;
  int2  sector;         // sector bottom linedef sd1 of step references
  int2  stepsector;     // sector of step
  int2  sidesector;     // sector for the sides of step

    // stairs
       int2 floorh = 0;     // depends on sector
       int2 ceilh  = 1024;

static int2 stepnum   =  8;  //6
static int2 stepheight= 16;
static int2 stepwidth = 128;
static int2 stepdepth =   0; //32;
    // extra for rotating stair
static int2 stepangle = 270; //300
static int2 stepradius=  64; //128
static int2 stepradinc=  14; //32
// old values
static int2 curvestepangle = 270; //300
static int2 curvestepradius=  64; //128
static int2 curvestepradinc=  14; //32

  int2     radiusinc  = 8;
  int2     angleinc   = 6;
  int2     rotateangle= 0;

  char     szmsg[80];

  Bool     FirstTime = TRUE;
  Bool     Redraw;
  Bool     processCmd;

  MSG      msg;
  HDC      hdc;
  HPALETTE hOldPal;
//  int      OldRop;
  HPEN     hOldPen;
  HFONT    hOldFont;
  UINT     KeyData;

//-----------------------------------------------------------------------------

 hdc = GetDC(hwnd);

 hOldPal  = SelectPalette(hdc, gd.hDoomPal, FALSE);
 RealizePalette(hdc);

 SetROP2(hdc, R2_COPYPEN);   //OldRop
 hOldPen  = SelectPen(hdc, GetStockPen(WHITE_PEN));
 hOldFont = SelectFont(hdc, gd.hSmallFont);

 SetCapture(hwnd);                           // Capture mouse input

 ResetMapResident(hwnd, FALSE, TRUE, TRUE);// Pics, Object, List

 Instructions("Stair Tool  +/-=Zoom, Up/Down=size, <>=rotate, PgUp/PgDn=width, F1=Help, ");

for (;;)
{
   if(!ToolContinue && !FirstTime)
   {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
       return;
   }


   MouseButton = 0;  // From Ctrl - A

   //Redraw = FALSE;
   FirstTime = FALSE;
   MouseMove();

   centerx = MAPX(PointerX);
   centery = MAPY(PointerY);

   if (Stype > 0)           // entry from object menu, these are the limits
   {
       floorh     = *Sfloorh;
       ceilh      = *Sceilh;
       stepnum    = *Sstepnum;
       stepheight = *Sstepheight;
       stepwidth  = *Sstepwidth;
       stepdepth  = *Sstepdepth;
   }


   startingsector = GetCurObject(NOCONVERT, OBJ_SECTORS,
                                 centerx,centery, centerx,centery,THINGNOT);

   // don't override what the user gave, although sector could move

   if (startingsector >=0 && !Stype)
   {
       ceilh  = level->Sectors[startingsector].ceilh;
       floorh = level->Sectors[startingsector].floorh;

   }

   //-------------------------------
   // first get the type of stair
   //-------------------------------

   stairtype = DialogBox( gd.hInst, "StairType", hwnd, StairTypeDlgProc );

   if(stairtype == psh3) // cancel
   {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
       return;
   }

   // curved
   if (stairtype == psh1)
   {    stepangle = 0;
        stepradius= 0;
        stepradinc= 0;
   }
   else
   // straight
   {
       stepangle  = curvestepangle;
       stepradius = curvestepradius;
       stepradinc = curvestepradinc;

       if (!Input3Numbers(hwnd, "Curve (0's for straight)",// msg,
			       "Angle ",       0, 720,&stepangle,
			       "Radius",       0,4096,&stepradius,
			       "Swirl",     -128, 128,&stepradinc))
       {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
		     return;
       }
       // save for next time
       curvestepangle  = stepangle;
       curvestepradius = stepradius;
       curvestepradinc = stepradinc;
   }

   radius = stepradius;
   angle  = stepangle;

   if (angle == 0 && radius == 0 && stepdepth < 1) stepdepth = 16;

   savedLnum = level->NumLineDefs;
   savedSnum = level->NumSectors;

   // Wait for Left Button ------------------------------------------------

   while (MouseButton != 1)         // Wait for L-BUTTON DOWN
   {
      WaitMessage();
      KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);

      if ( CheckEscape(&msg,KeyData) )
      {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
          return;
      }
      if (CheckF1(&msg,KeyData))
      {   PolyHelp(hwnd);
          continue;
      }

      if (MouseButton == 1) break;

      if (MouseButton & 2 )
      {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
		    return;
      }

      if (msg.message == WM_MOUSEMOVE)
      {   CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE);  //was comment   11-7-96
          XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
      }
   }

   //================

   // Move the cursor to the correct position

   SetCursorPos( PointerX, PointerY + gd.CaptionHeight );
   gd.CaptionHeight = GetSystemMetrics(SM_CYMENU)+GetSystemMetrics(SM_CYCAPTION);

   // now where are we

   centerx = MAPX(PointerX);
   centery = MAPY(PointerY);

   xnew = xold = PointerX;
   ynew = yold = PointerY;

   startingsector = GetCurObject(NOCONVERT, OBJ_SECTORS,
                                 centerx,centery, centerx,centery,THINGNOT);

   // Draw Initial Location of stairs on screen

   DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
                   radius, stepradinc, rotateangle, angle );

   // make sure button is gone

   while (MouseButton & 1)
   {     WaitMessage();
         GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
   }


   while ( MouseButton != 1 )
   {
         WaitMessage();
         KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
         Redraw = TRUE;

         if (CheckEscape(&msg,KeyData))
         {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
		       return;
         }
         if (CheckF1(&msg,KeyData))
         {   StairHelp(hwnd);
             continue;
         }

         radiusmore = 0;  // for fall thru on no valid keystroke

         switch(msg.message)
         {
           case(WM_MOUSEMOVE):
             XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList
             //erase
	          DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                 radius, stepradinc, rotateangle, angle );

             if (CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE))
             {   xold = PointerX;
                 yold = PointerY;
                 centerx = MAPX(PointerX);
                 centery = MAPY(PointerY);
             }

             xnew   = PointerX;
             ynew   = PointerY;
             Redraw = TRUE;
             //draw now so next erase is ok
	          DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                 radius, stepradinc, rotateangle, angle );
             break;

           case(WM_KEYDOWN):
             //erase
	          DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                 radius, stepradinc, rotateangle, angle );

             ProcessMapSizeCommands(hwnd,level,
                                  msg.message,
                                  KeyData,            // The Char Key '1' etc.
                                  FALSE, FALSE);      // no goto, nodrag   8-1-97

             Redraw = TRUE;

             switch(KeyData)
             {
               case VK_PRIOR:           //pgup
 		           stepwidth++;
                 break;
               case VK_NEXT:            //pgdown
		           stepwidth--;
		           if (stepwidth < 32 ) stepwidth = 32;
                 break;
               case VK_INSERT:          //ins
 		           stepdepth++;
                 break;
               case VK_DELETE:          //del
		           stepdepth--;
		           if (stepdepth < 0) stepdepth = 0;
                 break;
               case VK_HOME:            //home
		           stepheight++;
		           if (stepheight > 24) stepheight = 24;
                 break;
               case VK_END:             //end
		           stepheight--;
		           if (stepheight < 0) stepheight = 0;
                 break;
               case VK_RIGHT:           //right cursor
		           stepradinc++;
                 break;
               case VK_LEFT:            //left cursor
		           stepradinc--;
		           if (stepradinc <0 ) stepradinc = 0;
                 break;
               case VK_UP:              //up cursor
		           stepnum++;
		           if (stepnum > 200 ) stepnum= 200;    //200
                 break;
               case VK_DOWN:            //down cursor
		           stepnum--;
		           if (stepnum <  2) stepnum = 2;
                 break;

	            // rotate stairs
	            case VK_BACK:           // back  Space
		           rotateangle -=angleinc;
		           break;
	            case VK_TAB:            // forwardtab
		           rotateangle +=angleinc;
		           break;
             }

             //   3-1-97
             while ( (stepnum * stepheight) > (ceilh - floorh) )
	          {    if (stepnum > 2)       // reduce the steps
		            {   stepnum--;
		            }
		            else
		            {   stepheight--;
		            }

		            Instructions("#Sector height reached , # of steps adjusted");
	               SoundError();
	          }
             //draw
	          DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                 radius, stepradinc, rotateangle, angle );
             break;
           // character data
           case(WM_CHAR):
             //erase
	          DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                 radius, stepradinc, rotateangle, angle );

             Redraw = TRUE;
             processCmd = FALSE;

             switch(KeyData)
             {
                case VK_RETURN:
                  MouseButton = 1;
		            break;
                case VK_SPACE:
		            if (radiusinc==1 )  radiusinc= 8 ;
		            else                radiusinc= 1;

		            if (angleinc ==1  ) angleinc = 6 ;
		            else                angleinc = 1;
		            break;
                case ('>'):     //   >
                case ('.'):
                  angle +=angleinc;
		            break;
                case ('<'):     //  <
                case (','):
                  angle -=angleinc;
	               break;
                case('+'):
                case('='):
                  if (!gd.ShiftDown)
          		   {   radiusmore =+radiusinc;
                  }
                  else
                  {   processCmd= TRUE;
                  }
                  break;
                case('-'):
                case('_'):
                  if (!gd.ShiftDown)
          		   {   radiusmore =-radiusinc;
                  }
                  else
                  {   processCmd= TRUE;
                  }
                  break;
                default:
                  processCmd = TRUE;
                  break;
             }

             if (processCmd)
             {
               ProcessMapSizeCommands(hwnd,level,
                                      msg.message,
                                      KeyData,
                                      FALSE, FALSE);  // no goto, nodrag   8-1-97
             }

	          if (angle >  360) angle -=  360;
             if (angle < -360) angle +=  360;

             //draw
	          DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                 radius, stepradinc, rotateangle, angle );
             break;
         }  // End basic switch msg type


         // straight stairs move around with the mousemove

         if( ((xnew != xold || ynew != yold)
           && (stepangle == 0 && stepradius == 0 && stepradinc == 0) )  )
         {
              //draw
	           DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                       radius, stepradinc, rotateangle, angle );
              // new center
	           centerx = MAPX(PointerX);
	           centery = MAPY(PointerY);
              //draw
	           DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                       radius, stepradinc, rotateangle, angle );

               xnew = PointerX;
               ynew = PointerY;
               radiusmore = 0;
         }
         else  // must be curved stairs
         {  if(xnew > xold)         radiusmore =  radiusinc;
            else if(xnew < xold)    radiusmore = -radiusinc;
            else if(ynew > yold)    radiusmore =  radiusinc;
            else if(ynew < yold)    radiusmore =  -radiusinc;
         }

         xold = xnew;
         yold = ynew;
         /*
         while ( (stepnum * stepheight) > (ceilh - floorh) )
	      {    if (stepnum > 2)       // reduce the steps
		        {   stepnum--;
		        }
		        else
		        {   stepheight--;
		        }

		       Instructions("#Sector height reached , # of steps adjusted");
	          SoundError();
	      }*/

        //////////////////////////////////////////
	     // if 2nd mouse pressed, drag Polygon   //
        //////////////////////////////////////////
	     if (MouseButton & 2)
	     {
            while (MouseButton & 2)         // Wait for R-BUTTON UP
            {
                WaitMessage();
                KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
                Redraw  = TRUE;

                if (CheckEscape(&msg,KeyData))
                {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
                    return;
                }
                if (CheckF1(&msg,KeyData))
                {   DragHelp(hwnd);
                    continue;
                }

                if (msg.message == WM_CHAR
                ||  msg.message == WM_KEYDOWN)
                {
                    //erase
	                 DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                        radius, stepradinc, rotateangle, angle );

                    ProcessMapSizeCommands(hwnd,level,
                                           msg.message,
                                           KeyData,            // The Char Key '1' etc.
                                           FALSE, FALSE);      // no goto, nodrag   8-1-97
                    // draw
	                 DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                        radius, stepradinc, rotateangle, angle );
                }

                if(msg.message == WM_MOUSEMOVE)
                {
                   XYUpdate(hwnd, PointerX,PointerY);                  // update our position   3-1-97
                   if (!CheckMapAndScroll(hwnd, hdc, level, FALSE, FALSE) )
                   {  //erase
	                    DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                      radius, stepradinc, rotateangle, angle );
                   }

                   // new center
	                centerx = MAPX(PointerX);
	                centery = MAPY(PointerY);
                   //draw
	                DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                       radius, stepradinc, rotateangle, angle );

                   xnew = PointerX;
                   ynew = PointerY;
                   radiusmore = 0;
                }
            } //end while button =2
	     }  // END if (MouseButton == 2)

	     // if width/length or angle, show it
	     if (Redraw)
	     {
            //Redraw = FALSE;
            sector = GetCurObject(NOCONVERT, OBJ_SECTORS, centerx,centery, centerx,centery,THINGNOT);
            Sleep(50);
            //erase
	         DrawStairs(hdc, centerx,centery,stepnum,stepwidth,stepdepth,
				                   radius, stepradinc, rotateangle, angle );

	         radius  +=radiusmore;

	         if (radius < 0 ) radius = 0;

            SetTextType(hdc, 3);

	         if (sector >=0)
	         {   DrawScreenTextOutline(hdc, 10, 1,20,MenuInfoColor," Object Stair Sector %4hd",sector);
		          if (stepnum > 16 )
		          {   DrawScreenTextOutline(hdc, 10, 16,20,MenuInfoColor,"  Steps require Testing  ");
		          }
	         }
	         else
	         {   DrawScreenTextOutline(hdc, 10, 1,20,MenuInfoColor,"        Room Stair       ");
	         }

	         DrawScreenTextOutline(hdc,160 , 1,24,MenuInfoColor," #%3hd A%4hd  W=%4hd ",stepnum,   angle,     PolyLength);
	         DrawScreenTextOutline(hdc,160 ,16,24,MenuInfoColor,"      R%4hd  H=%4hd     ",            radius    ,stepheight);
	         DrawScreenTextOutline(hdc,160 ,31,24,MenuInfoColor,"     r+%4hd  D=%4hd     ",            stepradinc,stepdepth );
 	         DrawScreenTextOutline(hdc,160 ,46,24,MenuInfoColor," %4hd  Total H=%4hd ",rotateangle,stepheight*stepnum   );

            //draw
	         DrawStairs(hdc, centerx, centery, stepnum, stepwidth, stepdepth,
                            radius, stepradinc, rotateangle, angle );  // draw
        }
  	     if (MouseButton == 1) break;           // set by 0x000d

  } // END while( MouseButton != 1)


  if (!PolyVertexVerify(hwnd,level, int2( stepnum * 2+2),angle, FALSE) )
  {   ToolExit(hwnd, hdc, GridSnap, GridScale, hOldPal, hOldPen, hOldFont);
      return;
  }

  SelectPen(hdc, gd.hPens[VertexColor]);               // blue
  DrawTheStairs(hdc, centerx, centery,stepnum,stepwidth,stepdepth,
              radius, stepradinc, rotateangle, angle );

  CheckPointLevel(UndoBuffer, level); // ------ CHECKPOINT ---

  oldsector = GetCurObject(NOCONVERT, OBJ_SECTORS,
			                  centerx,centery, centerx,centery,THINGNOT);

  if (oldsector < 0)
  {   wsprintf (szmsg,"Room stair with 1 sided");
  }
  else
  {   wsprintf (szmsg,"Object stair with 2 sided");
  }

  if (oldsector != startingsector)
  {
      HelpMsg(hwnd,CENTER|HELPINFO, "Stair",
          "Stair Sector Changed",
          "",
		    "The sector where you started",
		    "and the current sector are",
		    "not the same!",
          "",
		    "Current sector will be used!",
          "",
		    szmsg,
		    "walls will be created.",
          NULL);
  }

  // created in an existing sector----------------------------------------------

  if (oldsector >=0)
  {
     // build stairs from the bottom up ----------
     // create baseline first

     InsertObject(level, OBJ_VERTEXES, -1, PolygonX[0] ,PolygonY[0]  );    // -4
     InsertObject(level, OBJ_VERTEXES, -1, PolygonX[1] ,PolygonY[1]  );    // -3

     sidesector = oldsector;                          // sector ref for sides
     sector     = oldsector;                          // for first linedef top

     n=2;
     for (i = 0; i < stepnum; i++)
     {  // create the next 2 vertexes

       InsertObject(level, OBJ_VERTEXES, -1,PolygonX[n] ,PolygonY[n]); // -2
       n++;
       InsertObject(level, OBJ_VERTEXES, -1,PolygonX[n] ,PolygonY[n]); // -1
       n++;
       // make a new sector, copy from current

       InsertObject(level, OBJ_SECTORS, -1, 0, 0);              // create step sector
       stepsector = int2(level->NumSectors -1);

       floorh += stepheight;

       level->Sectors[ stepsector].ceilh  = ceilh;                        // ceiling
       level->Sectors[ stepsector].floorh = floorh;                       // floor height
       strncpy(level->Sectors[ stepsector].ceilt,level->Sectors[sector].ceilt ,8); // copy floor ceiling tex
       strncpy(level->Sectors[ stepsector].floort, DefaultStairFloorTx,8);// floor ceiling tex

       for (l = 0; l < 3; l++)
       {
	    InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);        //+1 linedef

	    //  textures SD1
	    InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);        //+1 sidedef1
	    strncpy(level->SideDefs[level->NumSideDefs-1].tex1, NoTexture,8);          // upper
	    strncpy(level->SideDefs[level->NumSideDefs-1].tex3, NoTexture,8);          // normal
	    strncpy(level->SideDefs[level->NumSideDefs-1].tex2, DefaultStairSideTx,8); // lower

	    // textures SD2
	    InsertObject(level, OBJ_SIDEDEFS, -1,0, 0);         //+1 sidedef2
	    strncpy (level->SideDefs[level->NumSideDefs-1].tex3,NoTexture,8);      // normal

	    level->LineDefs[ level->NumLineDefs -1].sidedef1 = int2(level->NumSideDefs-2); //sd1
	    level->LineDefs[ level->NumLineDefs -1].flags = 4;                // 2-sided
	    level->LineDefs[ level->NumLineDefs -1].sidedef2 = int2(level->NumSideDefs-1); //sd2
	    // step side
	    if ( l == 1 )   // 1st bottom linedef
	    {
		 level->SideDefs[ level->NumSideDefs -2].sector = sector;      //sd1
		 level->SideDefs[ level->NumSideDefs -1].sector = stepsector;  //sd2
		 strncpy (level->SideDefs[ level->NumSideDefs -2].tex2,DefaultStairStepTx,8); //steplower
		 sector = stepsector;      // for next top linedef
	    }
	    else // the sides of stairs reference sector we are in
	    {
		 level->SideDefs[ level->NumSideDefs -2].sector = sidesector;   //sd1
		 level->SideDefs[ level->NumSideDefs -1].sector = stepsector;   //sd2
	    }
       }

       level->LineDefs[ level->NumLineDefs -3].start = int2(level->NumVertexes -2);   //leftside
       level->LineDefs[ level->NumLineDefs -3].end   = int2(level->NumVertexes -4);

       level->LineDefs[ level->NumLineDefs -2].start = int2(level->NumVertexes -4);   // bottom
       level->LineDefs[ level->NumLineDefs -2].end   = int2(level->NumVertexes -3);

       level->LineDefs[ level->NumLineDefs -1].start = int2(level->NumVertexes -3);  // rightside
       level->LineDefs[ level->NumLineDefs -1].end   = int2(level->NumVertexes -1);
     }

     // last step

     InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);        //+1 linedef
     level->LineDefs[ level->NumLineDefs -1].start = int2(level->NumVertexes -2);    // top
     level->LineDefs[ level->NumLineDefs -1].end   = int2(level->NumVertexes -1);
     //-------------later add upper stair texture option if ceiling is stepped
     // textures SD1
     InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);        //+1 sidedef1
     strncpy(level->SideDefs[level->NumSideDefs-1].tex1, NoTexture,8);          // upper
     strncpy(level->SideDefs[level->NumSideDefs-1].tex3, NoTexture,8);          // normal
     strncpy(level->SideDefs[level->NumSideDefs-1].tex2, DefaultStairStepTx,8); // steplower

     level->LineDefs[ level->NumLineDefs -1].sidedef1 = int2(level->NumSideDefs-1); //sd1
     level->SideDefs[ level->NumSideDefs -1].sector = stepsector;             //sd1
     level->LineDefs[ level->NumLineDefs -1].flags = 4;  // 2-sided

     // textures SD2
     InsertObject(level, OBJ_SIDEDEFS, -1,0, 0);         //+1 sidedef2
     strncpy(level->SideDefs[level->NumSideDefs-1].tex3, NoTexture,8);           // normal
     strncpy(level->SideDefs[level->NumSideDefs-1].tex2, DefaultLowerTexture,8); // lower

     level->LineDefs[ level->NumLineDefs -1].sidedef2 = int2(level->NumSideDefs-1); //sd2
     level->SideDefs[ level->NumSideDefs -1].sector = oldsector ;   //sd2

  }
  else
  // created in an new sector  ------------------------------------------------
  {
     // build stairs from the bottom up ----------
     // create 2 vertexes were we start (baseline)

     InsertObject(level, OBJ_SECTORS, -1, 0, 0);
     newsector = int2(level->NumSectors - 1);
     sector    = newsector;

     InsertObject(level, OBJ_VERTEXES, -1, PolygonX[0] ,PolygonY[0]  );    // -4
     InsertObject(level, OBJ_VERTEXES, -1, PolygonX[1] ,PolygonY[1]  );    // -3

     n=2;
     for (i = 0; i < stepnum; i++)
     {  // create the next 2 vertexes

       InsertObject(level, OBJ_VERTEXES, -1,PolygonX[n] ,PolygonY[n]); // -2
       n++;
       InsertObject(level, OBJ_VERTEXES, -1,PolygonX[n] ,PolygonY[n]); // -1
       n++;

       // make a new sector, copy from current

       InsertObject(level, OBJ_SECTORS, -1, 0, 0);      // create step sector
       stepsector = int2(level->NumSectors -1);
       sidesector = stepsector;                         // sector ref for sides

       floorh += stepheight;

       level->Sectors[ stepsector].ceilh  = ceilh;                        // ceiling
       level->Sectors[ stepsector].floorh = floorh;                       // floor height
       strncpy(level->Sectors[ stepsector].ceilt,  DefaultStairCeilTx,8); // floor ceiling tex
       strncpy(level->Sectors[ stepsector].floort, DefaultStairFloorTx,8);// floor ceiling tex

       // create 1 step
       for (l = 0; l < 3; l++)
       {
          InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);        //+1 linedef
          // SD1
          InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);        //+1 sidedef1

          level->LineDefs[ level->NumLineDefs -1].sidedef1 = int2(level->NumSideDefs-1); //sd1

          if ( l == 1 )   // 1st bottom linedef
          {  // create SD2
             InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);        //+1 sidedef2

	          level->LineDefs[ level->NumLineDefs -1].sidedef2 = int2(level->NumSideDefs-1); //sd2
	          level->LineDefs[ level->NumLineDefs -1].flags = 4;                // 2-sided
	          // SD1
	          level->SideDefs[ level->NumSideDefs -2].sector =    sector;        //sd1

	          // SD1 overide the textures
	          strncpy(level->SideDefs[level->NumSideDefs-2].tex1, NoTexture,8);          // upper
	          strncpy(level->SideDefs[level->NumSideDefs-2].tex2, DefaultStairStepTx,8); // step lower
	          strncpy(level->SideDefs[level->NumSideDefs-2].tex3, NoTexture,8);          // normal

	          level->SideDefs[ level->NumSideDefs -1].sector = stepsector;    //sd2
	          strncpy(level->SideDefs[level->NumSideDefs-1].tex3,NoTexture,8);      //none normal
	          sector = stepsector;      // for top linedef
	       }
	       else // the sides of stairs reference step sector (pointin)
	       {
	          level->SideDefs[ level->NumSideDefs -1].sector = sidesector;    //sd1
	       }
       }

       // left side
       level->LineDefs[ level->NumLineDefs -3].start = int2(level->NumVertexes -4);  //leftside
       level->LineDefs[ level->NumLineDefs -3].end   = int2(level->NumVertexes -2);
       // bottom
       level->LineDefs[ level->NumLineDefs -2].start = int2(level->NumVertexes -4);  // bottom
       level->LineDefs[ level->NumLineDefs -2].end   = int2(level->NumVertexes -3);
       // right side
       level->LineDefs[ level->NumLineDefs -1].start = int2(level->NumVertexes -1);  // rightside
       level->LineDefs[ level->NumLineDefs -1].end   = int2(level->NumVertexes -3);
       //
     }

     // last step

     InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);        //+1 linedef
     level->LineDefs[ level->NumLineDefs -1].start = int2(level->NumVertexes -2);  // top
     level->LineDefs[ level->NumLineDefs -1].end   = int2(level->NumVertexes -1);

     // textures SD1
     InsertObject(level, OBJ_SIDEDEFS, -1, 0, 0);        //+1 sidedef1
     strncpy(level->SideDefs[level->NumSideDefs-1].tex1, NoTexture,8);          // upper
     strncpy(level->SideDefs[level->NumSideDefs-1].tex3, NoTexture,8);          // normal
     strncpy(level->SideDefs[level->NumSideDefs-1].tex2, DefaultStairStepTx,8); // step lower

     level->LineDefs[ level->NumLineDefs -1].sidedef1 = int2(level->NumSideDefs-1); //sd1
     level->SideDefs[ level->NumSideDefs -1].sector = stepsector;             //sd1
     level->LineDefs[ level->NumLineDefs -1].flags = 4;  // 2-sided

     // textures SD2
     InsertObject(level, OBJ_SIDEDEFS, -1,0, 0);         //+1 sidedef2
     strncpy(level->SideDefs[level->NumSideDefs-1].tex3, NoTexture,8);           // normal
     strncpy(level->SideDefs[level->NumSideDefs-1].tex2, DefaultLowerTexture,8); // lower last one

     level->LineDefs[ level->NumLineDefs -1].sidedef2 = int2(level->NumSideDefs-1); //sd2
     level->SideDefs[ level->NumSideDefs -1].sector = newsector ;             //sd2

  }

  CalculateMinMax( level);                     //   3-1-97

  // only for Sector or LineDefs change the selectlist

  if (EditMode == OBJ_LINEDEFS )
  {   ForgetSelection(&SelectList);
      for (n = int2(savedLnum); n < level->NumLineDefs; n++)
      	   SelectObject( &SelectList, (int2)n);

  }
  else
  if (EditMode == OBJ_SECTORS)
  {   ForgetSelection(&SelectList);

      for (n = savedSnum; n < level->NumSectors; n++)
	        SelectObject( &SelectList,(int2)n);
  }

} // end for(;;)
}



static ImageData DrawingImageBuffer={0};             // for lengths when drawing

/////////////////////////////////////////////////////
// Added wrapper for Hiding the Cursor             //
/////////////////////////////////////////////////////
static void DrawTheLine(HWND hwnd, HDC hdc, int2 mode)
{ //static int i =0;
  //Trace(NULL,10,4,"%d mode =%hd",i++,mode);
  ShowCursor(FALSE);
  DrawingLines(hwnd, hdc, mode);
  ShowCursor(TRUE);
}

//----------------------------------------------------------------------------------
//   line drawing mode
//----------------------------------------------------------------------------------
void Deep_OnKey(HWND hwnd, UINT vk, BOOL fDown, int cRepeat, UINT flags);

void  DrawingTool(HWND hwnd, LevelPtr level)
{
static int2 OldEditMode;            // for linedrawing
  Bool     Continue;
  Bool     MouseProcess; //   3-1-97 reqd with new mouse chk, no move, no flicker
  HDC      hdc;
  MSG      msg;
  RECT     rc;
  HPALETTE hOldPal;
  UINT     KeyData;
  int4     oldPointerX;
  int4     oldPointerY;
//int2     DrawingRange = Range;            // Range to look for object

//----------------------------------------------------------------------------------
//  NotifyMsg(NULL,"mode = %hd",EditMode);
  // erase any current highlighted
  VertexObject    =-1;
  NewObject       =-1;
//if (DrawingRange > 3) DrawingRange = 3;   // so can draw tight stuff   10-1-97 (made option 3-1-98

  hdc = GetDC(hwnd);

  CopyRect(&rc,&gd.rcClient);
  InflateRect(&rc, 1, 1);                   //adjust for fillrect

  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  RealizePalette(hdc);
  SetCapture(hwnd);

  OldEditMode = EditMode;

  // erase current object line
  ResetMapResident(hwnd, FALSE, TRUE, TRUE);// Pics, Object, List

  // erase any line selected (GREEN)
  // also switch to Vertex Mode

  if (EditMode != OBJ_VERTEXES)
  {
      EditMode = OBJ_VERTEXES;            // switch to vertex mode
      ForgetSelection(&SelectList);       // no sector/linedefs count
      FillRect(hdc, &rc, GetStockBrush(BLACK_BRUSH) );
      DrawBigMap( PARTIALNO,hdc, FALSE);
  }

   CurObject = -1;  //++
   OldObject = -1;  //++

   MouseButton = 0;

   EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));        // force arrow always   8-1-97


for (;;)
{
  Instructions("LeftButton=Insert, RightButton=Close/Quit (F1=Help)");

  ForgetSelection( &SLineDefList);    // unselect linedefs/sectors(if any)
  ForgetSelection( &LineDefList);     // unselect linedefs(if any)
  ForgetSelection( &VertexList);      // unselect vertexes(if any)

  //------start line draw-------------------------------------------------

  DrawTheLine(hwnd,hdc,0);                          // 0 = start

  // wait for a button

  Continue = TRUE;
  GetMouseCoords(hwnd,&msg,NULL,&oldPointerX,&oldPointerY,&MouseButton);  //   11-7-96

  while ( Continue)
  {
	  // ----------- move just the map----------------

     MouseProcess = FALSE;
	  CheckMapAndScroll(hwnd, hdc, level,FALSE, FALSE);// sets DidScroll

     WaitMessage();
     KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);  //   11-7-96
     if (CheckEscape(&msg,KeyData) )
     {   MouseButton = 2;
         DrawTheLine(hwnd,hdc,3);            // quit , no close
         MouseButton = 0;
         break;
     }

     if (CheckF1(&msg,KeyData))
     {   DrawingHelp(hwnd);
         continue;
     }

     if (msg.message == WM_CHAR
     ||  msg.message == WM_KEYDOWN)
     {
         if (ProcessMapSizeCommands(hwnd,level,
                                   msg.message,
                                   KeyData,      // The Char Key '1' etc.
                                   FALSE, FALSE))//TRUE)) // no goto, nodrag   8-1-97
         {  DidScroll = TRUE;
            MouseProcess = TRUE;
         }
         else
         {
            switch(KeyData)        //   8-1-97 let keys move too
            {
               case VK_UP:        //up arrow  Y
               case VK_DOWN:      //down arrow Y
               case VK_LEFT:      //left arrow X
               case VK_RIGHT:     // right arrow X
                Deep_OnKey(hwnd, KeyData, TRUE, 0,0);
                DidScroll = TRUE;
                MouseProcess = TRUE;
                break;
            }
         }

         if (msg.message == WM_CHAR)
         {
             if(KeyData == VK_RETURN )
             {
		          MouseButton = 1;
		          DrawTheLine(hwnd,hdc,1);            // process where we are now
		          MouseButton = 0;
                MouseProcess = TRUE;
             }
         }

         if (msg.message == WM_KEYDOWN)
         {
             if(KeyData == VK_INSERT )              //0x5200)
             {
		          MouseButton = 2;
		          DrawTheLine(hwnd,hdc,2);            // close do not add 2-sided
		          MouseButton = 0;
		          Continue    = FALSE;
                MouseProcess = TRUE;
             }
         }

     }

    // ShowCursor puts msgs in the queue!! so don't move unless its for real     3-1-97

     if (!MouseButton
     &&  !MouseProcess
     &&  oldPointerX == PointerX
     &&  oldPointerY == PointerY) continue;

     oldPointerX = PointerX;
     oldPointerY = PointerY;

	  VertexObject = LookForObject(EditMode,DrawingRange);  // find a close vertex to link to

     if (DidScroll) NewObject = CurObject;                 // scroll put up CurObject   8-1-97

	  if (VertexObject >= 0 &&  VertexObject != NewObject || DidScroll)    // do current object + if scrolled   8-1-97
	  {
	      if (NewObject >=0)
	      {   HighlightObject(hdc, EditMode, NewObject, gd.hHiLightPen);   //YELLOW Erase
	      }

		   NewObject = VertexObject;
         HighlightObject(hdc, EditMode, NewObject, gd.hHiLightPen);

		   SoundBeep(SoundClick1);         // 55 different object

		   DisplayObjectInfo(hdc, NewObject);             // info on bottom
	  }
     //static int i=0;
     //Trace(NULL,10,4,"1b %d ",i++);
	  DrawTheLine(hwnd,hdc,1);                           // process where we are now

	  if (MouseButton == 2)    break;

	  if (!DrawingVertex)      break;                    // quit

  }  // while (continue)

  MouseButton = 0;
  if (!DrawingVertex) break;     // quit

} // for (;;)

  ForgetSelection( &SLineDefList);    // unselect linedefs/sectors(if any)
  ForgetSelection( &LineDefList);     // unselect linedefs(if any)
  ForgetSelection( &VertexList);      // unselect vertexes(if any)

  EditMode    = OldEditMode;          // restore editmode

  if (EditMode != OBJ_VERTEXES)
  {   CurObject = -1;
  }

  RedrawMap(hdc, &gd.rcClient, ERASEYES, TRUE);
  SelectPalette(hdc, hOldPal, FALSE);
  ReleaseDC(hwnd, hdc);
  Selecting = FALSE;
  InstructionsOff();

//  NotifyMsg(NULL,"mode = %hd",EditMode);
  ReleaseCapture();                  // moved as last   8-1-97
}

//----------------------------------------------------------------------
// insert vertex  (when = 0 first time
//                        1 next vertex, etc
//                        2 Ins key, close vertex make new sector+2-sided
//                          Ins key, only 2 vertexes ask if split
//                        3 Esc key, close vertex/quit
//----------------------------------------------------------------------
static void DrawingLines (HWND hwnd, HDC hdc, int2 when)                 // process where we are now
{

  static  int2   VertPointerX;                // track drawing of line
  static  int2   VertPointerY;

  static  Bool   Sharing;
  static  int2   FirstNewVertex = 0;               // whether we started on existing vertex
  static  int2   CurNewVertex   = 0;
  static  int2   TotalVertexCount=0;
  static  int2   VertexCount=0;

  static  int2   oldbutton=0;
  static  int2   xorg=0,yorg=0;

  static  int2   LastLineDef=0;

  static  double dx,dy;
  static  int4   distance;
  static  double angle;

  static  int2   FirstVertex=-1;           // first vertex inserted
  static  int2   LastVertex =-1;           // last vertex inserted

  static  int2   findsector;               // sector when drawing
          int2   newsector;

  LevelPtr level = &LevelWad.level;

  LDPtr   TempLineDef;

  Bool    SplitDo;
  SelPtr  cur;
  int4    l,n;
  int2    vstart,vend;
  int4    xVert,yVert;              // was int2 10-1-97
  int2    v;
  int4    ld;
  int2    newline;
  int4    TotalX;
  int4    TotalY;
  int4    TotalV;


//------------this is for saving the boxed area where line lengths are shown

static int2 txtlen;
static char msg[20];

 HPEN     hOldPen = NULL;
 SIZE     size;
 HDC      hTxtDC;
 HBITMAP  hOldBmp;
 HFONT    hOldFont;
 HPALETTE hOldPal;
 RECT     rc;
 RECT     rcBmp = {0,0,16,16}; // size of our Vertex

//---------------------------------------------------------------------

  //------------------------------------------------------------------------
  //  set up origin for vertex move
  //------------------------------------------------------------------------
  //  -- just let the snap-to control all of this, leave it up to user

  hOldFont = SelectFont(hdc, gd.hSmallFont);

  //
  // -- First Time ---
  //
  if  (when == 0)                              // 1st time
  {   oldbutton     = -1;
      findsector    = -1;
      Sharing       = FALSE;                   // not sharing any lines
      DrawingVertex = TRUE;

	   if (DrawingImageBuffer.imagenum == 1 )
      {
	       hTxtDC=CreateCompatibleDC(hdc);
	       hOldBmp=SelectBitmap(hTxtDC, DrawingImageBuffer.hbm);

	       BitBlt( hdc, DrawingImageBuffer.midx, DrawingImageBuffer.midy, DrawingImageBuffer.siz.cx, DrawingImageBuffer.siz.cy, hTxtDC, 0, 0, SRCINVERT);

	       SelectBitmap(hTxtDC, hOldBmp);  // select oldbitmap
	       DeleteDC(hTxtDC);               // delete dc
      }
                                          // Had some problems with the 2nd
                                          // sector - first vertex drawn would
      if(DrawingImageBuffer.hbm != NULL)         // leave a small amount of garbage on
      {   DeleteBitmap(DrawingImageBuffer.hbm);  // the screen at the last vertex pos.
          DrawingImageBuffer.hbm = NULL;         // added the ZeroMem commands  . . .
      }

      ZeroMemory(&DrawingImageBuffer, sizeof(DrawingImageBuffer));
      DrawingImageBuffer.imagenum == -1;

      VertPointerX = !PointerX;                // setup to detect mouse change
      VertPointerY = !PointerY;

      if (SelectCount >=2) ForgetSelection(&SelectList);  // safety drop

      // if any preselected (safety check selectlist is active)
      if (SelectCount && SelectList)
      {   vstart  = SelectList->objnum;            // first vertex active

          MoveOldPointerX = level->Vertexes[vstart].x;    // reference
	       MoveOldPointerY = level->Vertexes[vstart].y;    //

	       xorg = MoveOldPointerX;                  //
          yorg = MoveOldPointerY;
          SelectObject   ( &VertexList,  vstart);  // track old vertex
          VertexCount     = 1;                     // have 1 vertex
          FirstNewVertex  = 0;                     // started on existing vertex
          FirstVertex     = vstart;                // this is 1st
      }
      else
      {   VertexCount =  0;
          FirstVertex = -1;
      }

      TotalVertexCount =  VertexCount;
      LastVertex=FirstVertex;
  }

  //------------------------------------------------------------------------
  //  main vertex insert/move logic
  //------------------------------------------------------------------------

  // Ins (when=2),
  // Esc (when=3) key to close vertex makes button =2

  // see if creating a vertex

  if (MouseButton != 2)                 // not button 2
  {
    if (oldbutton != MouseButton)
    {
        if (MouseButton == 1)           // button 1 create a vertex
        {
	         // anchor the new vertex relative position

	         MoveOldPointerX = MAPX(PointerX);                  //reference
	         MoveOldPointerY = MAPY(PointerY);

	         // if no existing vertex is active - get a new one
	         CurNewVertex = 0;                                 // no new vertex
	         TotalVertexCount++;

	         if (VertexCount == 0)   CheckPointLevel(UndoBuffer, level);

	         if (VertexObject < 0)                              // not existing
	         {
                GridAlign(&MoveOldPointerX,&MoveOldPointerY);  // align
	             InsertObject(level, OBJ_VERTEXES,-1, MoveOldPointerX,MoveOldPointerY);

	             VertexObject = level->NumVertexes-1;           // this is current
	             CurNewVertex = 1;                              // made a vertex

	             // check if this Vertex is on a LineDef

	             for (ld = 0; ld < level->NumLineDefs; ld++)
	             {
		               if (IsLineDefInside(ld,int2(MoveOldPointerX -VertexTolerance),
				   	                          int2(MoveOldPointerY -VertexTolerance),
					                             int2(MoveOldPointerX +VertexTolerance),
					                             int2(MoveOldPointerY +VertexTolerance)))
		               {
		                   // erase old line
                         hOldPen = SelectPen(hdc, gd.hPens[BackColor]);
		                   TempLineDef = &level->LineDefs[ld];
                         //   8-1-97 (removed old)
			                DrawMapVector(hdc, level->Vertexes[TempLineDef->start].x ,
					                                 level->Vertexes[TempLineDef->start].y ,
					                                 level->Vertexes[TempLineDef->end  ].x ,
					                                 level->Vertexes[TempLineDef->end  ].y ,
					                                 VertexArrow,CONVERT  );

		                   newline = Split1LineDef(ld,VertexObject);  // split linedef at vertex, add linedef

		                   // show two sides of line

                         SelectPen(hdc, gd.hPens[DragColor]);      // RED
		                   SetROP2(hdc, R2_XORPEN);         // write mode xor OldRop

		                   TempLineDef = &level->LineDefs[ld];
		                   DrawMapVector(hdc, level->Vertexes[TempLineDef->start].x ,
				                                level->Vertexes[TempLineDef->start].y ,
				                                level->Vertexes[TempLineDef->end  ].x ,
                                            level->Vertexes[TempLineDef->end  ].y ,
				                                VertexArrow,CONVERT  );

		                   TempLineDef = &level->LineDefs[newline];
		                   DrawMapVector(hdc, level->Vertexes[TempLineDef->start].x ,
				                                level->Vertexes[TempLineDef->start].y ,
				                                level->Vertexes[TempLineDef->end  ].x ,
				                                level->Vertexes[TempLineDef->end  ].y ,
				                                VertexArrow ,CONVERT );

		                   //SetROP2(hdc, OldROP);        // write mode put
                         SetROP2(hdc, R2_COPYPEN);        // set writing mode for line drawing

		                   if (VertexCount == 0) FirstNewVertex = 0;// start on existing V

		                   break;
		               }
	             }

	             if (ld >= level->NumLineDefs)                     // no split done
	             {   if (VertexCount == 0) FirstNewVertex = 1;     // started on new vertex
	             }

	         }
	         else
	         {   if (VertexCount == 0) FirstNewVertex = 0;         // start on existing Vertex
	         }

	         MoveOldPointerX = level->Vertexes[VertexObject].x;    // aligned reference
	         MoveOldPointerY = level->Vertexes[VertexObject].y;    //

	         // NEW vertex     (see if made yet)
	         // SelectList contains vertex we already made (so it's only 1 or 2)
	         // Not in list , process
	         if ( !IsSelected( SelectList, VertexObject) )         // see if already in list
	         {

	             if (FirstVertex == -1) FirstVertex = VertexObject; // first new vertex
	             LastVertex = VertexObject;

	             // use these so alignment stays the same

	             xVert = (SCREENX(MoveOldPointerX) - VertexSize/2);
	             yVert = (SCREENY(MoveOldPointerY) - VertexSize/2);

	             // show the one just made if it's not already made.

 		          HDC     BitmapDC = CreateCompatibleDC(hdc);
 		          HBITMAP OldBitmap= SelectBitmap(BitmapDC, hVertexBmp);
                HPALETTE hOldPal = SelectPalette(BitmapDC, gd.hDoomPal, FALSE);

                FillRect(BitmapDC, &rcBmp, gd.hBrushes[VertexColor]);

                BitBlt(hdc, xVert, yVert, VertexSize, VertexSize, BitmapDC, 0, 0, SRCCOPY);

                SelectBitmap(BitmapDC, OldBitmap);	// Restore Old Bitmap Data
                SelectPalette(BitmapDC, hOldPal, FALSE);
                DeleteDC(BitmapDC);						// Delete Temp DC

	             SelectObject   ( &SelectList,  VertexObject);    // add a new vertex to list
	             SelectObject   ( &VertexList,  VertexObject);    // add a new vertex to list

	             VertexCount++;

	             // see if this is first vertex, anchor point

	             if (VertexCount == 1)                        // just started
	             {
                    xorg = MoveOldPointerX;                  // location of cursor
                    yorg = MoveOldPointerY;
                    // look as a THING get were we are starting
                    findsector = GetCurObject (NOCONVERT,OBJ_SECTORS,
				                                             xorg,yorg,xorg,yorg,THINGDRAW);

	             }
	             else
	             {   // This is 2nd vertex - so we need to make a new line

                    // Start was on existing vertex and current is a new one
                    // see where the new vertex is located.

                    if (!FirstNewVertex                    // started on existing Vertex
                    &&  CurNewVertex)                      // made a new vertex
                    {    // look as if a Thing so we get a sector
                        newsector = GetCurObject (NOCONVERT,OBJ_SECTORS,
				                                                MoveOldPointerX,MoveOldPointerY,
				                                                MoveOldPointerX,MoveOldPointerY,THINGDRAW); //FALSE

                        //Trace(0,20,"newsec %hd",newsector);

                        // see where new vertex is
                        // use it if this is a NEW vertex, since it should be correct

                        if (newsector != findsector  &&  newsector >=0)                     // 3-14-96
                        {   findsector = newsector;
                            FirstNewVertex = 2;                // so not 1
                        }
                    }

                    // create LineDefs between the Vertices //
                    // from the first to the second.........

                    vstart = SelectList->objnum;            // 2nd  vertex# (selectflips)
                    vend   = SelectList->next->objnum;      // 1st  vertex#

                    // make sure there is not a linedef already at coordinates
                    // if there is, don't make another one
                    TempLineDef = level->LineDefs;

                    for (v = 0; v < level->NumLineDefs; v++,TempLineDef+=1)
                    {
                         if ((TempLineDef->start == vstart && TempLineDef->end   == vend)
                         ||  (TempLineDef->start == vend   && TempLineDef->end   == vstart) )
                         { break;
                         }
		              }

       	           //  see if v is a new linedef
		              if (v >= level->NumLineDefs)                // new linedef required
		              {   InsertObject(level, OBJ_LINEDEFS, -1, 0, 0);

		                  v = int2(level->NumLineDefs - 1);       // one just added

		                  level->LineDefs[ v].start = vend;       // start v
		                  level->LineDefs[ v].end   = vstart;     // end   v
                        SetTextType(hdc, 5);
		                  DrawScreenTextOutline(hdc,20 ,(ht2*2+6),12,MenuInfoColor," NewLine %4hd ",v);
		                  LastLineDef = v;                        // track for SPLIT later
		              }
		              else
		              {  SetTextType(hdc, 5);
                       DrawScreenTextOutline(hdc,20 ,(ht2*2+6),12,MenuInfoColor," OldLine %4hd ",v);
		                 LastLineDef = -1;
                       Sharing     = TRUE;
		              }

		              ForgetSelection( &SelectList);              // unselect vertexes
		              SelectObject( &SelectList, vstart);         // keep last one
		              VertexCount = 1;

		              MoveOldPointerX = level->Vertexes[vstart].x;  //new reference
		              MoveOldPointerY = level->Vertexes[vstart].y;

	             }  // end select =2

                SetTextType(hdc, 5);
	             DrawScreenTextOutline(hdc,20 ,(ht2+4),12,MenuInfoColor," Sector  %4hd ",findsector);

	         } // valid vertex (created or old one)
	     }  // end button= 1

	     oldbutton = MouseButton;

    }  // end button change

    //---------------------------------------------------------------
    // if have a vertex already, show the line and the length of line
    //---------------------------------------------------------------

    if (VertexCount>0 )
    {   if ( (PointerX !=VertPointerX || PointerY !=VertPointerY) ||  DidScroll )
        {
            XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList

            VertPointerX = int2(PointerX);
	         VertPointerY = int2(PointerY);

            // erase old line (if no scroll occurred)

       	   if (!DidScroll)
        	   {  if (DrawingImageBuffer.imagenum == 1)
	            {
                  // restore prior image  (erase any box surround/restore)
	               if (NewObject >= 0)
		            {   HighlightObject(hdc, EditMode, NewObject, gd.hHiLightPen);//YELLOW);
		            }
	               hTxtDC=CreateCompatibleDC(hdc);

                  if(DrawingImageBuffer.hbm != NULL)
                  {
                     hOldBmp=SelectBitmap(hTxtDC, DrawingImageBuffer.hbm);
                     BitBlt( hdc, DrawingImageBuffer.midx, DrawingImageBuffer.midy,
                                  DrawingImageBuffer.siz.cx, DrawingImageBuffer.siz.cy,
                                  hTxtDC, 0, 0, SRCINVERT);
                     SelectBitmap(hTxtDC, hOldBmp);
                  }

	               DeleteDC(hTxtDC);

		            if (NewObject >= 0)
		            {  HighlightObject(hdc, EditMode, NewObject, gd.hHiLightPen);//YELLOW);
		            }
	            }

               SelectPen(hdc, gd.hDrawPen);     // YELLOW
		         SetROP2(hdc, R2_XORPEN);         // write mode xor OldRop
	            DrawMapVector(hdc, MoveOldPointerX, MoveOldPointerY,xorg,yorg,
				                                       VertexArrow,CONVERT);   // Line with an arrow
               SetROP2(hdc, R2_COPYPEN);        // set writing mode for line drawing

	         }

	         if ( DidScroll)  DidScroll = FALSE;              // reset

            if(DrawingImageBuffer.hbm != NULL)
            {  DeleteBitmap(DrawingImageBuffer.hbm);
               DrawingImageBuffer.hbm = NULL;
            }
            ZeroMemory(&DrawingImageBuffer, sizeof(DrawingImageBuffer));
	         DrawingImageBuffer.imagenum = -1;      // reset prior image where length was

	         // now show current line
            xorg = int2(MAPX(PointerX));
	         yorg = int2(MAPY(PointerY));
        	   GridAlign(&xorg,&yorg);         // align

            SelectPen(hdc, gd.hDrawPen);     // YELLOW
	         SetROP2(hdc, R2_XORPEN);         // write mode xor OldRop
	         DrawMapVector(hdc, MoveOldPointerX, MoveOldPointerY,xorg,yorg,
			    			           VertexArrow,CONVERT);   // Line with an arrow

            SetROP2(hdc, R2_COPYPEN);        // set writing mode for line drawing

	         // compute length of line and show below coordinates
            dx = (double) (xorg -MoveOldPointerX);
	         dy = (double) (yorg -MoveOldPointerY);
	         distance = hypot (dx, dy );     // length of line sqrt( (dx*dx) + (dy*dy) );

            // 3.141592654
            // radians to degrees : radian = pi*deg/180 =     .017453292
            // deg = (radian*180)/pi = 57.2957795
            if (dx != 0.0 || dy != 0.0)
            {   angle = ( atan2(dy,dx)*180.0 )/PI;
            }
            else
            {   angle = 0.0;
            }
            SetTextType(hdc, 5);
	         //DrawScreenTextOutline(hdc,20 ,2,12,MenuInfoColor," length  %4d ",distance);
	         DrawScreenTextOutline(hdc,20 ,2,12,MenuInfoColor," L %4d   %3.0f",distance,angle);

	         // show length when we draw
	         if (ShowDrawLength)
	         {
	             wsprintf(msg,"%d",distance);
	             txtlen = int2(lstrlen(msg));

                SetTextType(hdc, 2);
                GetTextExtentPoint32(hdc, msg, txtlen, &size);  // find length of txt

	             // erase any vertex select box so when we restore we don't get lines
	             if (NewObject >= 0)
	             {   HighlightObject(hdc, EditMode, NewObject, gd.hHiLightPen);//YELLOW);
	             }

	             // save data where length will be displayed
		          DrawingImageBuffer.siz.cx   = size.cx;             // store size
		          DrawingImageBuffer.siz.cy   = size.cy;

                hTxtDC = CreateCompatibleDC(hdc);           // Compat.DC
                                                            // Make Bitmap
                DrawingImageBuffer.hbm = CreateCompatibleBitmap(hdc, size.cx, size.cy);

                rc.left   = rc.top = 0;
                rc.right  = size.cx;
                rc.bottom = size.cy;

	             DrawingImageBuffer.midx = PointerX+12;       //
	             DrawingImageBuffer.midy = PointerY;         //
   	          DrawingImageBuffer.imagenum = 1;
                hOldBmp = SelectBitmap(hTxtDC, DrawingImageBuffer.hbm);
                hOldPal = SelectPalette(hTxtDC, gd.hDoomPal, FALSE);//TRUE);   12-13-96

                FillRect(hTxtDC, &rc, gd.hBrushes[BLACK] );
                SetTextColor(hTxtDC, PALETTEINDEX(GameColors[WHITE]) );
  	             SetBkColor(hTxtDC, RGB(0,0,0));
                SetBkMode (hTxtDC, TRANSPARENT);

                SetTextType(hTxtDC, 2);
                TextOut(hTxtDC, 0, 0, msg, txtlen);

                BitBlt(hdc, DrawingImageBuffer.midx, DrawingImageBuffer.midy,
                            DrawingImageBuffer.siz.cx,  DrawingImageBuffer.siz.cy,
                            hTxtDC, 0, 0, SRCINVERT);

                SelectBitmap(hTxtDC, hOldBmp);
                SelectPalette(hTxtDC, hOldPal, FALSE);
                DeleteDC(hTxtDC);

	             //restore selection
	             if (NewObject >= 0)                   // do current object
	             {   HighlightObject(hdc, EditMode, NewObject, gd.hHiLightPen);//YELLOW);
	             }
	         }
        }
    }
  } // end if(Not button 2
  else
  //-----------------------------------------------------------------
  // mouse button 2 pressed - close
  //-----------------------------------------------------------------
  {
    if (VertexCount>0 )
    {
       SelectPen(hdc, gd.hDrawPen);     // YELLOW
	    SetROP2(hdc, R2_XORPEN);         // write mode xor OldRop

       // erase old line (if no scroll occurred)

       if (!DidScroll)
	    {
	        DrawMapVector(hdc, MoveOldPointerX, MoveOldPointerY,xorg,yorg,
							    VertexArrow,CONVERT);   // Line with an arrow
	    }
	    //SetROP2(hdc, OldROP);
       SetROP2(hdc, R2_COPYPEN);        // set writing mode for line drawing
    }

    ForgetSelection( &LineDefList);      // unselect linedefs/sectors(if any)
    ForgetSelection( &SelectList);       // unselect vertexes (1/2 combo)

    // MouseRelease(&MouseButton);          // wait for final click
    SplitDo     = FALSE;

    if (VertexList )                     // list of vertexes we made
    {  // if normal end close it.

       // make sure vertex didn't get deleted

	    if ( IsSelected( VertexList, FirstVertex) &&  IsSelected( VertexList, LastVertex) )
       {
	         if (when != 3)                               // not ESC ---
	         {
	             if (TotalVertexCount == 2 && when != 2)   // not INS key
	             {
                    ShowCursor(TRUE);                              // for split menu   3-1-97

		              //if(!SplitSector (FirstVertex, LastVertex, LastLineDef) )
		              if(!SplitSector (FirstVertex, LastVertex,0) )  //  7-1-97 don't checkpoint again
		              {  ClosePolygon(hwnd,FirstVertex, LastVertex, TRUE);
		              }

                    ShowCursor(FALSE);

		              SplitDo  = TRUE;    // always here so no flip occurs
	             }
	             else
	             {   if (ClosePolygon(hwnd,FirstVertex,LastVertex,TRUE) ) // from vertex mode
                    {   Sharing = TRUE;
                    }
	             }
	         }
	    }

       // see if the vertex is not used anywhere so delete it

    	 for (cur=VertexList; cur; cur=cur->next)
    	 {
	        v=cur->objnum;

	        TempLineDef = level->LineDefs;

	        for (n = 0; n < level->NumLineDefs; n++, TempLineDef +=1)
	        {
		        if (v == TempLineDef->start
		        ||  v == TempLineDef->end )
	           {    break;
		        }
	        }

	        if (n >= level->NumLineDefs)  // not used anywhere
	        {
     	         xVert = (SCREENX(level->Vertexes[FirstVertex].x) - VertexSize/2);
	    	      yVert = (SCREENY(level->Vertexes[FirstVertex].y) - VertexSize/2);

 		         HDC     BitmapDC = CreateCompatibleDC(hdc);
 		         HBITMAP OldBitmap= SelectBitmap(BitmapDC, hVertexBmp);
               HPALETTE hOldPal = SelectPalette(BitmapDC, gd.hDoomPal, FALSE);

               FillRect(BitmapDC, &rcBmp, gd.hBrushes[VertexColor]);

               BitBlt(hdc, xVert, yVert, VertexSize, VertexSize, BitmapDC, 0, 0, SRCCOPY);

               SelectBitmap(BitmapDC, OldBitmap);	// Restore Old Bitmap Data
               SelectPalette(BitmapDC, hOldPal, FALSE);
               DeleteDC(BitmapDC);						// Delete Temp DC

		         DeleteObject(level, OBJ_VERTEXES, FirstVertex);
	        }
	    }

   	 // if more than 2 vertexes (not split above) make sides/sectors

	    if (! SplitDo)
       {
	        // convert the vertexes made into a linedef list (match with linedef)

	        TempLineDef = level->LineDefs;

	        for (l = 0; l < level->NumLineDefs; l++,TempLineDef+=1)
	        {    if (IsSelected( VertexList, TempLineDef->start)
                && IsSelected( VertexList, TempLineDef->end))
                {   SelectObject( &SelectList,(int2)l);   // get the linedef
                }
	        }

	        // get center of object just made and now find the sector
	        TotalX = 0;
	        TotalY = 0;
	        TotalV = 0;

	        for (cur=VertexList; cur; cur=cur->next)
	        {
                v=cur->objnum;
	             TotalX +=level->Vertexes[v].x;
		          TotalY +=level->Vertexes[v].y;
	             TotalV++;
	        }

	        TotalX /=TotalV;                     // approximate center
	        TotalY /=TotalV;
	        ForgetSelection( &VertexList);       // unselect vertexes

	        // this will be mislead if drawing a sector around another one
           // so we do this only if sharing something

           if (Sharing)                         // connecting to existing
           {
	            findsector = GetCurObject (NOCONVERT,OBJ_SECTORS,
				                             int2(TotalX),int2(TotalY),
			                                int2(TotalX),int2(TotalY),THINGDRAW); //FALSE
           }

	        DrawingCreateSector(hwnd,when,findsector);// make a NEW sector+sidedefs
       }

       ForgetSelection(&VertexList);               // unselect vertexes if not auto
       ForgetSelection(&SelectList);
    }
    else // end if vertex list
    {  DrawingVertex = FALSE;                     // tell caller to quit
    }
  }

  if(hOldPen)  SelectPen (hdc, hOldPen);
  if(hOldFont) SelectFont(hdc, hOldFont);
}





//                                                                            //
//                                                                            //
//       E N D  Routines for Inserting Basic Shapes                           //
//                                                                            //
//       Rectangle, Polygon, Stairs, Door, Line Draw                          //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//       Routines for Dragging Items on the Screen                            //
//                                                                            //

////////////////////////   DRAG   TOOL     /////////////////////////////////////
//                                                                            //
void FindThingsToDrag(LevelPtr level, SelPtr *ThingList)
{
  SelPtr cur;
  VPtr  TempVertex;
  TPtr  TempThing;
  int4  n;
  int2  d;

  int2  DragMinX =  32767;
  int2  DragMinY =  32767;
  int2  DragMaxX = -32767;
  int2  DragMaxY = -32767;

     // find out vertex limit of x/y and find the outer vertices

     for (cur=OldVertexList; cur; cur=cur->next)
     {
       	TempVertex = &level->Vertexes[cur->objnum];

	      if ( TempVertex->x  <  DragMinX )
      	{    DragMinX   = TempVertex->x;
	      }
       	if ( TempVertex->y  <  DragMinY )
      	{    DragMinY   = TempVertex->y;
	      }
      	if ( TempVertex->x  >  DragMaxX )
	      {    DragMaxX   = TempVertex->x;
	      }
	      if ( TempVertex->y  >  DragMaxY )
	      {    DragMaxY   = TempVertex->y;
	      }
     }

     TempThing = level->Things;

     for (n = 0; n < level->NumThings; n++, TempThing+=1)
     {
	  //ignore the Things that are outside the Rectangle enclosing all sectors

	  if (TempThing->xpos < DragMinX || TempThing->xpos > DragMaxX
	   || TempThing->ypos < DragMinY || TempThing->ypos > DragMaxY)
	  {   continue;
	  }

	  d = GetCurObject( NOCONVERT, OBJ_SECTORS,
				       TempThing->xpos,
				       TempThing->ypos,
				       TempThing->xpos,
				       TempThing->ypos,THINGREAL );

	  // if Things is in one of the sectors

	  if (IsSelected(SelectList, d))                   // check list of sectors
	  {
	      SelectObject(ThingList, int2(n));
	  }
     }
}


//==============================================================================
// common exit for dragtool
//==============================================================================

void DragExit(HWND hwnd, HDC hdc, HPALETTE hOldPal, SelPtr *ThingList)
{
  ForgetSelection(ThingList);
  SelectPalette(hdc, hOldPal, FALSE);
  ReleaseDC(hwnd, hdc);
  ReleaseCapture();
  Selecting = FALSE;
  InstructionsOff();

  if (SelectTemp)                      // temp selection   3-1-97
  {   SelectTemp = FALSE;
      ForgetSelection( &SelectList);
  }
}

//------------------------------------------------------------------------------
// Drag some objects around
//
// 1-25-97 bk Added ShowCursor to prevent Cursor from wiping drawn lines !
//--------------------------------------------------------------------------------

void DragTool (HWND hwnd, LevelPtr level)
{
  int2   l;
  int2   sd1,sd2;

  int4   OrgPointerX;       // remember for thing move
  int4   OrgPointerY;

  int4   ToolPointerX;
  int4   ToolPointerY;
  int4   OldPointerX= -1;
  int4   OldPointerY= -1;

  LDPtr  TempLineDef;

  int2   MoveMode;
  Bool   DidMove;
  Bool   Redraw=FALSE;   //   3-1-97
  UINT   KeyData;
  SelPtr NewSel   = NULL;
  SelPtr ThingList= NULL;

  HDC      hdc;
  MSG      msg;
  HPALETTE hOldPal;

//----------------------------------------------------------------------------------
#define MYVERTEX  -1         //internal use move vertex

  if (SelectList == NULL && CurObject < 0 ) return;      // nothing to do

  SetCapture(hwnd);

//  NotifyMsg(NULL,"select1 = %d",SelectCount);

  DidMove   = FALSE;               // init not done moving
  SelectTemp= FALSE;               //   3-1-97 make sure false, till chk it all

  hdc     = GetDC(hwnd);
  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  RealizePalette(hdc);

  CheckPointLevel(UndoBuffer, level);   // checkpoint now

  if (EditMode == OBJ_THINGS  ||  EditMode == OBJ_VERTEXES)
  {   MoveMode = EditMode;
  }
  else
  {   MoveMode = MYVERTEX;                    //
  }

  // set current origin for move to be done later

  OrgPointerX = (MAPX(PointerX));        // origin reference point
  OrgPointerY = (MAPY(PointerY));

  MoveObjectsToCoords( MoveMode, NULL, OrgPointerX,               // ORIGIN
				                           OrgPointerY, GridScale);

  // erase old stuff

  Instructions("Selecting Objects to Drag - Wait");

  EraseOldMap(hdc, &SelectList,&SelectTemp,2);  // creates select list as required+lengths

  if (!SelectList)
  {   DragExit(hwnd, hdc, hOldPal, &ThingList);
      return;                                  // none?? should not happen
  }

  if (EditMode == OBJ_SECTORS && DragThings == TRUE)
  {
     Instructions("Selecting Things to Drag - Wait");
     FindThingsToDrag(level,&ThingList );  // stores all things in Thinglist
  }

  Instructions("When done, Press C to Clear Selected Objects, F1=Help");

  //----------------------------------------------------------------------------
  MouseButton = 2;

  while(MouseButton & 2) // Mouse 2 button not up yet
  {  WaitMessage();
     KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);

     if (CheckEscape(&msg,KeyData))
     {
         CancelMove();                                     // cancel any move in progress
         ForgetSelection(&ThingList);

         if (DidMove)
         {   ResetMapResident(hwnd,TRUE,FALSE,RESETSELECT);// reset pics on bottom and selectlists
             RedrawMap(NULL, &gd.rcClient, ERASEYES, FALSE);   //   3-1-97
         }
         else
         {   DisplayNewMap(hwnd, hdc, &SelectList,&SelectTemp); // show new object locations
         }
         DragExit(hwnd, hdc, hOldPal, &ThingList);
		   return;
     }

     if (CheckF1(&msg,KeyData))
     {   DragHelp(hwnd);
         continue;
     }


	  // ----------- move just the map----------------
     if (msg.message == WM_CHAR
     ||  msg.message == WM_KEYDOWN)
     {   ProcessMapSizeCommands(hwnd,level,
                                msg.message,
                                KeyData,               // The Char Key '1' etc.
                                FALSE, TRUE);          // no goto, drag mode

     }

	  CheckMapAndScroll(hwnd, hdc, level, TRUE, FALSE); //drag is active, no redraw Map

	  // ----------- move objects ----------------

	  if (PointerX !=OldPointerX || PointerY !=OldPointerY)
	  {
        XYUpdate(hwnd, PointerX,PointerY);         //   12-18-96 update our position and SelectList

	     OldPointerX = PointerX;
	     OldPointerY = PointerY;

	     ToolPointerX = MAPX(PointerX);
	     ToolPointerY = MAPY(PointerY);

	     // test if move will take place --
	     if (MoveObjectsToCoords( 0,SelectList, ToolPointerX,
                                               ToolPointerY, GridScale))
	     {   DidMove = TRUE;
		      ShowLength = 1;                       // erase old lengths

            ShowCursor(FALSE);

		      DrawLittleMap(hdc, SelectList,3);     //  erase old

            //  -- move the objects --

		      MoveObjectsToCoords(MoveMode, OldVertexList,
                                          ToolPointerX,
				                              ToolPointerY, GridScale);

		      // reset current origin again -------

		      MoveObjectsToCoords(MoveMode, NULL, ToolPointerX,
				                                    ToolPointerY, GridScale);

		      ShowLength = 2;                       // show new length
		      DrawLittleMap(hdc, SelectList,3);     // draw new
		      ShowLength = 0;

            ShowCursor(TRUE);
	     }

	  }       // if (PointerX != OldPointerX)

  }          // End  while(mouse button 2 not up )

  //----------------------------------------------------------------------------

  Instructions("Moving Objects, Please Wait");

  DisplayNewMap(hwnd, hdc, &SelectList,&SelectTemp);       // show new object locations
  if(EditMode == OBJ_THINGS)  Redraw = TRUE;

  // if nothing done (0 displacement)
  //  keep things as they are, keep list
  if (!DidMove)
  {    DragExit(hwnd, hdc, hOldPal, &ThingList);
       return ;                                 // nothing moved!!
  }

  //if (SelectCount > 50    // temp test   8-1-97 might want something like this
  //&&  EditMode != OBJ_THINGS)
  //{
  //    if ( !Confirm( "Check Lines for overlap?") )
  //    {   DragExit(hwnd, hdc, hOldPal, &ThingList);
  //        return;
  //    }
  //}

  // wrong bg color causes bad  redisplay   3-1-97 else won't need next code
  // temp fix till that's figured out (see Edit, drawbigmap)

  //  if (GridShown && GridScale)   //<<<<<<<<<< temp hack till fixed
  //  {   Redraw = TRUE;            //   3-1-97 it's fixed I hope
  //  }

  // check for vertex overlap

  switch(EditMode)
  {  //  -- VERTEXES --
     case (OBJ_VERTEXES):
   	if (SelectList != NULL)
	   {    if(AutoMergeVertices( &SelectList,NOWAIT)) // confirmed = FALSE
              Redraw=TRUE;
	   }
      else
      if (CurObject >= 0)
    	{   SelectObject( &NewSel, CurObject);

	       if(AutoMergeVertices( &NewSel,NOWAIT))    // confirmed = FALSE
             Redraw=TRUE;

	       ForgetSelection  ( &NewSel);
	   }
      break;
     //  -- LINEDEFS --
     case(OBJ_LINEDEFS):
      // see if any added vertices are on top of linedefs
      // store all the vertices that belong to this
   	if (SelectList != NULL)
	   {
	       if(AutoMergeLineDefVertices( &SelectList, WAIT)) //
             Redraw = TRUE;
	   }
    	else
	   if (CurObject >= 0)
    	{
	       SelectObject(&NewSel,CurObject);
	       if(AutoMergeLineDefVertices( &NewSel, WAIT))
             Redraw = TRUE;

	       ForgetSelection  ( &NewSel);
     	}
    	break;
     //  -- SECTORS --
     case OBJ_SECTORS:
	   // find the sectors and the linedefs associated with sectors
	   if (SelectList != NULL)      // only for SelectList
      {
	     TempLineDef = level->LineDefs;

	     for (l = 0; l < level->NumLineDefs; l++, TempLineDef+=1)
	     {
	      	sd1= TempLineDef->sidedef1;

		      if (sd1 >= 0 && IsSelected( SelectList, level->SideDefs[sd1].sector))
		      {   SelectObject( &NewSel, l);
		      }
		      else
		      {  sd2 = TempLineDef->sidedef2;

		         if (sd2 >= 0 && IsSelected( SelectList, level->SideDefs[sd2].sector))
		         {   SelectObject( &NewSel, l);
		         }
		      }
	     }
	   }
	   else
	   if (CurObject >= 0)
	   {
	     TempLineDef = level->LineDefs;

	     for (l = 0; l < level->NumLineDefs; l++,TempLineDef+=1)
	     {
	    	 sd1= TempLineDef->sidedef1;

	    	 if (sd1 >= 0 && level->SideDefs[sd1].sector== CurObject )
		    {   SelectObject( &NewSel, l);
		    }
		    else
	    	 {  sd2 = TempLineDef->sidedef2;

		       if (sd2 >= 0 && level->SideDefs[sd2].sector== CurObject )
		       {   SelectObject( &NewSel, l);
		       }
	       }
	     }
	   }

	   if(AutoMergeLineDefVertices( &NewSel, WAIT))  // 8-1-97   was NOWAIT
         Redraw = TRUE;

	   if (DragThings == TRUE && DidMove)
	   {   // set origin
	       MoveObjectsToCoords( OBJ_THINGS, NULL,
					                            OrgPointerX,
					                            OrgPointerY, GridScale);

	       if (ThingList)                   // moving things in sectors to
	       {                                // final destination

        	     MoveObjectsToCoords( OBJ_THINGS, ThingList,
						                             ToolPointerX,
						                             ToolPointerY, GridScale);
	     	     Redraw = TRUE;            // things moved, redisplay
	       }
	   }

     	ForgetSelection  ( &NewSel);
	   break;
     default:
     	break;
  }
  /*
  if(EditMode == OBJ_THINGS)
  {  if(SelectList)           // Highlight the Select List
     {   HighlightSelection(hdc, EditMode, SelectList);
     }
     else                     // Try to find a Current Object to Highlight
     {   HighlightObject(hdc, EditMode, CurObject, gd.hHiLightPen);
     }
  } */

//  NotifyMsg(NULL,"selectexit = %d",SelectCount);
  DragExit(hwnd, hdc, hOldPal, &ThingList);

  if( Redraw )// hdc,  rect,    erase bk, palette,
  {   RedrawMap(NULL, &gd.rcClient, ERASEYES, FALSE);
  }
//  else    3-1-97 ???
//  { RedrawMap(NULL, &gd.rcClient, TRUE, FALSE);
//  }

}

//                                                                            //
//                                                                            //
//       E N D  Routines for Dragging                                         //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////




//----------------------------------------------------------------------//
//   rotate things direction only
//----------------------------------------------------------------------//
void RotateThings (HWND hwnd, int2 userdirection)//, int2 RotateColor )
{
  LevelPtr level = &LevelWad.level;

  SelPtr cur;

  int2   n;
  int2   direction;
  HDC    hdc;
  HPALETTE hOldPal;

//--------------------------------------------------------------------------

  hdc     = GetDC(hwnd);
  hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
  RealizePalette(hdc);

  direction = userdirection * 45;

  EraseOldMap(hdc, &SelectList,&SelectTemp,0);  // direct back, creates list
  if (!SelectList) return;                      // nothing

  for (cur = SelectList; cur; cur = cur->next)
  {    n = cur->objnum;
       // don't rotate polyobjects
       if (GameVersion == 4)
       {
	       if ( level->Things[ n].type >= 3000
            && level->Things[ n].type <= 3002 )
	       {  continue;
	       }
       }

       level->Things[ n].angle  +=direction;
       if ( level->Things[n].angle >=360 ) level->Things[n].angle =    0;
       if ( level->Things[n].angle < 0   ) level->Things[n].angle  = 315;

       DisplayThingObject (hdc, n );
   }

  DisplayNewMap(hwnd, hdc, &SelectList,&SelectTemp);//objTemp);          //++

  if (SelectTemp)                                   // temp selection   3-1-97
  {   SelectTemp = FALSE;                           // although this should
	   ForgetSelection( &SelectList);                // not occur since selectlist is set in DeeP.cpp now
  }

  SelectPalette(hdc, hOldPal, FALSE);
  ReleaseDC(hwnd, hdc);
}

///////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------
//  process mouse movement and keys for rotate/scale commands
//  return   0 = nothing done
//           1 = data to change map
//           2 = save and new command
//           3 = cancel
//---------------------------------------------------------------------
int2 ProcessRotateCommands(HWND hwnd, HDC hdc, LevelPtr level,
                           UINT KeyWaiting, UINT KeyType,
                           int2 *Button,
                           int4 *RotateCurPointerX, int4 *RotateCurPointerY,
                           int2  CurMode,  double *angle , double *scale)
{
    static double scaleinc = .01;
    static double angleinc = 1.0;

    int4   dx,dy;
    MSG    msg;
    Bool   Data = 0;
    Bool   DragMode = TRUE;

//-----------------------------------------------------------------------

    if (CurMode == OBJ_THINGS)           // not rotate thing (V,L,S only)
    {   DragMode = FALSE;
    }

    // detect and key strokes and process
    if (!KeyWaiting)
    {    KeyWaiting = GetMouseCoords(hwnd,&msg,NULL, &PointerX, &PointerY, Button);
         if (KeyWaiting)
         {   Data = 1;
         }
    }
    else
    {   msg.message        = KeyType;
        (UINT) msg.wParam  = KeyWaiting;
        Data = 1;
    }

    if (CheckEscape(&msg,KeyWaiting))
	 {   return 3;
    }

          dy = *RotateCurPointerY - PointerY;
          dx = *RotateCurPointerX - PointerX;

          if ( dx || dy )   // movement
          {  Data = 1;
	          if ( (dx +dy) > 0 )
	          {    *angle += 1.0;
	          }
	          else
	          {    *angle -= 1.0;
	          }
          }

       if (msg.message == WM_CHAR
       ||  msg.message == WM_KEYDOWN)
       {
          switch( KeyWaiting)//(UINT)msg.wParam )
          {
	         // change scale increment
	         case VK_SPACE:
	           if (scaleinc ==.01)
	           {   scaleinc = .04;
		            angleinc = 4.0;
	           }
	           else
	           {   scaleinc = .01;
		            angleinc = 1.0;
	           }
	           break;

            case VK_RETURN:
	           *Button = 1;
	           return 2;

            case '+':
            case '=':
	           if (gd.ShiftDown)   // works   11-10-96
              {  ProcessMapSizeCommands(hwnd,level,
                                        msg.message,
                                        KeyWaiting,
                                        TRUE, DragMode); // no goto, drag
	           }
              else
	           {   *scale += scaleinc; //.01;
	           }
              break;
            case '-':
            case '_':
	           if (gd.ShiftDown)   // works 11-10-96
              {  ProcessMapSizeCommands(hwnd,level,
                                        msg.message,
                                        KeyWaiting,
                                        TRUE, DragMode); // no goto, drag
	           }
              else
	           {   *scale -= scaleinc; //.01;
	           }
              break;
            case '<':                //  11-5-96
            case ',':
              *angle +=angleinc;
		        break;
            case '>':               //  11-5-96
            case '.':
              *angle -=angleinc;
		        break;
            case VK_F1:
              RotateHelp(hwnd);
		        break;
            default:
              ProcessMapSizeCommands(hwnd,level,
                                     msg.message,
                                     KeyWaiting,                     // The Char Key '1' etc.
                                     TRUE, DragMode);                // no goto, drag
              break;
          }

	    }

    if (*scale < .00) *scale  = .00;

    if (Data)
    {
       if (*angle >  360.0) *angle -=  360.0;
       if (*angle < -360.0) *angle +=  360.0;

       SetTextType(hdc, 3);
       DrawScreenTextOutline(hdc, 10 ,1,23,MenuInfoColor,"   Angle: %3hd,  Scale: %3hd%  ",-(int2)*angle,(int2)(*scale*100.0));
    }

    if (*Button == 2) Data = 1;

    return Data;
}


//----------------------------------------------------------------------//
//   rotate and scale a group of objects around the center of gravity
//----------------------------------------------------------------------//

void RotateAndScaleTool(HWND hwnd, int2 objtype, SelPtr objSlist, Bool DoClear)
{
   LevelPtr level = &LevelWad.level;
   LDPtr  TempLineDef;
   TPtr   TempThing;
   VPtr   TempVertex;

   SelPtr WrkVertices = NULL;
   SelPtr EntryList;               // original list at entry   3-1-97 was moved??

   int2   sd1,sd2;
   int2   sector1,sector2;

   int2   n, s;
   int2   vstart,vend;

static  double centerx=0.0, centery=0.0;
static  double angle = 0.0;

   double dx, dy;
   double accx=0.0, accy=0.0;
   double num = 0.0;
   double scale = 1.0;

   SelPtr cur  ;
   SelPtr curv ;
   SelPtr vertices   = NULL;

   SelPtr ThingList= NULL;         // list of things to move in sector rotate

   int4   RotateCurPointerX;
   int4   RotateCurPointerY;

   int4   dragx,dragy;
   int4   xnew,ynew;
   int4   oldx,oldy;

   int2   status;
   int2   Xwork,Ywork;

   HDC      hdc;
   RECT     rc;
   HPALETTE hOldPal;
   HFONT    hOldFont;
   MSG      msg1;
   UINT    KeyWaiting;
   UINT    KeyType;
   int4    CursorType;

//--------------------------------------------------------------------------

   if (!objSlist) return;

   level->MadeChanges++;
   CopyRect(&rc, &gd.rcClient);
   InflateRect(&rc, 1, 1);          // adjust for fillrect

   hdc     = GetDC(hwnd);
   hOldPal = SelectPalette(hdc, gd.hDoomPal, FALSE);
   RealizePalette(hdc);

   hOldFont = SelectFont(hdc, gd.hStockFont);//GetStockFont(ANSI_VAR_FONT));

   //Trace(hwnd, 400, 6, "Pal=%d, Font = %d, DC=%d", hOldPal, hOldFont, hdc);

   // always use DrawLittleMap with 0 as 2nd parm

   // DoClear = ERASE old stuff if first time entry from editobj

   EntryList  = objSlist;                  // copy original list

   if( DoClear)
   {
      angle = 0.0;
      CheckPointLevel(UndoBuffer, level); // ------ CHECKPOINT ---

      // This creates at least Selectlist (1 entry)
      EraseOldMap(hdc, &SelectList,&SelectTemp,0);
      if (!SelectList) return;                    // nothing

      if (EditMode == OBJ_SECTORS && DragThings == TRUE)
      {   Instructions("Selecting Things to Drag - Wait");
          FindThingsToDrag(level,&ThingList );
      }

      EntryList  = SelectList;                    // now this is the list
      Instructions("Rotate Tool <> = Rotate ,+ - = Scale, F1=Help");  // hides mouse+redo
   }

   switch (objtype)
   {  //------------------------THINGS ------------ (special)
      case OBJ_THINGS:
        break;
      //------------------------------------------------------------------
      // main workhorse, does all the scaling and rotating
      //------------------------------------------------------------------
      case OBJ_VERTEXES:
        // --------get center of this vertexes ---------------------------

        for (cur = EntryList; cur; cur = cur->next) // all the vertices
        {   n = cur->objnum;
            TempVertex = &level->Vertexes[n];
	         accx += (double) TempVertex->x;         // sum of X coord
	         accy += (double) TempVertex->y;         // sum of Y coord
	         num++;                                  // count
        }

        centerx = accx / num;                       // halfway . average
        centery = accy / num;

        MouseButton = 0;
        KeyWaiting = GetMouseCoords(hwnd,&msg1,NULL, &PointerX,&PointerY,&MouseButton);
        KeyType    = msg1.message;

        RotateCurPointerX = PointerX;
        RotateCurPointerY = PointerY;

	     while (MouseButton != 1)
	     {  // current mouse position
	        status = ProcessRotateCommands(hwnd, hdc, level,
                      KeyWaiting,KeyType,
                      &MouseButton,
					       &RotateCurPointerX, &RotateCurPointerY,
					       objtype, &angle, &scale);

	        if (status == 3) break;
	        if (status == 2) MouseButton = 1;     // save and exit

	        if (status==1)
	        {
	           ShowLength = 1;                     //erase
	           DrawLittleMap(hdc,EntryList, 3);    // erase old

	           // Loop through list of original vertex values and
              // ------ note that OldVertexXY is in same sequence as original
	           // stuff change in corresponding vertex

	           cur = EntryList;                // first

	           for (curv = OldVertexXY; curv; curv = curv->next)
	           {
	             dx   = (double)curv->objnum - centerx;       // x offset from center
	             curv = curv->next;
	             dy   = (double)curv->objnum - centery;       //  y offset from center

	             RotateAndScaleCoords( &dx, &dy, angle, scale);  // rotate checks for 0

	             Xwork = centerx + dx;
	             Ywork = centery + dy;

	             if (GridScale && GridSnap)
	             {
		             if (Xwork < 0)
		             {   Xwork= (Xwork - GridScale/2)/GridScale * GridScale;
		             }
		             else
		             {   Xwork= (Xwork + GridScale/2)/GridScale * GridScale;
		             }
		             if (Ywork < 0)
		             {   Ywork= (Ywork - GridScale/2)/GridScale * GridScale;
		             }
		             else
		             {   Ywork= (Ywork + GridScale/2)/GridScale * GridScale;
		             }
	             }

	             n  = cur->objnum;

	             level->Vertexes[ n ].x = Xwork;
	             level->Vertexes[ n ].y = Ywork;

	             cur = cur->next;               // next vertex
	           }

	           ShowLength = 2;                   // new -save old then show
	           DrawLittleMap(hdc,EntryList, 3);  // new map
	           ShowLength = 0;                   //

              SetCursorPos(SCREENX((int4)centerx), SCREENY((int4)centery)+gd.CaptionHeight); // set back

	           KeyWaiting = GetMouseCoords(hwnd,&msg1,NULL, &RotateCurPointerX,  &RotateCurPointerY, &MouseButton);
              KeyType    = msg1.message;
	           xnew = RotateCurPointerX;
	           ynew = RotateCurPointerY;
	           oldx = RotateCurPointerX;
	           oldy = RotateCurPointerY;

	           // current position to detect movement
              //KeyWaiting = 0;
              CursorType = 0;

	           while ( MouseButton!=1
	                && oldx == xnew
	                && oldy == ynew )
              {
                 if (KeyWaiting) break;
	              //WaitMessage();
	              KeyWaiting = GetMouseCoords(hwnd,&msg1,NULL, &RotateCurPointerX,  &RotateCurPointerY, &MouseButton);
                 KeyType    = msg1.message;
                 if (KeyWaiting) break;

	              xnew = RotateCurPointerX;
		           ynew = RotateCurPointerY;

                 // drag the map if button 2

                 if ( (MouseButton & 2)
                 &&    CursorType !=2)
                 {
                    EnableCursor(hwnd, gd.hCursorMove);
                    CursorType = 2;
                 }

		           if (MouseButton & 2 )
		           {
		              dragx = oldx - xnew;          //xorg
		              dragy = oldy - ynew;          //yorg

		              if (dragx || dragy)
		              {
		                  OrigX +=dragx/Scale;
		                  OrigY -=dragy/Scale;

                        SetCursorPos(SCREENX((int4)centerx), SCREENY((int4)centery)); // set back
		                  KeyWaiting= GetMouseCoords(hwnd,&msg1,NULL, &RotateCurPointerX,  &RotateCurPointerY, &MouseButton);// 
                        KeyType   = msg1.message;
		                  xnew = RotateCurPointerX;
		                  ynew = RotateCurPointerY;
		                  oldx  = xnew;
		                  oldy  = ynew;

                        FillRect(hdc, &rc, gd.hBrushes[BackColor]);
		                  DrawBigMap( PARTIALNO,hdc,FALSE);

		                  DrawLittleMap (hdc, EntryList,2 );    // 2 erase old area
		                  DrawLittleMap (hdc, EntryList,5 );    // 5 draw new solid black
		                  DrawLittleMap (hdc, EntryList,3 );    // 3 draw newcoordinates

		              } //end dragx

	              }   // end button2
              }

              if (CursorType == 2)
              {   EnableCursor(hwnd, (HCURSOR)LoadCursor(0, IDC_ARROW));
              }

	        }
	     }

        if (MouseButton == 1)
        {   level->MadeMapChanges = TRUE;
	         AutoMergeVertices( &EntryList, WAIT);        // check for overlap
        }

	     break;
   //--------------------------------------------------------------------------
   // linedefs
   // NOTE this routine must be the same as in Edit
   //--------------------------------------------------------------------------
   case OBJ_LINEDEFS:
   // objlist
	for (cur  = EntryList; cur; cur = cur->next)
	{    n = cur->objnum;                            // linedef number

        TempLineDef = &level->LineDefs[n];

	     vstart = TempLineDef->start;
	     vend   = TempLineDef->end;

	     if (!IsSelected( vertices, vstart))
	         SelectObject( &vertices,vstart);

	     if (!IsSelected( vertices, vend))
	         SelectObject( &vertices, vend);
	}

   if(hOldPal)      SelectPalette(hdc, hOldPal, FALSE);
   if(hOldFont)     SelectFont(hdc, hOldFont);
   if(hdc)          ReleaseDC(hwnd, hdc);

	RotateAndScaleTool(hwnd, OBJ_VERTEXES, vertices,CLEARNO);//, angle, scale);
	ForgetSelection( &vertices);
	break;

   //--------------------------------------------------------------------------
   // sectors (convert to linedefs and then vertices)
   // NOTE this routine must be the same as in Edit
   //--------------------------------------------------------------------------
   case OBJ_SECTORS:
	TempLineDef = level->LineDefs;

	for (n = 0; n < level->NumLineDefs; n++, TempLineDef +=1)
	{
       sd1 = TempLineDef->sidedef1;
	    sd2 = TempLineDef->sidedef2;
       sector1 = -3;
       sector2 = -3;
       if ( sd1 >= 0) sector1 = level->SideDefs[sd1].sector;
       if ( sd2 >= 0) sector2 = level->SideDefs[sd2].sector;

       // current list of sectors

	    for (cur = EntryList; cur; cur = cur->next)
	    {
           s = cur->objnum;                          // current sector

	       if (s == sector1
	       ||  s == sector2)
	       {
		        if (!IsSelected ( vertices,  TempLineDef->start)) // don't have
		            SelectObject( &vertices, TempLineDef->start); // yet addit

		        if (!IsSelected ( vertices,  TempLineDef->end))   // don't have
		            SelectObject( &vertices, TempLineDef->end);   // yet addit

              break;
	       }
	    }
   }
   SelectPalette(hdc, hOldPal, FALSE);
   SelectFont(hdc, hOldFont);
   ReleaseDC(hwnd, hdc);
	RotateAndScaleTool(hwnd, OBJ_VERTEXES, vertices,CLEARNO);

	ForgetSelection( &vertices);
	break;
   }

   // if this is primary entry and we are now exiting

   if( DoClear)
   {   if (MouseButton == 1)                      // rotate was OK
       {   // if Sector mode and drag things, do it
           if (ThingList)                         // moving things in sectors to
           {                                      // final destination
               ForgetSelection( &OldVertexXY);
               ForgetSelection( &OldVertexList);
               ForgetSelection( &SLineDefList);
               ForgetSelection( &LineDefList);     // unselect temp LineDefs

               // build vertex list for all things (use static value of entry!!)

               for ( cur= ThingList; cur; cur=cur->next)
               {    n = cur->objnum;
                    TempThing = &level->Things[n];

	            SelectObject( &WrkVertices,TempThing->xpos);
	            SelectObject( &WrkVertices,TempThing->ypos);
               }

	            // put in correct sequence to match original Thing select sequence

	            for (cur = WrkVertices; cur; cur = cur->next)
	            {    SelectObject( &OldVertexXY, cur->objnum);
	            }

	            ForgetSelection( &WrkVertices);

               cur = ThingList;            // this points to THINGS matches vertexes

               for ( curv= OldVertexXY; curv; curv=curv->next)
               {
	                dx   = (double)curv->objnum - centerx;       // x offset from center
	                curv = curv->next;
	                dy   = (double)curv->objnum - centery;       //  y offset from center

	                RotateAndScaleCoords( &dx, &dy, angle, scale);// rotate checks for 0

	                n  = cur->objnum;                  // Thing number

	                level->Things[ n ].xpos =(int2) ( centerx + dx  );
	                level->Things[ n ].ypos =(int2) ( centery + dy  );
	                cur = cur->next;                    // next Thing
	            }
	            //Redraw = TRUE;                      // things moved, redisplay
	        }
           else
           {   DisplayNewMap(hwnd, hdc,&SelectList,&SelectTemp);// original code (this only)
           }
       }
       else
       {   CancelMove();
       }

       if (SelectTemp)                      // temp selection
       {   ForgetSelection( &SelectList);
           SelectTemp = FALSE;
       }

       ForgetSelection(&ThingList);

       // created by DisplayLittleMap
       ForgetSelection( &OldVertexXY);
       ForgetSelection( &OldVertexList);
       ForgetSelection( &SLineDefList);
       ForgetSelection( &LineDefList);      // unselect temp LineDefs
  }

   SelectPalette(hdc, hOldPal, FALSE);
   SelectFont(hdc, hOldFont);
   ReleaseDC(hwnd, hdc);
   InstructionsOff();

}


//                                                                            //
//                                                                            //
//       E N D  Routines for Rotating                                         //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// process autotag commands
// uses last tag number (if available) and assigns it to current stuff
//------------------------------------------------------------------------------

void AutoTag (HWND hwnd)
{
    int2 oldeditmode = EditMode;

    LevelPtr level= &LevelWad.level;
    SelPtr cur;
    SelPtr OldSelect=SelectList;            // save old list
    HDC hdc;
    HPALETTE hOldPal;
    MSG msg;

    UINT KeyData;
//----------------------------------------------------------------------

    ForgetSelection( &TempList);            // safety
    hdc = GetDC(hwnd);
    hOldPal  = SelectPalette(hdc, gd.hDoomPal, FALSE);
    RealizePalette(hdc);

    if  (SelectList)
    {    TempList = SelectList;
    }
    else
    if (CurObject >= 0)
    {   SelectObject( &TempList, CurObject);
    }

    if (!TempList) return;

    SelectList = NULL;                      // drop current list (reset at exit)

    LastTagUsed=FindFreeTag() ;             // assign a new one

    if (EditMode == OBJ_LINEDEFS)
    {
        Instructions("Select Sector to have the same Tag   F1=Help");
        for (cur = TempList; cur; cur = cur->next)
        {    level->LineDefs[ cur->objnum].tag  = LastTagUsed;
        }

        level->MadeChanges++;
        EditMode = OBJ_SECTORS;

    }
    else
    if (EditMode == OBJ_SECTORS)
    {
        Instructions("Select LineDef to have the same Tag   F1=Help");
        for (cur = TempList; cur; cur = cur->next)
        {    level->Sectors[ cur->objnum].tag    = LastTagUsed;
        }

        level->MadeChanges++;
        EditMode = OBJ_LINEDEFS;
    }

    // release list
    if  (!OldSelect)
    {    ForgetSelection( &TempList);         // get rid of old list
    }
    else
    {    TempList = NULL;                     // reset since this is Selectlist
    }

    ResetMapResident(hwnd,TRUE,FALSE,FALSE);       // reset display oldtextures, but not object numbers

    FillRect(hdc, &gd.rcClient, gd.hBrushes[BackColor] );
    DrawBigMap(PARTIALNO, hdc,FALSE);              // draw //   11-5-96
    CurObject = LookForObject(EditMode,Range);
    DisplayObjectInfo(hdc, CurObject);            // info on bottom

    // look for next object to tag to
    MouseButton = 0;
    DidScroll = FALSE;

    while ( MouseButton !=1 || CurObject < 0 )
    {
         // ----------- move just the map----------------

         CheckMapAndScroll(hwnd, hdc, level,FALSE, FALSE);     // no dragging, sets DidScroll
         //if( CheckMapAndScroll(hwnd, hdc, level,FALSE, FALSE) )     // no dragging
         //{   DidScroll = TRUE;
         //}

         CurObject = LookForObject(EditMode,Range);
         if (CurObject != OldObject)
         {
             DisplayObjectInfo(hdc, CurObject);            // info on bottom

	          if (OldObject >= 0 )                          // clear old object 
	          {   HighlightObject(hdc, EditMode, OldObject, gd.hHiLightPen);//YELLOW);
             }

             //  SoundBeep(BeePFrequencyBrowse, 5);
             OldObject = CurObject;

    	       if (CurObject >= 0 )                             // do current object
             {   HighlightObject( hdc, EditMode, CurObject, gd.hHiLightPen);//YELLOW);
             }
         }

         // detect and key strokes and process

         KeyData = GetMouseCoords(hwnd,&msg,NULL,&PointerX,&PointerY,&MouseButton);
         if (CheckEscape(&msg,KeyData))
         {   MouseButton = 2;
         }
         if (CheckF1(&msg,KeyData))
         {   TagHelp(hwnd);
             continue;
         }

         if (msg.message == WM_CHAR
         ||  msg.message == WM_KEYDOWN)
          {
            ProcessMapSizeCommands(hwnd,level,
                                   msg.message,
                                   KeyData,                     // The Char Key '1' etc.
                                   FALSE, FALSE);               // no goto, nodrag   8-1-97
          }

          if (msg.message == WM_CHAR)
          {   if(KeyData == VK_RETURN )
              {  MouseButton = 1;
              }
          }

          if (MouseButton == 2) break;

    }  // END while(MouseButton != 1)

    // ----------------------------------------------------------------------
    if (MouseButton == 1 &&  CurObject >=0 )
    {
        // assign tag to new object

        if (EditMode == OBJ_LINEDEFS)
        {   level->LineDefs[ CurObject].tag  = LastTagUsed;
        }
        else
        {   level->Sectors[ CurObject].tag   = LastTagUsed;
	}

	level->MadeChanges++;
    }

    EditMode    = oldeditmode;                // back to prior mode
    SelectList  = OldSelect;                  // old list

    ResetMapResident(hwnd,TRUE,FALSE,FALSE);       // reset display oldtextures, but not object numbers
    CurObject   = LookForObject(EditMode,Range);

    RedrawMap(hdc, &gd.rcClient, ERASEYES, TRUE);
    SelectPalette(hdc, hOldPal, FALSE);
    ReleaseDC(hwnd, hdc);
    InstructionsOff();
}



//-------------------------------------------------------------------------
// merge any linedefs made (new 8.70) // put back orig   11-11-96
//-------------------------------------------------------------------------

void MergeTool(LevelPtr level,Bool Merge)
{
static int4 oldNumLineDefs;  // static since this is set, then used

   int4    n;
   SelPtr OldSelect;

   if (!Merge)
   {
       oldNumLineDefs = level->NumLineDefs;    // set starting # lines
   }
   else
   if (level->NumLineDefs > oldNumLineDefs)
   {    // only for linedefs do we merge
        if (EditMode == OBJ_LINEDEFS )
        {   ForgetSelection(&SelectList);      // get rid of old linedef list
        }
        else
        {   OldSelect = SelectList;            // remember old list
            SelectList = NULL;                 // so we can redo it
        }

        // select linedefs now

        for (n = oldNumLineDefs; n < level->NumLineDefs; n++)
        {    SelectObject( &SelectList,(int2) n);
        }

        if (ExpertMode)
        {   AutoMergeLineDefVertices( &SelectList, NOWAIT);  // don't ask
        }
        else
        {   AutoMergeLineDefVertices( &SelectList, WAIT);    // ask
        }

        if (EditMode != OBJ_LINEDEFS )
        {   ForgetSelection(&SelectList);      // get rid of old linedef list
            SelectList = OldSelect;            // remember old list
        }
        else
        {   CurObject    = level->NumLineDefs - 1;
            OldObject    = -1;
        }

        CalculateMinMax(level);//   new map limits
   }
}

//============================================================================
// For number boxes to invoke calculator automatically, later pass answer back
//============================================================================
Bool IsItCalculator(HWND hDlg,int TextID)
{
   char NumberText[9];

   GetDlgItemText(hDlg, TextID, NumberText,2);

   if (!strnicmp(NumberText,"A",1))
   {   Calculator(hDlg,NumberText);
       return TRUE;
   }
   return FALSE;
}

//////////////////////////////////////////////////////////////////////////////
//  The Deep INPUT Calculator Text Dialog Procedure
//////////////////////////////////////////////////////////////////////////////

#pragma argsused
BOOL CALLBACK CalculatorDlgProc(DIALOG_PARAMS)
{
  static char String   [34] ="";
  static char AnswerTxt[34] ="";
  int4  i;
  int4  ActiveCount;
  int2  parmlen;
  int4  Answer;
  char *option;
  char *startloc;
  char *endloc;

  int4  lastnumber;
  int4  numbers[24] ={0};
  char  Parms[24][12];
  char  nextfunc;

// LPDLG2OPTS lpd2;	// Our Long Pointer to Dialog Input 2 char strings later maybe

  ScreenSaverTimer = 0;                     // reset screensaver timer   8-1-97

   switch( msg )
   {
      case WM_INITDIALOG:
      {                                         //  Save string pointer passed here
			//SetWindowLong(hDlg, DWL_USER, lParam); // store our Pointer
         //lpd2 = (LPDLG2OPTS)lParam;				   // grab it for our use

         //Static_SetText( GetDlgItem( hDlg, stc1 ), lpd2->sz5 ); // Heading
         //Static_SetText( GetDlgItem( hDlg, stc2 ), lpd2->sz6 ); // Prompt

			//Edit_SetText( GetDlgItem( hDlg, edt1 ), lpd2->sz3 );   // init string
			Edit_SetText( GetDlgItem( hDlg, edt1 ), String );       // init string
         Static_SetText( GetDlgItem( hDlg, stc1 ), AnswerTxt );  // answer
         return TRUE;
      }

      // By request; exit dialogs on right button up
      case WM_RBUTTONUP:
   	  PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDCANCEL,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);//   6-1-97
        break;
      //   6-1-97    this is ok 7-1-97
      case WM_LBUTTONUP:
        PostMessage(hDlg,WM_COMMAND,MAKEWPARAM(IDOK,(WORD)GetWindowLong(hDlg, GWL_ID)), 0);
        break;

      case WM_COMMAND:

       //lpd2 = (LPDLG2OPTS)GetWindowLong(hDlg, DWL_USER);  	// grab our pointer

       switch LOWORD(wParam)
       {
         case IDCANCEL:
          EndDialog( hDlg, IDCANCEL );
          return IDCANCEL;

         case IDOK:                              // buffer  // # of chars
			 //Edit_GetText( GetDlgItem( hDlg, edt1 ), lpd2->sz3, lpd2->i1+1 );   // init string GetWindowText has bug   1-5-97 needs+1 in length
			 Edit_GetText( GetDlgItem( hDlg, edt1 ), String, 34 );   // init string GetWindowText has bug   1-5-97 needs+1 in length
          //EndDialog( hDlg, IDOK );
          Answer   = 0;          // zero total
          nextfunc ='\0';        // default to add

          i = 0;
          option = String;

          for (;;)
          {   memset (Parms[i],0,10);

              while (isspace( option[0]) ) option++;  // eat leading spaces
	           if (strlen (option) <=0 ) break;        // nothing
	           startloc = option;                      // starting loc

	           while (option[0]
	               && option[0] >= '0'
	               && option[0] <= '9')
              {    option++;                          // next loc
	           }

	           endloc  = option;
	           parmlen = endloc - startloc;

	           if (parmlen == 0 )                      // no number
	           {   if (!option[0]) break;              // end

	               if (option[0] == '+'
	               ||  option[0] == '-'
	               ||  option[0] == '/'
	               ||  option[0] == '*' )
	               {
		                memcpy(Parms[i],option,1);
		                i++;
	               }
	               option++;                          // next loc
	           }
	           else
	           {
	              if (parmlen > 9 ) continue;

	              strncpy(Parms[i],startloc,parmlen);
	              numbers[i] =atoi(Parms[i]);
	              i++;
	           }
	           if (i >=24) break;
          }

          ActiveCount = i;
          for (i=0; i< ActiveCount; i++)
          {
	          //Trace(0,i*20,"1 %s",Parms[i]);

	          switch(Parms[i][0])
	          {
	            case('+'):
	            case('-'):
	            case('*'):
	            case('/'):
		           nextfunc = Parms[i][0];
		           break;
	            default:
		           lastnumber = numbers[i];

	     	        switch(nextfunc)
	              {
		             default:
		             case('+'):
		               Answer +=lastnumber;
		               break;
		             case('-'):
		               Answer -=lastnumber;
		               break;
		             case('*'):
		               Answer *=lastnumber;
		               break;
		             case('/'):
		               if (lastnumber !=0)
		               {   Answer /=lastnumber;
		               }
		               break;
		           }
		           break;
	          }
          }

          //  show current value--------------------------------------
          sprintf (AnswerTxt,"Answer = %-14d   [%04X]",Answer,Answer);   // 3-1-98 hex
          Static_SetText( GetDlgItem( hDlg, stc1 ), AnswerTxt ); // answer
          return IDOK;
		 }
       break;
   }
   return FALSE ;
}

//--------------------------------------------------------------------------------
// calculator
//--------------------------------------------------------------------------------
#pragma argsused
void Calculator (HWND hwnd,char *TxtNumber)
{
   DialogBoxParam( gd.hInst, "CalculatorTxt", hwnd, CalculatorDlgProc, (LPARAM)0 );
}

//------------------------------------------------------------------------//
// end of file
//------------------------------------------------------------------------//

